SystemOrganization addCategory: #'Monticello-Base'!
SystemOrganization addCategory: #'Monticello-Chunk Format'!
SystemOrganization addCategory: #'Monticello-Environments'!
SystemOrganization addCategory: #'Monticello-Loading'!
SystemOrganization addCategory: #'Monticello-Merging'!
SystemOrganization addCategory: #'Monticello-Modeling'!
SystemOrganization addCategory: #'Monticello-Patching'!
SystemOrganization addCategory: #'Monticello-Repositories'!
SystemOrganization addCategory: #'Monticello-Storing'!
SystemOrganization addCategory: #'Monticello-UI'!
SystemOrganization addCategory: #'Monticello-Versioning'!

ProtoObject subclass: #MCInfoProxy
	instanceVariableNames: 'info repository workingCopy'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Repositories'!

!MCInfoProxy commentStamp: 'cmm 8/12/2013 21:51' prior: 0!
A MCInfoProxy takes the place of a MCVersionInfo with a large tree of ancestors that consume memory in the image, but are almost never accessed.  If they are, however, I will dynamically retrieve and become the Info from the original MC repository which includes the full ancestry tree.!

!MCInfoProxy class methodsFor: 'instance creation' stamp: 'cmm 8/16/2013 14:13'!
info: aMCVersionInfo workingCopy: aMCWorkingCopy repository: aMCRepository
	^ self new
		setInfo: aMCVersionInfo
		workingCopy: aMCWorkingCopy
		repository: aMCRepository! !

!MCInfoProxy methodsFor: 'private' stamp: 'abc 8/14/2013 12:58'!
becomeAncestry
	^ self becomeForward: (MCProxyMaterialization signalMaterializing: self)! !

!MCInfoProxy methodsFor: 'private' stamp: 'cmm 8/12/2013 21:25'!
doesNotUnderstand: aMessage
	^ aMessage sendTo: self becomeAncestry! !

!MCInfoProxy methodsFor: 'testing' stamp: 'cmm 8/16/2013 13:47'!
isMCInfoProxy
	^ true! !

!MCInfoProxy methodsFor: 'private' stamp: 'cmm 8/16/2013 14:45'!
materializeInfo
	workingCopy ancestry breadthFirstAncestorsDo:
		[ : each | (repository versionWithInfo: each) ifNotNil:
			[ : ver | ^ ver info allAncestorsDo:
				[ : eachAncestor | eachAncestor = info ifTrue: [ ^ eachAncestor ] ] ] ].
	nil error: 'Expected ' , info asString , ' to be an ancestor of one of ' , workingCopy ancestors asString! !

!MCInfoProxy methodsFor: 'initialize-release' stamp: 'cmm 8/16/2013 14:14'!
setInfo: aMCVersionInfo workingCopy: aMCWorkingCopy repository: aMCRepository 
	info := aMCVersionInfo.
	workingCopy := aMCWorkingCopy.
	repository := aMCRepository! !

ProtoObject subclass: #MCPackageInEnvironment
	instanceVariableNames: 'package environment'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Environments'!

!MCPackageInEnvironment commentStamp: 'jr 2/27/2017 23:05' prior: 0!
I am a decorator for an MCPackage, activating an Environment for relevant operations.

Instance Variables
	environment:		<Environment> should be the current one for some of my operations
	package:		<MCPackage> my substance!

!MCPackageInEnvironment class methodsFor: 'instance creation' stamp: 'jr 2/4/2017 00:16'!
decorating: aPackage in: anEnvironment
	| instance |
	instance := self new.
	instance initializeWithPackage: aPackage in: anEnvironment.
	^ instance! !

!MCPackageInEnvironment methodsFor: 'object behavior' stamp: 'jr 2/4/2017 00:18'!
basicInspect
	"Create and schedule an Inspector in which the user can examine the 
	receiver's variables. This method should not be overriden."
	^ToolSet basicInspect: self! !

!MCPackageInEnvironment methodsFor: 'delegating' stamp: 'jr 2/4/2017 00:03'!
doesNotUnderstand: aMessage
	^ aMessage sendTo: package! !

!MCPackageInEnvironment methodsFor: 'accessing' stamp: 'jr 2/4/2017 00:02'!
environment

	^ environment! !

!MCPackageInEnvironment methodsFor: 'accessing' stamp: 'jr 2/4/2017 00:02'!
environment: anObject

	environment := anObject! !

!MCPackageInEnvironment methodsFor: 'initialize-release' stamp: 'jr 2/28/2017 02:52'!
inEnvironment: anEnvironment
	environment == anEnvironment ifTrue: [^ self].
	^ MCPackageInEnvironment decorating: package in: anEnvironment! !

!MCPackageInEnvironment methodsFor: 'initialize-release' stamp: 'jr 2/4/2017 00:03'!
initializeWithPackage: aPackage in: anEnvironment
	package := aPackage.
	environment := anEnvironment.! !

!MCPackageInEnvironment methodsFor: 'delegating' stamp: 'jr 2/21/2017 22:44'!
respondsTo: aSymbol
	^ (MCPackageInEnvironment canUnderstand: aSymbol)
		or: [package respondsTo: aSymbol]! !

!MCPackageInEnvironment methodsFor: 'input/output' stamp: 'jr 2/4/2017 00:08'!
snapshot
	^ environment beCurrentDuring: [package snapshot]! !

Model subclass: #MCTool
	instanceVariableNames: 'morph label modalProcess modalValue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-UI'!

MCTool subclass: #MCCodeTool
	instanceVariableNames: 'items environmentInDisplayingImage'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-UI'!

!MCCodeTool commentStamp: 'nk 11/10/2003 22:00' prior: 0!
MCCodeTool is an abstract superclass for those Monticello browsers that display code.
It contains copies of the various CodeHolder methods that perform the various menu operations in the method list.
!

!MCCodeTool methodsFor: 'menus' stamp: 'ul 12/12/2009 14:10'!
adoptMessageInCurrentChangeset
	"Add the receiver's method to the current change set if not already there"

	self selectedClassOrMetaClass ifNotNil: [ :cl |
		self selectedMessageName ifNotNil: [ :sel |
			ChangeSet current adoptSelector: sel forClass: cl.
			self changed: #annotations ]]
! !

!MCCodeTool methodsFor: 'subclassResponsibility' stamp: 'nk 11/10/2003 22:01'!
annotations
	"Build an annotations string for the various browsers"
	^''! !

!MCCodeTool methodsFor: 'menus' stamp: 'mt 3/5/2020 15:07'!
browseClassHierarchy
	"Create and schedule a class list browser on the receiver's hierarchy."

	self systemNavigation
		spawnHierarchyForClass: self selectedClassOrMetaClass
		selector: self selectedMessageName	"OK if nil"! !

!MCCodeTool methodsFor: 'menus' stamp: 'ar 3/5/2010 20:57'!
browseFullProtocol
	"Open up a protocol-category browser on the value of the receiver's current selection.    If in mvc, an old-style protocol browser is opened instead.  Someone who still uses mvc might wish to make the protocol-category-browser work there too, thanks."

	| aClass |

	(Smalltalk isMorphic and: [Smalltalk hasClassNamed: #Lexicon]) ifFalse: [^ self spawnFullProtocol].
	(aClass := self selectedClassOrMetaClass) ifNotNil:
		[(Smalltalk at: #Lexicon) new openOnClass: aClass inWorld: ActiveWorld showingSelector: self selectedMessageName]! !

!MCCodeTool methodsFor: 'menus' stamp: 'nk 11/10/2003 21:26'!
browseMessages
	"Present a menu of the currently selected message, as well as all messages sent by it.  Open a message set browser of all implementors of the selector chosen."

	self systemNavigation browseAllImplementorsOf: (self selectedMessageName ifNil: [ ^nil ])! !

!MCCodeTool methodsFor: 'menus' stamp: 'fbs 9/13/2013 17:30'!
browseMethodFull
	"Create and schedule a full Browser and then select the current class and message, or just the current class if viewing a class definition."
	^ self selectedClassOrMetaClass ifNotNil: [ :cls |
		self selectedMessageName
			ifNil: [ToolSet browseClass: cls]
			ifNotNil: [:sel | ToolSet browse: cls selector: sel]]! !

!MCCodeTool methodsFor: 'menus' stamp: 'nk 11/10/2003 20:54'!
browseSendersOfMessages
	"Present a menu of the currently selected message, as well as all messages sent by it.  Open a message set browser of all senders of the selector chosen."

	self systemNavigation browseAllCallsOn: (self selectedMessageName ifNil: [ ^nil ])! !

!MCCodeTool methodsFor: 'menus' stamp: 'eem 1/7/2016 11:20'!
browseVersions
	"Create and schedule a message set browser on all versions of the currently selected message selector."
	(ToolSet
		browseVersionsOf: self selectedClassOrMetaClass
		selector: self selectedMessageName) ifNil: [self changed: #flash]! !

!MCCodeTool methodsFor: 'menus' stamp: 'mt 3/5/2020 15:14'!
classListMenu: aMenu 

	aMenu addList: #(
		-
		('browse full (b)'			browseMethodFull)
		('browse hierarchy (h)'		browseClassHierarchy)
		('browse protocol (p)'		browseFullProtocol)
"		-
		('printOut'					printOutClass)
		('fileOut'					fileOutClass)
"		-
		('show hierarchy'			methodHierarchy)
"		('show definition'			editClass)
		('show comment'			editComment)
"
"		-
		('inst var refs...'			browseInstVarRefs)
		('inst var defs...'			browseInstVarDefs)
		-
		('class var refs...'			browseClassVarRefs)
		('class vars'					browseClassVariables)
		('class refs (N)'				browseClassRefs)
		-
		('rename class ...'			renameClass)
		('copy class'				copyClass)
		('remove class (x)'			removeClass)
"
		-
		('find method...'				findMethodInChangeSets)).
							
	^aMenu! !

!MCCodeTool methodsFor: 'menus' stamp: 'nice 6/10/2017 17:32'!
copyReference
	"Copy the selected selector to the clipboard"
	self selectedMessageName ifNotNil:
		[ : selector | Clipboard clipboardText:
			(self selectedClassOrMetaClass
				ifNil: [ selector asString ]
				ifNotNilDo: [ : cls | (cls >> selector) reference ]) ]! !

!MCCodeTool methodsFor: 'menus' stamp: 'nk 11/10/2003 20:55'!
copySelector
	"Copy the selected selector to the clipboard"

	| selector |
	(selector := self selectedMessageName) ifNotNil:
		[Clipboard clipboardText: selector asString]! !

!MCCodeTool methodsFor: 'accessing' stamp: 'jr 1/14/2017 22:00'!
environmentInDisplayingImage
	^ environmentInDisplayingImage ifNil: [Smalltalk globals]! !

!MCCodeTool methodsFor: 'accessing' stamp: 'jr 12/13/2016 16:40'!
environmentInDisplayingImage: anEnvironment
	environmentInDisplayingImage := anEnvironment! !

!MCCodeTool methodsFor: 'menus' stamp: 'eem 12/30/2017 16:28'!
fileOutMessage
	"Put a description of the selected message on a file"

	| fileName |
	self selectedMessageName ifNotNil:
		[Cursor write showWhile:
			[self selectedClassOrMetaClass fileOutMethod: self selectedMessageName].
		 ^self].
	items isEmpty ifTrue:
		[^self].
	fileName := UIManager default saveFilenameRequest: 'File out on which file?' initialAnswer: 'methods'.
	fileName isEmptyOrNil ifTrue: [^self].
	Cursor write showWhile:
		[| definitionStream removalInitStream |
		definitionStream := WriteStream on: (String new: 1000).
		removalInitStream := WriteStream on: (String new: 100).
		definitionStream header; timeStamp.
		items do:
			[:patchOp| | def |
			def := patchOp definition.
			def isMethodDefinition ifTrue:
				[(def actualClass notNil
				  and: [def actualClass includesSelector: def selector])
					ifTrue:
						[def actualClass
							printMethodChunk: def selector
							withPreamble: true
							on: definitionStream
							moveSource: false
							toFile: nil.
						(def selector == #initialize and: [def classIsMeta]) ifTrue:
							[removalInitStream nextChunkPut: def className, ' initialize'; cr]]
					ifFalse:
						[removalInitStream nextChunkPut: def className, (def classIsMeta ifTrue: [' class'] ifFalse: ['']), ' removeSelector: ', def selector printString; cr]].
			def isClassDefinition ifTrue:
				[def actualClass
					ifNotNil:
						[definitionStream nextChunkPut: def actualClass definition.
						 def comment ifNotNil:
							[def actualClass organization
								putCommentOnFile: definitionStream
								numbered: 1
								moveSource: false
								forClass: def actualClass]]
					ifNil:
						[removalInitStream nextChunkPut: def className, ' removeFromSystem'; cr]]].
		definitionStream nextPutAll: removalInitStream contents.
		FileStream writeSourceCodeFrom: definitionStream baseName: fileName isSt: true useHtml: false]! !

!MCCodeTool methodsFor: 'menus' stamp: 'pre 8/22/2019 15:02'!
findMethodInChangeSets
	"Find and open a changeSet containing the current method."

	| aName |
	(aName := self selectedMessageName) ifNotNil: [
		ChangeSorter 
			browseChangeSetsWithClass: self selectedClassOrMetaClass
			selector: aName]! !

!MCCodeTool methodsFor: 'accessing' stamp: 'ab 7/16/2003 14:36'!
items
	^ items! !

!MCCodeTool methodsFor: 'menus' stamp: 'nk 7/30/2004 17:56'!
methodHierarchy
	"Create and schedule a method browser on the hierarchy of implementors."

	self systemNavigation methodHierarchyBrowserForClass: self selectedClassOrMetaClass
		selector: self selectedMessageName! !

!MCCodeTool methodsFor: 'menus' stamp: 'mt 3/5/2020 15:14'!
methodListKey: aKeystroke from: aListMorph 
	aKeystroke caseOf: {
		[$b] -> [self browseMethodFull].
		[$h] -> [self browseClassHierarchy].
		[$p] -> [self browseFullProtocol].
		[$o] -> [self fileOutMessage].
		[$c] -> [self copySelector].
		[$C] -> [self copyReference].
		[$n] -> [self browseSendersOfMessages].
		[$m] -> [self browseMessages].
		[$i] -> [self methodHierarchy].
		[$v] -> [self browseVersions]}
		 otherwise: []! !

!MCCodeTool methodsFor: 'menus' stamp: 'mt 3/5/2020 15:14'!
methodListMenu: aMenu
	"Build the menu for the selected method, if any."
	
	self selectedMessageName
	ifNil: [items notEmpty ifTrue:
		[aMenu addList:#(
			('browse full (b)' 						browseMethodFull)
			('fileOut (o)'								fileOutMessage))]]
	ifNotNil: [
	aMenu addList:#(
			('browse full (b)' 						browseMethodFull)
			('browse hierarchy (h)'					browseClassHierarchy)
			('browse protocol (p)'					browseFullProtocol)
			-
			('fileOut (o)'								fileOutMessage)
			('printOut'								printOutMessage)
			('copy selector (c)'						copySelector)
			('copy reference (C)'					copyReference)).
		aMenu addList: #(
			-
			('browse senders (n)'						browseSendersOfMessages)
			('browse implementors (m)'					browseMessages)
			('inheritance (i)'						methodHierarchy)
			('versions (v)'							browseVersions)
		('change sets with this method'			findMethodInChangeSets)
"		('x revert to previous version'				revertToPreviousVersion)"
		('remove from current change set'		removeFromCurrentChanges)
"		('x revert & remove from changes'		revertAndForget)"
		('add to current change set'				adoptMessageInCurrentChangeset)
"		('x copy up or copy down...'				copyUpOrCopyDown)"
"		('x remove method (x)'					removeMessage)"
		"-"
		).
	].
"	aMenu addList: #(
			('x inst var refs...'						browseInstVarRefs)
			('x inst var defs...'						browseInstVarDefs)
			('x class var refs...'						browseClassVarRefs)
			('x class variables'						browseClassVariables)
			('x class refs (N)'							browseClassRefs)
	).
"
	^ aMenu
! !

!MCCodeTool methodsFor: 'menus' stamp: 'nk 11/10/2003 20:55'!
perform: selector orSendTo: otherTarget 

	"Selector was just chosen from a menu by a user. If can respond, then  
	perform it on myself. If not, send it to otherTarget, presumably the  
	editPane from which the menu was invoked."

	(self respondsTo: selector)
		ifTrue: [^ self perform: selector]
		ifFalse: [^ super perform: selector orSendTo: otherTarget]! !

!MCCodeTool methodsFor: 'menus' stamp: 'nk 11/10/2003 20:58'!
printOutMessage
	"Write a file with the text of the selected message, for printing by a web browser"

	self selectedMessageName ifNotNil: [
		self selectedClassOrMetaClass fileOutMethod: self selectedMessageName
							asHtml: true]! !

!MCCodeTool methodsFor: 'menus' stamp: 'nk 11/10/2003 21:00'!
removeFromCurrentChanges
	"Tell the changes mgr to forget that the current msg was changed."

	ChangeSet current removeSelectorChanges: self selectedMessageName 
			class: self selectedClassOrMetaClass.
	self changed: #annotations! !

!MCCodeTool methodsFor: 'subclassResponsibility' stamp: 'nk 11/10/2003 22:02'!
selectedClass
	"Answer the class that is selected, or nil"
	self subclassResponsibility! !

!MCCodeTool methodsFor: 'subclassResponsibility' stamp: 'nk 11/10/2003 22:02'!
selectedClassOrMetaClass
	"Answer the class that is selected, or nil"
	self subclassResponsibility! !

!MCCodeTool methodsFor: 'subclassResponsibility' stamp: 'nk 11/10/2003 22:02'!
selectedMessageCategoryName
	"Answer the method category of the method that is selected, or nil"
	self subclassResponsibility! !

!MCCodeTool methodsFor: 'subclassResponsibility' stamp: 'nk 11/10/2003 22:02'!
selectedMessageName
	"Answer the name of the selected message"
	self subclassResponsibility! !

!MCCodeTool methodsFor: 'initialize-release' stamp: 'cmm 10/2/2013 17:08'!
setItems: aCollection
	items := aCollection! !

MCCodeTool subclass: #MCOperationsBrowser
	instanceVariableNames: 'selection reverts'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-UI'!

MCOperationsBrowser subclass: #MCMergeBrowser
	instanceVariableNames: 'conflicts merger ok'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-UI'!

!MCMergeBrowser class methodsFor: 'instance creation' stamp: 'ab 7/18/2003 17:52'!
resolveConflictsInMerger: aMerger
	| inst |
	inst := self new merger: aMerger.
	^ inst showModally ifNil: [false]! !

!MCMergeBrowser methodsFor: 'ui' stamp: 'tfel 7/25/2016 15:22'!
buttonSpecs
	^ #((Merge merge 'Proceed with the merge' canMerge)
		 (Cancel cancel 'Cancel the merge')
		('All Newer' chooseAllNewerConflicts 'Choose all newer conflict versions')
		('All Older' chooseAllOlderConflicts 'Choose all older conflict versions')
		('Rest Reject' chooseAllUnchosenLocal 'Choose local versions of all remaining conflicts')
		('Rest Accept' chooseAllUnchosenRemote 'Choose remote versions of all remaining conflicts')
		('Accept same source' chooseAllSameAST 'Choose all local conflicting versions that have essentially the same code')
)! !

!MCMergeBrowser methodsFor: 'testing' stamp: 'ab 7/22/2003 00:51'!
canMerge
	^ merger isMerged! !

!MCMergeBrowser methodsFor: 'actions' stamp: 'ab 7/18/2003 17:52'!
cancel
	self answer: false! !

!MCMergeBrowser methodsFor: 'actions' stamp: 'mt 8/3/2016 17:09'!
chooseAllNewerConflicts
	conflicts do: [ :ea | ea chooseNewer ].
	self changed: #text; changed: #list; changed: #canMerge.! !

!MCMergeBrowser methodsFor: 'actions' stamp: 'mt 8/3/2016 17:09'!
chooseAllOlderConflicts
	conflicts do: [ :ea | ea chooseOlder ].
	self changed: #text; changed: #list; changed: #canMerge.! !

!MCMergeBrowser methodsFor: 'actions' stamp: 'mt 8/3/2016 17:09'!
chooseAllSameAST
	conflicts do: [ :ea |
		ea chooseSameAST ].
	self changed: #text; changed: #list; changed: #canMerge.! !

!MCMergeBrowser methodsFor: 'actions' stamp: 'mt 8/3/2016 17:09'!
chooseAllUnchosenLocal
	conflicts do: [ :ea | ea isResolved ifFalse: [ ea chooseLocal ] ].
	self changed: #text; changed: #list; changed: #canMerge.! !

!MCMergeBrowser methodsFor: 'actions' stamp: 'mt 8/3/2016 17:09'!
chooseAllUnchosenRemote
	conflicts do: [ :ea | ea isResolved ifFalse: [ ea chooseRemote ] ].
	self changed: #text; changed: #list; changed: #canMerge.! !

!MCMergeBrowser methodsFor: 'actions' stamp: 'mt 8/3/2016 17:09'!
chooseLocal
	self conflictSelectionDo:
		[selection chooseLocal.
		self changed: #text; changed: #list; changed: #canMerge].
	self selectNextUnresolvedConflict! !

!MCMergeBrowser methodsFor: 'actions' stamp: 'mt 8/3/2016 17:10'!
chooseRemote
	self conflictSelectionDo:
		[selection chooseRemote.
		self changed: #text; changed: #list; changed: #canMerge].
	self selectNextUnresolvedConflict! !

!MCMergeBrowser methodsFor: 'actions' stamp: 'mt 8/3/2016 17:10'!
clearChoice
	self conflictSelectionDo:
		[selection clearChoice.
		self changed: #text; changed: #list; changed: #canMerge]! !

!MCMergeBrowser methodsFor: 'private-actions' stamp: 'nice 9/6/2010 22:59'!
conflictSelectionDo: aBlock
	self selectionIsConflicted
		ifTrue: aBlock! !

!MCMergeBrowser methodsFor: 'ui' stamp: 'ab 7/19/2003 21:31'!
defaultLabel
	^ 'Merge Browser'! !

!MCMergeBrowser methodsFor: 'menus' stamp: 'ab 7/18/2003 18:04'!
getConflictMenu: aMenu
	selection remoteChosen
		ifTrue: [aMenu add: 'undo keep change' target: self selector: #clearChoice]
		ifFalse: [aMenu add: 'keep change' target: self selector: #chooseRemote].
	selection localChosen
		ifTrue: [aMenu add: 'undo reject change' target: self selector: #clearChoice]	
		ifFalse: [aMenu add: 'reject change' target: self selector: #chooseLocal].
	^ aMenu! !

!MCMergeBrowser methodsFor: 'morphic ui' stamp: 'ab 7/18/2003 18:07'!
getMenu: aMenu
	selection ifNil: [^ aMenu].
	^ self selectionIsConflicted
		ifTrue: [self getConflictMenu: aMenu]
		ifFalse: [self getOperationMenu: aMenu]! !

!MCMergeBrowser methodsFor: 'menus' stamp: 'ab 7/18/2003 16:37'!
getOperationMenu: aMenu
	^ aMenu! !

!MCMergeBrowser methodsFor: 'private-ui' stamp: 'nice 9/22/2010 00:43'!
innerButtonRow
	^ self buttonRow: self innerButtonSpecs! !

!MCMergeBrowser methodsFor: 'private-ui' stamp: 'mt 2/5/2016 15:26'!
innerButtonSpecs
	^
		#((Accept chooseRemote 'Accept the selected incoming change. Overwrites local code.' )
		  (Reject chooseLocal 'Reject the selected incoming change. Retains local code.' ))! !

!MCMergeBrowser methodsFor: 'accessing' stamp: 'cmm 10/2/2013 17:34'!
items
	^ conflicts, super items! !

!MCMergeBrowser methodsFor: 'actions' stamp: 'ab 7/18/2003 17:52'!
merge
	merger isMerged
		ifFalse: [self inform: 'You must resolve all the conflicts first']
		ifTrue: [self answer: true] ! !

!MCMergeBrowser methodsFor: 'accessing' stamp: 'ul 3/13/2017 00:32'!
merger: aMerger
	merger := aMerger.
	items := aMerger operations sorted.
	conflicts := aMerger conflicts sort: [:a :b | a operation <= b operation].! !

!MCMergeBrowser methodsFor: 'menus' stamp: 'nice 9/22/2010 01:02'!
methodListKey: aKeystroke from: aListMorph 
	aKeystroke caseOf: {
		[$k] -> [self chooseRemote].
		[$r] -> [self chooseLocal]}
		 otherwise: [super methodListKey: aKeystroke from: aListMorph ]! !

!MCMergeBrowser methodsFor: 'actions' stamp: 'nice 9/6/2010 23:09'!
selectNextUnresolvedConflict
	"Try and select the next unresolved conflict starting at current selection in the list, wrapping at beginning.
	If there is no more unresolved conflict, then simply move to next non conflict item in the list.
	This method makes assumption that conflicts are always sorted before non conflicts items."
	(self findListMorph: #list)
		ifNotNil:
			[:aMorph |
			| currentIndex nextUnresolvedIndex |
			currentIndex := aMorph getCurrentSelectionIndex min: conflicts size.
			nextUnresolvedIndex := (currentIndex + 1 to: currentIndex + conflicts size - 1)
				detect:
					[:i |
					| nextItem |
					((nextItem := conflicts atWrap: i) isKindOf: MCConflict)
						and: [nextItem isResolved not]]
				ifNone: [0].
			nextUnresolvedIndex = 0
				ifTrue: [items size > 1 ifTrue: [self selection: (aMorph getCurrentSelectionIndex max: conflicts size)  - conflicts size \\ items size + conflicts size + 1]]
				ifFalse: [self selection: nextUnresolvedIndex - 1 \\ conflicts size + 1].].! !

!MCMergeBrowser methodsFor: 'testing' stamp: 'ab 7/18/2003 18:07'!
selectionIsConflicted
	^ selection isKindOf: MCConflict! !

!MCMergeBrowser methodsFor: 'ui' stamp: 'mt 4/30/2015 13:59'!
widgetSpecs
	Preferences annotationPanes ifFalse: [ ^#(
		((buttonRow) (0 0 1 0) (0 0 0 defaultButtonPaneHeight))
		((listMorph:selection:menu:keystroke: list selection methodListMenu: methodListKey:from:) (0 0 1 0.4) (0 defaultButtonPaneHeight 0 0))
		((innerButtonRow) (0 0.4 1 0.4) (0 0 0 defaultButtonPaneHeight))
		((textMorph: text) (0 0.4 1 1) (0 defaultButtonPaneHeight 0 0))
		)].

	^ {
		#((buttonRow) (0 0 1 0) (0 0 0 defaultButtonPaneHeight)).
		#((listMorph:selection:menu:keystroke: list selection methodListMenu: methodListKey:from:) (0 0 1 0.4) (0 defaultButtonPaneHeight 0 0)).
		#((innerButtonRow) (0 0.4 1 0.4) (0 0 0 defaultButtonPaneHeight)).
		{ #(textMorph: annotations). #(0 0.4 1 0.4). { 0. self defaultButtonPaneHeight. 0. self defaultButtonPaneHeight+self defaultAnnotationPaneHeight. }}.
		{ #(textMorph: text). #(0 0.4 1 1). { 0. self defaultButtonPaneHeight+self defaultAnnotationPaneHeight. 0. 0.}}
	}! !

!MCOperationsBrowser class methodsFor: 'instance creation' stamp: 'cmm 10/2/2013 17:44'!
items: aCollection 
	^ self new
		 setItems: aCollection ;
		 yourself! !

!MCOperationsBrowser class methodsFor: 'preferences' stamp: 'mt 8/16/2016 14:18'!
themeProperties

	^ super themeProperties, {
		{ #revertedOperationAttributes. 'Colors' . 'Text attributes to use for reverted operations in MC tools.' }.
		{ #ignoredOperationAttributes. 'Colors' . 'Text attributes to use for ignored operations in MC tools.' }.
		
		"{ #rejectedOperationAttributes. 'Colors' . 'Text attributes to use for rejected operations in MC tools.' }.
		{ #acceptedOperationAttributes. 'Colors' . 'Text attributes to use for accepted operations in MC tools.' }.
		{ #conflictingOperationAttributes. 'Colors' . 'Text attributes to use for conflicting operations in MC tools.' }."
	}! !

!MCOperationsBrowser methodsFor: 'selecting' stamp: 'pre 11/11/2015 15:26'!
advanceSelection

	self selection < items size
		ifTrue: [self selection: self selection + 1]! !

!MCOperationsBrowser methodsFor: 'accessing' stamp: 'nk 11/10/2003 21:41'!
annotations
	^selection ifNil: [ super annotations ]
		ifNotNil: [ selection annotations ]! !

!MCOperationsBrowser methodsFor: 'updating' stamp: 'mt 8/16/2016 14:07'!
applyUserInterfaceTheme

	super applyUserInterfaceTheme.
	
	self changed: #list.! !

!MCOperationsBrowser methodsFor: 'actions' stamp: 'cmm 6/22/2016 19:51'!
browseSelectionOrigin
	| mcDefinition |
	selection ifNil: [ UIManager inform: 'Make a selection.' ].
	mcDefinition := selection definition.
	Cursor wait showWhile:
		[ mcDefinition mcModel ifNotNil:
			[ : mcModel | (mcModel originOf: mcDefinition)
				ifNil: [ UIManager inform: mcDefinition asString , ' was not found in any historical MC repository.' ]
				ifNotNilDo:
					[ : version | version open ] ] ]! !

!MCOperationsBrowser methodsFor: 'ui' stamp: 'ab 8/22/2003 02:21'!
buttonSpecs
	^ #((Invert invert 'Show the reverse set of changes')
		 (Export export 'Export the changes as a change set'))! !

!MCOperationsBrowser methodsFor: 'accessing' stamp: 'nk 2/23/2005 08:04'!
changeSetNameForInstall
	"Answer the name of the change set into which my selection will be installed.
	Derive this from my label.
	If I have no label, use the current change set."

	| tokens |
	label ifNil: [ ^ChangeSet current name ].
	tokens := label findTokens: ' '.
	tokens removeAllFoundIn: { 'changes'. 'between'. 'and' }.
	(tokens size = 3 and: [ tokens second = '<working' ]) ifTrue: [ ^tokens first, '-to-working' ].
	tokens size = 2 ifFalse: [ ^'InstalledPatches' ].
	^'{1}-to-{2}' format: tokens ! !

!MCOperationsBrowser methodsFor: 'ui' stamp: 'cmm 11/27/2016 14:45'!
defaultLabel
	^ label ifNil: [ 'Revisions Browser' ]! !

!MCOperationsBrowser methodsFor: 'actions' stamp: 'eem 1/24/2020 12:19'!
filterOutUnchangedMethods
	"Remove from the list methods that only have changed timestamps"
	| unchangedMethods |
	unchangedMethods := self unchangedMethods.
	(self confirm: ('Ignore {1} methods that only differ in timestamp?' translated
		format: {unchangedMethods size}))
		ifTrue:
			[items := items reject: [:op| op isUnchangedMethod].
			 self changed: #list]
! !

!MCOperationsBrowser methodsFor: 'actions' stamp: 'pre 11/11/2015 15:26'!
installSelection
	| loader |
	selection ifNotNil:
		[loader := MCPackageLoader new.
		selection applyTo: loader.
		loader loadWithName: self changeSetNameForInstall.
		self reverts remove: selection ifAbsent: [].
		self changed: #list ]! !

!MCOperationsBrowser methodsFor: 'selecting' stamp: 'ul 3/13/2017 00:34'!
invert
	items replace: [:ea | ea inverse].
	self changed: #list; changed: #text; changed: #selection! !

!MCOperationsBrowser methodsFor: 'accessing' stamp: 'bf 11/25/2016 12:15'!
isViewingRevisions
	^self items size > 1 and:
		[self items allSatisfy:
			[:each | each definition isRevisionOf: self items first definition]]! !

!MCOperationsBrowser methodsFor: 'accessing' stamp: 'bf 11/25/2016 12:15'!
list
	| showRevision |
	showRevision := self isViewingRevisions.
	^ self items collect: [:each | | summary |
		summary := showRevision ifTrue: [each summaryAndRevision] ifFalse: [each summary].
		(self reverts includes: each)
			ifFalse: [summary]
			ifTrue: [Text
				string: '( ', summary, ' )'
				attributes: (self userInterfaceTheme revertedOperationAttributes ifNil: [{TextEmphasis struckOut}])]]! !

!MCOperationsBrowser methodsFor: 'menus' stamp: 'pre 11/11/2015 15:09'!
methodListKey: aKeystroke from: aListMorph
	aKeystroke caseOf: {
		[$x] -> [self revertSelection] } 
	otherwise: [super methodListKey: aKeystroke from: aListMorph ]! !

!MCOperationsBrowser methodsFor: 'menus' stamp: 'eem 1/24/2020 12:13'!
methodListMenu: aMenu
	selection ifNotNil:
		[aMenu addList: #(
			('install'	 installSelection)
			('revert (x)'	 revertSelection)
			('browse origin' browseSelectionOrigin) 
			-)].
	self unchangedMethods ifNotEmpty:
		[aMenu addList: #(
			('revert unchanged methods...'	revertUnchangedMethods) 
			('filter out unchanged methods...'	filterOutUnchangedMethods) 
			-)].
	super methodListMenu: aMenu.
	^ aMenu! !

!MCOperationsBrowser methodsFor: 'ui' stamp: 'nk 11/10/2003 20:55'!
perform: selector orSendTo: otherTarget
	"Selector was just chosen from a menu by a user.  If can respond, then
perform it on myself. If not, send it to otherTarget, presumably the
editPane from which the menu was invoked."

	(self respondsTo: selector)
		ifTrue: [^ self perform: selector]
		ifFalse: [^ otherTarget perform: selector]! !

!MCOperationsBrowser methodsFor: 'actions' stamp: 'jr 12/13/2016 16:58'!
revertSelection
	| loader |
	selection ifNotNil:
		[loader := MCPackageLoader new.
		selection inverse applyTo: loader.
		self environmentInDisplayingImage beCurrentDuring: [loader loadWithName: self changeSetNameForInstall].
		self reverts add: selection.
		self
			advanceSelection;
			changed: #list ]! !

!MCOperationsBrowser methodsFor: 'actions' stamp: 'eem 1/24/2020 12:15'!
revertUnchangedMethods
	"revert methods that only have changed timestamps"
	| loader unchangedMethods |
	unchangedMethods := self unchangedMethods.
	(self confirm: ('Revert {1} methods that only differ in timestamp?' translated
		format: {unchangedMethods size}))
		ifTrue:
			[loader := MCPackageLoader new.
			unchangedMethods do: [:op | op inverse applyTo: loader].
			loader loadWithName: self changeSetNameForInstall].
! !

!MCOperationsBrowser methodsFor: 'accessing' stamp: 'pre 11/11/2015 15:57'!
reverts
	^ reverts ifNil: [reverts := Set new]! !

!MCOperationsBrowser methodsFor: 'subclassResponsibility' stamp: 'jr 2/27/2017 22:59'!
selectedClass
	| definition |
	selection ifNil: [ ^nil ].
	(definition := selection definition) ifNil: [ ^nil ].
	definition isMethodDefinition ifFalse: [ ^nil ].
	definition className in: [:className | | environment |
		environment := self environmentInDisplayingImage.
		^ environment at: className ifAbsent: [environment valueOf: className]]! !

!MCOperationsBrowser methodsFor: 'subclassResponsibility' stamp: 'jr 2/28/2017 02:45'!
selectedClassOrMetaClass
	| definition |
	selection ifNil: [ ^nil ].
	(definition := selection definition) ifNil: [ ^nil ].
	(definition isMethodDefinition or: [definition isClassDefinition]) ifFalse: [ ^nil ].
	^ definition actualClassIn: self environmentInDisplayingImage! !

!MCOperationsBrowser methodsFor: 'subclassResponsibility' stamp: 'stephaneducasse 2/4/2006 20:47'!
selectedMessageCategoryName
	| definition |
	selection ifNil: [ ^nil ].
	(definition := selection definition) ifNil: [ ^nil ].
	definition isMethodDefinition ifFalse: [ ^nil ].
	^definition category! !

!MCOperationsBrowser methodsFor: 'subclassResponsibility' stamp: 'stephaneducasse 2/4/2006 20:47'!
selectedMessageName
	| definition |
	selection ifNil: [ ^nil ].
	(definition := selection definition) ifNil: [ ^nil ].
	definition isMethodDefinition ifFalse: [ ^nil ].
	^definition  selector! !

!MCOperationsBrowser methodsFor: 'selecting' stamp: 'ab 7/16/2003 14:30'!
selection
	^ selection 
		ifNil: [0]
		ifNotNil: [self items indexOf: selection]! !

!MCOperationsBrowser methodsFor: 'selecting' stamp: 'cmm 4/3/2015 18:15'!
selection: aNumber 
	selection := self items
		at: aNumber
		ifAbsent: [  ].
	self
		 changed: #selection ;
		 changed: #text ;
		 changed: #annotations! !

!MCOperationsBrowser methodsFor: 'text' stamp: 'ab 7/16/2003 14:40'!
text
	^ selection ifNil: [''] ifNotNil: [selection source]! !

!MCOperationsBrowser methodsFor: 'text' stamp: 'ab 7/16/2003 14:27'!
text: aTextOrString
	self changed: #text! !

!MCOperationsBrowser methodsFor: 'accessing' stamp: 'bf 6/13/2011 16:33'!
unchangedMethods
	^ items select: [:op | op isUnchangedMethod]! !

!MCOperationsBrowser methodsFor: 'ui' stamp: 'mt 4/30/2015 13:50'!
widgetSpecs
	Preferences annotationPanes ifFalse: [ ^#(
		((listMorph:selection:menu:keystroke: list selection methodListMenu: methodListKey:from:) (0 0 1 0.4) (0 0 0 0))
		((textMorph: text) (0 0.4 1 1))
		) ].

	^ #(
		((listMorph:selection:menu:keystroke: list selection methodListMenu: methodListKey:from:) (0 0 1 0.4) (0 0 0 0))
		((textMorph: annotations) (0 0.4 1 0.4) (0 0 0 defaultAnnotationPaneHeight))
		((textMorph: text) (0 0.4 1 1) (0 defaultAnnotationPaneHeight 0 0))
	)! !

MCOperationsBrowser subclass: #MCPatchBrowser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-UI'!

MCPatchBrowser subclass: #MCChangeSelector
	instanceVariableNames: 'kept'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-UI'!

!MCChangeSelector methodsFor: 'ui' stamp: 'avi 9/11/2004 16:07'!
buttonSpecs
	^ #((Select select 'Select these changes')
		 (Cancel cancel 'Cancel the operation')
		)! !

!MCChangeSelector methodsFor: 'actions' stamp: 'avi 9/11/2004 16:26'!
cancel
	self answer: nil! !

!MCChangeSelector methodsFor: 'ui' stamp: 'avi 9/14/2004 15:01'!
defaultLabel
	^ 'Change Selector'! !

!MCChangeSelector methodsFor: 'private-ui' stamp: 'nice 9/22/2010 00:43'!
innerButtonRow
	^ self buttonRow: self innerButtonSpecs! !

!MCChangeSelector methodsFor: 'private-ui' stamp: 'nice 9/22/2010 00:42'!
innerButtonSpecs
	^
		#(('Select All' selectAll 'select all changes')
		  ('Select None' selectNone 'select no changes'))! !

!MCChangeSelector methodsFor: 'accessing' stamp: 'avi 9/11/2004 16:23'!
kept
	^ kept ifNil: [kept := Set new]! !

!MCChangeSelector methodsFor: 'actions' stamp: 'avi 9/11/2004 16:22'!
listSelectionAt: aNumber
	^ self kept includes: (self items at: aNumber)! !

!MCChangeSelector methodsFor: 'actions' stamp: 'avi 9/11/2004 16:23'!
listSelectionAt: aNumber put: aBoolean
	| item |
	item := self items at: aNumber.
	aBoolean
		ifTrue: [self kept add: item ]
		ifFalse: [self kept remove: item ifAbsent: []]! !

!MCChangeSelector methodsFor: 'actions' stamp: 'avi 9/11/2004 16:26'!
select
	self answer: (MCPatch operations: kept)! !

!MCChangeSelector methodsFor: 'actions' stamp: 'avi 9/14/2004 15:00'!
selectAll
	kept addAll: self items.
	self changed: #list! !

!MCChangeSelector methodsFor: 'actions' stamp: 'avi 9/14/2004 14:59'!
selectNone
	kept := Set new.
	self changed: #list! !

!MCChangeSelector methodsFor: 'ui' stamp: 'mt 4/30/2015 14:03'!
widgetSpecs
	Preferences annotationPanes ifFalse: [ ^#(
		((buttonRow) (0 0 1 0) (0 0 0 defaultButtonPaneHeight))
		((multiListMorph:selection:listSelection:menu: list selection listSelectionAt: methodListMenu:) (0 0 1 0.4) (0 defaultButtonPaneHeight 0 0))
		((innerButtonRow) (0 0.4 1 0.4) (0 0 0 defaultButtonPaneHeight))
		((textMorph: text) (0 0.4 1 1) (0 defaultButtonPaneHeight 0 0))
		)].

	^ {
		#((buttonRow) (0 0 1 0) (0 0 0 defaultButtonPaneHeight)).
		#((multiListMorph:selection:listSelection:menu: list selection listSelectionAt: methodListMenu:) (0 0 1 0.4) (0 defaultButtonPaneHeight 0 0)).
		#((innerButtonRow) (0 0.4 1 0.4) (0 0 0 defaultButtonPaneHeight)).
		{ #(textMorph: annotations). #(0 0.4 1 0.4). { 0. self defaultButtonPaneHeight. 0. self defaultButtonPaneHeight+self defaultAnnotationPaneHeight. }}.
		{ #(textMorph: text). #(0 0.4 1 1). { 0. self defaultButtonPaneHeight+self defaultAnnotationPaneHeight. 0. 0.}}
	}! !

!MCPatchBrowser class methodsFor: 'instance creation' stamp: 'ab 7/16/2003 14:35'!
forPatch: aPatch
	^ self new patch: aPatch! !

!MCPatchBrowser methodsFor: 'styling' stamp: 'kb 2/26/2010 15:24'!
aboutToStyle: aStyler 
	
	selection ifNotNil: [
		selection isConflict ifTrue: [ ^false ].
		(selection isAddition or: [ selection isRemoval ]) ifTrue: [
			selection definition isOrganizationDefinition ifTrue: [ ^false ].
				aStyler classOrMetaClass: self selectedClassOrMetaClass.
				^true ] ].
	^false! !

!MCPatchBrowser methodsFor: 'ui' stamp: 'ab 7/19/2003 21:31'!
defaultLabel
	^ 'Patch Browser'! !

!MCPatchBrowser methodsFor: 'initialize-release' stamp: 'ul 3/13/2017 00:35'!
patch: aPatch
	items := aPatch operations sorted! !

!MCPatchBrowser methodsFor: 'ui' stamp: 'cmm 3/31/2015 13:39'!
representsSameBrowseeAs: anotherModel 
	^ self class = anotherModel class
	and: [ items = anotherModel items ]! !

MCPatchBrowser subclass: #MCSaveVersionDialog
	instanceVariableNames: 'name message ignore patchBlock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-UI'!

!MCSaveVersionDialog methodsFor: 'actions' stamp: 'ul 10/18/2018 01:33'!
accept
	| logMessage logMessageWidget |
	self updateItems.
	logMessage := (logMessageWidget := self findTextMorph: #logMessage) text asString.
	(logMessage isEmpty or: [logMessage beginsWith: 'empty log message'])
		ifTrue:
			[(UIManager confirm: 'the log message is empty; are you sure you want to commit') ifFalse: [^ self]]
		ifFalse: [logMessageWidget accept].
	self answer: {
		(self findTextMorph: #versionName) text asString.
		logMessage.
		ignore }! !

!MCSaveVersionDialog methodsFor: 'ui' stamp: 'ab 8/24/2003 20:10'!
buttonSpecs
	^ #((Accept accept 'accept version name and log message')
		(Cancel cancel 'cancel saving version')
		) ! !

!MCSaveVersionDialog methodsFor: 'actions' stamp: 'ab 8/24/2003 20:41'!
cancel
	self answer: nil! !

!MCSaveVersionDialog methodsFor: 'ui' stamp: 'topa 9/3/2014 13:07'!
defaultExtent 
	^ 700@600! !

!MCSaveVersionDialog methodsFor: 'ui' stamp: 'cmm 10/19/2015 16:46:29'!
defaultLabel
	^ 'Edit Version Name and Message:  ', self name! !

!MCSaveVersionDialog methodsFor: 'actions' stamp: 'bf 1/22/2013 15:59'!
ignore
	^ ignore ifNil: [ignore := Set new]! !

!MCSaveVersionDialog methodsFor: 'actions' stamp: 'pre 11/11/2015 15:26'!
ignoreSelection
	selection
		ifNil: [ignore size = items size
			ifFalse: [ignore addAll: items]
			ifTrue: [ignore removeAll]]
		ifNotNil: [
			ignore remove: selection ifAbsent: [
				ignore add: selection].
			self advanceSelection].
	self changed: #list
! !

!MCSaveVersionDialog methodsFor: 'actions' stamp: 'bf 1/22/2013 16:12'!
installSelection
	super installSelection.
	selection ifNotNil: [
		ignore remove: selection ifAbsent: [].
		self changed: #list].

! !

!MCSaveVersionDialog methodsFor: 'accessing' stamp: 'mt 8/17/2016 22:03'!
list
  ^ self items collect: [:each |
        (self reverts includes: each)
            ifFalse: [(self ignore includes: each)
                        ifFalse: [each summary]
                        ifTrue: [Text
					string: '( ', each summary, ' )'
					attributes: (self userInterfaceTheme ignoredOperationAttributes ifNil: [{TextColor color: Color gray}])]]
            ifTrue: [Text
			string: '( ', each summary, ' )'
			attributes: (self userInterfaceTheme revertedOperationAttributes ifNil: [ {TextEmphasis struckOut} ]) ]]! !

!MCSaveVersionDialog methodsFor: 'accessing' stamp: 'ab 8/24/2003 20:41'!
logMessage
	^ message ifNil: ['empty log message']! !

!MCSaveVersionDialog methodsFor: 'accessing' stamp: 'ab 8/24/2003 20:42'!
logMessage: aString
	message := aString.
	self changed: #logMessage! !

!MCSaveVersionDialog methodsFor: 'accessing' stamp: 'topa 9/3/2014 13:56'!
message

	^ message! !

!MCSaveVersionDialog methodsFor: 'accessing' stamp: 'topa 9/3/2014 13:56'!
message: anObject

	message := anObject! !

!MCSaveVersionDialog methodsFor: 'menus' stamp: 'cmm 10/19/2015 16:09:06'!
methodListKey: aKeystroke from: aListMorph 
	aKeystroke caseOf: {
		[$I] -> [self ignoreSelection].
		[$e] -> [self refresh].
	} otherwise: [super methodListKey: aKeystroke from: aListMorph ]! !

!MCSaveVersionDialog methodsFor: 'menus' stamp: 'cmm 10/19/2015 16:08:52'!
methodListMenu: aMenu
	super methodListMenu: aMenu.
	aMenu addList:#(-
		('ignore (I)'	ignoreSelection 'Toggle inclusion of this change when saving.')
		('refresh (e)'	refresh 'Refresh the list of changes to this package.')).
	^aMenu! !

!MCSaveVersionDialog methodsFor: 'accessing' stamp: 'cmm 10/19/2015 15:20:19'!
name
	^ name! !

!MCSaveVersionDialog methodsFor: 'accessing' stamp: 'topa 9/3/2014 13:56'!
name: anObject

	name := anObject! !

!MCSaveVersionDialog methodsFor: 'actions' stamp: 'cmm 7/29/2016 14:59'!
okToClose
	^ (self findTextMorph: #logMessage)
		ifNil: [true]
		ifNotNil:
			[:widget | widget canDiscardEdits or: [self confirm: 'Version notes are not saved.
Is it OK to discard those notes?' translated]]! !

!MCSaveVersionDialog methodsFor: 'accessing' stamp: 'topa 9/3/2014 13:56'!
patchBlock

	^ patchBlock! !

!MCSaveVersionDialog methodsFor: 'accessing' stamp: 'topa 9/3/2014 13:57'!
patchBlock: anObject

	patchBlock := anObject.
	self updateItems! !

!MCSaveVersionDialog methodsFor: 'actions' stamp: 'cmm 4/3/2015 18:28'!
refresh
	self
		 updateItems ;
		 changed: #list! !

!MCSaveVersionDialog methodsFor: 'ui' stamp: 'cmm 2/14/2010 09:37'!
representsSameBrowseeAs: anotherModel 
	^ self class = anotherModel class
	and: [ self versionName = anotherModel versionName ]! !

!MCSaveVersionDialog methodsFor: 'ui' stamp: 'topa 9/3/2014 13:57'!
updateItems
	" update our items using the patchBlock "
	self patch: patchBlock value! !

!MCSaveVersionDialog methodsFor: 'accessing' stamp: 'ab 8/24/2003 20:37'!
versionName
	^ name! !

!MCSaveVersionDialog methodsFor: 'accessing' stamp: 'ab 8/24/2003 20:37'!
versionName: aString
	name := aString.
	self changed: #versionName! !

!MCSaveVersionDialog methodsFor: 'ui' stamp: 'mt 4/30/2015 13:40'!
widgetSpecs
	^ #(	
		((listMorph:selection:menu:keystroke: list selection methodListMenu: methodListKey:from:) (0 0 0.5 0.6) (0 0 0 -47) )
		((inputMorph: versionName) (0.5 0 1 0) (0 0 0 defaultInputFieldHeight))
		((textMorph: logMessage) (0.5 0 1 0.6) (0 defaultInputFieldHeight 0 -47))
		((buttonRow) (0.5 0.6 1 0.6) (0 -47 0 0))
		((textMorph: annotations) (0 0.6 0.5 0.6) (0 -47 0 0))
		((textMorph: text) (0 0.6 1 1) (0 0 0 0))
		)! !

MCCodeTool subclass: #MCSnapshotBrowser
	instanceVariableNames: 'categorySelection classSelection protocolSelection methodSelection switch'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-UI'!

!MCSnapshotBrowser class methodsFor: 'instance creation' stamp: 'ab 7/19/2003 18:03'!
forSnapshot: aSnapshot
	^ self new snapshot: aSnapshot! !

!MCSnapshotBrowser methodsFor: 'styling' stamp: 'ul 4/3/2016 04:19'!
aboutToStyle: aStyler
	
	| classDefinition shouldStyle |
	classSelection ifNil: [ ^false ].
	self switchIsComment ifTrue: [ ^false ].
	methodSelection 
		ifNotNil: [ 
			classDefinition := items 
				detect: [:ea | 
					ea isClassDefinition and: [ ea className = classSelection ] ]
				ifNone: [ 
					(Smalltalk at: classSelection ifAbsent: [ Object ]) asClassDefinition ].
			shouldStyle := true ]
		ifNil: [ 
			classDefinition := nil.
			shouldStyle := categorySelection ~= self extensionsCategory ].
	(Smalltalk classNamed: #SHMCClassDefinition) 
		ifNil: [ ^false ]
		ifNotNil: [ :SHMCClassDefinition |
			aStyler 
				environment: self;
				classOrMetaClass: (classDefinition ifNotNil: [
					SHMCClassDefinition 
						classDefinition: classDefinition 
						items: items 
						meta: switch = #class ]) ].
	^shouldStyle! !

!MCSnapshotBrowser methodsFor: 'accessing' stamp: 'hmm 4/16/2010 22:07'!
allClassNames
	^ (items 
		select: [:ea | (ea isOrganizationDefinition | ea isScriptDefinition) not] 
		thenCollect: [:ea | ea className]) asSet.
! !

!MCSnapshotBrowser methodsFor: 'text' stamp: 'nk 7/24/2003 13:40'!
annotations
	methodSelection ifNotNil: [^ methodSelection annotations ].
	^ ''! !

!MCSnapshotBrowser methodsFor: 'text' stamp: 'nk 7/24/2003 13:41'!
annotations: stuff
	self changed: #annotations! !

!MCSnapshotBrowser methodsFor: 'binding' stamp: 'kb 2/26/2010 15:00'!
bindingOf: aSymbol

	(Smalltalk bindingOf: aSymbol) ifNotNil: [ :binding | ^binding ].
	items do: [ :each |
		(each isClassDefinition and: [
			each className = aSymbol ]) ifTrue: [ ^aSymbol -> each ] ].
	^nil! !

!MCSnapshotBrowser methodsFor: 'morphic ui' stamp: 'bf 5/27/2005 19:33'!
buttonSpecs
	^ #(('instance' switchBeInstance 'show instance' buttonEnabled switchIsInstance)
		('?' switchBeComment 'show comment' buttonEnabled switchIsComment)
		('class' switchBeClass 'show class' buttonEnabled switchIsClass))! !

!MCSnapshotBrowser methodsFor: 'listing' stamp: 'cwp 7/10/2003 18:33'!
categoryList
	^ self visibleCategories! !

!MCSnapshotBrowser methodsFor: 'menus' stamp: 'nk 4/17/2004 09:52'!
categoryListMenu: aMenu 
	categorySelection
		ifNotNil: [aMenu
				add: (categorySelection = '*Extensions'
						ifTrue: ['load all extension methods' translated]
						ifFalse: ['load class category {1}' translated format: {categorySelection}])
				action: #loadCategorySelection].
	^ aMenu! !

!MCSnapshotBrowser methodsFor: 'selecting' stamp: 'cwp 7/10/2003 18:33'!
categorySelection
	^ categorySelection ifNil: [0] ifNotNil: [self visibleCategories indexOf: categorySelection]! !

!MCSnapshotBrowser methodsFor: 'selecting' stamp: 'nk 7/24/2003 13:42'!
categorySelection: aNumber
	categorySelection := aNumber = 0 ifFalse: [self visibleCategories at: aNumber].
	self classSelection: 0.
	self changed: #categorySelection;
		changed: #annotations;
		changed: #classList.
! !

!MCSnapshotBrowser methodsFor: 'text' stamp: 'ab 7/18/2003 15:48'!
classCommentString
	^ (items 
		detect: [:ea | ea isClassDefinition and: [ea className = classSelection]]
		ifNone: [^ '']) comment.! !

!MCSnapshotBrowser methodsFor: 'text' stamp: 'bf 11/12/2004 14:54'!
classDefinitionString
	| defs |
	defs := items select: [:ea | (ea isClassDefinition or: [ea isClassDefinitionExtension])
			and: [ea className = classSelection]].

	defs isEmpty ifTrue: [^ 'This class is defined elsewhere.'].

	^ String streamContents: [:stream | 
		defs asArray sort 
			do: [:ea | ea printDefinitionOn: stream]
			separatedBy: [stream nextPut: $.; cr]
	].! !

!MCSnapshotBrowser methodsFor: 'listing' stamp: 'bf 9/9/2012 14:23'!
classList
	^ categorySelection = self extensionsCategory
		ifTrue: [self labeledClassnames: self visibleClasses]
		ifFalse: [self visibleClasses]! !

!MCSnapshotBrowser methodsFor: 'menus' stamp: 'jr 11/15/2016 15:56'!
classListMenu: aMenu 
	classSelection ifNil: [ ^aMenu ].

	super classListMenu: aMenu.

	aMenu
		addLine;
				add: ('load class {1}' translated format: {classSelection})
				action: #loadClassSelection;
				add: ('load class {1} into other Environment...' translated format: {classSelection})
				action: #loadClassSelectionIntoOtherEnvironment.
	^ aMenu! !

!MCSnapshotBrowser methodsFor: 'selecting' stamp: 'cwp 7/10/2003 18:28'!
classSelection
	^ classSelection ifNil: [0] ifNotNil: [self visibleClasses indexOf: classSelection]! !

!MCSnapshotBrowser methodsFor: 'selecting' stamp: 'eem 9/19/2012 14:09'!
classSelection: aNumber
	classSelection := aNumber = 0 ifFalse: [self visibleClasses at: aNumber].
	self protocolSelection: 0.
	self changed: #classSelection;  changed: #protocolList.
	(self protocolList size = 1
	 and: [self protocolList first first = $*])
		ifTrue:
			[self protocolSelection: 1]
		ifFalse:
			[self changed: #annotations;
				changed: #methodList]
! !

!MCSnapshotBrowser methodsFor: 'morphic ui' stamp: 'ab 7/18/2003 17:29'!
defaultExtent
	^ 650@400.! !

!MCSnapshotBrowser methodsFor: 'morphic ui' stamp: 'ab 7/19/2003 21:31'!
defaultLabel
	^ 'Snapshot Browser'! !

!MCSnapshotBrowser methodsFor: 'accessing' stamp: 'ul 3/13/2017 00:36'!
extensionClassNames
	^ (self allClassNames difference: self packageClassNames) sorted! !

!MCSnapshotBrowser methodsFor: 'accessing' stamp: 'ab 7/5/2003 23:41'!
extensionsCategory
	^ '*Extensions'! !

!MCSnapshotBrowser methodsFor: 'accessing' stamp: 'bf 8/21/2012 21:38'!
hasExtensions
	^self extensionClassNames notEmpty! !

!MCSnapshotBrowser methodsFor: 'menus' stamp: 'cwp 7/10/2003 18:03'!
inspectSelection
	^ self methodSelection inspect! !

!MCSnapshotBrowser methodsFor: 'private' stamp: 'bf 9/9/2012 14:38'!
labeledClassnames: classNames
	| methodClassification |
	methodClassification := Dictionary new.
	items do: [:def | def isMethodDefinition ifTrue: [
		(methodClassification at: def className ifAbsentPut: [Set new])
			add: (def classIsMeta ifFalse: [$i] ifTrue: [$c])]].
	^ classNames collect: [:className |
		String streamContents: [:s |
			s nextPutAll: className; space.
			(methodClassification at: className)
				do: [:each | each = $i
					ifTrue: [s nextPutAll: '*inst']
					ifFalse: [s nextPutAll: '*class']]
				separatedBy: [s nextPut: $,]]]
! !

!MCSnapshotBrowser methodsFor: 'menus' stamp: 'nk 4/17/2004 09:53'!
loadCategorySelection
	"Load the entire selected category"
	categorySelection ifNil: [ ^self ].
	self methodsForSelectedClassCategory do: [ :m | m load ].! !

!MCSnapshotBrowser methodsFor: 'menus' stamp: 'nk 4/30/2004 15:06'!
loadClassSelection
	classSelection ifNil: [ ^self ].
	(self packageClasses detect: [ :ea | ea className = classSelection ] ifNone: [ ^self ]) load.
	self methodsForSelectedClass do: [ :m | m load ].! !

!MCSnapshotBrowser methodsFor: 'menus' stamp: 'jr 2/27/2017 23:07'!
loadClassSelectionIntoOtherEnvironment
	| env |
	classSelection ifNil: [ ^self ].
	env := EnvironmentRequest signal.
	env beCurrentDuring: [
		(self packageClasses detect: [ :ea | ea className = classSelection ] ifNone: [ ^self ])
			load.
		self methodsForSelectedClass do: [ :m | m load ]].! !

!MCSnapshotBrowser methodsFor: 'menus' stamp: 'nk 4/17/2004 09:45'!
loadMethodSelection
	methodSelection ifNil: [ ^self ].
	methodSelection load.! !

!MCSnapshotBrowser methodsFor: 'menus' stamp: 'nk 4/17/2004 09:46'!
loadProtocolSelection
	protocolSelection ifNil: [ ^self ].
	self methodsForSelectedProtocol do: [ :m | m load ].! !

!MCSnapshotBrowser methodsFor: 'listing' stamp: 'cwp 7/13/2003 02:11'!
methodList
	^ self visibleMethods collect: [:ea | ea selector]! !

!MCSnapshotBrowser methodsFor: 'menus' stamp: 'ul 12/12/2009 14:09'!
methodListMenu: aMenu 
	super methodListMenu: aMenu.
	self selectedMessageName
		ifNotNil: [:msgName | aMenu addLine; add: 'load method' translated action: #loadMethodSelection].
	^ aMenu! !

!MCSnapshotBrowser methodsFor: 'selecting' stamp: 'cwp 7/10/2003 20:26'!
methodSelection
	^ methodSelection
			ifNil: [0] 
			ifNotNil: [self visibleMethods indexOf: methodSelection]! !

!MCSnapshotBrowser methodsFor: 'selecting' stamp: 'nk 7/24/2003 13:42'!
methodSelection: aNumber
	methodSelection := aNumber = 0 ifFalse: [self visibleMethods at: aNumber].
	self changed: #methodSelection; changed: #text; changed: #annotations! !

!MCSnapshotBrowser methodsFor: 'accessing' stamp: 'cmm 10/22/2015 11:37:18'!
methodsForSelectedClass
	^ items select:
		[ : ea | ea className = classSelection and: [ ea isMethodDefinition ] ]! !

!MCSnapshotBrowser methodsFor: 'accessing' stamp: 'cmm 10/22/2015 11:36:43'!
methodsForSelectedClassAndMetaSelection
	^ self methodsForSelectedClass select:
		[ : each | each classIsMeta = self switchIsClass ]! !

!MCSnapshotBrowser methodsFor: 'accessing' stamp: 'cmm 10/22/2015 11:45:10'!
methodsForSelectedClassCategory
	^ items select:
		[ : ea | (self visibleClasses includes: ea className) and: [ ea isMethodDefinition ] ]! !

!MCSnapshotBrowser methodsFor: 'accessing' stamp: 'ul 11/14/2018 00:08'!
methodsForSelectedProtocol
	| methods |
	protocolSelection ifNil: [^ Array empty].
	methods := self methodsForSelectedClassAndMetaSelection asOrderedCollection.
	(protocolSelection = '-- all --') 
		ifFalse: [methods removeAllSuchThat: [:ea | ea category ~= protocolSelection]].
	^ methods 
	
								! !

!MCSnapshotBrowser methodsFor: 'accessing' stamp: 'ab 7/18/2003 15:48'!
packageClassNames
	^ self packageClasses collect: [:ea | ea className]! !

!MCSnapshotBrowser methodsFor: 'accessing' stamp: 'ab 7/18/2003 15:48'!
packageClasses
	^ items select: [:ea | ea isClassDefinition]! !

!MCSnapshotBrowser methodsFor: 'accessing' stamp: 'bf 7/11/2011 12:27'!
packageOrganizations
	^ items select: [:ea | ea isOrganizationDefinition]! !

!MCSnapshotBrowser methodsFor: 'listing' stamp: 'cwp 7/10/2003 19:07'!
protocolList
	^ self visibleProtocols! !

!MCSnapshotBrowser methodsFor: 'menus' stamp: 'nk 4/17/2004 09:43'!
protocolListMenu: aMenu 
	protocolSelection
		ifNotNil: [aMenu
				add: ('load protocol ''{1}''' translated format: {protocolSelection})
				action: #loadProtocolSelection ].
	^ aMenu! !

!MCSnapshotBrowser methodsFor: 'selecting' stamp: 'cwp 7/10/2003 19:35'!
protocolSelection
	^ protocolSelection 
		ifNil: [0]
		ifNotNil: [self visibleProtocols indexOf: protocolSelection]! !

!MCSnapshotBrowser methodsFor: 'selecting' stamp: 'nk 7/24/2003 13:43'!
protocolSelection: anInteger
	protocolSelection := (anInteger = 0 ifFalse: [self visibleProtocols at: anInteger]).
	self methodSelection: 0.
	self changed: #protocolSelection;
		changed: #methodList;	
		changed: #annotations! !

!MCSnapshotBrowser methodsFor: 'text' stamp: 'tfel 8/28/2009 20:42'!
scriptDefinitionString
	| defs |
	defs := items select: [:ea | ea isScriptDefinition].
	defs isEmpty ifTrue: [^'(package defines no scripts)'].
	
	^ String streamContents: [:stream | 
		defs asArray sort 
			do: [:ea | stream nextPutAll: '---------- package ';
					nextPutAll: ea scriptSelector;
					nextPutAll: ' ----------'; cr;
					nextPutAll: ea script; cr]
			separatedBy: [stream cr]].! !

!MCSnapshotBrowser methodsFor: 'accessing' stamp: 'jr 2/27/2017 23:08'!
selectedClass
	| environment |
	classSelection ifNil: [ ^nil ].
	environment := self environmentInDisplayingImage.
	^ environment at: classSelection ifAbsent: [environment valueOf: classSelection]
! !

!MCSnapshotBrowser methodsFor: 'accessing' stamp: 'nk 10/11/2003 16:51'!
selectedClassOrMetaClass
	| class |
	classSelection ifNil: [ ^nil ].
	class := Smalltalk at: classSelection ifAbsent: [ ^nil ].
	^self switchIsClass ifTrue: [ class class ]
		ifFalse: [ class ].! !

!MCSnapshotBrowser methodsFor: 'accessing' stamp: 'nk 11/10/2003 21:29'!
selectedMessageCategoryName
	^protocolSelection! !

!MCSnapshotBrowser methodsFor: 'accessing' stamp: 'nk 10/11/2003 16:45'!
selectedMessageName
	^methodSelection ifNotNil: [^ methodSelection selector ].
! !

!MCSnapshotBrowser methodsFor: 'switch' stamp: 'cwp 7/12/2003 18:57'!
signalSwitchChanged
	self protocolSelection: 0.
	self 
		changed: #switchIsInstance;
		changed: #switchIsComment;
		changed: #switchIsClass;
		changed: #protocolList;
		changed: #methodList;
		changed: #text.! !

!MCSnapshotBrowser methodsFor: 'accessing' stamp: 'ul 3/13/2017 00:36'!
snapshot: aSnapshot
	items := aSnapshot definitions sorted.
	self categorySelection: 0.! !

!MCSnapshotBrowser methodsFor: 'switch' stamp: 'cwp 7/12/2003 18:11'!
switchBeClass
	switch := #class.
	self signalSwitchChanged.! !

!MCSnapshotBrowser methodsFor: 'switch' stamp: 'cwp 7/12/2003 18:12'!
switchBeComment
	switch := #comment.
	self signalSwitchChanged.! !

!MCSnapshotBrowser methodsFor: 'switch' stamp: 'cwp 7/12/2003 18:12'!
switchBeInstance
	switch := #instance.
	self signalSwitchChanged.! !

!MCSnapshotBrowser methodsFor: 'switch' stamp: 'cwp 7/12/2003 18:05'!
switchIsClass
	^ switch = #class! !

!MCSnapshotBrowser methodsFor: 'switch' stamp: 'cwp 7/12/2003 18:04'!
switchIsComment
	^ switch = #comment.! !

!MCSnapshotBrowser methodsFor: 'switch' stamp: 'cwp 7/12/2003 18:03'!
switchIsInstance
	switch ifNil: [switch := #instance].
	^ switch = #instance.! !

!MCSnapshotBrowser methodsFor: 'text' stamp: 'bf 8/12/2009 23:00'!
text
	self switchIsComment ifTrue: [^ self classCommentString].
	methodSelection ifNotNil: [^ methodSelection source].
	protocolSelection ifNotNil: [^ ''].
	classSelection ifNotNil: [^ self classDefinitionString].
	categorySelection ifNil: [^self scriptDefinitionString].
	^ ''! !

!MCSnapshotBrowser methodsFor: 'text' stamp: 'cwp 7/11/2003 00:30'!
text: aTextOrString
	self changed: #text! !

!MCSnapshotBrowser methodsFor: 'listing' stamp: 'ul 3/13/2017 00:38'!
visibleCategories

	| visibleCategories |
	visibleCategories := Set new.
	self packageOrganizations do: [ :each | visibleCategories addAll: each categories ].
	self packageClasses do: [ :each | visibleCategories add: each category ].
	self hasExtensions ifTrue: [ visibleCategories add: self extensionsCategory ].
	^visibleCategories sorted: [:each | each ifNil: ['~(put nils to the end)']] ascending! !

!MCSnapshotBrowser methodsFor: 'listing' stamp: 'ab 7/18/2003 15:48'!
visibleClasses
	^ categorySelection = self extensionsCategory
		ifTrue: [self extensionClassNames]
		ifFalse: [self packageClasses
					select: [:ea | ea category = categorySelection]
					thenCollect: [:ea | ea className]].! !

!MCSnapshotBrowser methodsFor: 'listing' stamp: 'cwp 7/10/2003 19:46'!
visibleMethods
	^ classSelection 
		ifNil: [#()]
		ifNotNil: [self methodsForSelectedProtocol]! !

!MCSnapshotBrowser methodsFor: 'listing' stamp: 'ul 11/14/2018 00:09'!
visibleProtocols
	| methods protocols |
	self switchIsComment ifTrue: [^ Array empty].
	methods := self methodsForSelectedClassAndMetaSelection.
	protocols := methods collect: [:ea | ea category] as: Set.
	(protocols size > 1) ifTrue: [protocols add: '-- all --'].
	^ protocols sorted! !

!MCSnapshotBrowser methodsFor: 'morphic ui' stamp: 'mt 4/30/2015 13:42'!
widgetSpecs

	Preferences annotationPanes ifFalse: [ ^#(
		((listMorph: category) (0 0 0.25 0.4))
		((listMorph: class) (0.25 0 0.50 0.4) (0 0 0 -30))
		((listMorph: protocol) (0.50 0 0.75 0.4))
		((listMorph:selection:menu:keystroke:  methodList methodSelection methodListMenu: methodListKey:from:) (0.75 0 1 0.4))
		((buttonRow) (0.25 0.4 0.5 0.4) (0 -30 0 0))
		((codePane: text) (0 0.4 1 1))
		) ].

	^#(
		((listMorph: category) (0 0 0.25 0.4))
		((listMorph: class) (0.25 0 0.50 0.4) (0 0 0 -30))
		((listMorph: protocol) (0.50 0 0.75 0.4))
		((listMorph:selection:menu:keystroke:  methodList methodSelection methodListMenu: methodListKey:from:) (0.75 0 1 0.4))

		((buttonRow) (0.25 0.4 0.5 0.4) (0 -30 0 0))

		((inputMorph: annotations) (0 0.4 1 0.4) (0 0 0 defaultInputFieldHeight))
		((codePane: text) (0 0.4 1 1) (0 defaultInputFieldHeight 0 0))
		)! !

!MCTool methodsFor: 'morphic ui' stamp: 'bf 4/11/2013 17:39'!
answer: anObject
	(modalProcess notNil and: [modalProcess ~~ Processor activeProcess])
		ifTrue: [^self inform: 'This modal dialog was interrupted, please close it'].
	modalValue := anObject.
	self close.! !

!MCTool methodsFor: 'toolbuilder' stamp: 'fbs 5/24/2013 17:24'!
buildWindow
	^ ToolBuilder build: self! !

!MCTool methodsFor: 'toolbuilder' stamp: 'mt 4/30/2015 13:15'!
buildWith: builder
	|  windowBuilder |

	windowBuilder := MCToolWindowBuilder builder: builder tool: self.
	self widgetSpecs do:
		[:spec | | send fractions offsets |
		send := spec first.
		fractions := (spec at: 2 ifAbsent: [#(0 0 1 1)]) copy.
		offsets := (spec at: 3 ifAbsent: [#(0 0 0 0)]) copy.
		
		fractions doWithIndex: [:numberOrSymbol :index |
			numberOrSymbol isSymbol
				ifTrue: [fractions at: index put: (self perform: numberOrSymbol)]].
		offsets doWithIndex: [:numberOrSymbol :index |
			numberOrSymbol isSymbol
				ifTrue: [offsets at: index put: (self perform: numberOrSymbol)]].
					
		windowBuilder frame: (LayoutFrame
			fractions: (fractions first @ fractions second corner: fractions third @ fractions fourth)
			offsets: (offsets first @ offsets second corner: offsets third @ offsets fourth)).
		windowBuilder perform: send first withArguments: send allButFirst].

	^ windowBuilder build
! !

!MCTool methodsFor: 'morphic ui' stamp: 'bf 5/27/2005 19:19'!
buttonEnabled
	^ true! !

!MCTool methodsFor: 'morphic ui' stamp: 'bf 5/27/2005 19:22'!
buttonSelected
	^ false! !

!MCTool methodsFor: 'morphic ui' stamp: 'ab 7/17/2003 15:23'!
buttonSpecs
	^ #()! !

!MCTool methodsFor: 'morphic ui' stamp: 'ab 7/22/2003 00:45'!
buttonState
	^ true! !

!MCTool methodsFor: 'morphic ui' stamp: 'ab 7/18/2003 17:42'!
close
	self window delete! !

!MCTool methodsFor: 'morphic ui' stamp: 'mt 4/30/2015 13:51'!
defaultAnnotationPaneHeight 
	"Answer the receiver's preferred default height for new annotation panes.
	Since MC compares two annotations, reserve a bit less than a double the traditional height."
	
	^ Preferences standardDefaultTextFont height * 3! !

!MCTool methodsFor: 'morphic ui' stamp: 'mt 4/30/2015 13:33'!
defaultButtonPaneHeight
	"Answer the user's preferred default height for new button panes."

	^ (Preferences standardButtonFont height * 2.5) rounded! !

!MCTool methodsFor: 'morphic ui' stamp: 'ab 7/17/2003 15:10'!
defaultExtent
	^ 500@500! !

!MCTool methodsFor: 'morphic ui' stamp: 'mt 4/30/2015 13:33'!
defaultInputFieldHeight

	^ Preferences standardDefaultTextFont height * 2! !

!MCTool methodsFor: 'morphic ui' stamp: 'ab 7/17/2003 15:11'!
defaultLabel
	^ self class name! !

!MCTool methodsFor: 'user interface' stamp: 'mt 7/9/2016 08:52'!
defaultWindowColor
	^ (Color r: 0.65 g: 0.691 b: 0.876)! !

!MCTool methodsFor: 'morphic ui' stamp: 'eem 4/13/2012 10:21'!
fillMenu: aMenu fromSpecs: anArray
	anArray do:
		[:spec |
		spec == #addLine
			ifTrue: [aMenu addLine]
			ifFalse:
				[aMenu
					add: spec first
					target: self
					selector: spec second
					argumentList: (spec allButFirst: 2)]].
	^aMenu! !

!MCTool methodsFor: 'morphic ui' stamp: 'bf 3/16/2005 14:48'!
findListMorph: aSymbol
	^ morph submorphs detect: [:ea | (ea respondsTo: #getListSelector) and: [ea getListSelector = aSymbol]] ifNone: []! !

!MCTool methodsFor: 'morphic ui' stamp: 'ab 8/24/2003 20:15'!
findTextMorph: aSymbol
	^ morph submorphs detect: [:ea | (ea respondsTo: #getTextSelector) and: [ea getTextSelector = aSymbol]] ifNone: []! !

!MCTool methodsFor: 'morphic ui' stamp: 'ab 7/18/2003 17:23'!
getMenu: aMenu
	^aMenu! !

!MCTool methodsFor: 'morphic ui' stamp: 'ab 7/18/2003 17:36'!
label
	^ label ifNil: [self defaultLabel]! !

!MCTool methodsFor: 'morphic ui' stamp: 'ab 7/18/2003 17:36'!
label: aString
	label := aString! !

!MCTool methodsFor: 'morphic ui' stamp: 'ab 7/18/2003 17:18'!
perform: selector orSendTo: otherTarget
	"Selector was just chosen from a menu by a user.  If can respond, then
perform it on myself. If not, send it to otherTarget, presumably the
editPane from which the menu was invoked."

	(self respondsTo: selector)
		ifTrue: [^ self perform: selector]
		ifFalse: [^ otherTarget perform: selector]! !

!MCTool methodsFor: 'morphic ui' stamp: 'lr 10/5/2003 09:09'!
performButtonAction: anActionSelector enabled: anEnabledSelector
	(self perform: anEnabledSelector) 
		ifTrue: [ self perform: anActionSelector ]! !

!MCTool methodsFor: 'morphic ui' stamp: 'jr 5/8/2020 23:10'!
refresh
	"Do nothing by default."! !

!MCTool methodsFor: 'morphic ui' stamp: 'bf 4/11/2013 17:26'!
show
	modalProcess := nil.
	Smalltalk at: #ToolBuilder ifPresent: [:tb | tb open: self. ^ self].
	^self window openInWorldExtent: self defaultExtent; yourself! !

!MCTool methodsFor: 'morphic ui' stamp: 'bf 4/11/2013 17:32'!
showLabelled: labelString
	modalProcess := nil.
	self label: labelString.
	^(self window)
		openInWorldExtent: self defaultExtent;
		yourself! !

!MCTool methodsFor: 'morphic ui' stamp: 'ct 1/14/2020 13:53'!
showModally
	modalProcess := Processor activeProcess.
	self window openInWorldExtent: self defaultExtent.
	ToolBuilder default runModal: self window.
	morph := nil.
	^ modalValue! !

!MCTool methodsFor: 'morphic ui' stamp: 'ab 7/17/2003 15:40'!
widgetSpecs
	^ #()! !

!MCTool methodsFor: 'morphic ui' stamp: 'ab 7/18/2003 17:38'!
window
	^ morph ifNil: [morph := self buildWindow]! !

MCTool subclass: #MCVersionHistoryBrowser
	instanceVariableNames: 'ancestry index repositoryGroup package infos'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-UI'!

!MCVersionHistoryBrowser methodsFor: 'accessing' stamp: 'avi 2/13/2004 01:11'!
ancestry: anAncestry
	ancestry := anAncestry! !

!MCVersionHistoryBrowser methodsFor: 'accessing' stamp: 'avi 2/13/2004 01:10'!
baseSnapshot
	^ self snapshotForInfo: ancestry! !

!MCVersionHistoryBrowser methodsFor: 'morphic ui' stamp: 'ab 7/17/2003 15:41'!
defaultExtent
	^ 440@169.
	! !

!MCVersionHistoryBrowser methodsFor: 'morphic ui' stamp: 'avi 2/13/2004 01:09'!
defaultLabel
	^ ancestry name, ' History'! !

!MCVersionHistoryBrowser methodsFor: 'morphic ui' stamp: 'fbs 1/22/2014 08:51'!
getMenu: aMenu
	| menuSpecs |
	self selection < 1 ifTrue: [^ aMenu]. "Nothing selected = nothing to do"
	menuSpecs := 	(self selectedInfo ancestors collect: [:parent |
			{'view changes from ', parent name . #viewChanges: . parent}]),
		#(('spawn history' spawnHistory)
		('search history' searchHistory)).
	self selection > 1 ifTrue: [menuSpecs := {{'view changes to ', ancestry name . #viewChanges}}, menuSpecs].
	self fillMenu: aMenu fromSpecs: menuSpecs.
	^ aMenu! !

!MCVersionHistoryBrowser methodsFor: 'accessing' stamp: 'ab 8/22/2003 01:41'!
index
	"Answer the value of index"

	^ index! !

!MCVersionHistoryBrowser methodsFor: 'accessing' stamp: 'nk 7/28/2003 18:17'!
index: anObject
	"Set the value of index"

	index := anObject! !

!MCVersionHistoryBrowser methodsFor: 'accessing' stamp: 'bf 12/8/2011 18:53'!
infos
	^ infos ifNil: [infos := ancestry topologicalAncestors]! !

!MCVersionHistoryBrowser methodsFor: 'accessing' stamp: 'avi 9/17/2005 16:10'!
list
	^ self infos collect: [:ea | ea name]! !

!MCVersionHistoryBrowser methodsFor: 'accessing' stamp: 'nk 7/28/2003 18:27'!
package: aMCPackage
	package := aMCPackage! !

!MCVersionHistoryBrowser methodsFor: 'accessing' stamp: 'ab 8/7/2003 21:27'!
repositoryGroup
	^ MCRepositoryGroup default! !

!MCVersionHistoryBrowser methodsFor: 'morphic ui' stamp: 'bf 11/26/2012 18:54'!
searchHistory
	(UIManager default
		edit: (String streamContents:
			[:s|
			self selectedInfo topologicalAncestors
				do: [:versionInfo | s nextPutAll: versionInfo summary]
				separatedBy: [s cr; cr]])
		label: 'Version History: ', self selectedInfo versionName)
			extent: (0.5@0.9) * Display height! !

!MCVersionHistoryBrowser methodsFor: 'accessing' stamp: 'avi 9/17/2005 16:09'!
selectedInfo
	^ self infos at: self selection ifAbsent: [nil]! !

!MCVersionHistoryBrowser methodsFor: 'accessing' stamp: 'ab 8/22/2003 01:39'!
selectedSnapshot
	^ self snapshotForInfo: self selectedInfo! !

!MCVersionHistoryBrowser methodsFor: 'accessing' stamp: 'ab 7/11/2003 23:24'!
selection
	^ index ifNil: [0]! !

!MCVersionHistoryBrowser methodsFor: 'accessing' stamp: 'ab 7/11/2003 23:31'!
selection: aNumber
	index := aNumber.
	self changed: #selection; changed: #summary! !

!MCVersionHistoryBrowser methodsFor: 'accessing' stamp: 'ab 8/22/2003 01:38'!
snapshotForInfo: aVersionInfo
	^ (self repositoryGroup versionWithInfo: aVersionInfo) snapshot! !

!MCVersionHistoryBrowser methodsFor: 'morphic ui' stamp: 'avi 2/13/2004 01:10'!
spawnHistory
	MCVersionHistoryBrowser new
		ancestry: self selectedInfo;
		package: package;
		show! !

!MCVersionHistoryBrowser methodsFor: 'accessing' stamp: 'ab 7/11/2003 23:34'!
summary
	| selInfo |
	selInfo := self selectedInfo.
	^ selInfo 
		ifNil: ['']
		ifNotNil: [selInfo summary]! !

!MCVersionHistoryBrowser methodsFor: 'morphic ui' stamp: 'bf 9/27/2011 15:00'!
viewChanges
	self viewChanges: ancestry snapshot: self baseSnapshot
		relativeTo: self selectedInfo snapshot: self selectedSnapshot! !

!MCVersionHistoryBrowser methodsFor: 'morphic ui' stamp: 'bf 9/27/2011 15:00'!
viewChanges: otherInfo
	self viewChanges: self selectedInfo snapshot: self selectedSnapshot
		relativeTo:  otherInfo snapshot: (self snapshotForInfo: otherInfo)! !

!MCVersionHistoryBrowser methodsFor: 'morphic ui' stamp: 'bf 9/27/2011 14:59'!
viewChanges: ancestorInfo snapshot: ancestorSnapshot relativeTo: baseInfo snapshot: baseSnapshot
	"Note that the patchLabel will be parsed in MCPatchBrowser>>installSelection, so don't translate it!!"
	| patch patchLabel |
	patchLabel := 'changes between {1} and {2}' format: { baseInfo name. ancestorInfo name }.
	patch := ancestorSnapshot patchRelativeToBase: baseSnapshot.
	(MCPatchBrowser forPatch: patch) label: patchLabel; show! !

!MCVersionHistoryBrowser methodsFor: 'morphic ui' stamp: 'nk 7/28/2003 18:05'!
widgetSpecs
	^ #(
		((listMorph:selection:menu: list selection getMenu:) (0 0 0.3 1))
		((textMorph: summary) (0.3 0 1 1))
	 	)! !

MCVersionHistoryBrowser subclass: #MCWorkingHistoryBrowser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-UI'!

!MCWorkingHistoryBrowser methodsFor: 'accessing' stamp: 'ab 8/22/2003 01:37'!
baseSnapshot
	^ package snapshot! !

MCTool subclass: #MCVersionInspector
	instanceVariableNames: 'version'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-UI'!

MCVersionInspector subclass: #MCRepositoryInspector
	instanceVariableNames: 'repository packageNames versionNames selectedPackage selectedVersion order versionInfo loaded newer inherited inheritedId'
	classVariableNames: 'BrowseBranchedVersionsSeparately Order'
	poolDictionaries: ''
	category: 'Monticello-UI'!

MCRepositoryInspector subclass: #MCFileRepositoryInspector
	instanceVariableNames: 'allVersionNames'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-UI'!

!MCFileRepositoryInspector methodsFor: 'private' stamp: 'cmm 3/4/2011 16:27'!
allVersionNames
	^ allVersionNames ifNil:
		[ self initializeVersionNames.
		allVersionNames ]! !

!MCFileRepositoryInspector methodsFor: 'private' stamp: 'cmm 3/4/2011 16:26'!
initializeVersionNames
	repository cacheAllFileNamesDuring:
		[ super initializeVersionNames.
		allVersionNames := repository readableFileNames ]! !

!MCFileRepositoryInspector methodsFor: 'morphic ui' stamp: 'cmm 1/4/2020 15:25'!
refresh
	allVersionNames := nil.
	super refresh! !

!MCFileRepositoryInspector methodsFor: 'private' stamp: 'stephaneducasse 2/4/2006 20:47'!
versionInfo
	^ versionInfo ifNil: [versionInfo := repository versionInfoFromFileNamed: selectedVersion]! !

!MCFileRepositoryInspector methodsFor: 'private' stamp: 'cmm 3/4/2011 16:21'!
versionNamesForNoPackageSelection
	^ self allVersionNames! !

!MCFileRepositoryInspector methodsFor: 'private' stamp: 'eem 6/7/2012 10:11'!
versionNamesForSelectedPackage
	^ self allVersionNames select:
		(self class browseBranchedVersionsSeparately
			ifTrue: [[:each| each packageAndBranchName = selectedPackage]]
			ifFalse: [[:each| each packageName = selectedPackage]])! !

!MCFileRepositoryInspector methodsFor: 'private' stamp: 'cmm 3/6/2011 15:21'!
versionSelection: aNumber 
	versionInfo := nil.
	super versionSelection: aNumber! !

!MCFileRepositoryInspector methodsFor: 'morphic ui' stamp: 'avi 9/17/2005 18:36'!
versionSummary
	^ version
		ifNotNil: [version summary]
		ifNil: [self versionInfo summary]! !

!MCRepositoryInspector class methodsFor: 'class initialization' stamp: 'eem 7/11/2012 15:35'!
browseBranchedVersionsSeparately

	<preference: 'Browse branched package versions separately'
	category: 'Monticello'
	description: 'If true, versions of packages on branches will be shown in a separate section from trunk package versions'
	type: #Boolean>
	^BrowseBranchedVersionsSeparately ifNil: [true]! !

!MCRepositoryInspector class methodsFor: 'class initialization' stamp: 'eem 6/7/2012 10:05'!
browseBranchedVersionsSeparately: aBoolean
	BrowseBranchedVersionsSeparately := aBoolean! !

!MCRepositoryInspector class methodsFor: 'class initialization' stamp: 'bf 3/16/2005 14:41'!
initialize
	"self initialize"

	self migrateInstances! !

!MCRepositoryInspector class methodsFor: 'class initialization' stamp: 'bf 3/16/2005 14:53'!
migrateInstances
	self allSubInstancesDo: [:inst |
		#(packageList versionList) do: [:each |
			[(inst findListMorph: each) highlightSelector: nil]
				on: Error do: [:ignore | ]]].! !

!MCRepositoryInspector class methodsFor: 'class initialization' stamp: 'stephaneducasse 2/4/2006 20:47'!
order
	Order isNil
		ifTrue: [ Order := 5 ].
	^Order! !

!MCRepositoryInspector class methodsFor: 'class initialization' stamp: 'stephaneducasse 2/4/2006 20:47'!
order: anInteger
	Order := anInteger! !

!MCRepositoryInspector class methodsFor: 'instance creation' stamp: 'avi 2/28/2004 18:51'!
repository: aFileBasedRepository workingCopy: aWorkingCopy
	^self new
		setRepository: aFileBasedRepository workingCopy: aWorkingCopy;
		yourself! !

!MCRepositoryInspector methodsFor: 'morphic ui' stamp: 'dtl 11/5/2018 21:50'!
defaultExtent
	^600@300! !

!MCRepositoryInspector methodsFor: 'morphic ui' stamp: 'avi 2/28/2004 18:51'!
defaultLabel
	^'Repository: ' , repository description! !

!MCRepositoryInspector methodsFor: 'morphic ui' stamp: 'avi 2/28/2004 20:20'!
hasVersion
	^ selectedVersion notNil! !

!MCRepositoryInspector methodsFor: 'morphic ui' stamp: 'nice 2/13/2019 16:07'!
identifyLoadedAndInherited: aMCWorkingCopy 
	| seen |
	seen := Set new.
	aMCWorkingCopy ancestors do:
		[ : ancestor | loaded add: ancestor versionName.
		seen add: ancestor.
		ancestor ancestorsDoWhileTrue:
			[ : heir | (seen includes: heir)
				ifTrue: [ false ]
				ifFalse:
					[ inherited add: heir versionName.
					inheritedId add: heir id.
					seen add: heir.
					true ] ] ]! !

!MCRepositoryInspector methodsFor: 'initialize-release' stamp: 'eem 7/11/2012 15:25'!
identifyNewerVersionsOf: aMCWorkingCopy 
	(repository possiblyNewerVersionsOfAnyOf: aMCWorkingCopy ancestry names) do:
		[ : eachNewerVersionName |
		newer add: (self class browseBranchedVersionsSeparately
						ifTrue: [ eachNewerVersionName packageAndBranchName ]
						ifFalse: [ eachNewerVersionName packageName ]) ]! !

!MCRepositoryInspector methodsFor: 'initialize-release' stamp: 'cmm 3/2/2011 23:05'!
initialize
	super initialize.
	self initializeEmphasis! !

!MCRepositoryInspector methodsFor: 'initialize-release' stamp: 'nice 2/13/2019 16:08'!
initializeEmphasis
	inherited := Set new.
	inheritedId := Set new.
	loaded := Set new! !

!MCRepositoryInspector methodsFor: 'initialize-release' stamp: 'ul 11/14/2018 00:08'!
initializeVersionNames
	repository cacheAllFileNamesDuring:
		[ versionNames := selectedPackage
			ifNil: [Array empty]
			ifNotNil: [repository versionNamesForPackageNamed: selectedPackage].
		self refreshEmphasis ]! !

!MCRepositoryInspector methodsFor: 'actions' stamp: 'avi 9/18/2005 10:54'!
load
	self hasVersion ifTrue:
		[self version isCacheable
			ifTrue: [version workingCopy repositoryGroup addRepository: repository].
		super load.
		self refresh].! !

!MCRepositoryInspector methodsFor: 'actions' stamp: 'bf 11/16/2004 11:56'!
merge
	super merge.
	self refresh.
! !

!MCRepositoryInspector methodsFor: 'morphic ui' stamp: 'stephaneducasse 2/4/2006 20:47'!
order: anInteger
	self class order: (order := anInteger).
	self changed: #versionList.! !

!MCRepositoryInspector methodsFor: 'morphic ui' stamp: 'mva 2/4/2017 10:33'!
orderSpecs
	^{
		'unchanged' -> nil.
		'order by package' -> [ :x :y | x packageName < y packageName ].
		'order by author' -> [ :x :y | x author caseInsensitiveLessOrEqual: y author ].
		'order by version-string' -> [ :x :y | x versionNumber asString < y versionNumber asString ].
		'order by version-number' -> [ :x :y | x versionNumber > y versionNumber ].
		'order by filename' -> [ :x :y | x < y ].
	}! !

!MCRepositoryInspector methodsFor: 'morphic ui' stamp: 'lr 9/26/2003 21:07'!
orderString: anIndex
	^String streamContents: [ :stream |
		order = anIndex
			ifTrue: [ stream nextPutAll: '<yes>' ]
			ifFalse: [ stream nextPutAll: '<no>' ].
		stream nextPutAll: (self orderSpecs at: anIndex) key ]! !

!MCRepositoryInspector methodsFor: 'morphic ui' stamp: 'eem 7/17/2014 12:23'!
packageHighlight: loadedPackageAndMaybeBranchName
	| packageName bold underline emphasis |
	bold := TextEmphasis bold emphasisCode.
	underline := TextEmphasis underlined emphasisCode.
	packageName := loadedPackageAndMaybeBranchName copyUpTo: $..
	emphasis := (loaded detect: [:each| each packageName = packageName] ifNone: nil) ifNotNil:
			[:mcVersionName|
			 self class browseBranchedVersionsSeparately
				ifTrue:
					[mcVersionName packageAndBranchName = loadedPackageAndMaybeBranchName
						ifTrue:
							[(newer includes: loadedPackageAndMaybeBranchName)
								ifTrue: [bold + underline]
								ifFalse: [underline]]
						ifFalse:
							[(newer includes: packageName)
								ifTrue: [bold]
								ifFalse: [nil]]]
				ifFalse:
					[(newer includes: loadedPackageAndMaybeBranchName)
						ifTrue: [bold + underline]
						ifFalse: [underline]]].
	^emphasis
		ifNil: [loadedPackageAndMaybeBranchName]
		ifNotNil:
			[Text
				string: loadedPackageAndMaybeBranchName
				attribute: (TextEmphasis new emphasisCode: emphasis)]! !

!MCRepositoryInspector methodsFor: 'morphic ui' stamp: 'eem 6/26/2014 11:50'!
packageList
	| result loadedPackages |
	packageNames ifNotNil: [ ^ packageNames ].
	repository cacheAllFileNamesDuring: 
		[ "Enjoy emphasis side-effects of populating my versionNames." 
		self versionNames.
		result := self class browseBranchedVersionsSeparately
					ifTrue: [ repository allPackageAndBranchNames ]
					ifFalse: [ repository allPackageNames ] ].
	"sort loaded packages first, then alphabetically"
	loadedPackages := loaded asSet collect: [ : each | each packageAndBranchName ].
	result := result asArray sort:
		[ : a : b | | loadedA loadedB |
		loadedA := loadedPackages includes: a.
		loadedB := loadedPackages includes: b.
		loadedA = loadedB
			ifTrue: [ a < b ]
			ifFalse: [ loadedA ] ].
	^ packageNames := result collect:
		[ : each | self packageHighlight: each ]! !

!MCRepositoryInspector methodsFor: 'morphic ui' stamp: 'avi 2/28/2004 18:51'!
packageListMenu: aMenu
	^aMenu! !

!MCRepositoryInspector methodsFor: 'morphic ui' stamp: 'lr 9/26/2003 20:17'!
packageSelection
	^self packageList indexOf: selectedPackage! !

!MCRepositoryInspector methodsFor: 'morphic ui' stamp: 'cmm 11/27/2011 17:30'!
packageSelection: aNumber
	selectedPackage := (aNumber between: 1 and: self packageList size)
		ifTrue: [ (self packageList at: aNumber) asString ].
	self versionSelection: 0.
	versionNames := nil.
	self changed: #packageSelection; changed: #versionList! !

!MCRepositoryInspector methodsFor: 'initialize-release' stamp: 'cmm 2/14/2010 20:26'!
postAcceptBrowseFor: aModel
	"Make the same selections as in aModel."
	self 
		packageSelection: aModel packageSelection ;
		versionSelection: aModel versionSelection! !

!MCRepositoryInspector methodsFor: 'actions' stamp: 'cmm 3/15/2011 13:47'!
refresh
	packageNames := versionNames := newer := nil.
	repository refresh.
	self
		 changed: #packageList ;
		 changed: #versionList.! !

!MCRepositoryInspector methodsFor: 'morphic ui' stamp: 'cmm 3/15/2011 13:55'!
refreshEmphasis
	| identifyNewer |
	identifyNewer := newer isNil.
	identifyNewer ifTrue: [ newer := Set new ].
	self initializeEmphasis.
	MCWorkingCopy allManagers do:
		[ : each | self identifyLoadedAndInherited: each.
		identifyNewer ifTrue: [ self identifyNewerVersionsOf: each ] ]! !

!MCRepositoryInspector methodsFor: 'access' stamp: 'cmm 2/14/2010 09:54'!
repository
	^ repository! !

!MCRepositoryInspector methodsFor: 'morphic ui' stamp: 'cmm 2/14/2010 10:10'!
representsSameBrowseeAs: anotherModel 
	^ self class = anotherModel class
	and: [ self repository = anotherModel repository ]! !

!MCRepositoryInspector methodsFor: 'testing' stamp: 'nice 2/13/2019 16:35'!
selectedVersionIsFalseAncestor
	"Answer true if selectedVersion is a false ancestor of working copy.
	An ancestor of working copy that has same name, but not same id is a false ancestor!!"
	^(selectedVersion notNil
			and: [(inherited includes: selectedVersion versionName)
					and: [(inheritedId includes: self versionInfo id) not]])! !

!MCRepositoryInspector methodsFor: 'initialize-release' stamp: 'eem 6/26/2014 11:52'!
setRepository: aFileBasedRepository workingCopy: aWorkingCopy 
	order := self class order.
	repository := aFileBasedRepository.
	self refresh.
	aWorkingCopy
		ifNil: [ self packageSelection: 1 ]
		ifNotNil:
			[ selectedPackage := aWorkingCopy ancestry ancestors ifNotEmpty:
				[ : ancestors | ancestors anyOne name asMCVersionName packageAndBranchName ] ].
	MCWorkingCopy addDependent: self! !

!MCRepositoryInspector methodsFor: 'morphic ui' stamp: 'nice 2/13/2019 17:00'!
summary
	self selectedVersionIsFalseAncestor
		ifTrue: [^ (Text string: 'Beware, this is a false ancestor whose name conflicts with a true one\' withCRs attributes: {TextColor red. TextEmphasis bold})
				, super summary].
	^ super summary! !

!MCRepositoryInspector methodsFor: 'morphic ui' stamp: 'cmm 3/6/2011 16:17'!
version
	^ version ifNil:
		[Cursor wait showWhile:
			[version := repository versionNamed: selectedVersion].
		version]! !

!MCRepositoryInspector methodsFor: 'morphic ui' stamp: 'nice 2/13/2019 16:51'!
versionHighlight: aMCVersionName 
	inherited ifNil: [inherited := #()].
	inheritedId ifNil: [inheritedId := #()].
	(selectedVersion notNil
			and: [selectedVersion = aMCVersionName
					and: [self selectedVersionIsFalseAncestor]])
		ifTrue: ["False ancestor might be dangerous, signal them in red"
			^ Text string: aMCVersionName attributes: {TextColor red. TextEmphasis bold}].
	^ Text
		string: aMCVersionName
		attribute: (TextEmphasis new
				emphasisCode: ((loaded includes: aMCVersionName)
						ifTrue: ["underlined" 4]
						ifFalse: [(inherited includes: aMCVersionName)
								ifTrue: [0]
								ifFalse: ["bold" 1]]))! !

!MCRepositoryInspector methodsFor: 'morphic ui' stamp: 'eem 3/13/2017 19:13'!
versionList
	| result |
	result := selectedPackage
				ifNil: [ self versionNamesForNoPackageSelection ]
				ifNotNil: [ self versionNamesForSelectedPackage ].
	(self orderSpecs at: order) value ifNotNil:
		[ : sortBlock |
		result sort: sortBlock ].
	^result collect:
		[ : each | self versionHighlight: each ]! !

!MCRepositoryInspector methodsFor: 'morphic ui' stamp: 'ct 9/17/2020 10:49'!
versionListMenu: aMenu

	1 to: self orderSpecs size do: [ :index |
		aMenu addUpdating: #orderString: target: self selector: #order: argumentList: { index } ].
	aMenu addLine.
	aMenu add: 'Changes against ...' translated action: [
		| ri versions seen |
		ri := aMenu defaultTarget.
		versions := ri versionList
			collect: [:name | MCVersionName on: name]
			as: OrderedCollection.
		seen := versions asSet.
		self version info breadthFirstAncestors do: [:ancestor |
			(seen includes: ancestor name) ifFalse: [
				versions add: ancestor.
				seen add: ancestor name]].
		(UIManager default
			chooseFrom: (versions collect: [:version | version name])
			values: versions
			title: 'Select version to show patch against ...' translated) ifNotNil: [:name |
			| target base |
			target := ri repository versionNamed: ri versionInfo name.
			base := name isString
				ifTrue: [ri repository versionNamed: name]
				ifFalse: [ri version workingCopy repositoryGroup versionWithInfo: name].
			(MCPatchBrowser
				forPatch: (target snapshot patchRelativeToBase: base snapshot))
			showLabelled: ('Changes from {1} to {2}' translated format: {name. ri versionInfo name})]].
	^aMenu! !

!MCRepositoryInspector methodsFor: 'access' stamp: 'cmm 3/4/2011 16:27'!
versionNames
	^ versionNames ifNil:
		[ self initializeVersionNames.
		versionNames ]! !

!MCRepositoryInspector methodsFor: 'access' stamp: 'cmm 3/4/2011 16:04'!
versionNamesForNoPackageSelection
	^ Array empty! !

!MCRepositoryInspector methodsFor: 'morphic ui' stamp: 'cmm 3/4/2011 16:06'!
versionNamesForSelectedPackage
	^ self versionNames! !

!MCRepositoryInspector methodsFor: 'morphic ui' stamp: 'lr 9/26/2003 20:18'!
versionSelection
	^self versionList indexOf: selectedVersion! !

!MCRepositoryInspector methodsFor: 'morphic ui' stamp: 'nice 2/13/2019 16:50'!
versionSelection: aNumber 
	selectedVersion := version := nil.
	aNumber isZero ifFalse: [ selectedVersion := (self versionList at: aNumber) asString ].
	self selectedVersionIsFalseAncestor ifTrue: [self changed: #versionList].
	self
		 changed: #versionSelection ;
		 changed: #summary ;
		 changed: #hasVersion! !

!MCRepositoryInspector methodsFor: 'morphic ui' stamp: 'mt 4/30/2015 13:39'!
widgetSpecs
	^#(	((buttonRow) (0 0 1 0) (0 0 0 defaultButtonPaneHeight))
		((listMorph: package) (0 0 0.5 0.6) (0 defaultButtonPaneHeight 0 0))
		((listMorph: version) (0.5 0 1 0.6) (0 defaultButtonPaneHeight 0 0))
		((textMorph: summary) (0 0.6 1 1) (0 0 0 0)) )! !

!MCVersionInspector methodsFor: 'accessing' stamp: 'avi 2/28/2004 20:19'!
adopt
	(self confirm:
'Modifying ancestry can be dangerous unless you know
what you are doing.  Are you sure you want to adopt
',self version info name, ' as an ancestor of your working copy?')
		ifTrue: [self version adopt]! !

!MCVersionInspector methodsFor: 'accessing' stamp: 'bf 3/22/2005 22:12'!
browse
	self version browse! !

!MCVersionInspector methodsFor: 'morphic ui' stamp: 'dtl 11/5/2018 21:36'!
buttonSpecs
	^#(
		('Refresh' refresh 'refresh the version-list')
		(Browse browse 'Browse this version' hasVersion)
		(History history 'Browse the history of this version' hasVersion)
		(Changes changes 'Browse the changes this version would make to the image' hasVersion)
		(Load load 'Load this version into the image' hasVersion)
		(Merge merge 'Merge this version into the image' hasVersion)
		(Adopt adopt 'Adopt this version as an ancestor of your working copy' hasVersion)
		(Reparent reparent 'Adopt this version as the sole ancestor of your working copy' hasVersion)
		(Copy save 'Copy this version to another repository' hasVersion)
		(Diff diff 'Create an equivalent version based on an earlier release' hasVersion)
	)! !

!MCVersionInspector methodsFor: 'accessing' stamp: 'nk 4/17/2004 10:05'!
changes
	(MCPatchBrowser forPatch: self version changes)
		showLabelled: 'Changes from ', self version info name! !

!MCVersionInspector methodsFor: 'morphic ui' stamp: 'avi 8/31/2003 00:45'!
defaultExtent
	^ 400@200! !

!MCVersionInspector methodsFor: 'morphic ui' stamp: 'avi 2/28/2004 20:18'!
defaultLabel
	^ 'Version: ', self version info name! !

!MCVersionInspector methodsFor: 'accessing' stamp: 'ul 12/12/2009 14:09'!
diff
	| ancestorVersion |
	self pickAncestor ifNotNil:
		[:ancestor |
		ancestorVersion := self version workingCopy repositoryGroup versionWithInfo: ancestor.
		(self version asDiffAgainst: ancestorVersion) open]! !

!MCVersionInspector methodsFor: 'accessing' stamp: 'lr 9/26/2003 20:15'!
hasVersion
	^version notNil! !

!MCVersionInspector methodsFor: 'accessing' stamp: 'avi 9/17/2005 17:14'!
history
	(MCVersionHistoryBrowser new ancestry: self versionInfo) show! !

!MCVersionInspector methodsFor: 'accessing' stamp: 'bf 3/14/2005 15:32'!
load
	Cursor wait showWhile: [self version load]! !

!MCVersionInspector methodsFor: 'accessing' stamp: 'avi 2/28/2004 20:19'!
merge
	self version merge! !

!MCVersionInspector methodsFor: 'morphic ui' stamp: 'ar 8/6/2009 18:25'!
pickAncestor
	| index versions |
	versions := self version info breadthFirstAncestors.
	index := UIManager default chooseFrom: (versions collect: [:ea | ea name])
				title: 'Ancestor:'.
	^ index = 0 ifFalse: [versions at: index]! !

!MCVersionInspector methodsFor: 'morphic ui' stamp: 'ar 8/6/2009 18:25'!
pickRepository
	| index |
	index := UIManager default chooseFrom: (self repositories collect: [:ea | ea description])
				title: 'Repository:'.
	^ index = 0 ifFalse: [self repositories at: index]! !

!MCVersionInspector methodsFor: 'accessing' stamp: 'ct 10/3/2019 02:27'!
reparent
	(self confirm:'Adopt ',self version info name, ' as the sole ancestor of your working copy?')
		ifTrue: [
			self version reparent.
			self refresh]! !

!MCVersionInspector methodsFor: 'morphic ui' stamp: 'avi 8/31/2003 00:44'!
repositories
	^ MCRepositoryGroup default repositories! !

!MCVersionInspector methodsFor: 'accessing' stamp: 'ul 12/12/2009 14:09'!
save
	self pickRepository ifNotNil:
		[:ea |
		ea storeVersion: self version]! !

!MCVersionInspector methodsFor: 'accessing' stamp: 'avi 9/17/2005 17:16'!
summary
	^self hasVersion
		ifTrue: [ self versionSummary ]
		ifFalse: [ String new ]! !

!MCVersionInspector methodsFor: 'accessing' stamp: 'avi 2/28/2004 20:19'!
version
	^ version! !

!MCVersionInspector methodsFor: 'accessing' stamp: 'ab 7/12/2003 00:13'!
version: aVersion
	version := aVersion! !

!MCVersionInspector methodsFor: 'accessing' stamp: 'avi 9/17/2005 17:16'!
versionInfo
	^ self version info! !

!MCVersionInspector methodsFor: 'accessing' stamp: 'avi 9/17/2005 17:16'!
versionSummary
	^ self version summary! !

!MCVersionInspector methodsFor: 'morphic ui' stamp: 'mt 4/30/2015 13:43'!
widgetSpecs
	^ #(
		((buttonRow) (0 0 1 0) (0 0 0 defaultButtonPaneHeight))
		((textMorph: summary) (0 0 1 1) (0 defaultButtonPaneHeight 0 0))
		)! !

MCTool subclass: #MCWorkingCopyBrowser
	instanceVariableNames: 'workingCopy workingCopyWrapper repository defaults'
	classVariableNames: 'CheckForNewerVersionsBeforeSave ExternalMenuEntries'
	poolDictionaries: ''
	category: 'Monticello-UI'!

!MCWorkingCopyBrowser class methodsFor: 'hooks' stamp: 'tfel 6/12/2010 14:55'!
addMenuSpec: aMCMenuSpec
	"Register a context menu entry in the monticello browser from an external package.
	 The MCWorkingCopyBrowser model is passed as argument."
	self externalMenuEntries 
		remove: aMCMenuSpec 
		ifAbsent: ["Remove any previous entry with description string"];
		add: aMCMenuSpec.! !

!MCWorkingCopyBrowser class methodsFor: 'preferences' stamp: 'ar 3/10/2010 21:12'!
checkForNewerVersionsBeforeSave
	"Preference accessor"
	<preference: 'Check for new versions before save'
		category: 'Monticello'
		description: 'If true, MC will warn before committing to repositories that have possibly newer versions of the package being saved.'
		type: #Boolean>
	^CheckForNewerVersionsBeforeSave ifNil:[true]! !

!MCWorkingCopyBrowser class methodsFor: 'preferences' stamp: 'ar 3/10/2010 21:12'!
checkForNewerVersionsBeforeSave: aBool
	"Sets the CheckForNewerVersionsBeforeSave preference"
	CheckForNewerVersionsBeforeSave := aBool! !

!MCWorkingCopyBrowser class methodsFor: 'hooks' stamp: 'tfel 6/12/2010 11:44'!
externalMenuEntries
	
	ExternalMenuEntries ifNil: [ExternalMenuEntries := Set new].
	^ ExternalMenuEntries! !

!MCWorkingCopyBrowser class methodsFor: 'class initialization' stamp: 'ab 7/19/2003 23:38'!
initialize
	 (TheWorldMenu respondsTo: #registerOpenCommand:)
         ifTrue: [TheWorldMenu registerOpenCommand: {'Monticello Browser'. {self. #open}}]! !

!MCWorkingCopyBrowser class methodsFor: 'instance creation' stamp: 'avi 9/13/2004 18:04'!
new
	^ self basicNew initialize! !

!MCWorkingCopyBrowser class methodsFor: 'instance creation' stamp: 'ab 7/19/2003 23:27'!
open
	self new show! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'nice 6/10/2017 17:32'!
addPackageRepository

	workingCopy ifNotNil:
		[
		(self pickRepositorySatisfying: [ :repos | (workingCopy repositoryGroup includes: repos) not ])
			ifNotNil:
				[:repos |
					workingCopy repositoryGroup addRepository: repos.
					self repository: repos.	
					self
						changed: #repositoryList;
						changed: #repositorySelection.
					self changedButtons]]! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'ul 12/12/2009 14:09'!
addRepository
	self newRepository ifNotNil:
		[:repos | self addRepository: repos ].
! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'bf 6/21/2005 15:58'!
addRepository: aRepository
	self repository: aRepository.
	self repositoryGroup addRepository: aRepository.
	self changed: #repositoryList; changed: #repositorySelection.
	self changedButtons.! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'ul 12/12/2009 14:12'!
addRepositoryToPackage
	self repository ifNotNil:
		[:repos |
		(self pickWorkingCopySatisfying: [ :p | (p repositoryGroup includes: repos) not ]) ifNotNil:
			[:wc |
			workingCopy := wc.
			workingCopy repositoryGroup addRepository: repos.
			self repository: repos.	
			self
				changed: #workingCopySelection;
				changed: #repositoryList;
				changed: #repositorySelection.
			self changedButtons]]! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'ul 12/12/2009 14:09'!
addRepositoryToWorkingCopy
	workingCopy ifNotNil:
		[:wc |
			workingCopy repositoryGroup addRepository: self repository.
			self
				changed: #workingCopySelection;
				changed: #repositoryList;
				changed: #repositorySelection.
			self changedButtons]! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'ul 12/12/2009 14:09'!
addRequiredPackage
	| chosen |
	workingCopy ifNotNil:
		[:wc |
		chosen := self pickWorkingCopySatisfying: 
			[:ea | ea ~= wc and: [(wc requiredPackages includes: ea package) not]].
		chosen ifNotNil:
			[wc requirePackage: chosen package.
			self workingCopyListChanged]]! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'fbs 4/22/2013 09:34'!
addWorkingCopy
	|name|
	name := UIManager default request: 'Name of package:'.
	name isEmptyOrNil ifFalse:
		[PackageInfo registerPackageName: name.
		workingCopy := MCWorkingCopy forPackage: (MCPackage new name: name).
		workingCopyWrapper := nil.
		self repositorySelection: 0].
	self workingCopyListChanged; changed: #workingCopySelection; changed: #repositoryList.
	self changedButtons.! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'ul 12/12/2009 14:09'!
backportChanges
	self canBackport ifFalse: [^self].
	workingCopy ifNotNil:
		[workingCopy needsSaving ifTrue: [^ self inform: 'You must save the working copy before backporting.'].
		self pickAncestorVersionInfo ifNotNil:
			[:baseVersionInfo |
			workingCopy backportChangesTo: baseVersionInfo]]! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'mt 8/14/2016 12:51'!
browseWorkingCopy
	workingCopy ifNotNil: [:wc | wc browse].! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'avi 2/28/2005 16:28'!
buttonSpecs
       ^ #(
               ('+Package' addWorkingCopy 'Add a new package and make it the working copy')
               (Browse browseWorkingCopy 'Browse the working copy of the selected package' hasWorkingCopy)
               (Scripts editLoadScripts 'Edit the load/unload scripts of this package' hasWorkingCopy)
               (History viewHistory 'View the working copy''s history' hasWorkingCopy)
               (Changes viewChanges 'View the working copy''s changes relative to the installed version from the repository' canSave)
               (Backport backportChanges 'Backport the working copy''s changes to an ancestor' canBackport)
               (Save saveVersion 'Save the working copy as a new version to the selected repository' canSave)
             ('+Repository' addRepository 'Add an existing repository to the list of those visible')
               (Open openRepository 'Open a browser on the selected repository' hasRepository)
               )! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'avi 9/14/2004 14:57'!
canBackport
	^ self hasWorkingCopy and: [workingCopy needsSaving not]! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'ab 8/7/2003 21:22'!
canSave
	^ self hasWorkingCopy and: [self hasRepository]! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'jr 1/15/2017 00:28'!
changeEnvironment
	workingCopy ifNil: [^ self].
	workingCopy environment: EnvironmentRequest signal! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'bf 6/21/2005 15:50'!
changedButtons
	self changed: #hasWorkingCopy.
	self changed: #canSave.
	self changed: #canBackport.
	self changed: #hasRepository.
! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'bf 9/11/2013 17:30'!
checkAllPackages
	MCWorkingCopy checkModified: true.
! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'ul 5/8/2011 19:44'!
checkForNewerVersions
	| newer |
	newer := workingCopy possiblyNewerVersionsIn: self repository.
	^ newer isEmpty or: [
		self confirm: 'CAUTION!! These versions in the repository may be newer:', 
			String cr, ((newer asCommaString withNoLineLongerThan:  150) truncateWithElipsisTo: 5 * 149), String cr,
			'Do you really want to save this version?'].! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'ul 12/12/2009 14:09'!
clearRequiredPackages
	workingCopy ifNotNil:
		[:wc |
		wc clearRequiredPackages.
		self workingCopyListChanged]! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'cmm 8/14/2011 14:44'!
copyImageVersions
	self repository ifNotNil: [ : repos | repos copyImageVersions ]! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'avi 2/28/2005 16:28'!
defaultExtent
	^ 550@200! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'ab 7/19/2003 23:38'!
defaultLabel
	^ 'Monticello Browser'! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'avi 11/16/2003 20:22'!
defaults
	^ defaults ifNil: [defaults := Dictionary new]! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'ar 2/14/2004 02:46'!
deleteWorkingCopy
	workingCopy unregister.
	self workingCopySelection: 0.
	self workingCopyListChanged.! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'cmm 10/31/2016 14:00'!
demoteRepository
	self repository ifNotNil:
		[ : repo | self repositoryGroup demoteRepository: repo.
		self
			 changed: #repositoryList ;
			 changed: #repositorySelection ]! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'fbs 4/22/2013 20:42'!
editLoadScripts

	| arg |
	self hasWorkingCopy ifFalse: [^self].
	arg := UIManager default
		chooseFrom: #('edit preamble' 'edit postscript' 'edit preambleOfRemoval' 'edit postscriptOfRemoval')
		values: #(#preamble #postscript #preambleOfRemoval #postscriptOfRemoval).

	arg ifNotNil: [
		self editScript: arg].! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'ul 1/12/2010 01:03'!
editRepository
	| newRepo |
	
	newRepo := self repository openAndEditTemplateCopy.
	newRepo ifNotNil: [ 
		newRepo class = self repository class
			ifTrue: [
				self repository copyFrom: newRepo.
				self changed: #repositoryList ]
			ifFalse: [self inform: 'Must not change repository type!!']]
! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'stephaneducasse 2/4/2006 20:47'!
editScript: scriptSymbol

| script |
script := workingCopy packageInfo perform: scriptSymbol.
script openLabel: scriptSymbol asString, ' of the Package ', workingCopy package name.! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'cmm 3/23/2015 14:09'!
flushCachedVersions
	| beforeBytes afterBytes beforeVersions afterVersions |
	Cursor wait showWhile: [
		Smalltalk garbageCollect.
		beforeBytes := Smalltalk bytesLeft: true.
		beforeVersions := MCVersion allSubInstances size.
		MCFileBasedRepository flushAllCaches.
		afterBytes := Smalltalk bytesLeft: true.
		afterVersions := MCVersion allSubInstances size.
	].
	^self inform: (beforeVersions - afterVersions) asString, ' versions flushed', String cr,
 		(beforeBytes - afterBytes) asBytesDescription, ' bytes reclaimed'! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'cmm 8/16/2013 14:55'!
flushCachedVersionsAncestry
	| beforeBytes afterBytes beforeVersions afterVersions beforeInfos afterInfos |
	Cursor wait showWhile: 
		[ beforeBytes := Smalltalk garbageCollect.
		beforeVersions := MCVersion allSubInstances size.
		beforeInfos := MCVersionInfo instanceCount.
		MCFileBasedRepository flushAllCaches.
		MCWorkingCopy stubAllAncestry.
		afterBytes := Smalltalk garbageCollect.
		afterVersions := MCVersion allSubInstances size.
		afterInfos := MCVersionInfo instanceCount ].
	self inform: 
		(beforeVersions-afterVersions) asString, ' versions flushed',
		String cr, (beforeInfos-afterInfos), ' infos flushed',
	 	String cr, (afterBytes-beforeBytes) asBytesDescription, ' reclaimed'! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'ab 7/22/2003 02:21'!
hasRepository
	^ self repository notNil! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'ab 7/22/2003 00:46'!
hasWorkingCopy
	^ workingCopy notNil! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'avi 11/16/2003 20:21'!
initialize
	MCWorkingCopy addDependent: self.
	self workingCopies do: [:ea | ea addDependent: self].! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'tfel 6/12/2010 13:47'!
insertExternalMenuEntries: aMenu
	
	self class externalMenuEntries ifNotEmpty: [
		aMenu addLine.
		self class externalMenuEntries do: [:each |
			aMenu 
				add: each entry 
				target: each target
				selector: each selector
				argument: self]].! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'avi 9/11/2004 15:32'!
inspectWorkingCopy
	workingCopy ifNotNil: [workingCopy inspect]! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'bf 6/21/2005 15:58'!
loadRepositories
	FileStream fileIn: 'MCRepositories.st'.
	self changed: #repositoryList.
	self changedButtons.
! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'ar 8/6/2009 18:25'!
newRepository
	| types index |
	types := MCRepository allConcreteSubclasses asArray.
	index := UIManager default chooseFrom: (types collect: [:ea | ea description])
				title: 'Repository type:'.
	^ index = 0 ifFalse: [(types at: index) morphicConfigure]! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'ul 12/12/2009 14:09'!
openRepository
	self repository ifNotNil: [:repos | repos morphicOpen: workingCopy ]! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'ar 8/6/2009 18:26'!
pickAncestorVersionInfo
	| ancestors index |
	ancestors := workingCopy ancestry breadthFirstAncestors.
	index := UIManager default chooseFrom: (ancestors collect: [:ea | ea name])
				title: 'Ancestor:'.
	^ index = 0 ifFalse: [ ancestors at: index]! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'ar 3/6/2006 18:16'!
pickRepositorySatisfying: aBlock
	| repos index |
	repos := MCRepositoryGroup default repositories select: aBlock.
	index := UIManager default chooseFrom: (repos collect: [:ea | ea description])
				title: 'Repository:'.
	^ index = 0 ifFalse: [repos at: index]! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'nk 3/9/2004 14:39'!
pickWorkingCopy
	^self pickWorkingCopySatisfying: [ :c | true ]! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'ar 8/6/2009 18:26'!
pickWorkingCopySatisfying: aBlock
	| copies index |
	copies := self workingCopies select: aBlock.
	copies isEmpty ifTrue: [ ^nil ].
	index := UIManager default chooseFrom: (copies collect: [:ea | ea packageName])
				title: 'Package:'.
	^ index = 0 ifFalse: [ copies at: index]! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'jr 3/2/2017 23:19'!
recompilePackage
	(workingCopy withEnvironmentActiveDo: [workingCopy package packageInfo methods])
		do: [:ea | ea actualClass recompile: ea methodSymbol]
		displayingProgress: 'Recompiling...'! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'ul 12/12/2009 14:09'!
removeRepository
	self repository ifNotNil:
		[:repos |
		self repositoryGroup removeRepository: repos.
		self repositorySelection: (1 min: self repositories size)].
	self changed: #repositoryList.
	self changedButtons.
! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'fbs 4/22/2013 09:35'!
renamePackage
	| newName |
	workingCopy ifNil:
		[ UIManager inform: 'Please select a package to be renamed.'.
		^ self ].
	workingCopy modified ifTrue:
		[ UIManager inform: 'Only unmodified packages should be renamed.'.
		^ self ].
	newName := UIManager default
		request: 'New name of package:'
		initialAnswer: workingCopy packageName.
	newName isEmptyOrNil ifFalse:
		[ | newWorkingCopy |
		newWorkingCopy := workingCopy renameToBe: newName.
		workingCopy package snapshot definitions
			ifEmpty:
				[ "It worked."
				workingCopy unregister.
				workingCopy := newWorkingCopy.
				self repositorySelection: 0.
				self
					 workingCopyListChanged ;
					 changed: #workingCopySelection ;
					 changed: #repositoryList.
				self changedButtons ]
			ifNotEmpty: 
				[ "It didn't work, browse the remaining definitions."
				self browseWorkingCopy ] ]! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'ab 8/7/2003 21:32'!
repositories
	^ self repositoryGroup repositories! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'avi 11/16/2003 20:21'!
repository
	workingCopy ifNotNil: [repository := self defaults at: workingCopy ifAbsent: []].
	^ repository! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'avi 11/16/2003 20:21'!
repository: aRepository
	repository := aRepository.
	workingCopy ifNotNil: [self defaults at: workingCopy put: aRepository]! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'ab 8/7/2003 21:32'!
repositoryGroup
	^ workingCopy
		ifNil: [MCRepositoryGroup default]
		ifNotNil: [workingCopy repositoryGroup]! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'ab 7/19/2003 22:04'!
repositoryList
	^ self repositories collect: [:ea | ea description]! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'cmm 1/19/2018 16:07'!
repositoryListMenu: aMenu
	"first add repository-specific commands"
	self repository ifNotNil:
		[ self
			fillMenu: aMenu
			fromSpecs:
				#(('open repository' #openRepository)
				('edit repository info' #editRepository)
				('add to package...' #addRepositoryToPackage)
				('remove repository' #removeRepository)
				('demote to bottom' #demoteRepository)
				('copy image versions here' #copyImageVersions)).
		aMenu
			add:
				(self repository alwaysStoreDiffs
					ifTrue: ['store full versions']
					ifFalse: ['store diffs'])
				target: self
				selector: #toggleDiffs ;
			addLine ].
	"then the non-specific commands"
	^self fillMenu: aMenu fromSpecs:
		#(	('load repositories' #loadRepositories)
		 	('save repositories' #saveRepositories)
			('flush cached versions' #flushCachedVersions))! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'ab 7/22/2003 02:22'!
repositorySelection
	^ self repositories indexOf: self repository! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'bf 6/21/2005 15:56'!
repositorySelection: aNumber
	aNumber = 0
		ifTrue: [self repository: nil]
		ifFalse: [self repository: (self repositories at: aNumber)].
	self changed: #repositorySelection.
	self changedButtons.
! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'cmm 2/13/2010 20:03'!
representsSameBrowseeAs: anotherModel
	^ self class = anotherModel class! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'ul 12/12/2009 14:09'!
revertPackage
	self pickAncestorVersionInfo ifNotNil: [:info |
		(self repositoryGroup versionWithInfo: info
			ifNone: [^self inform: 'No repository found for ', info name]
		) load]! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'nice 3/31/2010 22:30'!
saveRepositories
	FileStream forceNewFileNamed: 'MCRepositories.st' do: [:f |
		MCRepositoryGroup default repositoriesDo: [:r |
			r asCreationTemplate ifNotNil: [:template |
				f nextPutAll: 'MCRepositoryGroup default addRepository: (', template , ')!!'; cr]]]! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'mt 8/1/2016 15:08'!
saveVersion
	| repo |
	self canSave ifFalse: [^self].
	self checkForNewerVersions ifFalse: [^self].
	repo := self repository.
	(repo == MCRepository trunk and: [SystemVersion current isFeatureFreeze])
		ifTrue: [self inform: 'FEATURE FREEZE. A new release is being prepared.\Please do only do bugfixes, but no new features.' translated withCRs].
	(repo == MCRepository trunk and: [SystemVersion current isCodeFreeze])
		ifTrue: [self inform: 'CODE FREEZE. The new release is almost ready.\Please do only do URGENT fixes, if any.' translated withCRs].
				
	(self withRepository: repo do: [workingCopy newVersion]) ifNotNil:
		[:v |
		(MCVersionInspector new version: v) show.
		Cursor wait showWhile: [repo storeVersion: v].
		MCCacheRepository default cacheAllFileNamesDuring: 
			[repo cacheAllFileNamesDuring: 
				[v allAvailableDependenciesDo:
					[:dep |
					(repo includesVersionNamed: dep info name)
						ifFalse: [repo storeVersion: dep]]]]]! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'eem 4/26/2012 09:37'!
searchHistory
	workingCopy ifNotNil:
		[(UIManager default
			edit: (String streamContents:
					[:s|
					"Ignore the initial MCWorkingAncestry instance."
					workingCopy ancestry topologicalAncestors allButFirst
						do: [:versionInfo|
							s nextPutAll: versionInfo summary]
						separatedBy: [s cr; cr"; next: 32 put: $-; cr; cr"]])
			label: 'Version History: ',  workingCopy packageName)
				extent: (0.6@1) * Display height * 0.9]! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'avi 8/31/2004 01:14'!
toggleDiffs
	self repository alwaysStoreDiffs
		ifTrue: [self repository doNotAlwaysStoreDiffs]
		ifFalse: [self repository doAlwaysStoreDiffs]! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'ar 2/14/2004 02:46'!
unloadPackage
	workingCopy unload.
	self workingCopySelection: 0.
	self workingCopyListChanged.! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'avi 9/10/2004 17:54'!
unsortedWorkingCopies
	^ MCWorkingCopy allManagers ! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'avi 9/10/2004 17:54'!
update: aSymbol
	self unsortedWorkingCopies do: [:ea | ea addDependent: self].
	self workingCopyListChanged.! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'cmm 4/6/2019 16:35'!
viewChanges
	| patch |
	self canSave ifTrue:
		[patch := workingCopy changesRelativeToRepository: self repository withCache.
		patch isNil ifTrue: [ ^self ].
		patch isEmpty
			ifTrue: [ workingCopy modified: false.
				self inform: 'No changes' ]
			ifFalse:
				[ workingCopy modified: true.
				(MCPatchBrowser forPatch: patch)
					label: 'Patch Browser: ', workingCopy description;
					environmentInDisplayingImage: workingCopy environment;
					show ] ]! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'avi 2/13/2004 01:13'!
viewHistory
	workingCopy ifNotNil:
		[(MCWorkingHistoryBrowser new
				ancestry: workingCopy ancestry;
				package: workingCopy package)
			label:  'Version History: ',  workingCopy packageName;
			show]! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'mt 4/30/2015 13:43'!
widgetSpecs
	^ #(
		((buttonRow) (0 0 1 0) (0 0 0 defaultButtonPaneHeight))
		((treeOrListMorph: workingCopy) (0 0 0.5 1) (0 defaultButtonPaneHeight 0 0))
		((listMorph: repository) (0.5 0 1 1) (0 defaultButtonPaneHeight 0 0))
		)! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'bf 4/11/2013 16:50'!
withRepository: aRepository do: aBlock
	^aBlock
		on: MCRepositoryRequest
		do: [:req | req resume: aRepository]! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'ul 3/13/2017 00:42'!
workingCopies
	^ MCWorkingCopy allManagers sort:
		[ :a :b | a package name <= b package name ]! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'mt 10/14/2019 18:58'!
workingCopy: wc
	workingCopy := wc.
	self
		changed: #workingCopySelection;
		changed: #repositoryList;
		changed: #repositorySelection.
	self changedButtons.
! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'ul 10/18/2018 01:44'!
workingCopyList
	^ self workingCopies collect:
		[:ea |
		(workingCopy notNil and: [workingCopy requiredPackages includes: ea package])
			ifTrue: [Text string: ea description emphasis: { TextEmphasis bold }]
			ifFalse: [ea description]]! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'bf 6/21/2005 15:51'!
workingCopyListChanged
	self changed: #workingCopyList.
	self changed: #workingCopyTree.
	self changedButtons.
! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'jr 2/28/2017 02:29'!
workingCopyListMenu: aMenu
	workingCopy ifNil: [^ aMenu].
	self fillMenu: aMenu fromSpecs:
		#(('add required package' #addRequiredPackage)
			('clear required packages' #clearRequiredPackages)
			('add repository...' #addPackageRepository)
			('browse package' #browseWorkingCopy)
			('view changes' #viewChanges)
			('view history' #viewHistory)
			('search history' #searchHistory)
			('recompile package' #recompilePackage)
			('revert package...' #revertPackage)
			('unload package' #unloadPackage)
			('delete working copy' #deleteWorkingCopy)
			('inspect working copy' #inspectWorkingCopy)
			('rename package...' #renamePackage)
			('change environment...' #changeEnvironment)).
	(Smalltalk includesKey: #SARMCPackageDumper) ifTrue: [
		aMenu add: 'make SAR' target: self selector: #fileOutAsSAR
	].
	self fillMenu: aMenu fromSpecs:
		#(	addLine
			('check all packages for changes' #checkAllPackages)).
	self insertExternalMenuEntries: aMenu.
	^aMenu! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'ab 7/19/2003 22:05'!
workingCopySelection
	^ self workingCopies indexOf: workingCopy! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'ar 2/14/2004 02:48'!
workingCopySelection: aNumber
	self workingCopy: 
		(aNumber = 0 
			ifTrue:[nil]
			ifFalse:[self workingCopies at: aNumber]).	! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'ar 2/14/2004 02:38'!
workingCopySelectionWrapper
	^workingCopyWrapper! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'ar 2/14/2004 02:57'!
workingCopySelectionWrapper: aWrapper
	workingCopyWrapper := aWrapper.
	self changed: #workingCopySelectionWrapper.
	self workingCopy: (aWrapper ifNotNil:[aWrapper item])! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'ar 2/14/2004 02:30'!
workingCopyTree
	^ self workingCopies collect:[:each| MCDependentsWrapper with: each model: self].! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'bf 9/11/2013 17:26'!
workingCopyTreeMenu: aMenu
	^self workingCopyListMenu: aMenu
! !

!Environment methodsFor: '*Monticello-Loading' stamp: 'jr 10/19/2016 17:05'!
provisions
	"In contrast to #keys, return also the imported names"
	^ bindings keys! !

!ChangeList class methodsFor: '*monticello' stamp: 'nice 1/19/2010 01:52'!
recent: charCount on: origChangesFile 
	"Opens a changeList on the end of the specified changes log file"
	| changeList end changesFile |
	changesFile := origChangesFile readOnlyCopy.
	end := changesFile size.
	changeList := Cursor read
		showWhile: [self new
						scanFile: changesFile
						from: (0 max: end - charCount)
						to: end].
	changesFile close.
	^changeList! !

!ChangeList class methodsFor: '*monticello' stamp: 'ul 11/15/2010 11:34'!
recentLogOn: origChangesFile startingFrom: initialPos 
	"Prompt with a menu of how far back to go when browsing a changes file."

	| end banners positions pos chunk i changesFile |
	changesFile := origChangesFile readOnlyCopy.
	banners := OrderedCollection new.
	positions := OrderedCollection new.
	end := changesFile size.
	pos := initialPos.
	[pos = 0
		or: [banners size > 20]]
		whileFalse: [changesFile position: pos.
			chunk := changesFile nextChunk.
			i := chunk indexOfSubCollection: 'priorSource: ' startingAt: 1.
			i > 0
				ifTrue: [positions addLast: pos.
					banners
						addLast: (chunk copyFrom: 5 to: i - 2).
					pos := Number
								readFrom: (chunk copyFrom: i + 13 to: chunk size)]
				ifFalse: [pos := 0]].
	changesFile close.
	banners size = 0 ifTrue: [^self recent: end on: origChangesFile].

	pos := UIManager default chooseFrom: banners values: positions
				title: 'Browse as far back as...'.
	pos == nil
		ifTrue: [^ self].
	^self recent: end - pos on: origChangesFile! !

!ChangeList methodsFor: '*monticello' stamp: 'dvf 7/23/2003 14:44'!
changeTo: changeSubset
	| newList newChangeList |

	newChangeList := OrderedCollection new.
	newList := OrderedCollection new.

	1 to: changeList size do:
		[:i | (changeSubset includes: (changeList at: i)) ifTrue:
			[newChangeList add: (changeList at: i).
			newList add: (list at: i)]].
	newChangeList size < changeList size
		ifTrue:
			[changeList := newChangeList.
			list := newList.
			listIndex := 0.
			listSelections := Array new: list size withAll: false].
	self changed: #list

	! !

!String methodsFor: '*monticello' stamp: 'ul 2/27/2017 00:43'!
withLoadedPackageVersions
	"Everywhere the receiver contains a package version in the format:  PackageName-author.123 for which a version of that package is loaded in the image, answer a new String with each of those package versions changed to reflect the currently loaded version."
	| string |
	string := self.
	MCWorkingCopy allManagers do:
		[ : eachWorkingCopy | | start packageName |
		start := 1.
		packageName := eachWorkingCopy packageName.
		[ start := string
			findString: packageName
			startingAt: start.
		start > 0 and: [ string size > (start + packageName size + 4) ] ] whileTrue:
			[ | dot end versionName |
			dot := string
				indexOf: $.
				startingAt: (end := start + packageName size).
			dot > 0 ifTrue:
				[ end := (dot + 1 to: string size)
					detect: [ : n | (string at: n) isDigit not ]
					ifNone: [ 0 ].
				(end > (dot + 1) and:
					[ (dot + 1 to: end - 1) allSatisfy:
						[ : n | (string at: n) isDigit ] ])
					ifTrue:
						[ versionName := string
							copyFrom: start
							to: end - 1.
						versionName asMCVersionName packageName = packageName ifTrue:
							[ string := string
								copyReplaceTokens: versionName
								with: eachWorkingCopy ancestors first name ] ]
					ifFalse: [ end := start + packageName size ] ].
			start := end ] ].
	^ string! !

!Class methodsFor: '*monticello' stamp: 'al 3/26/2006 21:31'!
asClassDefinition
	^ MCClassDefinition
		name: self name
		superclassName: self superclass name
		traitComposition: self traitCompositionString
		classTraitComposition: self class traitCompositionString
		category: self category 
		instVarNames: self instVarNames
		classVarNames: self classVarNames
		poolDictionaryNames: self poolDictionaryNames
		classInstVarNames: self class instVarNames
		type: self typeOfClass
		comment: self organization classComment	 asString
		commentStamp: self organization commentStamp	! !

!Class methodsFor: '*monticello' stamp: 'ul 11/14/2018 00:05'!
classDefinitions

	^{ self asClassDefinition }! !

!Class methodsFor: '*monticello' stamp: 'cmm 3/8/2011 21:12'!
mcModel
	^ self asClassDefinition mcModel! !

!Class methodsFor: '*monticello' stamp: 'cmm 11/27/2016 16:17'!
mcPatchOperations
	"Answer a collection of MCPatchOperations that can be displayed in a MCPatchBrowser which will reflect the history of this methods definition in relation to each other (not simply compared to the image version)."
	^ (self mcRevisions reversed
		inject: OrderedCollection new
		into:
			[ : coll : each | coll
				ifEmpty:
					[ coll
						 add: (MCAddition of: each) ;
						 yourself ]
				ifNotEmpty:
					[ coll
						 add:
							(MCModification
								of:
									(coll last isAddition
										ifTrue: [ coll last definition ]
										ifFalse: [ coll last modification ])
								to: each) ;
						 yourself ] ]) reversed! !

!Class methodsFor: '*monticello' stamp: 'cmm 11/27/2016 16:16'!
mcRevisions
	"Answer a collection of MCClassDefinitions for this Class, which are in the first Magma-backed repository in the list of repositories of my working-copy."
	^ self mcModel ifNotNil: [ : mcmodel | mcmodel revisionsOf: self asClassDefinition ]! !

!Class methodsFor: '*monticello' stamp: 'bf 7/4/2012 15:12'!
poolDictionaryNames
	^ self sharedPools collect: [:ea | self environment keyAtIdentityValue: ea ifAbsent: [ea name]]! !

Exception subclass: #MCNoChangesException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

!MCNoChangesException methodsFor: 'handling' stamp: 'jf 8/21/2003 19:49'!
defaultAction
	self inform: 'No changes'! !

!ChangeSorter methodsFor: '*Monticello-changeSet menu' stamp: 'eem 8/27/2015 16:59'!
deleteMonticelloChangeSets
	"Destroy all change sets from Monticello loads, provided the currently selected change set is not one of those, and the user really wants to do this."

	myChangeSet isForPackageLoad ifTrue: "forms current changes for current project"
		[^self inform: 'current change set is a Monticello load change set.\Please select some other change set before deleting Monticello load change sets.' withCRs].
	(self confirm: 'Are you sure you want to delete all change sets due to Monticello package loads?') ifFalse:
		[^self].

	(ChangesOrganizer allChangeSets select: [:cs| cs isForPackageLoad]) do:
		[:cs| ChangesOrganizer removeChangeSet: cs].

	self changed: #changeSetList.
	self showChangeSet: ChangeSet current! !

!ChangeSorter methodsFor: '*Monticello-changeSet menu' stamp: 'eem 8/27/2015 16:51'!
monticelloChangeSetMenu: aMenu
	<changeSetMenuShifted: false>
	<menuPriority: 600>
	"Sigh, when implementing menu pragmas this is not what I had in mind..."
	aMenu add: 'delete Monticello load change sets' action: #deleteMonticelloChangeSets.
	(aMenu submorphs
						detect: [:m| m isMenuItemMorph and: [m contents beginsWith: 'destroy change set']]
						ifNone: []) ifNotNil:
		[:destroyItem| | item |
		aMenu removeMorph: (item := aMenu submorphs last).
		aMenu addMorph: item after: destroyItem].
	^aMenu! !

!BrowserRequestor methodsFor: '*monticello-revisions' stamp: 'cmm 11/27/2016 16:52'!
browseMcClassRevisions
	"Open a browser on all versions of this class available in the MC repository for this package."
	self getClass
		ifNil: [ UIManager inform: 'No class selected' ]
		ifNotNilDo:
			[ : theClass | theClass theNonMetaClass mcModel
				ifNil: [ UIManager inform: 'Only Magma-backed HTTP repositories (or MCMagmaRepositorys) support browsing Monticello revisions.' ]
				ifNotNil:
					[ : mcModel | (MCOperationsList operations: (Cursor wait showWhile: [ theClass theNonMetaClass mcPatchOperations ])) browse ] ]! !

!BrowserRequestor methodsFor: '*monticello-revisions' stamp: 'cmm 11/27/2016 16:23'!
browseMcMethodRevisions
	"Open a browser on all versions of this method available in the MC repository for this package."
	self selectedMethodReference
		ifNil: [ UIManager inform: 'No method selected' ]
		ifNotNilDo:
			[ : methodReference | methodReference mcModel
				ifNil: [ UIManager inform: 'Only Magma-backed HTTP repositories (or MCMagmaRepositorys) support browsing Monticello revisions.' ]
				ifNotNil: [ (MCOperationsList operations: (Cursor wait showWhile: [ methodReference mcPatchOperations ])) browse ] ]! !

!BrowserRequestor methodsFor: '*monticello-revisions' stamp: 'eem 7/26/2014 07:00'!
selectedMethodReference
	^ MethodReference
		class: self getClass
		selector: self getBrowser selectedMessageName! !

!PseudoClass methodsFor: '*monticello' stamp: 'ul 3/13/2017 01:26'!
asClassDefinition
	^ MCClassDefinition
		name: self name
		superclassName: self superclass name
		category: self category 
		instVarNames: self instVarNames
		classVarNames: self classVarNames
		poolDictionaryNames: self poolDictionaryNames
		classInstVarNames: self classInstVarNames
		type: self typeOfClass
		comment: self organization classComment asString
		commentStamp: self organization commentStamp	! !

!PseudoClass methodsFor: '*monticello' stamp: 'jr 2/11/2017 17:24'!
asMCDefinitionBy: aMCReader
	^ aMCReader classDefinitionFrom: self! !

!PseudoClass methodsFor: '*monticello-override' stamp: 'nk 2/18/2004 18:30'!
isMeta
	^false! !

Notification subclass: #MCChangeSelectionRequest
	instanceVariableNames: 'patch label'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

!MCChangeSelectionRequest methodsFor: 'handling' stamp: 'abc 10/2/2013 15:43'!
defaultAction
	^ (MCChangeSelector forPatch: patch)
		 label: label ;
		 showModally! !

!MCChangeSelectionRequest methodsFor: 'accessing' stamp: 'avi 9/14/2004 15:02'!
label
	^ label! !

!MCChangeSelectionRequest methodsFor: 'accessing' stamp: 'avi 9/14/2004 15:01'!
label: aString
	label := aString! !

!MCChangeSelectionRequest methodsFor: 'accessing' stamp: 'avi 9/11/2004 15:12'!
patch
	^ patch! !

!MCChangeSelectionRequest methodsFor: 'accessing' stamp: 'avi 9/11/2004 15:12'!
patch: aPatch
	patch := aPatch! !

Notification subclass: #MCMergeResolutionRequest
	instanceVariableNames: 'merger'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

!MCMergeResolutionRequest methodsFor: 'handling' stamp: 'bf 4/26/2005 14:25'!
defaultAction
	^ (MCMergeBrowser new
		merger: merger;
		label: messageText) showModally! !

!MCMergeResolutionRequest methodsFor: 'accessing' stamp: 'ab 7/18/2003 18:19'!
merger
	^ merger! !

!MCMergeResolutionRequest methodsFor: 'accessing' stamp: 'ab 7/18/2003 18:18'!
merger: aMerger
	merger := aMerger! !

Notification subclass: #MCProxyMaterialization
	instanceVariableNames: 'proxy'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Repositories'!

!MCProxyMaterialization class methodsFor: 'as yet unclassified' stamp: 'cmm 8/13/2013 15:58'!
signalMaterializing: aMCInfoProxy
	^ self new
		setProxy: aMCInfoProxy;
		signal! !

!MCProxyMaterialization methodsFor: 'handling' stamp: 'cmm 8/13/2013 15:59'!
defaultAction
	self resume: proxy materializeInfo! !

!MCProxyMaterialization methodsFor: 'initialize-release' stamp: 'cmm 8/13/2013 15:57'!
setProxy: aMCInfoProxy
	proxy := aMCInfoProxy! !

Notification subclass: #MCRepositoryRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-UI'!

Notification subclass: #MCVersionNameAndMessageRequest
	instanceVariableNames: 'suggestion initialMessage patchBlock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

!MCVersionNameAndMessageRequest methodsFor: 'handling' stamp: 'cmm 3/27/2015 15:16'!
defaultAction
	^ MCSaveVersionDialog new
		versionName: suggestion;
		logMessage: initialMessage;
		patchBlock: patchBlock;
		selection: 1;
		showModally! !

!MCVersionNameAndMessageRequest methodsFor: 'accessing' stamp: 'bf 9/4/2010 15:28'!
initialMessage
	^ initialMessage! !

!MCVersionNameAndMessageRequest methodsFor: 'accessing' stamp: 'bf 9/4/2010 15:28'!
initialMessage: aString
	initialMessage := aString! !

!MCVersionNameAndMessageRequest methodsFor: 'accessing' stamp: 'topa 9/3/2014 13:54'!
patchBlock
	^ patchBlock! !

!MCVersionNameAndMessageRequest methodsFor: 'accessing' stamp: 'topa 9/3/2014 13:55'!
patchBlock: aBlock
	patchBlock := aBlock
! !

!MCVersionNameAndMessageRequest methodsFor: 'accessing' stamp: 'ab 7/10/2003 01:07'!
suggestedName
	^ suggestion! !

!MCVersionNameAndMessageRequest methodsFor: 'accessing' stamp: 'ab 7/10/2003 01:07'!
suggestedName: aString
	suggestion := aString! !

!MethodReference methodsFor: '*monticello' stamp: 'ab 8/22/2003 17:55'!
asMethodDefinition
	^ MCMethodDefinition forMethodReference: self! !

!MethodReference methodsFor: '*monticello' stamp: 'al 10/9/2005 20:05'!
isLocalSelector
	^self actualClass
		includesLocalSelector: self methodSymbol! !

!MethodReference methodsFor: '*monticello' stamp: 'cmm 3/8/2011 21:11'!
mcModel
	"If my package is in a MCMagmaRepository, answer its mcModel."
	^ self asMethodDefinition mcModel! !

!MethodReference methodsFor: '*monticello' stamp: 'cmm 11/27/2016 16:18'!
mcPatchOperations
	"Answer a collection of MCPatchOperations that can be displayed in a MCPatchBrowser which will reflect the history of this objects definition in relation to each other (not simply compared to the image version)."
	^(self mcRevisions reversed
			inject: OrderedCollection new
			into:
				[ : coll : each | coll
					ifEmpty:
						[ coll
							 add: (MCAddition of: each) ;
							 yourself ]
					ifNotEmpty:
						[ coll
							 add:
								(MCModification
									of:
										(coll last isAddition
											ifTrue: [ coll last definition ]
											ifFalse: [ coll last modification ])
									to: each) ;
							 yourself ] ]) reversed! !

!MethodReference methodsFor: '*monticello' stamp: 'cmm 11/27/2016 16:16'!
mcRevisions
	"Answer a collection of MCMethodDefinitions for this method, which are in the first Magma-backed repository in the list of repositories of my working-copy."
	^ self mcModel ifNotNil: [ : mcmodel | mcmodel revisionsOf: self asMethodDefinition ]! !

!MethodReference methodsFor: '*monticello' stamp: 'cmm 9/7/2013 11:07'!
packageInfo
	"Answer the PackageInfo containing this method."
	| methodCategory classCategory |
	methodCategory := self category.
	classCategory := methodCategory first = $* ifFalse: [ self actualClass theNonMetaClass category ].
	^ PackageInfo allPackages 
		detect: 
			[ : each | 
			"detect: [ : each | each methods includes: self ]" "<-- too slow"
			(each isYourClassExtension: methodCategory) or: 
				[ classCategory notNil and: [ each systemCategories includes: classCategory ] ] ]
		ifFound: [ : foundPackage | PackageInfo named: foundPackage packageName ]
		ifNone: [ nil ]! !

!MethodReference methodsFor: '*monticello' stamp: 'cmm 11/28/2016 22:42'!
workingCopy
	"Answer the MCWorkingCopy in which I am defined."
	^ self packageInfo ifNotNil: [ : pi | pi workingCopy ]! !

Warning subclass: #MCEmptyVersion
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Storing'!

MCEmptyVersion subclass: #MCEmptyDiffyVersion
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Storing'!

!MCEmptyDiffyVersion class methodsFor: 'exceptionInstantiator' stamp: 'mt 5/12/2016 11:39'!
signal

	self signal: 'About to serialize an empty diffy version.' ! !

!MCEmptyVersion class methodsFor: 'exceptionInstantiator' stamp: 'mt 5/12/2016 11:39'!
signal

	self signal: 'About to serialize an empty mcz version.'! !

!ClassDescription methodsFor: '*monticello' stamp: 'cmm 4/22/2019 13:20'!
packageInfo
	^ (PackageInfo allPackages select: [ : each | each includesClass: self ])
		ifEmpty: [ nil ]
		ifNotEmpty:
			[ : myPackages | "Select the most-qualified match."
			myPackages detectMax: [ : each | each packageName size ] ]! !

!ClassDescription methodsFor: '*monticello' stamp: 'cmm 12/7/2016 22:34'!
workingCopy
	"Answer the MCWorkingCopy in which I am defined."
	^ self packageInfo ifNotNil: [ : pi | pi workingCopy ]! !

!ByteSymbol methodsFor: '*monticello' stamp: 'cmm 3/7/2011 14:23'!
asMCVersionName
	^ self asString asMCVersionName! !

!TimeStamp class methodsFor: '*monticello-instance creation' stamp: 'nk 10/21/2003 23:07'!
fromMethodTimeStamp: aString
	| stream |
	stream := ReadStream on: aString.
	stream skipSeparators.
	stream skipTo: Character space.
	^self readFrom: stream.! !

!TimeStamp class methodsFor: '*monticello-instance creation' stamp: 'cbc 2/4/2004 21:30'!
fromString: aString
	"Answer a new instance for the value given by aString.

	 TimeStamp fromString: '1-10-2000 11:55:00 am'. 
	"

	^self readFrom: (ReadStream on: aString).! !

!TimeStamp class methodsFor: '*monticello-instance creation' stamp: 'nk 10/21/2003 23:04'!
readFrom: stream
	| date time |
	stream skipSeparators.
	date := Date readFrom: stream.
	stream skipSeparators.
	time := Time readFrom: stream.
	^self 
		date: date
		time: time! !

!PseudoMetaclass methodsFor: '*monticello-override' stamp: 'nk 2/18/2004 18:30'!
isMeta
	^true! !

Object subclass: #MCAncestry
	instanceVariableNames: 'ancestors stepChildren'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

!MCAncestry commentStamp: '<historical>' prior: 0!
Abstract superclass of records of ancestry.!

!MCAncestry class methodsFor: 'instance creation' stamp: 'avi 2/12/2004 21:02'!
new
	^ self basicNew initialize! !

!MCAncestry methodsFor: 'ancestry' stamp: 'ul 5/8/2011 19:45'!
allAncestors
	
	| allAncestors |
	allAncestors := Set new.
	self allAncestorsDo: [ :each | ] visitedAncestors: allAncestors.
	^allAncestors ! !

!MCAncestry methodsFor: 'ancestry' stamp: 'ul 5/8/2011 19:08'!
allAncestorsDo: aBlock
	
	self allAncestorsDo: aBlock visitedAncestors: Set new! !

!MCAncestry methodsFor: 'ancestry' stamp: 'eem 6/15/2017 11:45'!
allAncestorsDo: aBlock visitedAncestors: visitedAncestors
	self ancestors do: [ :each |
		(visitedAncestors ifAbsentAdd: each) ifTrue: [
			aBlock value: each.
			each allAncestorsDo: aBlock visitedAncestors: visitedAncestors ] ]! !

!MCAncestry methodsFor: 'ancestry' stamp: 'avi 2/12/2004 20:57'!
allAncestorsOnPathTo: aVersionInfo
	^ MCFilteredVersionSorter new
		target: aVersionInfo;
		addAllVersionInfos: self ancestors;
		sortedVersionInfos! !

!MCAncestry methodsFor: 'ancestry' stamp: 'avi 2/12/2004 20:57'!
ancestorString
	^ String streamContents:
		[:s | self ancestors do: [:ea | s nextPutAll: ea name] separatedBy: [s nextPutAll: ', ']]! !

!MCAncestry methodsFor: 'ancestry' stamp: 'bp 11/21/2010 18:33'!
ancestorStringWithout: packageName
	^ String streamContents:
		[:s | self ancestors do: [:ea | s nextPutAll: (ea nameWithout: packageName)] separatedBy: [s nextPutAll: ', ']]! !

!MCAncestry methodsFor: 'ancestry' stamp: 'cmm 3/10/2011 20:04'!
ancestors
	^ ancestors ifNil: [ Array empty ]! !

!MCAncestry methodsFor: 'ancestry' stamp: 'bf 12/22/2004 21:55'!
ancestorsDoWhileTrue: aBlock
	self ancestors do:
		[:ea |
		(aBlock value: ea) ifTrue: 
			[ea ancestorsDoWhileTrue: aBlock]]! !

!MCAncestry methodsFor: 'ancestry' stamp: 'avi 9/17/2005 16:03'!
breadthFirstAncestors
	^ Array streamContents: [:s | self breadthFirstAncestorsDo: [:ea | s nextPut: ea]]! !

!MCAncestry methodsFor: 'ancestry' stamp: 'stephaneducasse 2/4/2006 20:47'!
breadthFirstAncestorsDo: aBlock
	| seen todo next |
	seen := Set with: self.
	todo := OrderedCollection with: self.
	[todo isEmpty] whileFalse:
		[next := todo removeFirst.
		next ancestors do:
			[:ea |
			(seen includes: ea) ifFalse:
				[aBlock value: ea.
				seen add: ea.
				todo add: ea]]]! !

!MCAncestry methodsFor: 'copying' stamp: 'cmm 11/20/2016 15:27'!
clearAncestors
	ancestors := Array empty! !

!MCAncestry methodsFor: 'copying' stamp: 'cmm 11/20/2016 15:44'!
clearStepChildren
	stepChildren := Array empty! !

!MCAncestry methodsFor: 'ancestry' stamp: 'avi 2/12/2004 20:58'!
commonAncestorWith: aNode
	| commonAncestors |
	commonAncestors := self commonAncestorsWith: aNode.
	^ commonAncestors at: 1 ifAbsent: [nil]! !

!MCAncestry methodsFor: 'ancestry' stamp: 'avi 9/11/2004 14:18'!
commonAncestorsWith: aVersionInfo

	| sharedAncestors mergedOrder sorter |
	sorter := MCVersionSorter new
						addVersionInfo: self;
						addVersionInfo: aVersionInfo.
	mergedOrder := sorter sortedVersionInfos.
	sharedAncestors := (sorter allAncestorsOf: self) intersection: (sorter allAncestorsOf: aVersionInfo).
	^ mergedOrder select: [:ea | sharedAncestors includes: ea]! !

!MCAncestry methodsFor: 'ancestry' stamp: 'cmm 3/12/2011 17:11'!
hasAncestor: aVersionInfo
	"Answer whether the receiver has ancestor, aVersionInfo."
"Would it be more performant to use a Set for alreadySeen:?"
	^ self
		hasAncestor: aVersionInfo
		alreadySeen: OrderedCollection new! !

!MCAncestry methodsFor: 'ancestry' stamp: 'jrp 7/12/2004 08:16'!
hasAncestor: aVersionInfo alreadySeen: aList
	(aList includes: self) ifTrue: [^ false].
	aList add: self.

	^ self = aVersionInfo or: [self ancestors anySatisfy: [:ea | ea hasAncestor: aVersionInfo alreadySeen: aList]]
! !

!MCAncestry methodsFor: 'initialize-release' stamp: 'avi 9/11/2004 10:42'!
initialize
	ancestors := #().
	stepChildren := #()! !

!MCAncestry methodsFor: 'testing' stamp: 'cmm 8/16/2013 13:47'!
isMCInfoProxy
	^ false! !

!MCAncestry methodsFor: 'ancestry' stamp: 'avi 2/12/2004 20:58'!
isRelatedTo: aVersionInfo
	^ aVersionInfo timeStamp < self timeStamp
		ifTrue: [self hasAncestor: aVersionInfo]
		ifFalse: [aVersionInfo hasAncestor: self]! !

!MCAncestry methodsFor: 'progeny' stamp: 'nice 2/14/2019 13:31'!
lastMergeOf: aNode and: anotherNode
	"Find the last merge of two Nodes starting from myself.
	Answer nil if there is none.
	Note: there might have been several merges, in which case we take the common merge of merges.
	Example: if c is a merge of a and b, d is also a merge of a and b, and e is a merge of c and d, then asnwer e."

	| common |
	(self hasAncestor: aNode) ifFalse: [^nil].
	(self hasAncestor: anotherNode) ifFalse: [^nil].
	common := ancestors collect: [:e | e lastMergeOf: aNode and: anotherNode] as: Set.
	common remove: nil ifAbsent: [].
	common size = 1 ifTrue: [^common anyOne].
	^self
	! !

!MCAncestry methodsFor: 'ancestry' stamp: 'cmm 3/4/2011 14:41'!
names
	"The names of the ancestors."
	^ self ancestors collect: [ : each | each name asMCVersionName ]! !

!MCAncestry methodsFor: 'copying' stamp: 'cmm 11/20/2016 16:00'!
postCopyWithTrimmedAncestry
	ancestors ifNotNil:
		[ ancestors := ancestors collect:
			[ : each | each copy
				 clearAncestors ;
				 clearStepChildren ;
				 yourself ] ].
	stepChildren ifNotNil:
		[ stepChildren := stepChildren collect:
			[ : each | each copy
				 clearAncestors ;
				 clearStepChildren ;
				 yourself ] ]! !

!MCAncestry methodsFor: 'ancestry' stamp: 'avi 9/11/2004 15:08'!
stepChildren
	^ stepChildren ifNil: [#()]! !

!MCAncestry methodsFor: 'ancestry' stamp: 'avi 9/14/2004 15:21'!
stepChildrenString
	^ String streamContents:
		[:s | self stepChildren do: [:ea | s nextPutAll: ea name] separatedBy: [s nextPutAll: ', ']]! !

!MCAncestry methodsFor: 'initialize-release' stamp: 'cmm 8/16/2013 14:13'!
stubAncestryFor: aMCWorkingCopy using: aMCRepository 
	"childInfo was retrieved from aMCRepository.  Replace my ancestry with a Proxy that can retrieve the full tree from these two elements."
	ancestors := ancestors collect:
		[ : each | each isMCInfoProxy
			ifTrue: [ each ]
			ifFalse:
				[ MCInfoProxy
					info: each copyWithTrimmedAncestry
					workingCopy: aMCWorkingCopy
					repository: aMCRepository ] ].
	stepChildren := stepChildren collect:
		[ : each | each isMCInfoProxy
			ifTrue: [ each ]
			ifFalse:
				[ MCInfoProxy
					info: each copyWithTrimmedAncestry
					workingCopy: aMCWorkingCopy
					repository: aMCRepository ] ]! !

!MCAncestry methodsFor: 'ancestry' stamp: 'bf 12/8/2011 23:51'!
topologicalAncestors
	"Answer all ancestors in topological order (that is, children always come before all their ancestors). If multiple such orderings are possible, arbitrate by time stamp."	
	^ Array streamContents:
		[:s | | frontier next |
		frontier := MCFrontier frontierOn: self.
		next := self.
		[	frontier remove: next.
			s nextPut: next.
			next := frontier frontier detectMax: [:v | v timeStamp].
			next isNil] whileFalse]! !

!MCAncestry methodsFor: 'ancestry' stamp: 'ul 10/18/2018 01:49'!
withBreadthFirstAncestors
	^ { self }, self breadthFirstAncestors! !

MCAncestry subclass: #MCVersionInfo
	instanceVariableNames: 'id name message date time author'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

!MCVersionInfo commentStamp: '<historical>' prior: 0!
Adds to the record of ancestry, other identifying details.!

!MCVersionInfo class methodsFor: 'instance creation' stamp: 'avi 9/11/2004 10:44'!
name: vName id: id message: message date: date time: time author: author ancestors: ancestors
	^ self 
		name: vName
		id: id
		message: message
		date: date
		time: time
		author: author
		ancestors: ancestors
		stepChildren: #()! !

!MCVersionInfo class methodsFor: 'instance creation' stamp: 'avi 9/11/2004 10:43'!
name: vName id: id message: message date: date time: time author: author ancestors: ancestors stepChildren: stepChildren
	^ self new
		initializeWithName: vName
		id: id
		message: message
		date: date
		time: time
		author: author
		ancestors: ancestors
		stepChildren: stepChildren! !

!MCVersionInfo methodsFor: 'comparing' stamp: 'ab 7/5/2003 14:23'!
= other
	^ other species = self species
		and: [other hasID: id]! !

!MCVersionInfo methodsFor: 'converting' stamp: 'bf 4/18/2010 23:25'!
asDictionary
	^ Dictionary new
		at: #name put: name;
		at: #id put: id asString;
		at: #message put: message;
		at: #date put: date;
		at: #time put: time;
		at: #author put: author;
		at: #ancestors put: (self ancestors collect: [:a | a asDictionary]);
		yourself! !

!MCVersionInfo methodsFor: 'pillaging' stamp: 'cwp 8/1/2003 00:26'!
author
	^ author! !

!MCVersionInfo methodsFor: 'initialize-release' stamp: 'cmm 8/13/2013 15:37'!
copyWithTrimmedAncestry
	^ self copy postCopyWithTrimmedAncestry! !

!MCVersionInfo methodsFor: 'pillaging' stamp: 'cwp 8/1/2003 00:26'!
date
	^ date! !

!MCVersionInfo methodsFor: 'accessing' stamp: 'cmm 3/12/2011 17:10'!
dateAndTime
	^ DateAndTime
		date: date
		time: (time ifNil: [ Time midnight ])! !

!MCVersionInfo methodsFor: 'private' stamp: 'ab 7/5/2003 14:10'!
hasID: aUUID
	^ id = aUUID! !

!MCVersionInfo methodsFor: 'comparing' stamp: 'ab 7/5/2003 14:09'!
hash
	^ id hash! !

!MCVersionInfo methodsFor: 'pillaging' stamp: 'cwp 8/1/2003 00:24'!
id
	^ id ! !

!MCVersionInfo methodsFor: 'initialize-release' stamp: 'cmm 3/7/2011 14:22'!
initializeWithName: vName id: aUUID message: aString date: aDate time: aTime author: initials ancestors: aCollection stepChildren: stepCollection
	name := vName asString asMCVersionName.
	id := aUUID.
	message := aString.
	date := aDate.
	time := aTime.
	author := initials.
	ancestors :=  aCollection.
	stepChildren := stepCollection! !

!MCVersionInfo methodsFor: 'accessing' stamp: 'ab 7/12/2003 00:04'!
message
	^ message ifNil: ['']! !

!MCVersionInfo methodsFor: 'accessing' stamp: 'ab 7/11/2003 23:33'!
name
	^ name ifNil: ['<working copy>']! !

!MCVersionInfo methodsFor: 'accessing' stamp: 'bp 11/21/2010 19:27'!
nameWithout: packageName
	| result |
	result := self name.
	(result beginsWith: packageName , '-') ifTrue: [
		result := result copyFrom: packageName size + 2 to: result size].
	^result! !

!MCVersionInfo methodsFor: 'accessing' stamp: 'bf 2/24/2012 14:42'!
packageAndBranchName
	^ name
		ifNil: [ self name ]
		ifNotNil: [ name asMCVersionName packageAndBranchName ]! !

!MCVersionInfo methodsFor: 'printing' stamp: 'ab 7/5/2003 18:00'!
printOn: aStream
	super printOn: aStream.
	aStream nextPut: $(; nextPutAll: self name; nextPut: $)
	! !

!MCVersionInfo methodsFor: 'accessing' stamp: 'avi 1/20/2004 16:08'!
summary
	^ String streamContents:
		[:s |
		s
			nextPutAll: self summaryHeader; cr; cr;
			nextPutAll: self message.
		]! !

!MCVersionInfo methodsFor: 'accessing' stamp: 'avi 9/14/2004 15:22'!
summaryHeader
	^ String streamContents:
		[:s |
		s
			nextPutAll: 'Name: '; nextPutAll: self name; cr.
		date ifNotNil:
			[s
				nextPutAll: 'Author: '; nextPutAll: author; cr;
				nextPutAll: 'Time: '; nextPutAll:  date asString, ', ', time asString; cr].
		id ifNotNil:
			[s nextPutAll: 'UUID: '; nextPutAll: id asString; cr].
		s
			nextPutAll: 'Ancestors: '; nextPutAll: self ancestorString.
		self stepChildren isEmpty ifFalse:
			[s cr; nextPutAll: 'Backported From: '; nextPutAll: self stepChildrenString].
		]! !

!MCVersionInfo methodsFor: 'pillaging' stamp: 'cwp 8/1/2003 00:26'!
time
	^ time! !

!MCVersionInfo methodsFor: 'accessing' stamp: 'eem 11/6/2018 18:23'!
timeStamp
	^ TimeStamp date: date time: (time ifNil: [Time fromString: '00:00'])! !

!MCVersionInfo methodsFor: 'accessing' stamp: 'avi 9/17/2003 11:24'!
timeString
	^ date asString, ', ', time asString! !

!MCVersionInfo methodsFor: 'accessing' stamp: 'cmm 3/3/2011 14:34'!
versionName
	^ name
		ifNil: [ self name ]
		ifNotNil: [ name asMCVersionName ]! !

!MCVersionInfo methodsFor: 'accessing' stamp: 'cmm 3/5/2011 15:18'!
versionNumber
	^ self versionName versionNumber! !

MCAncestry subclass: #MCWorkingAncestry
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

!MCWorkingAncestry commentStamp: '<historical>' prior: 0!
The interim record of ancestry for a working copy, gets merged version added to the ancestry, and is used to create the VersionInfo when the working copy becomes a version. !

!MCWorkingAncestry methodsFor: 'accessing' stamp: 'stephaneducasse 2/4/2006 20:47'!
addAncestor: aNode
	ancestors := (self ancestors reject: [:each | aNode hasAncestor: each])
		copyWith: aNode! !

!MCWorkingAncestry methodsFor: 'accessing' stamp: 'avi 9/11/2004 10:43'!
addStepChild: aVersionInfo
	stepChildren := stepChildren copyWith: aVersionInfo! !

!MCWorkingAncestry methodsFor: 'versions' stamp: 'avi 9/11/2004 16:31'!
infoWithName: nameString message: messageString
	^ MCVersionInfo
		name: nameString
		id: UUID new
		message: messageString
		date: Date today
		time: Time now
		author: Utilities authorInitials
		ancestors: ancestors asArray
		stepChildren: self stepChildren asArray! !

!MCWorkingAncestry methodsFor: 'testing' stamp: 'avi 2/13/2004 01:14'!
name
	^ '<working copy>'! !

!MCWorkingAncestry methodsFor: 'operations' stamp: 'ul 11/14/2018 00:09'!
reparent: aNode
	"Let aNode be the sole parent of this version"
	ancestors := { aNode }
! !

!MCWorkingAncestry methodsFor: 'accessing' stamp: 'avi 2/13/2004 01:14'!
summary
	^ 'Ancestors: ', self ancestorString! !

Object subclass: #MCConflict
	instanceVariableNames: 'operation chooseRemote'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Merging'!

!MCConflict class methodsFor: 'instance creation' stamp: 'ab 6/1/2003 13:07'!
operation: anOperation
	^ self new operation: anOperation	! !

!MCConflict methodsFor: 'accessing' stamp: 'ul 12/12/2009 14:10'!
annotations
	^operation ifNotNil: [ :op | op annotations ]! !

!MCConflict methodsFor: 'applying' stamp: 'ab 6/1/2003 13:04'!
applyTo: anObject
	self isResolved ifFalse: [self error: 'Cannot continue until this conflict has been resolved'].
	self remoteChosen ifTrue: [operation applyTo: anObject].! !

!MCConflict methodsFor: 'resolving ' stamp: 'ab 6/1/2003 13:03'!
chooseLocal
	chooseRemote := false! !

!MCConflict methodsFor: 'resolving ' stamp: 'nk 10/21/2003 23:16'!
chooseNewer
	self isLocalNewer ifTrue: [ self chooseLocal ]
		ifFalse: [ self isRemoteNewer ifTrue: [ self chooseRemote ]]! !

!MCConflict methodsFor: 'resolving ' stamp: 'nk 10/21/2003 23:22'!
chooseOlder
	self isRemoteNewer ifTrue: [ self chooseLocal ]
		ifFalse: [ self isLocalNewer ifTrue: [ self chooseRemote ]]! !

!MCConflict methodsFor: 'resolving ' stamp: 'ab 6/1/2003 13:03'!
chooseRemote
	chooseRemote := true! !

!MCConflict methodsFor: 'resolving ' stamp: 'tfel 7/25/2016 15:20'!
chooseSameAST
	| fromSrc toSrc |
	(self definition isNil or: [self definition isMethodDefinition not])
		ifTrue: [^ self].
	fromSrc := (Parser new parse: operation fromSource class: nil class)
		generate decompile asString.
	toSrc := (Parser new parse: operation toSource class: nil class)
		generate decompile asString.
	fromSrc = toSrc ifTrue: [self chooseLocal].! !

!MCConflict methodsFor: 'resolving ' stamp: 'ab 7/18/2003 16:39'!
clearChoice
	chooseRemote := nil! !

!MCConflict methodsFor: 'accessing' stamp: 'ul 12/12/2009 14:10'!
definition
	^operation ifNotNil: [ :op | op definition ]! !

!MCConflict methodsFor: 'testing' stamp: 'dvf 8/10/2004 23:24'!
isConflict
	^true! !

!MCConflict methodsFor: 'testing' stamp: 'nk 10/21/2003 23:11'!
isLocalNewer
	^ self localDefinition fullTimeStamp > self remoteDefinition fullTimeStamp! !

!MCConflict methodsFor: 'testing' stamp: 'nk 10/21/2003 23:15'!
isRemoteNewer
	^ self localDefinition fullTimeStamp < self remoteDefinition fullTimeStamp! !

!MCConflict methodsFor: 'testing' stamp: 'ab 6/1/2003 13:02'!
isResolved
	^ chooseRemote notNil! !

!MCConflict methodsFor: 'testing' stamp: 'ab 6/2/2003 01:45'!
localChosen
	^ chooseRemote notNil and: [chooseRemote not]! !

!MCConflict methodsFor: 'accessing' stamp: 'ab 6/1/2003 13:10'!
localDefinition
	^ operation baseDefinition! !

!MCConflict methodsFor: 'accessing' stamp: 'avi 9/19/2005 02:19'!
operation
	^ operation! !

!MCConflict methodsFor: 'accessing' stamp: 'ab 6/1/2003 13:07'!
operation: anOperation
	operation := anOperation! !

!MCConflict methodsFor: 'testing' stamp: 'ab 6/2/2003 01:45'!
remoteChosen
	^ chooseRemote notNil and: [chooseRemote]! !

!MCConflict methodsFor: 'testing' stamp: 'ab 6/1/2003 13:10'!
remoteDefinition
	^ operation targetDefinition! !

!MCConflict methodsFor: 'accessing' stamp: 'ab 7/18/2003 16:47'!
source
	^ self localChosen
		ifTrue: [operation fromSource]
		ifFalse: [operation source]! !

!MCConflict methodsFor: 'accessing' stamp: 'ab 6/1/2003 13:48'!
status
	^ self isResolved
		ifFalse: ['']
		ifTrue: [self remoteChosen
					ifFalse: ['L']
					ifTrue: ['R']]! !

!MCConflict methodsFor: 'accessing' stamp: 'ab 7/18/2003 16:54'!
summary
	| attribute |
	attribute := 
		self isResolved
			ifTrue: [self remoteChosen ifTrue: [#underlined] ifFalse: [#struckOut]]
			ifFalse: [#bold].
	^ Text string: operation summary attribute: (TextEmphasis perform: attribute)! !

Object subclass: #MCDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Base'!

MCDefinition class
	instanceVariableNames: 'instances'!

!MCDefinition commentStamp: 'hjh 1/22/2013 22:25' prior: 0!
A MCDefinition is a model for Smalltalk code. A package snapshot (MCSnapshot) is a collection of these definitions.

!

MCDefinition class
	instanceVariableNames: 'instances'!

MCDefinition subclass: #MCClassDefinition
	instanceVariableNames: 'name superclassName variables category type comment commentStamp traitComposition classTraitComposition'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Modeling'!

!MCClassDefinition class methodsFor: 'obsolete' stamp: 'ab 4/1/2003 01:22'!
name: nameString
superclassName: superclassString
category: categoryString 
instVarNames: ivarArray
classVarNames: cvarArray
poolDictionaryNames: poolArray
classInstVarNames: civarArray
comment: commentString
	^ self	name: nameString
			superclassName: superclassString
			category: categoryString 
			instVarNames: ivarArray
			classVarNames: cvarArray
			poolDictionaryNames: poolArray
			classInstVarNames: civarArray
			type: #normal
			comment: commentString
! !

!MCClassDefinition class methodsFor: 'obsolete' stamp: 'cwp 8/10/2003 16:33'!
name: nameString
superclassName: superclassString
category: categoryString 
instVarNames: ivarArray
classVarNames: cvarArray
poolDictionaryNames: poolArray
classInstVarNames: civarArray
type: typeSymbol
comment: commentString
	^ self 
		name: nameString
		superclassName: superclassString
		category: categoryString 
		instVarNames: ivarArray
		classVarNames: cvarArray
		poolDictionaryNames: poolArray
		classInstVarNames: civarArray
		type: typeSymbol
		comment: commentString
		commentStamp: nil! !

!MCClassDefinition class methodsFor: 'instance creation' stamp: 'al 10/9/2005 19:16'!
name: nameString
superclassName: superclassString
category: categoryString 
instVarNames: ivarArray
classVarNames: cvarArray
poolDictionaryNames: poolArray
classInstVarNames: civarArray
type: typeSymbol
comment: commentString
commentStamp: stampString
	^ self instanceLike:
		(self new initializeWithName: nameString
					superclassName: superclassString
					traitComposition: '{}'
					classTraitComposition: '{}'
					category: categoryString 
					instVarNames: ivarArray
					classVarNames: cvarArray
					poolDictionaryNames: poolArray
					classInstVarNames: civarArray
					type: typeSymbol
					comment: commentString
					commentStamp: stampString)! !

!MCClassDefinition class methodsFor: 'obsolete' stamp: 'ab 4/1/2003 01:22'!
name: nameString
superclassName: superclassString
category: categoryString 
instVarNames: ivarArray
comment: commentString
	^ self	name: nameString
			superclassName: superclassString
			category: categoryString 
			instVarNames: ivarArray
			classVarNames: #()
			poolDictionaryNames: #()
			classInstVarNames: #()
			comment: commentString
! !

!MCClassDefinition class methodsFor: 'instance creation' stamp: 'al 10/10/2005 13:58'!
name: nameString
superclassName: superclassString
traitComposition: traitCompositionString
classTraitComposition: classTraitCompositionString
category: categoryString 
instVarNames: ivarArray
classVarNames: cvarArray
poolDictionaryNames: poolArray
classInstVarNames: civarArray
type: typeSymbol
comment: commentString
commentStamp: stampString
	
	^ self instanceLike:
		(self new initializeWithName: nameString
					superclassName: superclassString
					traitComposition: traitCompositionString
					classTraitComposition: classTraitCompositionString
					category: categoryString 
					instVarNames: ivarArray
					classVarNames: cvarArray
					poolDictionaryNames: poolArray
					classInstVarNames: civarArray
					type: typeSymbol
					comment: commentString
					commentStamp: stampString)! !

!MCClassDefinition methodsFor: 'visiting' stamp: 'nice 10/31/2009 13:11'!
= aDefinition
	^(super = aDefinition)
		and: [superclassName = aDefinition superclassName
		and: [self traitCompositionString = aDefinition traitCompositionString
		and: [self classTraitCompositionString = aDefinition classTraitCompositionString
		and: [category = aDefinition category
		and: [type = aDefinition type
		and: [self sortedVariables = aDefinition sortedVariables
		and: [comment = aDefinition comment]]]]]]]! !

!MCClassDefinition methodsFor: 'visiting' stamp: 'al 10/9/2005 19:33'!
accept: aVisitor
	aVisitor visitClassDefinition: self.
	(self hasClassInstanceVariables or: [self hasClassTraitComposition])
		ifTrue: [aVisitor visitMetaclassDefinition: self].
! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'jr 2/28/2017 02:44'!
actualClass
	^ self actualClassIn: Environment current! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'jr 2/28/2017 02:44'!
actualClassIn: anEnvironment
	^anEnvironment classNamed: self className! !

!MCClassDefinition methodsFor: 'initializing' stamp: 'ul 3/13/2017 00:26'!
addVariables: aCollection ofType: aClass

	aCollection do: [ :variable |
		variables add: (aClass name: variable asString ) ]! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'cwp 11/24/2002 06:23'!
category
	^ category! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'cwp 7/7/2003 23:32'!
classInstVarNames
	^ self selectVariables: #isClassInstanceVariable! !

!MCClassDefinition methodsFor: 'printing' stamp: 'cwp 7/7/2003 23:35'!
classInstanceVariablesString
	^ self stringForVariablesOfType: #isClassInstanceVariable! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'ab 11/13/2002 01:52'!
className
	^ name! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'al 10/9/2005 20:07'!
classTraitComposition
	^classTraitComposition! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'al 10/9/2005 21:55'!
classTraitCompositionString
	^self classTraitComposition ifNil: ['{}'].! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'nice 10/20/2009 22:03'!
classVarNames
	^(self selectVariables: #isClassVariable) asArray sort! !

!MCClassDefinition methodsFor: 'printing' stamp: 'cwp 7/7/2003 23:36'!
classVariablesString
	^ self stringForVariablesOfType: #isClassVariable! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'ab 1/15/2003 13:42'!
comment
	^ comment! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'cwp 8/10/2003 16:40'!
commentStamp
	^ commentStamp! !

!MCClassDefinition methodsFor: 'installing' stamp: 'ul 11/14/2018 00:07'!
createClass
	| environment superClass class composition |
	environment := Environment current.
	superClass := superclassName == #nil ifFalse:
					[environment valueOf: superclassName
						ifAbsent: [(KeyNotFound key: superclassName) signal]].
	[class := (ClassBuilder new)
			name: name 
			inEnvironment: environment
			subclassOf: superClass
			type: type 
			instanceVariableNames: self instanceVariablesString 
			classVariableNames: self classVariablesString 
			poolDictionaries: self sharedPoolsString
			category: category.
	] on: Warning, DuplicateVariableError do:[:ex| ex resume].

	"The following is written to support traits unloading"
	composition := self traitComposition ifNil: [Array empty] ifNotNil: [:traitComposition | Compiler evaluate: traitComposition].
	(composition isCollection and:[composition isEmpty and:[class traitComposition isEmpty]]) ifFalse:[
		class setTraitComposition: composition asTraitComposition.
	].

	composition := self classTraitComposition ifNil: [Array new] ifNotNil: [:traitComposition | Compiler evaluate: traitComposition].
	(composition isCollection and:[composition isEmpty and:[class class traitComposition isEmpty]]) ifFalse:[
		class class setTraitComposition: composition asTraitComposition.
	].

	^class! !

!MCClassDefinition methodsFor: 'initializing' stamp: 'cwp 8/10/2003 17:39'!
defaultCommentStamp
	^ String new

	"The version below avoids stomping on stamps already in the image

	^ (Smalltalk at: name ifPresent: [:c | c organization commentStamp])
		ifNil: ['']
	"
! !

!MCClassDefinition methodsFor: 'printing' stamp: 'ul 11/6/2010 05:50'!
definitionAndCommentString
	^ String streamContents: [:stream |
		self printDefinitionOn: stream.
		stream cr; cr.
		self printClassSideDefinitionOn: stream.    
		stream cr; cr.
		self printCommentOn: stream]! !

!MCClassDefinition methodsFor: 'printing' stamp: 'cwp 8/2/2003 02:03'!
definitionString
	^ String streamContents: [:stream | self printDefinitionOn: stream]! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'ul 10/18/2018 01:48'!
description
	^{ name }
! !

!MCClassDefinition methodsFor: 'renaming' stamp: 'cmm 6/8/2011 13:36'!
handlePackageRename: oldPackageName to: newPackageName 
	(self actualClass category beginsWith: newPackageName) ifFalse:
		[ self actualClass category:
			(self
				newCategoryNameFor: self actualClass category
				givenRenameFrom: oldPackageName
				to: newPackageName) ]! !

!MCClassDefinition methodsFor: 'testing' stamp: 'cwp 8/2/2003 02:54'!
hasClassInstanceVariables
	^ (self selectVariables: #isClassInstanceVariable) isEmpty not! !

!MCClassDefinition methodsFor: 'testing' stamp: 'al 10/9/2005 21:59'!
hasClassTraitComposition
	^self classTraitCompositionString ~= '{}'! !

!MCClassDefinition methodsFor: 'testing' stamp: 'al 10/9/2005 20:13'!
hasComment
	^ comment isEmptyOrNil not! !

!MCClassDefinition methodsFor: 'testing' stamp: 'al 3/29/2006 00:27'!
hasTraitComposition
	^self traitCompositionString ~= '{}'! !

!MCClassDefinition methodsFor: 'comparing' stamp: 'ul 11/2/2009 03:32'!
hash

	| hash |
	hash := name hashWithInitialHash: 0.
	hash := superclassName hashWithInitialHash: hash.
	hash := self traitCompositionString hashWithInitialHash: hash.
	hash := self classTraitComposition asString hashWithInitialHash: hash.
	hash := (category ifNil: ['']) hashWithInitialHash: hash.
	hash := type hashWithInitialHash: hash.
	variables do: [ :v |
		hash := v name hashWithInitialHash: hash ].
	^hash! !

!MCClassDefinition methodsFor: 'initializing' stamp: 'eem 8/9/2017 09:57'!
initializeWithName: nameString
superclassName: superclassString
category: categoryString 
instVarNames: ivarArray
classVarNames: cvarArray
poolDictionaryNames: poolArray
classInstVarNames: civarArray
type: typeSymbol
comment: commentString
commentStamp: stampStringOrNil
	name := nameString asSymbol.
	superclassName := superclassString ifNil: ['nil'] ifNotNil: [superclassString asSymbol].
	category := categoryString.
	type := typeSymbol.
	comment := commentString withSqueakLineEndings.
	commentStamp := stampStringOrNil ifNil: [self defaultCommentStamp].
	variables := OrderedCollection  new.
	self addVariables: ivarArray ofType: MCInstanceVariableDefinition.
	self addVariables: cvarArray sorted ofType: MCClassVariableDefinition.
	self addVariables: poolArray sorted ofType: MCPoolImportDefinition.
	self addVariables: civarArray ofType: MCClassInstanceVariableDefinition! !

!MCClassDefinition methodsFor: 'initializing' stamp: 'eem 8/9/2017 09:58'!
initializeWithName: nameString
superclassName: superclassString
traitComposition: traitCompositionString
classTraitComposition: classTraitCompositionString
category: categoryString 
instVarNames: ivarArray
classVarNames: cvarArray
poolDictionaryNames: poolArray
classInstVarNames: civarArray
type: typeSymbol
comment: commentString
commentStamp: stampStringOrNil
	name := nameString asSymbol.
	superclassName := superclassString ifNil: ['nil'] ifNotNil: [superclassString asSymbol].
	traitComposition := traitCompositionString.
	classTraitComposition := classTraitCompositionString.
	category := categoryString.
	type := typeSymbol.
	comment := commentString withSqueakLineEndings.
	commentStamp := stampStringOrNil ifNil: [self defaultCommentStamp].
	variables := OrderedCollection  new.
	self addVariables: ivarArray ofType: MCInstanceVariableDefinition.
	self addVariables: cvarArray sorted ofType: MCClassVariableDefinition.
	self addVariables: poolArray sorted ofType: MCPoolImportDefinition.
	self addVariables: civarArray ofType: MCClassInstanceVariableDefinition! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'cwp 7/7/2003 23:33'!
instVarNames
	^ self selectVariables: #isInstanceVariable! !

!MCClassDefinition methodsFor: 'printing' stamp: 'cwp 7/7/2003 23:35'!
instanceVariablesString
	^ self stringForVariablesOfType: #isInstanceVariable! !

!MCClassDefinition methodsFor: 'testing' stamp: 'ab 12/4/2002 21:52'!
isClassDefinition
	^ true! !

!MCClassDefinition methodsFor: 'testing' stamp: 'ab 5/24/2003 13:49'!
isCodeDefinition
	^ true! !

!MCClassDefinition methodsFor: 'printing' stamp: 'eem 10/17/2016 12:16'!
kindOfSubclass
	type = #normal ifTrue: [^' subclass: '].
	type = #variable ifTrue: [^' variableSubclass: '].
	type = #bytes ifTrue: [^' variableByteSubclass: '].
	type = #compiledMethod ifTrue: [^' variableByteSubclass: ' ].
	type = #shorts ifTrue: [^' variableDoubleByteSubclass: '].
	type = #words ifTrue: [^' variableWordSubclass: '].
	type = #longs ifTrue: [^' variableDoubleWordSubclass: '].
	type = #weak ifTrue: [^' weakSubclass: ' ].
	type = #ephemeron ifTrue: [^' ephemeronSubclass: ' ].
	type = #immediate ifTrue: [^' immediateSubclass: ' ].
	self error: 'Unrecognized class type'
! !

!MCClassDefinition methodsFor: 'installing' stamp: 'ul 12/12/2009 14:10'!
load
	 self createClass ifNotNil:
		[:class |
		class class instanceVariableNames: self classInstanceVariablesString.
		self hasComment ifTrue: [class classComment: comment stamp: commentStamp]]! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'cwp 7/7/2003 23:53'!
poolDictionaries
	^ self selectVariables: #isPoolImport! !

!MCClassDefinition methodsFor: 'annotations' stamp: 'nk 7/24/2003 16:05'!
printAnnotations: requests on: aStream
	"Add a string for an annotation pane, trying to fulfill the annotation requests.
	These might include anything that
		Preferences defaultAnnotationRequests 
	might return. Which includes anything in
		Preferences annotationInfo
	To edit these, use:"
	"Preferences editAnnotations"

	requests do: [ :aRequest |
		aRequest == #requirements ifTrue: [
			self requirements do: [ :req | aStream nextPutAll: req ] separatedBy: [ aStream space ]]
	] separatedBy: [ aStream space ].! !

!MCClassDefinition methodsFor: 'printing' stamp: 'mtf 10/26/2014 06:27'!
printClassSideDefinitionOn: stream
	
	stream
		nextPutAll: self className;
		nextPutAll: ' class ';
		cr; tab.
	self hasClassTraitComposition ifTrue: [
		stream 
			nextPutAll: 'uses: ';
	 		nextPutAll: self classTraitCompositionString;
			cr; tab ].
	stream
		nextPutAll: 'instanceVariableNames: ';
		store: self classInstanceVariablesString! !

!MCClassDefinition methodsFor: 'printing' stamp: 'dtl 5/5/2010 21:36'!
printCommentOn: stream
	stream
		nextPut: $";
		nextPutAll: self comment asString;
		nextPut: $"
! !

!MCClassDefinition methodsFor: 'printing' stamp: 'al 3/28/2006 23:42'!
printDefinitionOn: stream
		stream 
			nextPutAll: self superclassName;
			nextPutAll: self kindOfSubclass;
			nextPut: $# ;
			nextPutAll: self className;
			cr; tab.
		self hasTraitComposition ifTrue: [
			stream 
				nextPutAll: 'uses: ';
		 		nextPutAll: self traitCompositionString;
				cr; tab ].
		stream
			nextPutAll: 'instanceVariableNames: ';
			store: self instanceVariablesString;
			cr; tab;
			nextPutAll: 'classVariableNames: ';
			store: self classVariablesString;
			cr; tab;
			nextPutAll: 'poolDictionaries: ';
			store: self sharedPoolsString;
			cr; tab;
			nextPutAll: 'category: ';
			store: self category asString! !

!MCClassDefinition methodsFor: 'comparing' stamp: 'ul 10/18/2018 01:48'!
provisions
	^{ name }! !

!MCClassDefinition methodsFor: 'comparing' stamp: 'jr 5/30/2020 02:20'!
requirements
	^superclassName == #nil
		ifTrue: [self poolDictionaries]
		ifFalse: [{ superclassName }, self poolDictionaries,
			(self hasTraitComposition
				ifTrue: [(Scanner new scanTokens: self traitComposition) flattened select: [:each | each first isUppercase]]
				ifFalse: [#()])].! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'cwp 7/7/2003 23:28'!
selectVariables: aSelector
	^ variables select: [:v | v perform: aSelector] thenCollect: [:v | v name]! !

!MCClassDefinition methodsFor: 'printing' stamp: 'cwp 7/7/2003 23:54'!
sharedPoolsString
	^ self stringForVariablesOfType: #isPoolImport! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'ab 7/19/2003 18:00'!
sortKey
	^ self className! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'ul 3/13/2017 00:31'!
sortedVariables
	"sort variables for comparison purposes"

	| orderDependents toSort |
	orderDependents := OrderedCollection new: variables size.
	toSort := OrderedCollection new.
	variables do: [ :variable |
		variable isOrderDependend
			ifTrue: [ orderDependents addLast: variable ]
			ifFalse: [ toSort addLast: variable ] ].
	toSort sort: [ :a :b | a name <= b name ].
	^orderDependents
		addAllLast: toSort;
		yourself! !

!MCClassDefinition methodsFor: 'printing' stamp: 'dtl 5/5/2010 20:33'!
source
	^ self definitionAndCommentString! !

!MCClassDefinition methodsFor: 'serializing' stamp: 'al 7/4/2006 10:14'!
storeDataOn: aDataStream
	| instVarSize |
	instVarSize := (self hasTraitComposition or: [ self hasClassTraitComposition ])
		ifTrue: [ self class instSize ]
		ifFalse: [ self class instSize - 2 ].
	aDataStream
		beginInstance: self class
		size: instVarSize.
	1 to: instVarSize do: [ :index |
		aDataStream nextPut: (self instVarAt: index) ].! !

!MCClassDefinition methodsFor: 'installing' stamp: 'cwp 2/3/2004 21:35'!
stringForVariablesOfType: aSymbol
	^ String streamContents:
		[:stream |
		(self selectVariables: aSymbol) 
			do: [:ea | stream nextPutAll: ea]
			separatedBy: [stream space]]! !

!MCClassDefinition methodsFor: 'printing' stamp: 'ab 11/16/2002 17:33'!
summary
	^ name! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'ab 11/13/2002 17:41'!
superclassName
	^ superclassName! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'al 10/9/2005 20:16'!
traitComposition
	^traitComposition! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'al 10/9/2005 21:55'!
traitCompositionString
	^self traitComposition ifNil: ['{}'].! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'cwp 11/24/2002 22:35'!
type
	^ type! !

!MCClassDefinition methodsFor: 'installing' stamp: 'jr 10/19/2016 15:20'!
unload
	Environment current removeClassNamed: name! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'cwp 11/25/2002 06:51'!
variables
	^ variables! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'cmm 3/8/2011 21:00'!
workingCopy
	^ self actualClass workingCopy! !

MCClassDefinition subclass: #MCTraitDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Modeling'!

!MCTraitDefinition class methodsFor: 'instance creation' stamp: 'al 10/9/2005 20:28'!
name: classNameString traitComposition:  traitCompositionString category:  categoryString comment:  commentString commentStamp:   commentStamp
	^ self instanceLike:
		(self new initializeWithName: classNameString 
			traitComposition:  traitCompositionString
			category:  categoryString
			comment:  commentString  
			commentStamp:   commentStamp)
! !

!MCTraitDefinition methodsFor: 'visiting' stamp: 'nice 10/31/2009 13:10'!
= aDefinition
	self flag: #traits. "Ugly we harcoded the super superclass method.  We will have to refactor the definition hierarchy"
	
	^ (aDefinition isKindOf: MCDefinition)
		and: [(self isRevisionOf: aDefinition)
		and: [self traitCompositionString = aDefinition traitCompositionString
		and: [category = aDefinition category
		and: [comment = aDefinition comment]]]]! !

!MCTraitDefinition methodsFor: 'visiting' stamp: 'al 10/9/2005 20:28'!
accept: aVisitor
	^ aVisitor visitTraitDefinition: self
! !

!MCTraitDefinition methodsFor: 'accessing' stamp: 'al 6/5/2006 14:05'!
classTraitCompositionString
	^self traitComposition ifNil: ['{}'].! !

!MCTraitDefinition methodsFor: 'visiting' stamp: 'jr 12/6/2016 17:20'!
createClass
	^ClassDescription
		newTraitNamed: name
		uses: (Compiler evaluate: self traitCompositionString)
		category: category
		in: Environment current
		
! !

!MCTraitDefinition methodsFor: 'testing' stamp: 'al 10/9/2005 20:28'!
hasClassInstanceVariables
	^ false

! !

!MCTraitDefinition methodsFor: 'testing' stamp: 'al 6/5/2006 14:13'!
hasTraitComposition
	^self traitCompositionString ~= '{}'! !

!MCTraitDefinition methodsFor: 'comparing' stamp: 'ul 11/2/2009 04:55'!
hash

	| hash |
	hash := name hashWithInitialHash: 0.
	hash := self traitCompositionString hashWithInitialHash: hash.
	hash := (category ifNil: ['']) hashWithInitialHash: hash.
	^hash
! !

!MCTraitDefinition methodsFor: 'initializing' stamp: 'al 6/5/2006 14:14'!
initializeWithName: classNameString 
	traitComposition:  traitCompositionString
	category:  categoryString
	comment:  commentString  
	commentStamp:   commentStampString
					
		name := classNameString asSymbol.
		traitComposition := traitCompositionString.
	     category := categoryString.
		comment := commentString withSqueakLineEndings.
		commentStamp :=  commentStampString ifNil: [self defaultCommentStamp]
! !

!MCTraitDefinition methodsFor: 'installing' stamp: 'ul 12/12/2009 14:09'!
load
	 self createClass ifNotNil: [:trait |
		self hasComment ifTrue: [trait classComment: comment stamp: commentStamp]]! !

!MCTraitDefinition methodsFor: 'printing' stamp: 'mtf 10/26/2014 06:29'!
printClassSideDefinitionOn: stream
	stream
		nextPutAll: self className;
		nextPutAll: ' classTrait ';
		cr; tab;
		nextPutAll: 'uses: ';
 		nextPutAll: self classTraitCompositionString! !

!MCTraitDefinition methodsFor: 'printing' stamp: 'al 6/5/2006 14:15'!
printDefinitionOn: stream
	stream nextPutAll: 'Trait named: #', self className;
		 cr;
		 tab;
		 nextPutAll: 'uses: ';
		 nextPutAll: self traitCompositionString;
		 cr;
		 tab;
		 nextPutAll: 'category: ';
		 store: self category asString
! !

!MCTraitDefinition methodsFor: 'comparing' stamp: 'ul 11/14/2018 00:09'!
requirements
	"Assuming that traits in a composition can be identified by
	testing for the first character beeing an uppercase character
	(and thus not a special character such as {, # etc.)"

	| tokens traitNames |
	self hasTraitComposition ifFalse: [ ^Array empty ].
	tokens := (Scanner new scanTokens: self traitComposition) flattened.
	traitNames := tokens select: [:each | each first isUppercase].
	^traitNames asArray! !

!MCTraitDefinition methodsFor: 'accessing' stamp: 'al 10/9/2005 20:29'!
traitComposition
	^traitComposition! !

MCDefinition subclass: #MCClassTraitDefinition
	instanceVariableNames: 'baseTrait classTraitComposition'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Modeling'!

!MCClassTraitDefinition class methodsFor: 'instance creation' stamp: 'al 10/9/2005 20:22'!
baseTraitName: aString classTraitComposition: classTraitCompositionString
	^self instanceLike: (
		self new
			initializeWithBaseTraitName: aString
			classTraitComposition: classTraitCompositionString).! !

!MCClassTraitDefinition methodsFor: 'accessing' stamp: 'nice 10/31/2009 13:11'!
= aDefinition
	^ (super = aDefinition)
		and: [baseTrait = aDefinition baseTrait
		and: [self classTraitCompositionString = aDefinition classTraitCompositionString]]

! !

!MCClassTraitDefinition methodsFor: 'visiting' stamp: 'al 10/9/2005 20:23'!
accept: aVisitor
	^ aVisitor visitClassTraitDefinition: self.! !

!MCClassTraitDefinition methodsFor: 'accessing' stamp: 'al 10/9/2005 20:23'!
baseTrait
	^baseTrait
! !

!MCClassTraitDefinition methodsFor: 'accessing' stamp: 'al 12/15/2005 11:31'!
className
	^self baseTrait! !

!MCClassTraitDefinition methodsFor: 'accessing' stamp: 'al 10/9/2005 20:23'!
classTraitComposition
	^classTraitComposition

! !

!MCClassTraitDefinition methodsFor: 'accessing' stamp: 'al 10/10/2005 10:12'!
classTraitCompositionString
	^self classTraitComposition ifNil: ['{}'].

! !

!MCClassTraitDefinition methodsFor: 'accessing' stamp: 'al 10/9/2005 21:59'!
definitionString
	^self baseTrait , ' classTrait
	uses: ' , self classTraitCompositionString.
! !

!MCClassTraitDefinition methodsFor: 'accessing' stamp: 'pre 11/14/2018 15:34'!
description
	^{
		baseTrait .
		classTraitComposition }! !

!MCClassTraitDefinition methodsFor: 'accessing' stamp: 'ul 11/2/2009 03:32'!
hash

	| hash |
	hash := baseTrait hashWithInitialHash: 0.
	hash := self classTraitCompositionString hashWithInitialHash: hash.
	^hash
! !

!MCClassTraitDefinition methodsFor: 'initialization' stamp: 'stephaneducasse 2/4/2006 20:47'!
initializeWithBaseTraitName: aTraitName classTraitComposition: aString
	baseTrait := aTraitName.
	classTraitComposition := aString.! !

!MCClassTraitDefinition methodsFor: 'installing' stamp: 'jr 5/30/2020 23:52'!
load	
	Compiler evaluate: self definitionString environment: Environment current! !

!MCClassTraitDefinition methodsFor: 'accessing' stamp: 'ul 10/18/2018 01:48'!
requirements
	^{ baseTrait }! !

!MCClassTraitDefinition methodsFor: 'accessing' stamp: 'al 10/9/2005 20:25'!
sortKey
	^ self baseTrait name , '.classTrait'! !

!MCClassTraitDefinition methodsFor: 'accessing' stamp: 'al 10/9/2005 20:25'!
source
	^self definitionString! !

!MCClassTraitDefinition methodsFor: 'accessing' stamp: 'al 10/9/2005 20:25'!
summary
	^self baseTrait , ' classTrait'
! !

!MCDefinition class methodsFor: 'class initialization' stamp: 'ar 2/26/2010 23:03'!
cleanUp
	"Flush caches"

	self clearInstances.! !

!MCDefinition class methodsFor: 'as yet unclassified' stamp: 'ul 2/26/2010 22:10'!
clearInstances

	instances := nil.
	self subclassesDo: #clearInstances! !

!MCDefinition class methodsFor: 'as yet unclassified' stamp: 'ul 2/26/2010 22:10'!
instanceLike: aDefinition

	^(instances ifNil: [ instances := WeakSet new ])
		like: aDefinition
		ifAbsent: [ instances add: aDefinition ]! !

!MCDefinition methodsFor: 'comparing' stamp: 'ab 7/19/2003 17:59'!
<= other
	^ self sortKey <= other sortKey! !

!MCDefinition methodsFor: 'comparing' stamp: 'nice 10/31/2009 13:08'!
= aDefinition
	^(aDefinition isKindOf: MCDefinition) and: [self isRevisionOf: aDefinition]! !

!MCDefinition methodsFor: 'accessing' stamp: 'ul 11/7/2010 22:45'!
actualClass

	^nil! !

!MCDefinition methodsFor: 'accessing' stamp: 'jr 2/28/2017 02:43'!
actualClassIn: anEnvironment

	^nil! !

!MCDefinition methodsFor: 'installing' stamp: 'rej 2/26/2007 18:45'!
addMethodAdditionTo: aCollection
  Transcript show: self printString.
  self load! !

!MCDefinition methodsFor: 'annotations' stamp: 'nk 7/24/2003 12:27'!
annotations
	^self annotations: Preferences defaultAnnotationRequests! !

!MCDefinition methodsFor: 'annotations' stamp: 'nk 7/24/2003 12:26'!
annotations: requests
	"Answer a string for an annotation pane, trying to fulfill the annotation requests.
	These might include anything that
		Preferences defaultAnnotationRequests 
	might return. Which includes anything in
		Preferences annotationInfo
	To edit these, use:"
	"Preferences editAnnotations"

	^String streamContents: [ :s | self printAnnotations: requests on: s ].! !

!MCDefinition methodsFor: 'comparing' stamp: 'ab 7/19/2003 18:14'!
description
	self subclassResponsibility! !

!MCDefinition methodsFor: 'comparing' stamp: 'nk 10/21/2003 23:18'!
fullTimeStamp
	^TimeStamp current! !

!MCDefinition methodsFor: 'renaming' stamp: 'cmm 6/8/2011 13:22'!
handlePackageRename: oldPackageName to: newPackageName
	"Overridden as necessary in subclasses."! !

!MCDefinition methodsFor: 'comparing' stamp: 'ab 7/19/2003 18:14'!
hash
	^ self description hash! !

!MCDefinition methodsFor: 'testing' stamp: 'ab 12/4/2002 21:51'!
isClassDefinition
	^false! !

!MCDefinition methodsFor: 'testing' stamp: 'bf 11/12/2004 14:46'!
isClassDefinitionExtension
	"Answer true if this definition extends the regular class definition"
	^false! !

!MCDefinition methodsFor: 'testing' stamp: 'ab 12/4/2002 21:51'!
isMethodDefinition
	^false! !

!MCDefinition methodsFor: 'testing' stamp: 'cwp 7/11/2003 01:32'!
isOrganizationDefinition
	^false! !

!MCDefinition methodsFor: 'comparing' stamp: 'ab 12/5/2002 21:24'!
isRevisionOf: aDefinition
	^ aDefinition description = self description! !

!MCDefinition methodsFor: 'comparing' stamp: 'ab 7/19/2003 18:25'!
isSameRevisionAs: aDefinition
	^ self = aDefinition! !

!MCDefinition methodsFor: 'testing' stamp: 'bf 8/12/2009 22:55'!
isScriptDefinition
	^false! !

!MCDefinition methodsFor: 'installing' stamp: 'ab 7/18/2003 21:31'!
load
	! !

!MCDefinition methodsFor: 'installing' stamp: 'avi 2/17/2004 13:19'!
loadOver: aDefinition
	self load
	! !

!MCDefinition methodsFor: 'private' stamp: 'nice 6/10/2017 17:32'!
mcModel
	"Find my WorkingCopy, use the first mcModel-capable repository in its reposigoryGroup."
	self repositoryGroup ifNotNil:
		[ : group | group repositoriesDo:
			[ : each | each mcModel ifNotNil:
				[ : mcModel | ^ mcModel ] ] ].
	^ nil! !

!MCDefinition methodsFor: 'renaming' stamp: 'cmm 6/6/2011 15:22'!
newCategoryNameFor: categoryName givenRenameFrom: oldPackageName to: newPackageName 
	^ oldPackageName size < categoryName size
		ifTrue:
			[ newPackageName ,
				(categoryName
					copyFrom: oldPackageName size + 1
					to: categoryName size) ]
		ifFalse: [ newPackageName ]! !

!MCDefinition methodsFor: 'installing' stamp: 'ab 7/18/2003 19:48'!
postload! !

!MCDefinition methodsFor: 'installing' stamp: 'avi 2/17/2004 13:19'!
postloadOver: aDefinition
	self postload! !

!MCDefinition methodsFor: 'annotations' stamp: 'nk 11/10/2003 21:46'!
printAnnotations: requests on: aStream
	"Add a string for an annotation pane, trying to fulfill the annotation requests.
	These might include anything that
		Preferences defaultAnnotationRequests 
	might return. Which includes anything in
		Preferences annotationInfo
	To edit these, use:"
	"Preferences editAnnotations"

	aStream nextPutAll: 'not yet implemented'! !

!MCDefinition methodsFor: 'printing' stamp: 'ab 7/18/2003 19:43'!
printOn: aStream
	super printOn: aStream.
	aStream nextPutAll: '(', self summary, ')'! !

!MCDefinition methodsFor: 'accessing' stamp: 'ab 5/24/2003 14:12'!
provisions
	^ #()! !

!MCDefinition methodsFor: 'repositories' stamp: 'cmm 11/28/2016 22:43'!
repositoryGroup
	"Answer the MCRepositoryGroup from which this this object was loaded."
	^ self workingCopy ifNotNil: [ : wc | wc repositoryGroup ]! !

!MCDefinition methodsFor: 'accessing' stamp: 'ab 5/24/2003 14:12'!
requirements
	^ #()! !

!MCDefinition methodsFor: 'comparing' stamp: 'ab 7/19/2003 18:04'!
sortKey
	self subclassResponsibility ! !

!MCDefinition methodsFor: 'printing' stamp: 'ab 7/19/2003 18:23'!
summary
	self subclassResponsibility ! !

!MCDefinition methodsFor: 'printing' stamp: 'bf 11/25/2016 11:30'!
summaryAndRevision
	^self summary! !

!MCDefinition methodsFor: 'printing' stamp: 'bf 11/1/2011 11:42'!
summarySuffixOver: previousDefinition
	^self source = previousDefinition source
		ifTrue: [ ' (source same but rev changed)' ]
		ifFalse: [ ' (changed)' ]! !

!MCDefinition methodsFor: 'installing' stamp: 'ab 11/14/2002 00:08'!
unload! !

!MCDefinition methodsFor: 'testing' stamp: 'ct 4/18/2020 16:37'!
wantsToBeOutermost
	"Outermost definitions are installed after all other definitions. In particular, this hook is crucial for the correct behavior of removal preambles. See MCDependencySorter >> #addDeferred:."

	^ false! !

!MCDefinition methodsFor: 'repositories' stamp: 'cmm 3/8/2011 21:13'!
workingCopy
	self subclassResponsibility! !

MCDefinition subclass: #MCMethodDefinition
	instanceVariableNames: 'classIsMeta source category selector className timeStamp'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Modeling'!

MCMethodDefinition class
	instanceVariableNames: 'definitions'!

MCMethodDefinition class
	instanceVariableNames: 'definitions'!

!MCMethodDefinition class methodsFor: 'accessing' stamp: 'ul 2/26/2010 17:42'!
cachedDefinitions
	
	^definitions ifNil: [ definitions := WeakIdentityKeyDictionary new ]! !

!MCMethodDefinition class methodsFor: 'create' stamp: 'ab 7/26/2003 02:05'!
className: classString
classIsMeta: metaBoolean
selector: selectorString
category: catString
timeStamp: timeString
source: sourceString
	^ self instanceLike:
		(self new initializeWithClassName: classString
					classIsMeta: metaBoolean
					selector: selectorString
					category: catString
					timeStamp: timeString
					source: sourceString)! !

!MCMethodDefinition class methodsFor: 'create' stamp: 'ab 4/1/2003 01:40'!
className: classString
selector: selectorString
category: catString
timeStamp: timeString
source: sourceString
	^ self	className: classString
			classIsMeta: false
			selector: selectorString
			category: catString
			timeStamp: timeString
			source: sourceString! !

!MCMethodDefinition class methodsFor: 'class initialization' stamp: 'ar 2/26/2010 23:03'!
cleanUp
	"Flush caches"

	self shutDown.! !

!MCMethodDefinition class methodsFor: 'create' stamp: 'nice 1/1/2010 22:10'!
forMethodReference: aMethodReference
	| definition |
	definition := self cachedDefinitions at: aMethodReference compiledMethod ifAbsent: [].
	(definition isNil
		or: [definition selector ~= aMethodReference methodSymbol
		or: [definition className ~= aMethodReference classSymbol
		or: [definition classIsMeta ~= aMethodReference classIsMeta
		or: [definition category ~= aMethodReference category]]]])
			ifTrue: [definition := self 
						className: aMethodReference classSymbol
						classIsMeta: aMethodReference classIsMeta
						selector: aMethodReference methodSymbol
						category: aMethodReference category
						timeStamp: aMethodReference timeStamp
						source: aMethodReference source.
					self cachedDefinitions at: aMethodReference compiledMethod put: definition].
	^ definition
	! !

!MCMethodDefinition class methodsFor: 'class initialization' stamp: 'ab 8/22/2003 18:14'!
initialize
	Smalltalk addToShutDownList: self! !

!MCMethodDefinition class methodsFor: 'class initialization' stamp: 'ul 2/26/2010 17:42'!
shutDown
	
	definitions := nil.! !

!MCMethodDefinition methodsFor: 'comparing' stamp: 'nice 10/31/2009 13:12'!
= aDefinition
	^(super = aDefinition)
		and: [aDefinition source = self source
		and: [aDefinition category = self category
		and: [aDefinition timeStamp = self timeStamp]]]! !

!MCMethodDefinition methodsFor: 'visiting' stamp: 'ab 7/18/2003 21:47'!
accept: aVisitor
	^ aVisitor visitMethodDefinition: self! !

!MCMethodDefinition methodsFor: 'accessing' stamp: 'jr 2/28/2017 02:44'!
actualClass
	^ self actualClassIn: Environment current! !

!MCMethodDefinition methodsFor: 'accessing' stamp: 'jr 2/28/2017 02:44'!
actualClassIn: anEnvironment
	^ (anEnvironment at: className ifAbsent: [anEnvironment valueOf: className])
		ifNotNil: [:class |
			class isBehavior ifTrue: [classIsMeta ifTrue: [class classSide] ifFalse: [class]]]! !

!MCMethodDefinition methodsFor: 'installing' stamp: 'cmm 3/8/2011 20:25'!
addMethodAdditionTo: aCollection 
	aCollection
		 add: self asMethodAddition createCompiledMethod ;
		 yourself! !

!MCMethodDefinition methodsFor: 'converting' stamp: 'jr 12/23/2019 18:04'!
asMethodAddition
	^MethodAddition new
		compile: source
		classified: category
		withStamp: timeStamp
		notifying: nil
		logSource: SystemChangeNotifier uniqueInstance isBroadcasting
		inClass: self actualClass.! !

!MCMethodDefinition methodsFor: 'converting' stamp: 'cmm 3/8/2011 19:44'!
asMethodReference
	^ MethodReference
		class: self actualClass
		selector: self selector! !

!MCMethodDefinition methodsFor: 'accessing' stamp: 'ab 11/13/2002 01:59'!
category
	^ category! !

!MCMethodDefinition methodsFor: 'accessing' stamp: 'cwp 11/25/2002 07:26'!
classIsMeta
	^ classIsMeta! !

!MCMethodDefinition methodsFor: 'accessing' stamp: 'ab 11/15/2002 01:12'!
className
	^className! !

!MCMethodDefinition methodsFor: 'printing' stamp: 'ul 10/3/2018 00:48'!
description
	
	^{ className. selector. classIsMeta }! !

!MCMethodDefinition methodsFor: 'printing' stamp: 'bf 3/28/2011 18:41'!
fullClassName
	"Using #class selector for classes for backwards compatibility"

	^ self classIsMeta
		ifFalse: [self className]
		ifTrue: [
			(self actualClass notNil and: [ self actualClass isTrait ])
				ifFalse: [self className, ' class']
				ifTrue: [self className, ' classSide']]! !

!MCMethodDefinition methodsFor: 'accessing' stamp: 'nk 10/21/2003 23:08'!
fullTimeStamp
	^TimeStamp fromMethodTimeStamp: timeStamp! !

!MCMethodDefinition methodsFor: 'renaming' stamp: 'cmm 12/2/2011 13:17'!
handlePackageRename: oldPackageName to: newPackageName 
	"If I'm an extension or override method, rename the category to be prefixed with newPackageName."
	((self isExtensionMethod or: [ self isOverrideMethod ]) and: [ (self category allButFirst beginsWith: newPackageName asLowercase) not ]) ifTrue:
		[ self actualClass organization
			renameCategory: self category
			toBe:
				'*' ,
					(self
						newCategoryNameFor: self category allButFirst
						givenRenameFrom: oldPackageName
						to: newPackageName) asLowercase ]! !

!MCMethodDefinition methodsFor: 'comparing' stamp: 'ul 11/2/2009 03:33'!
hash
	| hash |
	hash := classIsMeta asString hashWithInitialHash: 0.
	hash := source hashWithInitialHash: hash.
	hash := category hashWithInitialHash: hash.
	hash := className hashWithInitialHash: hash.
	^ hash! !

!MCMethodDefinition methodsFor: 'serializing' stamp: 'eem 2/4/2014 16:35'!
initializeWithClassName: classString
classIsMeta: metaBoolean
selector: selectorString
category: catString
timeStamp: timeString
source: sourceString
	className := classString asSymbol.
	selector := selectorString asSymbol.
	category := catString ifNil: [Categorizer default] ifNotNil: [catString asSymbol].
	timeStamp := timeString.
	classIsMeta := metaBoolean.
	source := sourceString withSqueakLineEndings! !

!MCMethodDefinition methodsFor: 'testing' stamp: 'ab 5/24/2003 13:49'!
isCodeDefinition
	^ true! !

!MCMethodDefinition methodsFor: 'installing' stamp: 'avi 9/17/2003 22:27'!
isExtensionMethod
	^ category beginsWith: '*'! !

!MCMethodDefinition methodsFor: 'testing' stamp: 'bf 1/5/2006 15:56'!
isExternalStructureFieldDefinition
	^ selector = #fields
		and: [classIsMeta
			and: [
				(Smalltalk at: #ExternalStructure ifPresent: [:externalStructure |
					self actualClass theNonMetaClass inheritsFrom: externalStructure]) == true]]
			
	! !

!MCMethodDefinition methodsFor: 'testing' stamp: 'ab 8/8/2003 17:05'!
isInitializer
	^ selector = #initialize and: [classIsMeta]
	! !

!MCMethodDefinition methodsFor: 'testing' stamp: 'ab 12/4/2002 21:52'!
isMethodDefinition
	^true! !

!MCMethodDefinition methodsFor: 'installing' stamp: 'avi 11/10/2003 15:45'!
isOverrideMethod
	"this oughta check the package"
	^ self isExtensionMethod and: [category endsWith: '-override']! !

!MCMethodDefinition methodsFor: 'accessing' stamp: 'jr 6/8/2020 21:40'!
load
	| class |
	class := self actualClass.
	class
		compile: source
		classified: category
		withStamp: timeStamp
		notifying: nil
		logSource: (SystemChangeNotifier uniqueInstance isBroadcasting and: [class acceptsLoggingOfCompilation])! !

!MCMethodDefinition methodsFor: 'installing' stamp: 'mt 6/20/2020 13:22'!
postload
	self isInitializer
		ifTrue: [self actualClass theNonMetaClass initialize].
	self isExternalStructureFieldDefinition
		ifTrue: [[self actualClass theNonMetaClass compileFields]
			ifError: [:msg | Transcript showln: '[FFI] ', msg]].! !

!MCMethodDefinition methodsFor: 'annotations' stamp: 'nk 7/24/2003 16:06'!
printAnnotations: requests on: aStream
	"Add a string for an annotation pane, trying to fulfill the annotation requests.
	These might include anything that
		Preferences defaultAnnotationRequests 
	might return. Which includes anything in
		Preferences annotationInfo
	To edit these, use:"
	"Preferences editAnnotations"

	requests do: [ :aRequest |
		aRequest == #timeStamp ifTrue: [ aStream nextPutAll: self timeStamp ].
		aRequest == #messageCategory ifTrue: [ aStream nextPutAll: self category ].
		aRequest == #requirements ifTrue: [
			self requirements do: [ :req |
				aStream nextPutAll: req ] separatedBy: [ aStream space ]].
	] separatedBy: [ aStream space ].! !

!MCMethodDefinition methodsFor: 'accessing' stamp: 'TonelReaderTest 6/15/2018 12:38'!
protocol
	"Answer in which protocol (conceptual groups of methods) the receiver is grouped into."
	^category! !

!MCMethodDefinition methodsFor: 'installing' stamp: 'ar 12/12/2009 17:05'!
removeSelector: aSelector fromClass: aClass
	"Safely remove the given selector from the target class.
	Be careful not to remove the selector when it has wondered
	to another package."
	| newCategory |
	newCategory := aClass organization categoryOfElement: aSelector.
	newCategory ifNotNil:[
		"If moved to and fro extension, ignore removal"
		(category beginsWith: '*') = (newCategory beginsWith: '*') ifFalse:[^self].
		"Check if moved between different extension categories"
		((category beginsWith: '*') and:[category ~= newCategory]) ifTrue:[^self]].
	aClass removeSelector: aSelector.
! !

!MCMethodDefinition methodsFor: 'comparing' stamp: 'ul 10/18/2018 01:47'!
requirements
	^{ className }! !

!MCMethodDefinition methodsFor: 'installing' stamp: 'bf 5/4/2012 16:27'!
scanForPreviousVersions
	"Answer change records for other versions of this method, i.e. other overrides and the original version"
	| versions |
	versions := OrderedCollection new.
	PackageInfo default changeRecordsForMethod: self asMethodReference do: [:record |
		record category = category ifFalse: [versions add: record]].
	^versions
! !

!MCMethodDefinition methodsFor: 'accessing' stamp: 'ab 11/15/2002 01:11'!
selector
	^selector! !

!MCMethodDefinition methodsFor: 'comparing' stamp: 'ab 7/19/2003 18:01'!
sortKey
	^ self className, '.', (self classIsMeta ifTrue: ['meta'] ifFalse: ['nonmeta']), '.', self selector! !

!MCMethodDefinition methodsFor: 'accessing' stamp: 'ab 11/13/2002 01:59'!
source
	^ source! !

!MCMethodDefinition methodsFor: 'printing' stamp: 'ab 4/8/2003 18:04'!
summary
	^ self fullClassName , '>>' , selector! !

!MCMethodDefinition methodsFor: 'printing' stamp: 'bf 11/25/2016 11:35'!
summaryAndRevision
	^String streamContents:
		[:s | s nextPutAll: self timeStamp;
			space; nextPutAll: self summary;
			nextPutAll: ' {'; nextPutAll: self category;
			nextPut: $}]! !

!MCMethodDefinition methodsFor: 'printing' stamp: 'bf 11/1/2011 11:49'!
summarySuffixOver: previousDefinition
	| sourceChanged categoryChanged timeStampChanged |
	sourceChanged := self source ~= previousDefinition source.
	timeStampChanged := self timeStamp ~= previousDefinition timeStamp.
	categoryChanged := self category ~= previousDefinition category.
	sourceChanged | timeStampChanged | categoryChanged
		ifFalse: [ ^super summarySuffixOver: previousDefinition ].
	sourceChanged ifTrue: [
		^categoryChanged
			ifTrue: [ ' (changed and recategorized)' ]
			ifFalse: [ ' (changed)' ] ].
	timeStampChanged & categoryChanged
		ifTrue: [^ ' (recategorized and different time stamp)' ].
	^categoryChanged
		ifTrue: [ ' (only recategorized)' ]
		ifFalse: [ ' (only different time stamp)' ]
! !

!MCMethodDefinition methodsFor: 'accessing' stamp: 'nice 9/22/2010 00:48'!
timeStamp
	^ timeStamp ifNil: ['']! !

!MCMethodDefinition methodsFor: 'installing' stamp: 'bf 5/4/2012 17:11'!
unload
	| previousVersions |
	self isOverrideMethod ifTrue: [
		previousVersions := self scanForPreviousVersions].
	previousVersions isEmptyOrNil 
		ifTrue: [self actualClass ifNotNil: [:class |
			self removeSelector: selector fromClass: class]]
		ifFalse: [
			"remove our override from history, even if buried below other overrides"
			previousVersions reverseDo: [:version | version fileIn]] ! !

!MCMethodDefinition methodsFor: 'accessing' stamp: 'cmm 3/8/2011 20:28'!
workingCopy
	"Answer the working copy of which this object is defined."
	^ self asMethodReference workingCopy! !

MCDefinition subclass: #MCOrganizationDefinition
	instanceVariableNames: 'categories'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Modeling'!

!MCOrganizationDefinition class methodsFor: 'instance creation' stamp: 'ab 7/26/2003 02:06'!
categories: anArray
	^ self instanceLike: (self new categories: anArray)! !

!MCOrganizationDefinition methodsFor: 'testing' stamp: 'ab 7/19/2003 18:25'!
= aDefinition
	^ (super = aDefinition)
		and: [categories = aDefinition categories]! !

!MCOrganizationDefinition methodsFor: 'actions' stamp: 'ab 7/18/2003 21:47'!
accept: aVisitor
	^ aVisitor visitOrganizationDefinition: self! !

!MCOrganizationDefinition methodsFor: 'accessing' stamp: 'ab 5/24/2003 13:51'!
categories
	^ categories! !

!MCOrganizationDefinition methodsFor: 'accessing' stamp: 'ab 5/24/2003 13:39'!
categories: anArray
	categories := anArray! !

!MCOrganizationDefinition methodsFor: 'accessing' stamp: 'avi 9/30/2004 21:56'!
commonPrefix
	| stream |
	categories isEmpty ifTrue: [^ ''].
	
	stream := String new writeStream.
	categories first withIndexDo:
		[:c :i|
		categories do:
			[:ea |
			(ea at: i ifAbsent: []) = c ifFalse: [^ stream contents]].
		stream nextPut: c].
	^ stream contents! !

!MCOrganizationDefinition methodsFor: 'accessing' stamp: 'ul 10/18/2018 01:47'!
description
	^{ #organization }
! !

!MCOrganizationDefinition methodsFor: 'actions' stamp: 'cmm 6/8/2011 13:59'!
handlePackageRename: oldPackageName to: newPackageName 
	categories do:
		[ : each | (each beginsWith: newPackageName) ifFalse:
			[ | newCategoryName |
			newCategoryName := self
				newCategoryNameFor: each
				givenRenameFrom: oldPackageName
				to: newPackageName.
			(SystemOrganizer default categories includes: newCategoryName) ifTrue: [ SystemOrganizer default removeCategory: newCategoryName ].
			SystemOrganizer default
				renameCategory: each
				toBe: newCategoryName ] ]! !

!MCOrganizationDefinition methodsFor: 'testing' stamp: 'cwp 7/11/2003 01:33'!
isOrganizationDefinition
	^ true! !

!MCOrganizationDefinition methodsFor: 'actions' stamp: 'avi 2/22/2004 13:46'!
postloadOver: oldDefinition
	SystemOrganization categories:
		(self
			reorderCategories: SystemOrganization categories
			original: (oldDefinition ifNil: [#()] ifNotNil: [oldDefinition categories]))! !

!MCOrganizationDefinition methodsFor: 'actions' stamp: 'bf 4/11/2012 10:09'!
reorderCategories: allCategories original: oldCategories
	| first locallyAddedCategories |
	first := allCategories detect: [:ea | categories includes: ea]
		ifNone: [^ allCategories, categories].
	locallyAddedCategories := (oldCategories copyWithoutAll: categories) select: [:cat |
		(SystemOrganization listAtCategoryNamed: cat) notEmpty].
	^ 	((allCategories copyUpTo: first) copyWithoutAll: oldCategories, categories),
		categories,
		locallyAddedCategories,
		((allCategories copyAfter: first) copyWithoutAll: oldCategories, categories)
! !

!MCOrganizationDefinition methodsFor: 'accessing' stamp: 'ab 7/19/2003 18:01'!
sortKey
	^ '<organization>'! !

!MCOrganizationDefinition methodsFor: 'accessing' stamp: 'ab 7/22/2003 01:14'!
source
	^ String streamContents:
		[:s |
		categories do: [:ea | s nextPutAll: ea] separatedBy: [s cr]]! !

!MCOrganizationDefinition methodsFor: 'accessing' stamp: 'ab 5/24/2003 13:55'!
summary
	^ categories asArray printString! !

!MCOrganizationDefinition methodsFor: 'actions' stamp: 'bf 4/11/2012 11:00'!
unload
	| empty |
	empty := categories select: [:ea |
		(SystemOrganization listAtCategoryNamed: ea) isEmpty].
	SystemOrganization categories:
		(SystemOrganization categories copyWithoutAll: empty)! !

MCDefinition subclass: #MCScriptDefinition
	instanceVariableNames: 'script packageName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Modeling'!

MCScriptDefinition subclass: #MCPostscriptDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Modeling'!

!MCPostscriptDefinition class methodsFor: 'private - instance creation' stamp: 'avi 2/28/2005 17:04'!
scriptSelector
	^ #postscript! !

!MCPostscriptDefinition methodsFor: 'installing' stamp: 'avi 2/28/2005 17:16'!
postload
	self evaluate! !

!MCPostscriptDefinition methodsFor: 'accessing' stamp: 'avi 2/28/2005 17:16'!
sortKey
	^ 'zzz' "force to the end so it gets loaded late"! !

!MCPostscriptDefinition methodsFor: 'nil' stamp: 'ct 2/6/2020 17:54'!
wantsToBeOutermost

	^ true! !

MCScriptDefinition subclass: #MCPreambleDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Modeling'!

!MCPreambleDefinition class methodsFor: 'private - instance creation' stamp: 'avi 2/28/2005 17:04'!
scriptSelector
	^ #preamble! !

!MCPreambleDefinition methodsFor: 'installing' stamp: 'avi 2/28/2005 17:15'!
load
	super load.
	self evaluate! !

MCScriptDefinition subclass: #MCRemovalPostscriptDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Modeling'!

!MCRemovalPostscriptDefinition class methodsFor: 'private - instance creation' stamp: 'avi 2/28/2005 17:04'!
scriptSelector
	^ #postscriptOfRemoval ! !

!MCRemovalPostscriptDefinition methodsFor: 'installing' stamp: 'avi 2/28/2005 17:15'!
unload
	super unload.
	self evaluate! !

MCScriptDefinition subclass: #MCRemovalPreambleDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Modeling'!

!MCRemovalPreambleDefinition class methodsFor: 'private - instance creation' stamp: 'avi 2/28/2005 17:04'!
scriptSelector
	^ #preambleOfRemoval! !

!MCRemovalPreambleDefinition methodsFor: 'accessing' stamp: 'avi 2/28/2005 17:14'!
sortKey
	^ 'zzz' "force to the end so it gets unloaded early"! !

!MCRemovalPreambleDefinition methodsFor: 'installing' stamp: 'avi 2/28/2005 17:15'!
unload
	super unload.
	self evaluate! !

!MCRemovalPreambleDefinition methodsFor: 'testing' stamp: 'ct 2/6/2020 17:54'!
wantsToBeOutermost

	^ true! !

!MCScriptDefinition class methodsFor: 'instance creation' stamp: 'bf 4/4/2005 12:20'!
from: aPackageInfo
	^ self script: (aPackageInfo perform: self scriptSelector) contents asString packageName: aPackageInfo name! !

!MCScriptDefinition class methodsFor: 'instance creation' stamp: 'avi 2/28/2005 16:59'!
script: aString packageName: packageString
	^ self instanceLike: (self new initializeWithScript: aString packageName: packageString)! !

!MCScriptDefinition class methodsFor: 'private - instance creation' stamp: 'avi 2/28/2005 17:00'!
scriptSelector
	self subclassResponsibility! !

!MCScriptDefinition class methodsFor: 'instance creation' stamp: 'bf 8/13/2009 00:24'!
scriptSelector: selectorString script: aString packageName: packageString
	^ (self subclassForScriptSelector: selectorString)
		script: aString packageName: packageString! !

!MCScriptDefinition class methodsFor: 'private - instance creation' stamp: 'bf 8/13/2009 00:25'!
subclassForScriptSelector: selectorString
	^self allSubclasses detect: [:ea | ea scriptSelector = selectorString]! !

!MCScriptDefinition methodsFor: 'comparing' stamp: 'avi 2/28/2005 16:55'!
= aDefinition
	^ (super = aDefinition)
		and: [script = aDefinition script]! !

!MCScriptDefinition methodsFor: 'visiting' stamp: 'bf 8/12/2009 21:41'!
accept: aVisitor
	aVisitor visitScriptDefinition: self! !

!MCScriptDefinition methodsFor: 'accessing' stamp: 'ul 10/18/2018 01:43'!
description
	^{ packageName. self scriptSelector }! !

!MCScriptDefinition methodsFor: 'installing' stamp: 'jr 5/30/2020 23:51'!
evaluate
	Compiler evaluate: script environment: Environment current! !

!MCScriptDefinition methodsFor: 'initializing' stamp: 'stephaneducasse 2/4/2006 20:47'!
initializeWithScript: aString packageName: packageString
	script := aString.
	packageName := packageString! !

!MCScriptDefinition methodsFor: 'installing' stamp: 'avi 2/28/2005 17:11'!
installScript
	self installScript: script! !

!MCScriptDefinition methodsFor: 'installing' stamp: 'bf 10/25/2005 18:55'!
installScript: aString
	| sel pi |
	sel := (self scriptSelector, ':') asSymbol.
	pi := self packageInfo.
	(pi respondsTo: sel)
		ifTrue: [pi perform: sel with: aString]! !

!MCScriptDefinition methodsFor: 'testing' stamp: 'bf 8/12/2009 22:55'!
isScriptDefinition
	^true! !

!MCScriptDefinition methodsFor: 'installing' stamp: 'avi 2/28/2005 17:12'!
load
	self installScript! !

!MCScriptDefinition methodsFor: 'accessing' stamp: 'avi 2/28/2005 17:22'!
packageInfo
	^ PackageInfo named: packageName! !

!MCScriptDefinition methodsFor: 'accessing' stamp: 'bf 8/13/2009 00:09'!
packageName
	^ packageName! !

!MCScriptDefinition methodsFor: 'accessing' stamp: 'avi 2/28/2005 16:54'!
script
	^ script! !

!MCScriptDefinition methodsFor: 'accessing' stamp: 'avi 2/28/2005 17:10'!
scriptSelector
	^ self class scriptSelector! !

!MCScriptDefinition methodsFor: 'accessing' stamp: 'bf 10/25/2005 19:05'!
sortKey
	^ '!!', self scriptSelector "force to the front so it gets loaded first"! !

!MCScriptDefinition methodsFor: 'accessing' stamp: 'avi 2/28/2005 17:04'!
source
	^ script! !

!MCScriptDefinition methodsFor: 'accessing' stamp: 'bf 7/31/2012 11:43'!
summary
	^ packageName, ' package ', self scriptSelector! !

!MCScriptDefinition methodsFor: 'installing' stamp: 'avi 2/28/2005 17:12'!
unload
	self installScript: nil! !

Object subclass: #MCDefinitionIndex
	instanceVariableNames: 'definitions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Patching'!

!MCDefinitionIndex commentStamp: 'ul 10/22/2018 22:39' prior: 0!
I'm just a wrapper for a KeyedSet with keyBlock [ :definition | definition description ] translating #add: to #put:.!

!MCDefinitionIndex class methodsFor: 'instance creation' stamp: 'ul 10/3/2018 00:40'!
definitions: aCollection

	^self basicNew
		initialize: aCollection size;
		addAll: aCollection;
		yourself! !

!MCDefinitionIndex methodsFor: 'adding' stamp: 'ul 10/22/2018 22:52'!
add: aDefinition

	^definitions put: aDefinition! !

!MCDefinitionIndex methodsFor: 'adding' stamp: 'ul 10/22/2018 22:52'!
addAll: aCollection

	aCollection do: [ :each |
		definitions put: each ]! !

!MCDefinitionIndex methodsFor: 'accessing' stamp: 'ul 10/22/2018 22:52'!
definitionLike: aDefinition ifPresent: foundBlock ifAbsent: absentBlock

	^(definitions like: aDefinition ifAbsent: nil)
		ifNil: [ absentBlock value ]
		ifNotNil: [ :definition |
			foundBlock value: definition ]! !

!MCDefinitionIndex methodsFor: 'accessing' stamp: 'ul 10/22/2018 22:53'!
definitions
	
	^definitions asArray! !

!MCDefinitionIndex methodsFor: 'enumerating' stamp: 'ul 10/17/2018 19:25'!
definitionsDo: aBlock
	
	definitions do: aBlock! !

!MCDefinitionIndex methodsFor: 'initialize-release' stamp: 'ul 10/3/2018 00:41'!
initialize

	self initialize: 10! !

!MCDefinitionIndex methodsFor: 'initialize-release' stamp: 'ul 10/22/2018 22:53'!
initialize: initialCapacity

	definitions := (KeyedSet new: initialCapacity)
		keyBlock: [ :object | object description ];
		yourself! !

!MCDefinitionIndex methodsFor: 'removing' stamp: 'ul 10/22/2018 22:53'!
remove: aDefinition

	^definitions remove: aDefinition ifAbsent: nil! !

Object subclass: #MCDependencySorter
	instanceVariableNames: 'nondeferred deferred required provided'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Loading'!

!MCDependencySorter commentStamp: 'ct 4/18/2020 16:38' prior: 0!
I sort a set of passed MCDefinitions, resolving all dependency conflicts automatically.

To use me, first add some items to me using the building protocol or my class-side #items: selector. After that, send #orderedItems to me and I will tell you all items in a loadable order.

Instance Variables
	nondeferred:	<SequenceableCollection>
		Usual items that do not require special sorting mechanics.
	deferred:		<SequenceableCollection>
		Items that are added after all nondeferred items. See MCDefinition >> #wantsToBeOutermost.
	required:		<Set>
	provided:		<Set>!

!MCDependencySorter class methodsFor: 'instance creation' stamp: 'ab 5/23/2003 14:17'!
items: aCollection
	^ self new addAll: aCollection! !

!MCDependencySorter class methodsFor: 'instance creation' stamp: 'avi 9/13/2004 18:03'!
new
	^ self basicNew initialize! !

!MCDependencySorter class methodsFor: 'utility' stamp: 'ct 4/18/2020 15:58'!
sortItems: aCollection
	| sorter |
	sorter := self items: aCollection.
	sorter externalRequirements do: [:req  | sorter addProvision: req].
	^ sorter orderedItems! !

!MCDependencySorter methodsFor: 'building' stamp: 'ct 2/6/2020 17:56'!
add: anItem
	| requirements |
	(anItem wantsToBeOutermost)
		ifTrue: [^self addDeferred: anItem].
	requirements := self unresolvedRequirementsFor: anItem.
	requirements isEmpty
		ifTrue: [self addToOrder: anItem]
		ifFalse: [self addRequirements: requirements for: anItem].
	^anItem! !

!MCDependencySorter methodsFor: 'building' stamp: 'bf 11/12/2004 14:50'!
addAll: aCollection
	aCollection asArray sort do: [:ea | self add: ea]! !

!MCDependencySorter methodsFor: 'private' stamp: 'ct 4/18/2020 16:14'!
addDeferred: anItem
	"Add an item to the derred list. Deferred items will be appended to #orderedItems after all nondeferred items. See #wantsToBeOutermost."

	^ deferred add: anItem! !

!MCDependencySorter methodsFor: 'building' stamp: 'avi 10/7/2004 22:47'!
addExternalProvisions: aCollection
	(aCollection intersection: self externalRequirements)
		do: [:ea | self addProvision: ea]! !

!MCDependencySorter methodsFor: 'private' stamp: 'ab 5/22/2003 23:25'!
addProvision: anObject
	| newlySatisfied |
	provided add: anObject.
	newlySatisfied := required removeKey: anObject ifAbsent: [#()].
	self addAll: newlySatisfied.! !

!MCDependencySorter methodsFor: 'private' stamp: 'ab 5/22/2003 23:24'!
addRequirement: reqObject for: itemObject
	(self itemsRequiring: reqObject) add: itemObject! !

!MCDependencySorter methodsFor: 'private' stamp: 'ab 5/22/2003 23:23'!
addRequirements: aCollection for: anObject
	aCollection do: [:ea | self addRequirement: ea for: anObject]! !

!MCDependencySorter methodsFor: 'private' stamp: 'ct 4/18/2020 15:57'!
addToOrder: anItem
	nondeferred add: anItem.
	anItem provisions do: [:ea | self addProvision: ea].! !

!MCDependencySorter methodsFor: 'accessing' stamp: 'dvf 9/8/2004 00:49'!
externalRequirements
	| unloaded providedByUnloaded |
	unloaded := self itemsWithMissingRequirements.
	providedByUnloaded := (unloaded gather: [:e | e provisions]) asSet.
	^ required keys reject: [:ea | providedByUnloaded includes: ea ]! !

!MCDependencySorter methodsFor: 'initialize-release' stamp: 'ct 4/18/2020 15:57'!
initialize
	provided := Set new.
	required := Dictionary new.
	nondeferred := OrderedCollection new.
	deferred := OrderedCollection new.! !

!MCDependencySorter methodsFor: 'private' stamp: 'ab 5/22/2003 23:24'!
itemsRequiring: anObject
	^ required at: anObject ifAbsentPut: [Set new]! !

!MCDependencySorter methodsFor: 'accessing' stamp: 'ab 5/25/2003 01:15'!
itemsWithMissingRequirements
	| items |
	items := Set new.
	required do: [:ea | items addAll: ea].
	^ items
! !

!MCDependencySorter methodsFor: 'accessing' stamp: 'ct 4/18/2020 16:14'!
orderedItems
	"Return all ordered items, first nondeferred items, second deferred (outermost) items."

	^ nondeferred , deferred! !

!MCDependencySorter methodsFor: 'private' stamp: 'ab 5/22/2003 23:22'!
unresolvedRequirementsFor: anItem
	^ anItem requirements difference: provided! !

Object subclass: #MCDoItParser
	instanceVariableNames: 'source'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Chunk Format'!

MCDoItParser subclass: #MCClassTraitParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Modeling'!

!MCClassTraitParser class methodsFor: 'constants' stamp: 'al 10/9/2005 20:43'!
pattern
	^ '*classTrait*uses:*'! !

!MCClassTraitParser methodsFor: 'reader' stamp: 'stephaneducasse 2/4/2006 20:47'!
addDefinitionsTo: aCollection
	| tokens  definition traitCompositionString |
	tokens := Scanner new scanTokens: source.
	traitCompositionString := ((ReadStream on: source)
		match: 'uses:';
		upToEnd) withBlanksTrimmed.
	definition := MCClassTraitDefinition
		baseTraitName: (tokens at: 1) 
		classTraitComposition: traitCompositionString.
	aCollection add: definition
! !

!MCDoItParser class methodsFor: 'accessing class hierarchy' stamp: 'avi 3/10/2004 12:29'!
concreteSubclasses
	^ self allSubclasses reject: [:c | c isAbstract]! !

!MCDoItParser class methodsFor: 'instance creation' stamp: 'ul 12/12/2009 14:10'!
forDoit: aString
	^ (self subclassForDoit: aString) ifNotNil: [:c | c new source: aString]! !

!MCDoItParser class methodsFor: 'testing' stamp: 'avi 3/10/2004 12:51'!
isAbstract
	^ self pattern isNil! !

!MCDoItParser class methodsFor: 'constants' stamp: 'avi 3/10/2004 12:30'!
pattern
	^ nil! !

!MCDoItParser class methodsFor: 'private' stamp: 'avi 3/10/2004 12:30'!
subclassForDoit: aString
	^ self concreteSubclasses detect: [:ea | ea pattern match: aString] ifNone: []! !

!MCDoItParser methodsFor: 'evaluating' stamp: 'avi 3/10/2004 12:40'!
addDefinitionsTo: aCollection
	self subclassResponsibility ! !

!MCDoItParser methodsFor: 'accessing' stamp: 'avi 3/10/2004 12:40'!
source
	^ source! !

!MCDoItParser methodsFor: 'accessing' stamp: 'avi 3/10/2004 12:40'!
source: aString
	source := aString! !

MCDoItParser subclass: #MCScriptParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Chunk Format'!

!MCScriptParser class methodsFor: 'constants' stamp: 'bf 8/13/2009 00:07'!
pattern
	^'(PackageInfo named: *'! !

!MCScriptParser methodsFor: 'evaluating' stamp: 'bf 8/13/2009 00:13'!
addDefinitionsTo: aCollection
	| tokens  definition |
	tokens := Scanner new scanTokens: source.
	definition := MCScriptDefinition
		scriptSelector: tokens second allButLast
		script: tokens third
		packageName: tokens first third.
	aCollection add: definition.! !

MCDoItParser subclass: #MCSystemCategoryParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Chunk Format'!

!MCSystemCategoryParser class methodsFor: 'constants' stamp: 'avi 3/10/2004 12:41'!
pattern
	^ 'SystemOrganization*'! !

!MCSystemCategoryParser methodsFor: 'evaluating' stamp: 'avi 3/10/2004 12:49'!
addDefinitionsTo: aCollection
	| definition |
	definition := aCollection detect: [:ea | ea isOrganizationDefinition ] ifNone: [aCollection add: (MCOrganizationDefinition categories: #())].
	definition categories: (definition categories copyWith: self category).! !

!MCSystemCategoryParser methodsFor: 'evaluating' stamp: 'avi 3/10/2004 12:42'!
category
	| tokens  |
	tokens := Scanner new scanTokens: source.
	tokens size = 3 ifFalse: [self error: 'Unrecognized category definition'].
	^ tokens at: 3! !

MCDoItParser subclass: #MCTraitParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Modeling'!

!MCTraitParser class methodsFor: 'constants' stamp: 'al 10/9/2005 21:09'!
pattern
	^ 'Trait named:*'! !

!MCTraitParser methodsFor: 'reader' stamp: 'stephaneducasse 2/4/2006 20:47'!
addDefinitionsTo: aCollection
	| tokens  definition traitCompositionString |
	tokens := Scanner new scanTokens: source.
	traitCompositionString := ((ReadStream on: source)
		match: 'uses:';
		upToAll: 'category:') withBlanksTrimmed.
	definition := MCTraitDefinition
		name: (tokens at: 3) 
		traitComposition: traitCompositionString
		category:  tokens last
		comment:  ''  
		commentStamp:   ''.
	aCollection add: definition.! !

Object subclass: #MCFrontier
	instanceVariableNames: 'frontier bag'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

!MCFrontier class methodsFor: 'instance creation' stamp: 'ul 10/18/2018 01:48'!
frontierOn: aVersionInfo
	^ self frontierOnAll: { aVersionInfo }! !

!MCFrontier class methodsFor: 'instance creation' stamp: 'ul 10/18/2018 01:42'!
frontierOn: aVersionInfo and: otherVersionInfo
	^ self frontierOnAll: { aVersionInfo. otherVersionInfo }! !

!MCFrontier class methodsFor: 'instance creation' stamp: 'stephaneducasse 2/4/2006 20:47'!
frontierOnAll: aCollection
	| remaining  allVersions |
	remaining := Bag new.
	allVersions := (aCollection gather: [:ea | ea withBreadthFirstAncestors]) asSet.
	allVersions do: [:ea | remaining addAll: ea ancestors].
	^self new frontier: aCollection bag: remaining! !

!MCFrontier methodsFor: 'accessing' stamp: 'bf 12/8/2011 19:35'!
frontier
	^frontier asArray	"not safe to hand out the frontier itself"! !

!MCFrontier methodsFor: 'initialization' stamp: 'avi 9/17/2005 22:11'!
frontier: f bag: remaining
	frontier := f asOrderedCollection.
	bag := remaining! !

!MCFrontier methodsFor: 'advancing' stamp: 'avi 9/17/2005 22:13'!
remove: aVersionInfo
	frontier remove: aVersionInfo.
	aVersionInfo ancestors  do:
		[ :ancestor |
			bag remove: ancestor.
			(bag occurrencesOf: ancestor) = 0
				ifTrue: [frontier add: ancestor]].
	^aVersionInfo! !

!MCFrontier methodsFor: 'advancing' stamp: 'avi 9/17/2005 22:02'!
removeAll: collection
	collection do: [ :n | self remove: n]! !

Object subclass: #MCMenuSpec
	instanceVariableNames: 'entry target selector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-UI'!

!MCMenuSpec commentStamp: 'tfel 6/12/2010 14:57' prior: 0!
A MCMenuSpec holds information to add menu entries to the monticello browser menus from external classes.
Required is the entry string (#entry), the call target and the selector be called.
An external class may use the MCWorkingCopyBrowser class>>addMenuSpec: method to add it's own menu entry to the monticello browser context menu.

Note that MCMenuSpecs are compared via their menu entry string and if multiple MCMenuSpecs are added to the MCWorkingCopyBrowser, the last one takes precedence.!

!MCMenuSpec methodsFor: 'comparing' stamp: 'tfel 6/12/2010 11:43'!
= aMCMenuSpec

	^ self class == aMCMenuSpec class and: [self entry = aMCMenuSpec entry].! !

!MCMenuSpec methodsFor: 'accessing' stamp: 'tfel 6/12/2010 11:38'!
entry

	^entry! !

!MCMenuSpec methodsFor: 'accessing' stamp: 'tfel 6/12/2010 11:38'!
entry: anObject

	entry := anObject! !

!MCMenuSpec methodsFor: 'comparing' stamp: 'tfel 6/12/2010 11:42'!
hash

	^ self entry hash! !

!MCMenuSpec methodsFor: 'accessing' stamp: 'tfel 6/12/2010 11:38'!
selector

	^selector! !

!MCMenuSpec methodsFor: 'accessing' stamp: 'tfel 6/12/2010 11:38'!
selector: anObject

	selector := anObject! !

!MCMenuSpec methodsFor: 'accessing' stamp: 'tfel 6/12/2010 11:38'!
target

	^target! !

!MCMenuSpec methodsFor: 'accessing' stamp: 'tfel 6/12/2010 11:38'!
target: anObject

	target := anObject! !

Object subclass: #MCMergeRecord
	instanceVariableNames: 'version packageSnapshot ancestorInfo ancestor ancestorSnapshot imagePatch mergePatch'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

!MCMergeRecord class methodsFor: 'instance creation' stamp: 'abc 2/13/2004 15:52'!
version: aVersion
	^ self basicNew initializeWithVersion: aVersion! !

!MCMergeRecord methodsFor: 'accessing' stamp: 'abc 2/13/2004 15:24'!
ancestorInfo
	^ ancestorInfo ifNil: [ancestorInfo := version info commonAncestorWith: version workingCopy ancestry]! !

!MCMergeRecord methodsFor: 'accessing' stamp: 'abc 2/13/2004 15:23'!
ancestorSnapshot
	^ ancestorSnapshot ifNil: [ancestorSnapshot := version workingCopy findSnapshotWithVersionInfo: self ancestorInfo]! !

!MCMergeRecord methodsFor: 'testing' stamp: 'nice 1/1/2010 21:52'!
imageIsClean
	| ancestors |
	ancestors := version workingCopy ancestors.
	^ ancestors size = 1
		and: [ancestors first = self ancestorInfo
		and: [self imagePatch isEmpty]]! !

!MCMergeRecord methodsFor: 'accessing' stamp: 'abc 2/13/2004 15:31'!
imagePatch
	^ imagePatch ifNil: [imagePatch := self packageSnapshot patchRelativeToBase: self ancestorSnapshot]! !

!MCMergeRecord methodsFor: 'initialize-release' stamp: 'abc 2/13/2004 15:21'!
initializeWithVersion: aVersion
	version := aVersion! !

!MCMergeRecord methodsFor: 'testing' stamp: 'abc 2/13/2004 17:14'!
isAncestorMerge
	^ version workingCopy ancestry hasAncestor: version info! !

!MCMergeRecord methodsFor: 'accessing' stamp: 'abc 2/13/2004 15:31'!
mergePatch
	^ mergePatch ifNil: [mergePatch := version snapshot patchRelativeToBase: self ancestorSnapshot]! !

!MCMergeRecord methodsFor: 'accessing' stamp: 'abc 2/13/2004 15:21'!
packageSnapshot
	^ packageSnapshot ifNil: [packageSnapshot := version package snapshot]! !

!MCMergeRecord methodsFor: 'operations' stamp: 'abc 2/13/2004 17:14'!
updateWorkingCopy
	self isAncestorMerge ifFalse:
		[self imageIsClean
			ifTrue: [version workingCopy loaded: version]
			ifFalse: [version workingCopy merged: version]]! !

!MCMergeRecord methodsFor: 'accessing' stamp: 'abc 2/13/2004 15:52'!
version
	^ version! !

Object subclass: #MCMerger
	instanceVariableNames: 'conflicts'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Merging'!

!MCMerger methodsFor: 'accessing' stamp: 'ab 6/2/2003 01:10'!
addConflictWithOperation: anOperation
	self conflicts add: (MCConflict operation: anOperation)! !

!MCMerger methodsFor: 'applying' stamp: 'ab 6/2/2003 01:01'!
applyTo: anObject
	self isMerged ifFalse: [self error: 'You must resolve all the conflicts first'].
	conflicts do: [:ea | ea applyTo: anObject]! !

!MCMerger methodsFor: 'accessing' stamp: 'ab 6/2/2003 01:02'!
conflicts
	^ conflicts ifNil: [conflicts := OrderedCollection new]! !

!MCMerger methodsFor: 'testing' stamp: 'ab 6/5/2003 19:09'!
isMerged
	^ self conflicts allSatisfy: [:ea | ea isResolved]! !

!MCMerger methodsFor: 'loading' stamp: 'avi 10/6/2004 15:19'!
load
	| loader |
	loader := MCPackageLoader new.
	loader provisions addAll: self provisions.
	self applyTo: loader.
	loader load! !

!MCMerger methodsFor: 'loading' stamp: 'stephaneducasse 2/4/2006 20:47'!
loadWithNameLike: baseName
	| loader |
	loader := MCPackageLoader new.
	loader provisions addAll: self provisions.
	self applyTo: loader.
	loader loadWithNameLike: baseName! !

!MCMerger methodsFor: 'accessing' stamp: 'ab 6/2/2003 01:11'!
mergedSnapshot
	^ MCPatcher apply: self to: self baseSnapshot! !

!MCMerger methodsFor: 'accessing' stamp: 'ab 7/18/2003 16:34'!
operations
	^ #()! !

!MCMerger methodsFor: 'accessing' stamp: 'avi 10/6/2004 15:19'!
provisions
	^ #()! !

MCMerger subclass: #MCThreeWayMerger
	instanceVariableNames: 'index operations provisions redundantAdds'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Merging'!

!MCThreeWayMerger class methodsFor: 'instance creation' stamp: 'avi 2/13/2004 01:53'!
base: aSnapshot patch: aPatch
	aPatch isEmpty ifTrue: [MCNoChangesException signal].
	^ self new
		addBaseSnapshot: aSnapshot;
		applyPatch: aPatch;
		yourself
		! !

!MCThreeWayMerger class methodsFor: 'instance creation' stamp: 'ab 6/2/2003 01:09'!
base: aSnapshot target: targetSnapshot ancestor: ancestorSnapshot
	^ self base: aSnapshot patch: (targetSnapshot patchRelativeToBase: ancestorSnapshot)! !

!MCThreeWayMerger class methodsFor: 'instance creation' stamp: 'avi 2/13/2004 01:52'!
new
	^ self basicNew initialize! !

!MCThreeWayMerger methodsFor: 'as yet unclassified' stamp: 'avi 10/6/2004 15:18'!
addBaseSnapshot: aSnapshot
	aSnapshot definitions do:
		[:ea |
		index add: ea.
		provisions addAll: ea provisions]! !

!MCThreeWayMerger methodsFor: 'as yet unclassified' stamp: 'ul 12/12/2009 14:09'!
addDefinition: aDefinition
	index
		definitionLike: aDefinition
		ifPresent: [:other |
			(self removalForDefinition: aDefinition)
				ifNotNil:
					[:op |
					self addOperation: (MCModification of: other to: aDefinition).
					self removeOperation: op.
					^ self].
			other = aDefinition
				ifFalse: [self addConflictWithOperation: (MCModification of: other to: aDefinition)]
				ifTrue: [self redundantAdds add: aDefinition]]
		ifAbsent: [self addOperation: (MCAddition of: aDefinition)]! !

!MCThreeWayMerger methodsFor: 'as yet unclassified' stamp: 'ab 6/2/2003 01:30'!
addOperation: anOperation
	self operations add: anOperation! !

!MCThreeWayMerger methodsFor: 'applying' stamp: 'avi 2/13/2004 01:52'!
applyPatch: aPatch
	aPatch applyTo: self! !

!MCThreeWayMerger methodsFor: 'applying' stamp: 'ab 6/2/2003 01:32'!
applyTo: anObject
	super applyTo: anObject.
	self operations do: [:ea | ea applyTo: anObject]! !

!MCThreeWayMerger methodsFor: 'as yet unclassified' stamp: 'avi 2/13/2004 01:49'!
baseSnapshot
	^ (MCSnapshot fromDefinitions: index definitions)! !

!MCThreeWayMerger methodsFor: 'initialize-release' stamp: 'avi 10/6/2004 15:18'!
initialize
	index := MCDefinitionIndex new.
	provisions := Set new! !

!MCThreeWayMerger methodsFor: 'as yet unclassified' stamp: 'avi 9/19/2005 02:22'!
modificationConflictForDefinition: aDefinition
	^ conflicts ifNotNil:
		[conflicts detect:
			[:ea | (ea definition isRevisionOf: aDefinition) and:
				[ea operation isModification]] ifNone: []]! !

!MCThreeWayMerger methodsFor: 'as yet unclassified' stamp: 'ab 6/2/2003 01:34'!
modifyDefinition: baseDefinition to: targetDefinition
	index
		definitionLike: baseDefinition
		ifPresent: [:other | other = baseDefinition
								ifTrue: [self addOperation: (MCModification of:  baseDefinition to: targetDefinition)]
								ifFalse: [other = targetDefinition
											ifFalse: [self addConflictWithOperation:
														(MCModification of: other to: targetDefinition)]]]
		ifAbsent: [self addConflictWithOperation: (MCAddition of: targetDefinition)]! !

!MCThreeWayMerger methodsFor: 'accessing' stamp: 'ab 6/2/2003 01:30'!
operations
	^ operations ifNil: [operations := OrderedCollection new]! !

!MCThreeWayMerger methodsFor: 'accessing' stamp: 'avi 10/6/2004 15:19'!
provisions
	^ provisions! !

!MCThreeWayMerger methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
redundantAdds
	^ redundantAdds ifNil: [redundantAdds := Set new]! !

!MCThreeWayMerger methodsFor: 'as yet unclassified' stamp: 'avi 9/19/2005 02:40'!
removalForDefinition: aDefinition
	^ operations ifNotNil:
		[operations
			detect: [:ea | (ea definition isRevisionOf: aDefinition) and: [ea isRemoval]]
			ifNone: []]! !

!MCThreeWayMerger methodsFor: 'as yet unclassified' stamp: 'avi 9/19/2005 02:40'!
removeConflict: aConflict
	conflicts remove: aConflict! !

!MCThreeWayMerger methodsFor: 'as yet unclassified' stamp: 'ul 12/12/2009 14:09'!
removeDefinition: aDefinition
	index
		definitionLike: aDefinition
		ifPresent: [:other | other = aDefinition
								ifTrue:
									[(self modificationConflictForDefinition: aDefinition)
										ifNotNil:
											[:c |
											self addOperation: c operation.
											self removeConflict: c.
											^ self]. 
									(self redundantAdds includes: aDefinition)
										ifFalse: [self addOperation: (MCRemoval of: aDefinition)]]
								ifFalse:
									[self addConflictWithOperation: (MCRemoval of: other)]]
		ifAbsent: []! !

!MCThreeWayMerger methodsFor: 'as yet unclassified' stamp: 'avi 9/19/2005 02:40'!
removeOperation: anOperation
	operations remove: anOperation! !

Object subclass: #MCOperationsList
	instanceVariableNames: 'operations'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Patching'!

!MCOperationsList class methodsFor: 'instance creation' stamp: 'avi 9/11/2004 15:50'!
operations: aCollection
	^ self basicNew initializeWithOperations: aCollection! !

!MCOperationsList methodsFor: 'ui' stamp: 'cmm 10/2/2013 17:39'!
browse
	(self browserClass items: operations) show! !

!MCOperationsList methodsFor: 'ui' stamp: 'cmm 10/1/2013 14:40'!
browserClass
	^ MCOperationsBrowser! !

!MCOperationsList methodsFor: 'intializing' stamp: 'stephaneducasse 2/4/2006 20:47'!
initializeWithOperations: aCollection
	operations := aCollection! !

!MCOperationsList methodsFor: 'querying' stamp: 'cwp 6/9/2003 11:53'!
isEmpty
	^ operations isEmpty! !

!MCOperationsList methodsFor: 'accessing' stamp: 'ab 5/13/2003 12:18'!
operations
	^ operations! !

MCOperationsList subclass: #MCPatch
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Patching'!

!MCPatch class methodsFor: 'instance creation' stamp: 'avi 9/11/2004 15:49'!
fromBase: baseSnapshot target: targetSnapshot
	^ self new initializeWithBase: baseSnapshot target: targetSnapshot! !

!MCPatch methodsFor: 'applying' stamp: 'ab 5/24/2003 16:12'!
applyTo: anObject
	operations do: [:ea | ea applyTo: anObject].
! !

!MCPatch methodsFor: 'ui' stamp: 'cmm 10/2/2013 17:06'!
browse
	(self browserClass forPatch: self) show! !

!MCPatch methodsFor: 'ui' stamp: 'cmm 10/1/2013 14:40'!
browserClass
	^ MCPatchBrowser! !

!MCPatch methodsFor: 'accessing' stamp: 'bf 1/22/2013 17:01'!
ignoring: ignoredOperations
	^ MCPatch operations: (operations difference: ignoredOperations)! !

!MCPatch methodsFor: 'initialize-release' stamp: 'ul 10/18/2018 01:37'!
initializeWithBase: baseSnapshot target: targetSnapshot
	| base target |	
	operations := OrderedCollection new.
	base := MCDefinitionIndex definitions: baseSnapshot definitions.
	target := MCDefinitionIndex definitions: targetSnapshot definitions.
	
	target definitionsDo:
		[:t |
		base
			definitionLike: t
			ifPresent: [:b | (b isSameRevisionAs: t) ifFalse: [operations add: (MCModification of: b to: t)]]
			ifAbsent: [operations add: (MCAddition of: t)]].
		
	base definitionsDo:
		[:b |
		target
			definitionLike: b
			ifPresent: [:t]
			ifAbsent: [operations add: (MCRemoval of: b)]]		! !

Object subclass: #MCPackage
	instanceVariableNames: 'name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Base'!

!MCPackage commentStamp: 'hjh 1/22/2013 22:22' prior: 0!
MCPackage uses PackageInfo to find out which methods and classes belong to a package.


!

!MCPackage class methodsFor: 'instance creation' stamp: 'ab 7/10/2003 01:17'!
named: aString
	^ self new name: aString! !

!MCPackage methodsFor: 'testing' stamp: 'ar 4/26/2005 21:57'!
= other
	^ other species = self species and: [other name sameAs: name]! !

!MCPackage methodsFor: 'testing' stamp: 'bf 4/19/2005 16:26'!
hasWorkingCopy
	^ MCWorkingCopy registry includesKey: self! !

!MCPackage methodsFor: 'testing' stamp: 'ar 4/26/2005 21:57'!
hash
	^ name asLowercase hash! !

!MCPackage methodsFor: 'environments' stamp: 'jr 2/28/2017 00:34'!
inEnvironment: anEnvironment
	"Answer a decorator for me that activates anEnvironment for certain operations."
	^ MCPackageInEnvironment decorating: self in: anEnvironment! !

!MCPackage methodsFor: 'accessing' stamp: 'ab 7/7/2003 00:57'!
name
	^ name! !

!MCPackage methodsFor: 'accessing' stamp: 'ab 7/7/2003 00:57'!
name: aString
	name := aString! !

!MCPackage methodsFor: 'accessing' stamp: 'jr 2/28/2017 02:50'!
packageInfo
	"Activate my working copy's environment so the PackageInfo is added to the
	correct EnvironmentInfo's packages."
	| getPackageInfo |
	getPackageInfo := [PackageInfo named: name].
	^ self hasWorkingCopy
		ifTrue: [self workingCopy withEnvironmentActiveDo: getPackageInfo]
		ifFalse: getPackageInfo! !

!MCPackage methodsFor: 'printing' stamp: 'nk 7/28/2003 13:30'!
printOn: aStream
	super printOn: aStream.
	aStream
		nextPut: $(;
		nextPutAll: name;
		nextPut: $)! !

!MCPackage methodsFor: 'input/output' stamp: 'ul 3/17/2011 01:29'!
snapshot
	| packageInfo definitions categories |
	packageInfo := self packageInfo.
	definitions := OrderedCollection new.
	categories := packageInfo systemCategories.
	categories isEmpty ifFalse: [ definitions add: (MCOrganizationDefinition categories: categories) ].
	CurrentReadOnlySourceFiles cacheDuring: [
		packageInfo methods do: [:ea | definitions add: ea asMethodDefinition] displayingProgress: 'Snapshotting methods...'.
		(packageInfo respondsTo: #overriddenMethods) ifTrue:
			[packageInfo overriddenMethods
				do: [:ea | definitions add:
						(packageInfo changeRecordForOverriddenMethod: ea) asMethodDefinition]
				displayingProgress: 'Searching for overrides...'].
		packageInfo classes do: [:ea | definitions addAll: ea classDefinitions] displayingProgress: 'Snapshotting classes...' ].
	(packageInfo respondsTo: #hasPreamble) ifTrue: [
		packageInfo hasPreamble ifTrue: [definitions add: (MCPreambleDefinition from: packageInfo)].
		packageInfo hasPostscript ifTrue: [definitions add: (MCPostscriptDefinition from: packageInfo)].
		packageInfo hasPreambleOfRemoval ifTrue: [definitions add: (MCRemovalPreambleDefinition from: packageInfo)].
		packageInfo hasPostscriptOfRemoval ifTrue: [definitions add: (MCRemovalPostscriptDefinition from: packageInfo)]]. 
	^ MCSnapshot fromDefinitions: definitions
! !

!MCPackage methodsFor: 'input/output' stamp: 'ab 7/10/2003 01:13'!
storeOn: aStream
	aStream
		nextPutAll: 'MCPackage';
		space; nextPutAll: 'named: '; store: name.! !

!MCPackage methodsFor: 'input/output' stamp: 'cwp 11/13/2003 13:32'!
unload
	^ self workingCopy unload! !

!MCPackage methodsFor: 'accessing' stamp: 'cwp 11/13/2003 13:33'!
workingCopy
	^ MCWorkingCopy forPackage: self.! !

Object subclass: #MCPackageCache
	instanceVariableNames: 'sorter fileNames'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Repositories'!

!MCPackageCache class methodsFor: 'instance creation' stamp: 'avi 1/22/2004 18:12'!
new
	^ self basicNew initialize! !

!MCPackageCache methodsFor: 'initialize-release' stamp: 'avi 1/22/2004 18:12'!
initialize
	sorter := MCVersionSorter new.
	fileNames := Dictionary new.! !

!MCPackageCache methodsFor: 'recording' stamp: 'avi 1/22/2004 18:25'!
recordVersionInfo: aVersionInfo forFileNamed: aString
	Transcript cr; show: aString.
	fileNames at: aVersionInfo put: aString.
	sorter addVersionInfo: aVersionInfo! !

!MCPackageCache methodsFor: 'accessing' stamp: 'avi 1/22/2004 18:21'!
versionInfos
	^ sorter sortedVersionInfos ! !

Object subclass: #MCPackageLoader
	instanceVariableNames: 'requirements unloadableDefinitions obsoletions additions removals errorDefinitions provisions methodAdditions preamble'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Loading'!

!MCPackageLoader commentStamp: 'rej 2/26/2007 07:35' prior: 0!
A MCPackageLoader is responsible for loading packages.  It gets used by VersionLoader, so it is eventually responsible for loading everything.

Instance Variables
	additions:		<Definitions>  Definitions that need to be added
	errorDefinitions:		<Object>
	obsoletions:		<Object>
	provisions:		<Object>
	removals:		<Object>
	requirements:		<Object>
	unloadableDefinitions:		<Object>
	methodAdditions  <MethodAdditions> MethodDefinitions corresponding to the Definitions in "additions" that have been added so far.

additions
	- xxxxx

errorDefinitions
	- xxxxx

obsoletions
	- xxxxx

provisions
	- xxxxx

removals
	- xxxxx

requirements
	- xxxxx

unloadableDefinitions
	- xxxxx
!

!MCPackageLoader class methodsFor: 'loading' stamp: 'ab 7/6/2003 23:30'!
installSnapshot: aSnapshot
	self new
		installSnapshot: aSnapshot;
		load! !

!MCPackageLoader class methodsFor: 'instance creation' stamp: 'avi 9/13/2004 18:03'!
new
	^ self basicNew initialize! !

!MCPackageLoader class methodsFor: 'unloading' stamp: 'bf 12/5/2004 12:00'!
unloadPackage: aPackage
	self new
		unloadPackage: aPackage;
		loadWithNameLike: aPackage name, '-unload'! !

!MCPackageLoader class methodsFor: 'loading' stamp: 'ab 7/7/2003 12:11'!
updatePackage: aPackage withSnapshot: aSnapshot
	self new
		updatePackage: aPackage withSnapshot: aSnapshot;
		load! !

!MCPackageLoader methodsFor: 'patch ops' stamp: 'ab 5/24/2003 16:13'!
addDefinition: aDefinition
	additions add: aDefinition! !

!MCPackageLoader methodsFor: 'private' stamp: 'nice 4/8/2017 16:48'!
analyze
	| sorter |
	sorter := self sorterForItems: additions.
	additions := sorter orderedItems.
	requirements := sorter externalRequirements.
	unloadableDefinitions := sorter itemsWithMissingRequirements sorted.
	
	self forgetSuperfluousMethodRemovals.
	sorter := self sorterForItems: removals.
	removals := sorter orderedItems reversed.! !

!MCPackageLoader methodsFor: 'public' stamp: 'bf 5/11/2012 13:23'!
appendToPreamble: aString
	preamble
		ifNil: [preamble := aString]
		ifNotNil: [preamble := preamble, aString].
! !

!MCPackageLoader methodsFor: 'private' stamp: 'cmm 10/3/2013 14:22'!
basicLoad
	"Load the contents of some package. This is the core loading method
	in Monticello. Be wary about modifying it unless you understand the details
	and dependencies of the various entities being modified."
	| pkgName |
	errorDefinitions := OrderedCollection new.
	"Obviously this isn't the package name but we don't have anything else
	to use here. ChangeSet current name will generally work since a CS is 
	usually installed prior to installation."
	pkgName := ChangeSet current name.
	preamble ifNotNil: [ChangeSet current preambleString: (self preambleAsCommentNamed: pkgName)].

RecentMessages default suspendWhile: [
	[CurrentReadOnlySourceFiles cacheDuring: [[
	"Pass 1: Load everything but the methods,  which are collected in methodAdditions."
	additions do: [:ea | 
		ea isMethodDefinition 
			ifTrue:[methodAdditions add: ea asMethodAddition]
			ifFalse:[[ea load]on: Error do: [errorDefinitions add: ea]].
	] displayingProgress: 'Reshaping ', pkgName.

	"Try again any delayed definitions"
	self shouldWarnAboutErrors ifTrue: [self warnAboutErrors].
	errorDefinitions do: [:ea | ea load] 
		displayingProgress: 'Reloading ', pkgName.

	"Pass 2: We compile new / changed methods"
	methodAdditions do:[:ea| ea createCompiledMethod] 
		displayingProgress: 'Compiling ', pkgName.

	'Installing ', pkgName displayProgressFrom: 0 to: 2 during:[:bar|
		"There is no progress *during* installation since a progress bar update
		will redraw the world and potentially call methods that we're just trying to install."
		bar value: 1.

		"Pass 3: Install the new / changed methods
		(this is a separate pass to allow compiler changes to be loaded)"
		methodAdditions do:[:ea| ea installMethod].

		"Pass 4: Remove the obsolete methods"
		removals do:[:ea| ea unload].
	].

	"Finally, notify observers for the method additions"
	methodAdditions do: [:each | each notifyObservers] 
		"the message is fake but actually telling people how much time we spend
		in the notifications is embarrassing so lie instead"
		displayingProgress: 'Installing ', pkgName.

	additions do: [:ea | ea postloadOver: (self obsoletionFor: ea)] 
		displayingProgress: 'Initializing ', pkgName.

	] on: InMidstOfFileinNotification do: [:n | n resume: true]
	]] ensure: [self flushChangesFile]
]! !

!MCPackageLoader methodsFor: 'private' stamp: 'ab 5/25/2003 01:24'!
dependencyWarning
	^ String streamContents:
		[:s |
		s nextPutAll: 'This package depends on the following classes:'; cr.
		requirements do: [:ea | s space; space; nextPutAll: ea; cr].
		s nextPutAll: 'You must resolve these dependencies before you will be able to load these definitions: '; cr.
		unloadableDefinitions do: [:ea | s space; space; nextPutAll: ea summary; cr]] ! !

!MCPackageLoader methodsFor: 'private' stamp: 'avi 1/24/2004 17:44'!
errorDefinitionWarning
	^ String streamContents:
		[:s |
		s nextPutAll: 'The following definitions had errors while loading.  Press Proceed to try to load them again (they may work on a second pass):'; cr.
		errorDefinitions do: [:ea | s space; space; nextPutAll: ea summary; cr]] ! !

!MCPackageLoader methodsFor: 'private' stamp: 'cwp 11/13/2003 02:01'!
flushChangesFile
	"The changes file is second in the SourceFiles array"

	(SourceFiles at: 2) flush! !

!MCPackageLoader methodsFor: 'private' stamp: 'nice 4/8/2017 16:34'!
forgetSuperfluousMethodRemovals
	|  removedClasses |
	removedClasses := (removals select: #isClassDefinition) collect: #actualClass.
	removedClasses addAll: (removedClasses collect: #class).
	removals := removals reject: [:e | e isMethodDefinition and: [removedClasses includes: e actualClass]]! !

!MCPackageLoader methodsFor: 'private' stamp: 'abc 2/26/2007 21:39'!
initialize
	additions := OrderedCollection new.
	removals := OrderedCollection new.
	obsoletions := Dictionary new.
	methodAdditions := OrderedCollection new. 
! !

!MCPackageLoader methodsFor: 'public' stamp: 'ab 7/6/2003 23:30'!
installSnapshot: aSnapshot
	| patch |
	patch := aSnapshot patchRelativeToBase: MCSnapshot empty.
	patch applyTo: self.
! !

!MCPackageLoader methodsFor: 'public' stamp: 'ab 8/24/2003 01:03'!
load
	self analyze.
	unloadableDefinitions isEmpty ifFalse: [self warnAboutDependencies].
	self useNewChangeSetDuring: [self basicLoad]! !

!MCPackageLoader methodsFor: 'private' stamp: 'ar 8/30/2009 20:25'!
loadClassDefinition: aDefinition
	[aDefinition isClassDefinition ifTrue:[aDefinition load]] on: Error do: [errorDefinitions add: aDefinition].! !

!MCPackageLoader methodsFor: 'public' stamp: 'nk 2/23/2005 07:51'!
loadWithName: baseName
	self analyze.
	unloadableDefinitions isEmpty ifFalse: [self warnAboutDependencies].
	self useChangeSetNamed: baseName during: [self basicLoad]! !

!MCPackageLoader methodsFor: 'public' stamp: 'nk 8/30/2004 08:39'!
loadWithNameLike: baseName
	self analyze.
	unloadableDefinitions isEmpty ifFalse: [self warnAboutDependencies].
	self useNewChangeSetNamedLike: baseName during: [self basicLoad]! !

!MCPackageLoader methodsFor: 'patch ops' stamp: 'avi 2/17/2004 13:14'!
modifyDefinition: old to: new
	self addDefinition: new.
	obsoletions at: new put: old.! !

!MCPackageLoader methodsFor: 'private' stamp: 'avi 2/17/2004 13:15'!
obsoletionFor: aDefinition
	^ obsoletions at: aDefinition ifAbsent: [nil]! !

!MCPackageLoader methodsFor: 'private' stamp: 'ab 5/25/2003 01:19'!
orderDefinitionsForLoading: aCollection
	^ (self sorterForItems: aCollection) orderedItems! !

!MCPackageLoader methodsFor: 'private' stamp: 'ab 5/24/2003 16:52'!
orderedAdditions
	^ additions! !

!MCPackageLoader methodsFor: 'private' stamp: 'bf 5/11/2012 13:22'!
preambleAsCommentNamed: pkgName
	^
'"Changeset:	{1}
Date:	{2}
Author:	(generated by MC)

{3}
"' format: {pkgName. Date today. preamble copyReplaceAll: '"' with: ''''''}! !

!MCPackageLoader methodsFor: 'private' stamp: 'jr 10/19/2016 17:05'!
provisions
	^ provisions ifNil: [provisions := Set withAll: Environment current provisions] ! !

!MCPackageLoader methodsFor: 'patch ops' stamp: 'ab 5/24/2003 16:14'!
removeDefinition: aDefinition
	removals add: aDefinition! !

!MCPackageLoader methodsFor: 'private' stamp: 'avi 1/25/2004 13:32'!
shouldWarnAboutErrors
	^ errorDefinitions isEmpty not and: [false "should make this a preference"]! !

!MCPackageLoader methodsFor: 'private' stamp: 'avi 10/7/2004 22:49'!
sorterForItems: aCollection
	| sorter |
	sorter := MCDependencySorter items: aCollection.
	sorter addExternalProvisions: self provisions.
	^ sorter! !

!MCPackageLoader methodsFor: 'private' stamp: 'rej 2/26/2007 10:41'!
tryToLoad: aDefinition
	[aDefinition addMethodAdditionTo: methodAdditions] on: Error do: [errorDefinitions add: aDefinition].! !

!MCPackageLoader methodsFor: 'public' stamp: 'avi 10/5/2003 11:09'!
unloadPackage: aPackage
	self updatePackage: aPackage withSnapshot: MCSnapshot empty! !

!MCPackageLoader methodsFor: 'public' stamp: 'avi 9/1/2004 01:09'!
updatePackage: aPackage withSnapshot: aSnapshot
	|  patch packageSnap |
	packageSnap := aPackage snapshot.
	patch := aSnapshot patchRelativeToBase: packageSnap.
	patch applyTo: self.
	packageSnap definitions do: [:ea | self provisions addAll: ea provisions]
! !

!MCPackageLoader methodsFor: 'private' stamp: 'pre 8/27/2019 11:38'!
useChangeSetNamed: baseName during: aBlock
	"Use the named change set, or create one with the given name."
	| changeHolder oldChanges newChanges |
	changeHolder := (ChangeSet respondsTo: #newChanges:)
						ifTrue: [ChangeSet]
						ifFalse: [Smalltalk].
	oldChanges := (ChangeSet respondsTo: #current)
						ifTrue: [ChangeSet current]
						ifFalse: [Smalltalk changes].

	newChanges := (ChangeSet named: baseName) ifNil: [ ChangeSet new name: baseName ].
	changeHolder newChanges: newChanges.
	aBlock ensure: [changeHolder newChanges: oldChanges].
! !

!MCPackageLoader methodsFor: 'private' stamp: 'nk 8/30/2004 08:38'!
useNewChangeSetDuring: aBlock
	^self useNewChangeSetNamedLike: 'MC' during: aBlock! !

!MCPackageLoader methodsFor: 'private' stamp: 'nk 2/23/2005 07:50'!
useNewChangeSetNamedLike: baseName during: aBlock
	^self useChangeSetNamed: (ChangeSet uniqueNameLike: baseName) during: aBlock! !

!MCPackageLoader methodsFor: 'private' stamp: 'ab 5/25/2003 01:22'!
warnAboutDependencies 
	self notify: self dependencyWarning! !

!MCPackageLoader methodsFor: 'private' stamp: 'avi 1/24/2004 17:42'!
warnAboutErrors
	self notify: self errorDefinitionWarning.
! !

Object subclass: #MCPackageManager
	instanceVariableNames: 'package modified'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

MCPackageManager class
	instanceVariableNames: 'registry'!

MCPackageManager class
	instanceVariableNames: 'registry'!

!MCPackageManager class methodsFor: 'private' stamp: 'ab 3/31/2003 20:45'!
allManagers
	^ self registry values! !

!MCPackageManager class methodsFor: 'system changes' stamp: 'ar 4/27/2005 14:28'!
classModified: anEvent
	self managersForClass: anEvent item do:[:mgr| mgr modified: true].! !

!MCPackageManager class methodsFor: 'system changes' stamp: 'ar 4/26/2005 21:31'!
classMoved: anEvent
	self classModified: anEvent.
	self managersForCategory: anEvent oldCategory do:[:mgr| mgr modified: true].! !

!MCPackageManager class methodsFor: 'system changes' stamp: 'ar 4/26/2005 21:31'!
classRemoved: anEvent
	self classModified: anEvent! !

!MCPackageManager class methodsFor: 'instance creation' stamp: 'ab 7/7/2003 16:28'!
forPackage: aPackage
	^ self registry at: aPackage ifAbsent:
		[|mgr|
		mgr := self new initializeWithPackage: aPackage.
		self registry at: aPackage put: mgr.
		self changed: #allManagers.
		mgr]! !

!MCPackageManager class methodsFor: 'instance creation' stamp: 'cmm 7/26/2014 17:29'!
forPackageNamed: aString 
	^ self registry detect:
		[ : each | each packageName = aString ]! !

!MCPackageManager class methodsFor: 'class initialization' stamp: 'ar 4/27/2005 02:09'!
initialize
	"Remove this later"
	Smalltalk at: #SystemChangeNotifier ifPresent:[:cls|
		(cls uniqueInstance) noMoreNotificationsFor: self.
	].! !

!MCPackageManager class methodsFor: 'system changes' stamp: 'md 3/14/2006 23:11'!
managersForCategory: aSystemCategory do: aBlock
	"Got to be careful here - we might get method categories where capitalization is problematic."
	| cat foundOne index |
	foundOne := false.
	cat := aSystemCategory ifNil:[^nil]. "yes this happens; for example in eToy projects"
	"first ask PackageInfos, their package name might not match the category"
	self registry do: [:mgr | 
		(mgr packageInfo includesSystemCategory: aSystemCategory)	ifTrue: [
			aBlock value: mgr.
			foundOne := true.
		]
	].
	foundOne ifTrue: [^self].
	["Loop over categories until we found a matching one"
	self registry at: (MCPackage named: cat) ifPresent:[:mgr|
		aBlock value: mgr.
		foundOne := true.
	].
	index := cat lastIndexOf: $-.
	index > 0]whileTrue:[
		"Step up to next level package"
		cat := cat copyFrom: 1 to: index-1.
	].
	foundOne ifFalse:[
		"Create a new (but only top-level)"
		aBlock value: (MCWorkingCopy forPackage: (MCPackage named: (aSystemCategory copyUpTo: $-) capitalized)).
	].! !

!MCPackageManager class methodsFor: 'system changes' stamp: 'ar 4/27/2005 14:11'!
managersForClass: aClass category: methodCategory do: aBlock
	(methodCategory isEmptyOrNil or:[methodCategory first ~= $*]) ifTrue:[
		"Not an extension method"
		^self managersForClass: aClass do: aBlock.
	].
	self managersForCategory: methodCategory allButFirst do: aBlock.! !

!MCPackageManager class methodsFor: 'system changes' stamp: 'bf 5/20/2005 16:50'!
managersForClass: aClass do: aBlock

	self registry do: [:mgr |
		(mgr packageInfo includesClass: aClass)
			ifTrue: [aBlock value: mgr]]! !

!MCPackageManager class methodsFor: 'system changes' stamp: 'ar 4/26/2005 21:40'!
managersForClass: aClass selector: aSelector do: aBlock
	^self managersForClass: aClass category: (aClass organization categoryOfElement: aSelector) do: aBlock! !

!MCPackageManager class methodsFor: 'system changes' stamp: 'ar 4/26/2005 21:40'!
methodModified: anEvent
	^self managersForClass: anEvent itemClass selector: anEvent itemSelector do:[:mgr| mgr modified: true].! !

!MCPackageManager class methodsFor: 'system changes' stamp: 'ar 4/26/2005 21:40'!
methodMoved: anEvent
	self managersForClass: anEvent itemClass category: anEvent oldCategory do:[:mgr| mgr modified: true].
	self methodModified: anEvent.! !

!MCPackageManager class methodsFor: 'system changes' stamp: 'ar 4/26/2005 22:12'!
methodRemoved: anEvent
	self managersForClass: anEvent itemClass category: anEvent itemProtocol do:[:mgr| mgr modified: true].
! !

!MCPackageManager class methodsFor: 'system changes' stamp: 'eem 7/3/2009 11:15'!
registerForNotifications
	Smalltalk
		at: #SystemChangeNotifier
		ifPresent:
			[:cls|
			cls uniqueInstance noMoreNotificationsFor: self.
			self reregisterForNotificationsWith: cls uniqueInstance]! !

!MCPackageManager class methodsFor: 'private' stamp: 'ab 5/9/2003 12:59'!
registry
	^ registry ifNil: [registry := Dictionary new]! !

!MCPackageManager class methodsFor: 'system changes' stamp: 'eem 7/3/2009 11:14'!
reregisterForNotificationsWith: aSystemChangeNotifier
	aSystemChangeNotifier
		notify: self ofSystemChangesOfItem: #class change: #Added using: #classModified:;
		notify: self ofSystemChangesOfItem: #class change: #Modified using: #classModified:;
		notify: self ofSystemChangesOfItem: #class change: #Renamed using: #classModified:;
		notify: self ofSystemChangesOfItem: #class change: #Commented using: #classModified:;
		notify: self ofSystemChangesOfItem: #class change: #Recategorized using: #classMoved:;
		notify: self ofSystemChangesOfItem: #class change: #Removed using: #classRemoved:;
		notify: self ofSystemChangesOfItem: #method change: #Added using: #methodModified:;
		notify: self ofSystemChangesOfItem: #method change: #Modified using: #methodModified:;
		notify: self ofSystemChangesOfItem: #method change: #Recategorized using: #methodMoved:;
		notify: self ofSystemChangesOfItem: #method change: #Removed using: #methodRemoved:! !

!MCPackageManager methodsFor: 'system changes' stamp: 'ar 4/26/2005 22:17'!
classModified: anEvent
	"obsolete - remove this later"! !

!MCPackageManager methodsFor: 'system changes' stamp: 'ar 4/26/2005 22:17'!
classMoved: anEvent
	"obsolete - remove this later"! !

!MCPackageManager methodsFor: 'system changes' stamp: 'ar 4/26/2005 22:17'!
classRemoved: anEvent
	"obsolete - remove this later"! !

!MCPackageManager methodsFor: 'initialize-release' stamp: 'avi 3/4/2004 16:43'!
initialize
	modified := false.
	self registerForNotifications.! !

!MCPackageManager methodsFor: 'initialize-release' stamp: 'jr 2/28/2017 00:36'!
initializeWithPackage: aPackage
	package := aPackage inEnvironment: Environment current.
	self initialize.! !

!MCPackageManager methodsFor: 'system changes' stamp: 'ar 4/26/2005 22:17'!
methodModified: anEvent
	"obsolete - remove this later"! !

!MCPackageManager methodsFor: 'system changes' stamp: 'ar 4/26/2005 22:17'!
methodMoved: anEvent 
	"obsolete - remove this later"! !

!MCPackageManager methodsFor: 'system changes' stamp: 'ar 4/26/2005 22:17'!
methodRemoved: anEvent
	"obsolete - remove this later"! !

!MCPackageManager methodsFor: 'accessing' stamp: 'cwp 11/13/2003 14:12'!
modified
	^ modified! !

!MCPackageManager methodsFor: 'accessing' stamp: 'ul 12/12/2009 14:10'!
modified: aBoolean
     modified = aBoolean ifTrue: [^ self].
	modified := aBoolean.
	self changed: #modified.
	
	modified ifFalse:
		[(((Smalltalk classNamed: 'SmalltalkImage') ifNotNil: [:si | si current]) ifNil: [Smalltalk])
			logChange: '"', self packageName, '"'].! !

!MCPackageManager methodsFor: 'accessing' stamp: 'ab 7/7/2003 16:47'!
package
	^ package! !

!MCPackageManager methodsFor: 'accessing' stamp: 'ab 7/7/2003 13:33'!
packageInfo
	^ package packageInfo! !

!MCPackageManager methodsFor: 'accessing' stamp: 'ab 7/7/2003 12:18'!
packageName
	^ package name! !

!MCPackageManager methodsFor: 'accessing' stamp: 'ab 7/5/2003 23:18'!
packageNameWithStar
	^ modified
		ifTrue: ['* ', self packageName]
		ifFalse: [self packageName]! !

!MCPackageManager methodsFor: 'system changes' stamp: 'ar 4/26/2005 22:17'!
registerForNotifications
	"obsolete - remove this later"! !

!MCPackageManager methodsFor: 'system changes' stamp: 'ar 4/26/2005 22:17'!
registerForNotificationsFrom: aNotifier
	"obsolete - remove this later"! !

!MCPackageManager methodsFor: 'system changes' stamp: 'ar 4/26/2005 22:17'!
systemChange: anEvent
	"obsolete - remove this later"! !

!MCPackageManager methodsFor: 'operations' stamp: 'cmm 6/8/2011 13:39'!
unregister
	self class registry
		removeKey: package
		ifAbsent: [ "Should not complain when trying to clean up." ].
	self class changed: #allManagers! !

!MCPackageManager methodsFor: 'operations' stamp: 'dtl 1/11/2011 23:30'!
unregisterSubpackages
	(self class registry keys
		select: [:p | self packageName , '-*' match: p name])
		do: [:k | self class registry removeKey: k]! !

!MCPackageManager methodsFor: 'system changes' stamp: 'fbs 9/10/2013 08:32'!
update: aSymbol
	InMidstOfFileinNotification signal ifFalse: [
	[((aSymbol = #recentMethodSubmissions)
		and: [self packageInfo
				includesMethodReference: RecentMessages default mostRecent])
					ifTrue: [self modified: true]]
		on: Error do: []]! !

MCPackageManager subclass: #MCWorkingCopy
	instanceVariableNames: 'versionInfo ancestry counter repositoryGroup requiredPackages environment'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

!MCWorkingCopy class methodsFor: 'operations' stamp: 'avi 9/13/2004 18:00'!
adoptVersionInfoFrom: anInstaller
	|viCache|
	viCache := Dictionary new.
	anInstaller versionInfo keysAndValuesDo: [:packageName :info |
		(self forPackage: (MCPackage named: packageName))
			versionInfo: (self infoFromDictionary:  info cache: viCache)].
	[anInstaller clearVersionInfo] on: Error do: ["backwards compat"].! !

!MCWorkingCopy class methodsFor: 'operations' stamp: 'avi 2/17/2004 01:23'!
ancestorsFromArray: anArray cache: aDictionary
	^ anArray ifNotNil: [anArray collect: [:dict | self infoFromDictionary: dict cache: aDictionary]]! !

!MCWorkingCopy class methodsFor: 'operations' stamp: 'bp 8/5/2014 09:29'!
checkModified: thorough
	"Verify that all working copies with a modified flag are really modified, by comparing them to their stored snapshot. If the 'thorough' argument is true, check all packages, even unmodified ones."
	"MCWorkingCopy checkModified: true"
	| workingCopies |
	workingCopies := self allManagers.
	thorough ifFalse: [
		workingCopies := workingCopies select: [:wc | wc modified]].
	workingCopies do: [:wc | wc checkModified]
		displayingProgress: [:wc | 'Checking package {1} for changes ...' translated format: {wc packageName}]! !

!MCWorkingCopy class methodsFor: 'class initialization' stamp: 'cmm 1/28/2014 11:10'!
cleanUp 
	self
"		 stubAllAncestry ;"
		 flushObsoletePackageInfos! !

!MCWorkingCopy class methodsFor: 'cleanup' stamp: 'cmm 1/22/2014 21:19'!
flushObsoletePackageInfos
	PackageOrganizer default flushObsoletePackages: [ : eachPackageInfo | eachPackageInfo class isObsolete ]! !

!MCWorkingCopy class methodsFor: 'operations' stamp: 'bf 4/19/2010 00:39'!
infoFromDictionary: aDictionary cache: cache
	| id |
	id := (aDictionary at: #id) asString.
	^ cache at: id ifAbsentPut:
		[MCVersionInfo
			name: (aDictionary at: #name ifAbsent: [''])
			id: (UUID fromString: id)
			message: (aDictionary at: #message ifAbsent: [''])
			date: ([Date fromString: (aDictionary at: #date)] ifError: [nil])
			time: ([Time fromString: (aDictionary at: #time)] ifError: [nil])
			author: (aDictionary at: #author ifAbsent: [''])
			ancestors: (self ancestorsFromArray: (aDictionary at: #ancestors ifAbsent: []) cache: cache)
			stepChildren: (self ancestorsFromArray: (aDictionary at: #stepChildren ifAbsent: []) cache: cache)]! !

!MCWorkingCopy class methodsFor: 'class initialization' stamp: 'eem 7/3/2009 11:21'!
initialize
	Smalltalk 
		at: #MczInstaller
		ifPresent: [:installer | self adoptVersionInfoFrom: installer].
	self updateInstVars.
	"Temporary conversion code -- remove later"
	registry ifNotNil:[registry rehash]. "changed #="
	self allInstancesDo:[:each| "moved notifications"
		Smalltalk at: #SystemChangeNotifier ifPresent:[:cls|
			cls uniqueInstance noMoreNotificationsFor: each.
		].
	].
	self registerForNotifications.
	Smalltalk addToStartUpList: self! !

!MCWorkingCopy class methodsFor: 'system startup' stamp: 'eem 7/3/2009 11:34'!
startUp: resuming
	"Ensure Monticello is receiving system change notifications."

	resuming ifTrue:
		[Smalltalk
			at: #SystemChangeNotifier
			ifPresent: [:scn| self reregisterForNotificationsWith: scn uniqueInstance]]
! !

!MCWorkingCopy class methodsFor: 'cleanup' stamp: 'cmm 8/14/2013 13:20'!
stubAllAncestry
	"Replace VersionInfo instances which are more than 10 versions behind with a MCInfoProxy which will dynamically re-retrieve them from the current repository if necessary."
	MCWorkingCopy allManagers do: [ : each | each stubAncestry ]! !

!MCWorkingCopy class methodsFor: 'system maintenance' stamp: 'avi 2/17/2004 02:36'!
updateInstVars
	self allInstances do: [:ea | ea updateInstVars]! !

!MCWorkingCopy methodsFor: 'operations' stamp: 'avi 2/13/2004 01:07'!
adopt: aVersion
	ancestry addAncestor: aVersion info.
	self changed.! !

!MCWorkingCopy methodsFor: 'accessing' stamp: 'avi 2/13/2004 01:07'!
ancestors
	^ ancestry ancestors! !

!MCWorkingCopy methodsFor: 'accessing' stamp: 'avi 2/13/2004 01:13'!
ancestry
	^ ancestry! !

!MCWorkingCopy methodsFor: 'operations' stamp: 'ul 12/12/2009 14:09'!
backportChangesTo: aVersionInfo
	| baseVersion fullPatch currentVersionInfo currentVersion newSnapshot newAncestry |
	currentVersionInfo := self currentVersionInfo.
	baseVersion := self repositoryGroup versionWithInfo: aVersionInfo.
	currentVersion := self repositoryGroup versionWithInfo: currentVersionInfo.
	fullPatch := currentVersion snapshot patchRelativeToBase: baseVersion snapshot.
	(MCChangeSelectionRequest new
		patch: fullPatch;
		label: 'Changes to Backport';
		signal ) ifNotNil:
		[:partialPatch |
		newSnapshot := MCPatcher apply: partialPatch to: baseVersion snapshot.
		newAncestry := MCWorkingAncestry new
							addAncestor: aVersionInfo;
							addStepChild: currentVersionInfo;
							yourself.
		MCPackageLoader updatePackage: package withSnapshot: newSnapshot.
		ancestry := newAncestry.
		self modified: false; modified: true]! !

!MCWorkingCopy methodsFor: 'ui' stamp: 'mt 8/14/2016 12:51'!
browse

	(MCSnapshotBrowser forSnapshot: self package snapshot)
			label: 'Snapshot Browser: ', self packageName;
			show.! !

!MCWorkingCopy methodsFor: 'operations' stamp: 'cmm 4/6/2019 15:42'!
changesRelativeToRepository: aRepository 
	| ancestorVersion ancestorSnapshot |
	ancestorVersion := aRepository withCache
		closestAncestorVersionFor: ancestry
		ifNone: [ nil ].
	ancestorSnapshot := ancestorVersion
		ifNil: [ MCSnapshot empty ]
		ifNotNil: [ ancestorVersion snapshot ].
	^ package snapshot patchRelativeToBase: ancestorSnapshot! !

!MCWorkingCopy methodsFor: 'operations' stamp: 'bf 8/14/2013 22:30'!
checkModified
	self ancestors size = 1
		ifFalse: [
			self modified: true]
		ifTrue: [
			| cleanSnapshot currentSnapshot modification |
			cleanSnapshot := self findSnapshotWithVersionInfo: self ancestors first.
			currentSnapshot := self package snapshot.
			modification := currentSnapshot patchRelativeToBase: cleanSnapshot.
			self modified: modification isEmpty not].
! !

!MCWorkingCopy methodsFor: 'accessing' stamp: 'avi 1/19/2004 16:30'!
clearRequiredPackages
	requiredPackages := nil! !

!MCWorkingCopy methodsFor: 'accessing' stamp: 'avi 2/13/2004 20:01'!
currentVersionInfo
	^ (self needsSaving or: [ancestry ancestors isEmpty])
		ifTrue: [self newVersion info]
		ifFalse: [ancestry ancestors first]! !

!MCWorkingCopy methodsFor: 'accessing' stamp: 'bp 11/21/2010 18:33'!
description
	^ self packageNameWithStar, ' (', (ancestry ancestorStringWithout: self packageName), ')'! !

!MCWorkingCopy methodsFor: 'accessing' stamp: 'jr 2/2/2017 18:01'!
environment
	^ environment ifNil: [Smalltalk globals]! !

!MCWorkingCopy methodsFor: 'accessing' stamp: 'jr 2/28/2017 00:35'!
environment: anEnvironment
	"Anything that is loaded to me should go into anEnvironment from now on."
	environment := anEnvironment.
	package := package inEnvironment: anEnvironment.! !

!MCWorkingCopy methodsFor: 'private' stamp: 'nice 12/25/2009 14:39'!
findSnapshotWithVersionInfo: aVersionInfo
	^ aVersionInfo
		ifNil: [MCSnapshot empty]
		ifNotNil: [(self repositoryGroup versionWithInfo: aVersionInfo)
			ifNil: [MCSnapshot empty]
			ifNotNil: [:aVersion | aVersion snapshot]]! !

!MCWorkingCopy methodsFor: 'private' stamp: 'avi 2/13/2004 01:07'!
initialize
	super initialize.
	ancestry := MCWorkingAncestry new! !

!MCWorkingCopy methodsFor: 'operations' stamp: 'avi 2/13/2004 01:06'!
loaded: aVersion
	ancestry := MCWorkingAncestry new addAncestor: aVersion info.
	requiredPackages := OrderedCollection withAll: (aVersion dependencies collect: [:ea | ea package]).
	self modified: false.
	self changed! !

!MCWorkingCopy methodsFor: 'operations' stamp: 'nice 1/1/2010 21:51'!
merge: targetVersion
	| ancestorInfo merger ancestorSnapshot packageSnapshot |
	targetVersion dependencies do: [:ea | ea resolve merge].
	ancestorInfo := targetVersion info commonAncestorWith: ancestry.
	
	ancestorInfo = targetVersion info ifTrue: [^ MCNoChangesException signal].
	
	packageSnapshot := package snapshot.
	ancestorSnapshot := ancestorInfo
							ifNotNil: [(self findSnapshotWithVersionInfo: ancestorInfo)]
							ifNil: [self notifyNoCommonAncestorWith: targetVersion.  MCSnapshot empty].
	
	(ancestry ancestors size = 1
		and: [ancestry ancestors first = ancestorInfo
		and: [(packageSnapshot patchRelativeToBase: ancestorSnapshot) isEmpty]])
				ifTrue: [^ targetVersion load].
	
	merger := MCThreeWayMerger 
				base: packageSnapshot
				target: targetVersion snapshot
				ancestor: ancestorSnapshot.
	((MCMergeResolutionRequest new merger: merger)
		signal: 'Merging ', targetVersion info name) = true ifTrue:
			[merger loadWithNameLike: targetVersion info name.
			ancestry addAncestor: targetVersion info].
	self changed! !

!MCWorkingCopy methodsFor: 'operations' stamp: 'abc 2/13/2004 15:57'!
merged: aVersion
	ancestry addAncestor: aVersion info.
	self changed! !

!MCWorkingCopy methodsFor: 'accessing' stamp: 'avi 2/13/2004 20:02'!
needsSaving
	^ self modified or: [self requiredPackages anySatisfy: [:ea | ea workingCopy needsSaving]]! !

!MCWorkingCopy methodsFor: 'repositories' stamp: 'cmm 2/13/2014 10:56'!
newRepositoryGroupIfDefault
	repositoryGroup == MCRepositoryGroup default
		ifTrue: [repositoryGroup := MCRepositoryGroup new].
	^ repositoryGroup! !

!MCWorkingCopy methodsFor: 'operations' stamp: 'dtl 11/30/2014 14:45'!
newVersion
	| packageSnapshot parentSnapshot patch patchBlock |
	parentSnapshot := self parentSnapshot.
	patchBlock :=  [patch := (packageSnapshot := package snapshot) patchRelativeToBase: parentSnapshot].
	patchBlock value. "Ensure that this is called at least once."
	^ (self requestVersionNameAndMessageWithSuggestion: self uniqueVersionName
		initialMessage: self patchMessageSuggestion
		patchBlock: patchBlock
	) ifNotNil: [:tuple |
		self newVersionWithName: tuple first withBlanksTrimmed
			message: (self patchMessageStripped: tuple second)
			snapshot: ((tuple size >= 3 and: [tuple third notNil and: [tuple third notEmpty]])
				ifTrue: [	MCPatcher apply: (patch ignoring: tuple third) to: parentSnapshot]
				ifFalse: [packageSnapshot])]! !

!MCWorkingCopy methodsFor: 'operations' stamp: 'bf 1/22/2013 13:57'!
newVersionWithName: nameString message: messageString
	^self newVersionWithName: nameString message: messageString snapshot: package snapshot! !

!MCWorkingCopy methodsFor: 'operations' stamp: 'bf 1/23/2013 17:02'!
newVersionWithName: nameString message: messageString snapshot: aSnapshot
	| info deps clean |
	info := ancestry infoWithName: nameString message: messageString.
	ancestry := MCWorkingAncestry new addAncestor: info.
	clean := (package snapshot patchRelativeToBase: aSnapshot) isEmpty.
	self modified: clean; modified: clean not. "hack to ensure label is updated"
	
	deps := self requiredPackages collect:
		[:ea | 
		MCVersionDependency
			package: ea
			info: ea workingCopy currentVersionInfo].

	^ MCVersion
		package: package
		info: info
		snapshot: aSnapshot
		dependencies: deps! !

!MCWorkingCopy methodsFor: 'private' stamp: 'bf 3/8/2012 18:30'!
nextVersionName
	| branch oldName |
	ancestry ancestors isEmpty
		ifTrue:
			[ counter ifNil: [ counter := 0 ].
			branch := package name ]
		ifFalse:
			[ oldName := ancestry ancestors first versionName.
			branch := oldName packageAndBranchName.
			counter ifNil:
				[ counter := (ancestry ancestors detectMax:
					[ : eachVersionInfo | eachVersionInfo versionNumber ])
					ifNil: [ 0 ]
					ifNotNil:
						[ : highestNumbered | highestNumbered versionNumber ] ] ].
	counter := counter + 1.
	^ branch , '-' , Utilities authorInitials , '.' , counter asString! !

!MCWorkingCopy methodsFor: 'operations' stamp: 'avi 2/13/2004 01:07'!
notifyNoCommonAncestorWith: aVersion
	self notify:
'Could not find a common ancestor between (',
aVersion info name,
') and (',
ancestry ancestorString, ').
Proceeding with this merge may cause spurious conflicts.'! !

!MCWorkingCopy methodsFor: 'private' stamp: 'bf 4/11/2013 17:06'!
parentSnapshot
	"prefer parent in selected repository"
	MCRepositoryRequest signal ifNotNil: [:repo |
		self ancestors do: [:ancestor |
			(repo versionWithInfo: ancestor)
				ifNotNil: [:ver | ^ver snapshot]]].
	"otherwise, look in all repositories"
	self ancestors do: [:ancestor |
		(self repositoryGroup versionWithInfo: ancestor)
			ifNotNil: [:ver | ^ver snapshot]].
	"otherwise"
	^MCSnapshot empty! !

!MCWorkingCopy methodsFor: 'operations' stamp: 'topa 9/3/2014 14:27'!
patchMessageAncestry
	^ String streamContents: [:strm |
		strm nextPutAll:	ancestry summary; cr.
		self ancestors do: [:ancestor |
			strm cr.
			strm nextPutAll: ancestor name; nextPut: $:; crtab.
			strm nextPutAll: ancestor message; cr.]]
! !

!MCWorkingCopy methodsFor: 'operations' stamp: 'bf 1/22/2013 14:48'!
patchMessageChanges
	| changes |
	changes := package snapshot patchRelativeToBase: self parentSnapshot.
	^ (MCPatchMessage new patch: changes) message! !

!MCWorkingCopy methodsFor: 'operations' stamp: 'bf 9/4/2010 19:00'!
patchMessageChangesDelimiter
	^'=== text below is ignored ==='! !

!MCWorkingCopy methodsFor: 'operations' stamp: 'bf 9/4/2010 19:04'!
patchMessageChangesHeader
	^ancestry summary, String cr,
	'Added, Modified, Deleted',
	(self ancestors ifEmpty: [''] ifNotEmpty: [' vs. ', self ancestors first name]), ':'! !

!MCWorkingCopy methodsFor: 'operations' stamp: 'bf 9/4/2010 18:15'!
patchMessageDefault
	^ 'empty log message'! !

!MCWorkingCopy methodsFor: 'operations' stamp: 'bf 9/4/2010 18:31'!
patchMessageStripped: aString
	| pos |
	pos := aString findString: self patchMessageChangesDelimiter.
	^ (pos > 0
		ifTrue: [aString first: pos - 1]
		ifFalse: [aString]) withBlanksTrimmed! !

!MCWorkingCopy methodsFor: 'operations' stamp: 'eem 11/20/2014 15:56'!
patchMessageSuggestion
	^ String streamContents: [:strm | strm
		nextPutAll: self patchMessageDefault; cr;cr;
		nextPutAll: self patchMessageChangesDelimiter; cr;
		nextPutAll: self patchMessageAncestry; cr; cr;
		nextPutAll: self patchMessageChanges]! !

!MCWorkingCopy methodsFor: 'private' stamp: 'bf 9/8/2005 10:58'!
possiblyNewerVersions

	^Array streamContents: [:strm |
		self repositoryGroup repositories do: [:repo |
			strm nextPutAll: (self possiblyNewerVersionsIn: repo)]]! !

!MCWorkingCopy methodsFor: 'private' stamp: 'ul 11/14/2018 00:10'!
possiblyNewerVersionsIn: aRepository 
	^ aRepository possiblyNewerVersionsOfAnyOf:
		((self ancestors collect: [:each | each versionName])
			ifEmpty: [Array empty]
			ifNotEmpty: [:ancestors | {ancestors detectMax: [:v | v versionNumber ]}])! !

!MCWorkingCopy methodsFor: 'accessing' stamp: 'bp 11/21/2010 17:12'!
printOn: stream
	super printOn: stream.
	stream
		nextPut: $(;
		nextPutAll: self packageName;
		nextPut: $)! !

!MCWorkingCopy methodsFor: 'operations' stamp: 'cmm 6/8/2011 14:25'!
renameToBe: newPackageName 
	| newWorkingCopy definitions |
	self modified ifTrue: [ self error: 'Should only rename an unmodified package.' ].
	(MCWorkingCopy allManagers anySatisfy:
		[ : each | each packageName = newPackageName ]) ifTrue: [ self error: newPackageName , ' is already taken.' ].
	definitions := self package snapshot definitions.
	PackageInfo registerPackageName: newPackageName.
	newWorkingCopy := (MCWorkingCopy forPackage: (MCPackage new name: newPackageName))
		 repositoryGroup: self repositoryGroup ;
		 requiredPackages: self requiredPackages copy ;
		 modified: true ;
		 yourself.
	definitions do:
		[ : each | each
			handlePackageRename: self package name
			to: newPackageName ].
	self ancestors do:
		[ : each | newWorkingCopy ancestry addAncestor: each ].
	newWorkingCopy package snapshot definitions size = definitions size ifTrue:
		[ PackageOrganizer default unregisterPackage: (PackageInfo named: self packageName).
		self unregister ].
	^ newWorkingCopy! !

!MCWorkingCopy methodsFor: 'operations' stamp: 'cmm 5/18/2019 15:57'!
reparent: aVersion 
	"Let aNode be the sole parent of this version"
	ancestry reparent: aVersion info.
	self
		 checkModified ;
		 changed! !

!MCWorkingCopy methodsFor: 'repositories' stamp: 'avi 8/31/2003 00:14'!
repositoryGroup
	^ repositoryGroup ifNil: [repositoryGroup := MCRepositoryGroup new]! !

!MCWorkingCopy methodsFor: 'repositories' stamp: 'ab 7/22/2003 00:20'!
repositoryGroup: aRepositoryGroup
	repositoryGroup := aRepositoryGroup! !

!MCWorkingCopy methodsFor: 'private' stamp: 'topa 9/3/2014 13:54'!
requestVersionNameAndMessageWithSuggestion: nameString initialMessage: msgString patchBlock: aPatchBlock
	^ (MCVersionNameAndMessageRequest new
		suggestedName: nameString;
		initialMessage: msgString;
		patchBlock: aPatchBlock
		) signal! !

!MCWorkingCopy methodsFor: 'accessing' stamp: 'avi 1/20/2004 16:04'!
requirePackage: aPackage
	(self requiredPackages includes: aPackage) ifFalse: [requiredPackages add: aPackage]! !

!MCWorkingCopy methodsFor: 'accessing' stamp: 'avi 1/19/2004 16:30'!
requiredPackages
	^ requiredPackages ifNil: [requiredPackages := OrderedCollection new]! !

!MCWorkingCopy methodsFor: 'private' stamp: 'cmm 6/6/2011 19:26'!
requiredPackages: aCollection
	requiredPackages := aCollection! !

!MCWorkingCopy methodsFor: 'operations' stamp: 'cmm 8/16/2013 14:11'!
stubAncestry
	"Stub all but the most recent 10 of my ancestry tree to save memory with a proxy which can be transparently accessed later, if necessary."
	self ancestors ifEmpty: [ ^ self ].
	repositoryGroup ifNotNil:
		[ | tenAgo count |
		count := 0.
		ancestry ancestorsDoWhileTrue:
			[ : each | tenAgo := each.
			(count := count + 1) < 10 ].
		tenAgo
			stubAncestryFor: self
			using: repositoryGroup ]! !

!MCWorkingCopy methodsFor: 'private' stamp: 'mt 6/16/2020 14:02'!
uniqueVersionName
	|versionName|
	counter := nil.
	[versionName := self nextVersionName.
	(MCRepository packageCache includesVersionNamed: versionName)
		or: [self repositoryGroup includesVersionNamed: versionName]] whileTrue.
	^ versionName! !

!MCWorkingCopy methodsFor: 'operations' stamp: 'jr 1/14/2017 23:46'!
unload
	self withEnvironmentActiveDo: [MCPackageLoader unloadPackage: self package].
	self unregisterSubpackages.
	self unregister.! !

!MCWorkingCopy methodsFor: 'migration' stamp: 'avi 2/17/2004 02:36'!
updateInstVars
	ancestry ifNil:
		[ancestry := MCWorkingAncestry new.
		versionInfo ifNotNil:
			[versionInfo ancestors do: [:ea | ancestry addAncestor: ea].
			versionInfo := nil]]! !

!MCWorkingCopy methodsFor: 'accessing' stamp: 'avi 2/13/2004 01:06'!
versionInfo: aVersionInfo
	ancestry := MCWorkingAncestry new addAncestor: aVersionInfo! !

!MCWorkingCopy methodsFor: 'private' stamp: 'avi 2/4/2004 14:11'!
versionSeparator
	^ $_! !

!MCWorkingCopy methodsFor: 'private' stamp: 'jr 1/15/2017 00:02'!
withEnvironmentActiveDo: aBlock
	^ self environment beCurrentDuring: aBlock! !

Object subclass: #MCPatchMessage
	instanceVariableNames: 'stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

!MCPatchMessage methodsFor: 'patch operations' stamp: 'bf 9/4/2010 18:48'!
addDefinition: aDefinition
	stream nextPutAll: 'A'; tab; nextPutAll: aDefinition summary; cr! !

!MCPatchMessage methodsFor: 'accessing' stamp: 'bf 9/4/2010 16:29'!
message
	^stream contents
! !

!MCPatchMessage methodsFor: 'patch operations' stamp: 'bf 9/4/2010 18:57'!
modifyDefinition: oldDefinition to: newDefinition
	stream nextPutAll: 'M'; tab; nextPutAll: newDefinition summary; cr! !

!MCPatchMessage methodsFor: 'accessing' stamp: 'ul 3/13/2017 00:35'!
patch: aPatch
	stream ifNil: [stream := WriteStream on: (String new: 100)].
	aPatch operations sorted
		do: [:op | op applyTo: self]! !

!MCPatchMessage methodsFor: 'patch operations' stamp: 'bf 9/4/2010 18:57'!
removeDefinition: aDefinition
	stream nextPutAll: 'D'; tab; nextPutAll: aDefinition summary; cr! !

Object subclass: #MCPatchOperation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Patching'!

MCPatchOperation subclass: #MCAddition
	instanceVariableNames: 'definition'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Patching'!

!MCAddition class methodsFor: 'instance creation' stamp: 'cwp 11/27/2002 10:03'!
of: aDefinition
	^ self new intializeWithDefinition: aDefinition! !

!MCAddition methodsFor: 'comparing' stamp: 'topa 9/3/2014 14:10'!
= other
	^ other isAddition and: [definition = other definition]! !

!MCAddition methodsFor: 'applying' stamp: 'ab 5/24/2003 16:11'!
applyTo: anObject
	anObject addDefinition: definition! !

!MCAddition methodsFor: 'accessing' stamp: 'ab 6/1/2003 13:09'!
baseDefinition
	^ nil! !

!MCAddition methodsFor: 'accessing' stamp: 'cwp 11/27/2002 09:32'!
definition
	^ definition! !

!MCAddition methodsFor: 'accessing' stamp: 'ab 7/18/2003 16:45'!
fromSource
	^ ''! !

!MCAddition methodsFor: 'comparing' stamp: 'topa 9/3/2014 14:10'!
hash
	^ definition hash! !

!MCAddition methodsFor: 'initializing' stamp: 'cwp 11/27/2002 10:01'!
intializeWithDefinition: aDefinition
	definition := aDefinition! !

!MCAddition methodsFor: 'accessing' stamp: 'ab 8/22/2003 02:26'!
inverse
	^ MCRemoval of: definition! !

!MCAddition methodsFor: 'testing' stamp: 'cwp 11/28/2002 07:22'!
isAddition
	^ true! !

!MCAddition methodsFor: 'testing' stamp: 'nk 2/25/2005 17:28'!
isClassPatch
	^definition isClassDefinition! !

!MCAddition methodsFor: 'accessing' stamp: 'nk 2/23/2005 18:39'!
sourceString
	^(self toSource asText)
		addAttribute: TextColor red;
		yourself! !

!MCAddition methodsFor: 'accessing' stamp: 'ab 5/13/2003 12:18'!
summary
	^ definition summary! !

!MCAddition methodsFor: 'accessing' stamp: 'nk 2/23/2005 18:17'!
targetClass
	^definition actualClass ! !

!MCAddition methodsFor: 'accessing' stamp: 'ab 6/1/2003 13:09'!
targetDefinition
	^ definition! !

!MCAddition methodsFor: 'accessing' stamp: 'ab 7/18/2003 16:45'!
toSource
	^ definition source! !

MCPatchOperation subclass: #MCModification
	instanceVariableNames: 'obsoletion modification'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Patching'!

!MCModification class methodsFor: 'instance creation' stamp: 'cwp 11/28/2002 07:19'!
of: base to: target
	^ self new initializeWithBase: base target: target! !

!MCModification methodsFor: 'comparing' stamp: 'topa 9/3/2014 14:11'!
= other
	^ other isModification 
	and: [obsoletion = other obsoletion
	and: [modification = other modification]]! !

!MCModification methodsFor: 'applying' stamp: 'ab 5/24/2003 16:12'!
applyTo: anObject
	anObject modifyDefinition: obsoletion to: modification! !

!MCModification methodsFor: 'accessing' stamp: 'ab 6/1/2003 13:10'!
baseDefinition
	^ obsoletion! !

!MCModification methodsFor: 'accessing' stamp: 'cwp 11/28/2002 06:55'!
definition
	^ modification! !

!MCModification methodsFor: 'accessing' stamp: 'ab 7/18/2003 16:44'!
fromSource
	^ obsoletion source! !

!MCModification methodsFor: 'comparing' stamp: 'topa 9/3/2014 14:12'!
hash
	^ obsoletion hash bitXor: modification hash! !

!MCModification methodsFor: 'initializing' stamp: 'cwp 11/28/2002 07:18'!
initializeWithBase: base target: target
	obsoletion := base.
	modification := target.! !

!MCModification methodsFor: 'accessing' stamp: 'ab 8/22/2003 02:27'!
inverse
	^ MCModification of: modification to: obsoletion! !

!MCModification methodsFor: 'testing' stamp: 'nk 2/25/2005 17:29'!
isClassPatch
	^obsoletion isClassDefinition! !

!MCModification methodsFor: 'testing' stamp: 'cwp 11/28/2002 07:23'!
isModification
	^ true! !

!MCModification methodsFor: 'testing' stamp: 'bf 6/13/2011 16:27'!
isUnchangedMethod
	"true if this is a modification of a method where only the timestamp changed"
	^ obsoletion isMethodDefinition
		and: [obsoletion source = modification source
			and: [obsoletion category = modification category] ]! !

!MCModification methodsFor: 'accessing' stamp: 'cwp 11/27/2002 09:46'!
modification
	^ modification! !

!MCModification methodsFor: 'accessing' stamp: 'cwp 11/27/2002 09:48'!
obsoletion
	^ obsoletion! !

!MCModification methodsFor: 'accessing' stamp: 'nk 11/10/2003 21:44'!
printAnnotations: request on: aStream
	aStream nextPutAll: 'old: '.
	obsoletion printAnnotations: request on: aStream.
	aStream cr.
	aStream nextPutAll: 'new: '.
	modification printAnnotations: request on: aStream.! !

!MCModification methodsFor: 'accessing' stamp: 'bf 11/1/2011 11:42'!
summarySuffix
	^ modification summarySuffixOver: obsoletion
! !

!MCModification methodsFor: 'accessing' stamp: 'nk 2/23/2005 18:18'!
targetClass
	^ obsoletion actualClass! !

!MCModification methodsFor: 'accessing' stamp: 'ab 6/1/2003 13:10'!
targetDefinition
	^ modification! !

!MCModification methodsFor: 'accessing' stamp: 'ab 7/18/2003 16:44'!
toSource
	^ modification source! !

!MCPatchOperation methodsFor: 'comparing' stamp: 'ab 7/19/2003 18:11'!
<= other
	^ self definition <= other definition! !

!MCPatchOperation methodsFor: 'accessing' stamp: 'nk 11/10/2003 21:38'!
annotations
	^self annotations: Preferences defaultAnnotationRequests! !

!MCPatchOperation methodsFor: 'accessing' stamp: 'nk 11/10/2003 21:39'!
annotations: requests
	"Answer a string for an annotation pane, trying to fulfill the annotation requests.
	These might include anything that
		Preferences defaultAnnotationRequests 
	might return. Which includes anything in
		Preferences annotationInfo
	To edit these, use:"
	"Preferences editAnnotations"

	^String streamContents: [ :s | self printAnnotations: requests on: s ].! !

!MCPatchOperation methodsFor: 'accessing' stamp: 'cwp 11/28/2002 06:59'!
definition
	^ self subclassResponsibility ! !

!MCPatchOperation methodsFor: 'accessing' stamp: 'cwp 11/27/2002 09:26'!
inverse
	self subclassResponsibility! !

!MCPatchOperation methodsFor: 'testing' stamp: 'cwp 11/27/2002 09:30'!
isAddition
	^ false! !

!MCPatchOperation methodsFor: 'testing' stamp: 'nk 2/25/2005 17:28'!
isClassPatch
	^false! !

!MCPatchOperation methodsFor: 'testing' stamp: 'cwp 11/27/2002 09:30'!
isModification
	^ false! !

!MCPatchOperation methodsFor: 'testing' stamp: 'cwp 11/27/2002 09:30'!
isRemoval
	^ false! !

!MCPatchOperation methodsFor: 'testing' stamp: 'bf 6/13/2011 16:24'!
isUnchangedMethod
	"true if this is a modification of a method where only the timestamp changed"
	^false! !

!MCPatchOperation methodsFor: 'accessing' stamp: 'avi 8/31/2003 17:53'!
prefixForOperation: aSymbol
	aSymbol == #insert ifTrue: [^ '+'].
	aSymbol == #remove ifTrue: [^ '-'].
	^ ' '! !

!MCPatchOperation methodsFor: 'accessing' stamp: 'nk 11/10/2003 21:40'!
printAnnotations: requests on: aStream
	"Add a string for an annotation pane, trying to fulfill the annotation requests.
	These might include anything that
		Preferences defaultAnnotationRequests 
	might return. Which includes anything in
		Preferences annotationInfo
	To edit these, use:"
	"Preferences editAnnotations"

	self definition printAnnotations: requests on: aStream.! !

!MCPatchOperation methodsFor: 'accessing' stamp: 'avi 8/31/2003 17:55'!
source
	^ self sourceText! !

!MCPatchOperation methodsFor: 'accessing' stamp: 'nk 2/25/2005 17:26'!
sourceString
	^self sourceText asString! !

!MCPatchOperation methodsFor: 'accessing' stamp: 'ul 11/6/2010 05:55'!
sourceText

	^(self isClassPatch
		ifFalse: [ TextDiffBuilder ]
		ifTrue: [ ClassDiffBuilder ])
			buildDisplayPatchFrom: self fromSource
			to: self toSource
			inClass: self targetClass
			prettyDiffs: (
				Preferences diffsWithPrettyPrint and: [
					self targetClass notNil and: [
					self isClassPatch not ] ])! !

!MCPatchOperation methodsFor: 'accessing' stamp: 'ab 7/6/2003 00:06'!
summary
	^ self definition summary, self summarySuffix! !

!MCPatchOperation methodsFor: 'accessing' stamp: 'bf 11/25/2016 11:29'!
summaryAndRevision
	^ self definition summaryAndRevision, self summarySuffix! !

!MCPatchOperation methodsFor: 'accessing' stamp: 'ab 7/6/2003 00:06'!
summarySuffix
	^ ''! !

!MCPatchOperation methodsFor: 'accessing' stamp: 'nk 2/23/2005 18:17'!
targetClass
	self subclassResponsibility.! !

MCPatchOperation subclass: #MCRemoval
	instanceVariableNames: 'definition'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Patching'!

!MCRemoval class methodsFor: 'instance creation' stamp: 'cwp 11/27/2002 10:03'!
of: aDefinition
	^ self new intializeWithDefinition: aDefinition! !

!MCRemoval methodsFor: 'comparing' stamp: 'topa 9/3/2014 14:10'!
= other
	^ other isRemoval and: [definition = other definition]! !

!MCRemoval methodsFor: 'accessing' stamp: 'ab 5/24/2003 16:11'!
applyTo: anObject
	anObject removeDefinition: definition! !

!MCRemoval methodsFor: 'accessing' stamp: 'ab 6/1/2003 13:10'!
baseDefinition
	^ definition! !

!MCRemoval methodsFor: 'accessing' stamp: 'cwp 11/27/2002 10:02'!
definition
	^ definition! !

!MCRemoval methodsFor: 'accessing' stamp: 'ab 7/18/2003 16:44'!
fromSource
	^ definition source! !

!MCRemoval methodsFor: 'comparing' stamp: 'topa 9/3/2014 14:10'!
hash
	^ definition hash! !

!MCRemoval methodsFor: 'initializing' stamp: 'cwp 11/27/2002 10:02'!
intializeWithDefinition: aDefinition
	definition := aDefinition! !

!MCRemoval methodsFor: 'accessing' stamp: 'ab 8/22/2003 02:26'!
inverse
	^ MCAddition of: definition! !

!MCRemoval methodsFor: 'testing' stamp: 'nk 2/25/2005 17:28'!
isClassPatch
	^definition isClassDefinition! !

!MCRemoval methodsFor: 'testing' stamp: 'cwp 11/28/2002 07:24'!
isRemoval
	^ true! !

!MCRemoval methodsFor: 'accessing' stamp: 'nk 2/23/2005 18:38'!
sourceString
	^self fromSource asText
		addAttribute: TextEmphasis struckOut;
		addAttribute: TextColor blue;
		yourself! !

!MCRemoval methodsFor: 'accessing' stamp: 'ab 7/6/2003 00:05'!
summarySuffix
	^ ' (removed)'! !

!MCRemoval methodsFor: 'accessing' stamp: 'nk 2/25/2005 17:23'!
targetClass
	^ definition actualClass! !

!MCRemoval methodsFor: 'accessing' stamp: 'ab 6/1/2003 13:10'!
targetDefinition
	^ nil! !

!MCRemoval methodsFor: 'accessing' stamp: 'ab 7/18/2003 16:44'!
toSource
	^ ''! !

Object subclass: #MCPatcher
	instanceVariableNames: 'definitions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Patching'!

!MCPatcher class methodsFor: 'applying' stamp: 'ab 6/2/2003 00:53'!
apply: aPatch to: aSnapshot
	| loader |
	loader := self snapshot: aSnapshot.
	aPatch applyTo: loader.
	^ loader patchedSnapshot! !

!MCPatcher class methodsFor: 'instance creation' stamp: 'ab 6/1/2003 14:22'!
snapshot: aSnapshot
	^ self new initializeWithSnapshot: aSnapshot! !

!MCPatcher methodsFor: 'accessing' stamp: 'ab 6/2/2003 00:46'!
addDefinition: aDefinition
	definitions add: aDefinition! !

!MCPatcher methodsFor: 'initialize-release' stamp: 'ab 7/6/2003 23:49'!
initializeWithSnapshot: aSnapshot
	definitions := MCDefinitionIndex definitions: aSnapshot definitions! !

!MCPatcher methodsFor: 'applying' stamp: 'ab 6/1/2003 14:23'!
modifyDefinition: baseDefinition to: targetDefinition
	self addDefinition: targetDefinition! !

!MCPatcher methodsFor: 'applying' stamp: 'ab 7/6/2003 23:48'!
patchedSnapshot
	^ MCSnapshot fromDefinitions: definitions definitions! !

!MCPatcher methodsFor: 'accessing' stamp: 'ab 6/2/2003 00:46'!
removeDefinition: aDefinition
	definitions remove: aDefinition! !

Object subclass: #MCReader
	instanceVariableNames: 'stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Storing'!

!MCReader class methodsFor: 'testing' stamp: 'avi 1/21/2004 19:00'!
canReadFileNamed: fileName
	^ (fileName endsWith: self extension)! !

!MCReader class methodsFor: 'testing' stamp: 'ul 3/22/2011 22:34'!
concreteSubclassesDo: aBlock

	self allSubclassesDo: [ :each |
		each isAbstract ifFalse: [ aBlock value: each ] ]! !

!MCReader class methodsFor: 'testing' stamp: 'avi 1/21/2004 19:01'!
isAbstract
	^ (self respondsTo: #extension) not! !

!MCReader class methodsFor: 'instance creation' stamp: 'avi 1/21/2004 19:02'!
new
	^ self basicNew initialize! !

!MCReader class methodsFor: 'instance creation' stamp: 'avi 1/21/2004 19:02'!
on: aStream
	^ self new stream: aStream! !

!MCReader class methodsFor: 'instance creation' stamp: 'avi 1/21/2004 19:02'!
on: aStream name: aFileName
	| class |
	class := self readerClassForFileNamed: aFileName.
	^ class
		ifNil: [self error: 'Unsupported format: ', aFileName]
		ifNotNil: [class on: aStream]! !

!MCReader class methodsFor: 'testing' stamp: 'ul 3/22/2011 21:23'!
readerClassForFileNamed: fileName
	
	self concreteSubclassesDo: [ :each |
		(each canReadFileNamed: fileName) ifTrue: [ ^each ] ].
	^nil! !

!MCReader methodsFor: 'accessing' stamp: 'avi 1/21/2004 19:00'!
stream: aStream
	stream := aStream! !

MCReader subclass: #MCSnapshotReader
	instanceVariableNames: 'definitions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Storing'!

!MCSnapshotReader class methodsFor: 'converting' stamp: 'avi 1/21/2004 22:56'!
snapshotFromStream: aStream
	^ (self on: aStream) snapshot! !

!MCSnapshotReader methodsFor: 'loading' stamp: 'avi 1/21/2004 23:09'!
definitions
	definitions ifNil: [self loadDefinitions].
	^ definitions! !

!MCSnapshotReader methodsFor: 'accessing' stamp: 'avi 1/21/2004 23:10'!
snapshot
	^ MCSnapshot fromDefinitions: self definitions! !

MCSnapshotReader subclass: #MCStReader
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Chunk Format'!

!MCStReader class methodsFor: 'constants' stamp: 'avi 1/20/2004 00:17'!
extension
	^ 'st'! !

!MCStReader methodsFor: 'private' stamp: 'ul 12/12/2009 14:09'!
addDefinitionsFromDoit: aString
	(MCDoItParser forDoit: aString) ifNotNil:
		[:parser |
		parser addDefinitionsTo: definitions]! !

!MCStReader methodsFor: 'private' stamp: 'avi 1/20/2004 00:02'!
categoryFromDoIt: aString
	| tokens  |
	tokens := Scanner new scanTokens: aString.
	tokens size = 3 ifFalse: [self error: 'Unrecognized category definition'].
	^ tokens at: 3! !

!MCStReader methodsFor: 'converting' stamp: 'dtl 12/27/2017 23:34'!
classDefinitionFrom: aPseudoClass
	| tokens traitCompositionString lastIndex classTraitCompositionString typeOfSubclass className |
	tokens := Scanner new scanTokens: aPseudoClass definition.
	traitCompositionString := ((ReadStream on: aPseudoClass definition)
		match: 'uses:';
		upToAll: 'instanceVariableNames:') withBlanksTrimmed.
	classTraitCompositionString := ((ReadStream on: aPseudoClass metaClass definition asString)
		match: 'uses:';
		upToAll: 'instanceVariableNames:') withBlanksTrimmed.
	traitCompositionString isEmpty ifTrue: [traitCompositionString := '{}'].
	classTraitCompositionString isEmpty ifTrue: [classTraitCompositionString := '{}'].
	lastIndex := tokens size.

	className := tokens at: 3.
	typeOfSubclass := self typeOfSubclass: (tokens at: 2).
	"Compiled code classes are special cases of the #bytes class type"
	(#bytes == typeOfSubclass and: [self compiledCodeClassNames includes: className])
		ifTrue: [typeOfSubclass := #compiledMethod].

	^ MCClassDefinition
		name: className
		superclassName: (tokens at: 1)
		traitComposition: traitCompositionString
		classTraitComposition: classTraitCompositionString
		category: (tokens at: lastIndex)
		instVarNames: ((tokens at: lastIndex - 6) findTokens: ' ')
		classVarNames: ((tokens at: lastIndex - 4) findTokens: ' ')
		poolDictionaryNames: ((tokens at: lastIndex - 2) findTokens: ' ')
		classInstVarNames: (self classInstVarNamesFor: aPseudoClass)
		type: typeOfSubclass
		comment: (self commentFor: aPseudoClass)
		commentStamp: (self commentStampFor: aPseudoClass)! !

!MCStReader methodsFor: 'converting' stamp: 'stephaneducasse 2/4/2006 20:47'!
classInstVarNamesFor: aPseudoClass
	| tokens |
	
	self flag: #traits.
	aPseudoClass metaClass hasDefinition ifFalse: [^ #()].
	tokens := Scanner new scanTokens: aPseudoClass metaClass definition.
	"tokens size = 4 ifFalse: [self error: 'Unrecognized metaclass definition']."
	^ tokens last findTokens: ' '! !

!MCStReader methodsFor: 'converting' stamp: 'avi 2/10/2004 14:52'!
commentFor: aPseudoClass
	| comment |
	comment := aPseudoClass organization classComment.
	^ comment asString = ''
		ifTrue: [comment]
		ifFalse: [comment string]! !

!MCStReader methodsFor: 'converting' stamp: 'avi 3/8/2004 21:09'!
commentStampFor: aPseudoClass
	| comment |
	comment := aPseudoClass organization classComment.
	^  [comment stamp] on: MessageNotUnderstood do: [nil]! !

!MCStReader methodsFor: 'private' stamp: 'dtl 12/27/2017 23:33'!
compiledCodeClassNames
	"Answer the names of classes for which the type is #compiledMethod. Traditionally,
	this was only class CompiledMehod, but later refactorings require that CompiledCode
	and its subclasses be treated as type #compiledMethod."

	^{ #CompiledCode . #CompiledBlock . #CompiledMethod }! !

!MCStReader methodsFor: 'evaluating' stamp: 'jr 2/11/2017 17:34'!
loadDefinitions
	| filePackage |
	filePackage :=
		FilePackage new
			fullName: 'ReadStream';
			fileInFrom: self readStream.
	definitions := OrderedCollection new.
	filePackage classes do:
		[:pseudoClass |
		pseudoClass hasDefinition
			ifTrue: [definitions add:
					(pseudoClass asMCDefinitionBy: self)].
		definitions addAll: (self methodDefinitionsFor: pseudoClass).
		definitions addAll: (self methodDefinitionsFor: pseudoClass metaClass)].
	filePackage doIts do:
		[:ea |
		self addDefinitionsFromDoit: ea string].
	! !

!MCStReader methodsFor: 'converting' stamp: 'avi 3/3/2004 15:23'!
methodDefinitionsFor: aPseudoClass
	^ aPseudoClass selectors collect: 
		[:ea |
		 MCMethodDefinition
			className: aPseudoClass name
			classIsMeta: aPseudoClass isMeta
			selector: ea
			category: (aPseudoClass organization categoryOfElement: ea)
			timeStamp: (aPseudoClass stampAt: ea)
			source: (aPseudoClass sourceCodeAt: ea)]! !

!MCStReader methodsFor: 'evaluating' stamp: 'avi 1/21/2004 14:21'!
readStream
	^ ('!!!!

', stream contents) readStream! !

!MCStReader methodsFor: 'converting' stamp: 'avi 1/20/2004 00:15'!
systemOrganizationFromRecords: changeRecords
	| categories |
	categories := changeRecords
					select: [:ea | 'SystemOrganization*' match: ea string]
					thenCollect: [:ea | (self categoryFromDoIt: ea string)].
	^ categories isEmpty ifFalse: [MCOrganizationDefinition categories: categories asArray]! !

!MCStReader methodsFor: 'converting' stamp: 'jr 2/11/2017 17:29'!
traitDefinitionFrom: aPseudoTrait
	| tokens traitCompositionString lastIndex |
	tokens := Scanner new scanTokens: aPseudoTrait definition.
	traitCompositionString := ((ReadStream on: aPseudoTrait definition)
		match: 'uses:';
		upToAll: 'category:') withBlanksTrimmed.
	traitCompositionString isEmpty ifTrue: [traitCompositionString := '{}'].
	lastIndex := tokens size.
	^ MCTraitDefinition
		name: (tokens at: 3)
		traitComposition: traitCompositionString
		category: (tokens at: lastIndex)
		comment: (self commentFor: aPseudoTrait)
		commentStamp: (self commentStampFor: aPseudoTrait)! !

!MCStReader methodsFor: 'private' stamp: 'nice 10/20/2016 21:37'!
typeOfSubclass: aSymbol
	#(
		(subclass: normal)
		(variableSubclass: variable)
		(variableByteSubclass: bytes)
		(variableDoubleByteSubclass: shorts)
		(variableWordSubclass: words)
		(variableDoubleWordSubclass: longs)
		(weakSubclass: weak)
		(ephemeronSubclass: ephemeron)
		(immediateSubclass: immediate)
		) do: [:ea | ea first = aSymbol ifTrue: [^ ea second]].
	self error: 'Unrecognized class definition'! !

MCReader subclass: #MCVersionReader
	instanceVariableNames: 'package info definitions dependencies stepChildren'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Storing'!

MCVersionReader subclass: #MCMczReader
	instanceVariableNames: 'zip infoCache'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Storing'!

MCMczReader subclass: #MCMcdReader
	instanceVariableNames: 'baseInfo patch'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Storing'!

!MCMcdReader class methodsFor: 'accessing' stamp: 'avi 2/13/2004 23:09'!
extension
	^ 'mcd'! !

!MCMcdReader methodsFor: 'accessing' stamp: 'avi 2/14/2004 21:33'!
baseInfo
	^ baseInfo ifNil: [self loadBaseInfo]! !

!MCMcdReader methodsFor: 'accessing' stamp: 'avi 2/13/2004 23:41'!
basicVersion
	^ MCDiffyVersion
		package: self package
		info: self info
		dependencies: self dependencies
		baseInfo: self baseInfo
		patch: self patch! !

!MCMcdReader methodsFor: 'private-loading' stamp: 'avi 2/14/2004 21:37'!
buildPatchFrom: oldDefinitions to: newDefinitions
	^ MCPatch
		fromBase: (MCSnapshot fromDefinitions: oldDefinitions)
		target: (MCSnapshot fromDefinitions: newDefinitions)! !

!MCMcdReader methodsFor: 'loading' stamp: 'avi 2/20/2004 00:13'!
loadBaseInfo
	^ baseInfo := self extractInfoFrom: (self parseMember: 'base')! !

!MCMcdReader methodsFor: 'loading' stamp: 'ul 12/12/2009 14:10'!
loadPatch
	| old new |
	(self zip memberNamed: 'patch.bin') ifNotNil:
		[:m | [^ patch := (DataStream on: m contentStream) next ]
			on: Error do: [:fallThrough ]].
	definitions := OrderedCollection new.
	(self zip membersMatching: 'old/*')
		do: [:m | self extractDefinitionsFrom: m].
	old := definitions asArray.
	definitions := OrderedCollection new.
	(self zip membersMatching: 'new/*')
		do: [:m | self extractDefinitionsFrom: m].
	new := definitions asArray.
	^ patch := self buildPatchFrom: old to: new.
	! !

!MCMcdReader methodsFor: 'accessing' stamp: 'avi 2/14/2004 21:34'!
patch
	^ patch ifNil: [self loadPatch]! !

!MCMczReader class methodsFor: 'accessing' stamp: 'cwp 8/1/2003 14:59'!
extension
	^ 'mcz'! !

!MCMczReader class methodsFor: 'testing' stamp: 'avi 1/19/2004 14:48'!
supportsDependencies
	^ true! !

!MCMczReader class methodsFor: 'testing' stamp: 'cwp 8/1/2003 12:19'!
supportsVersions
	^ true! !

!MCMczReader methodsFor: 'utilities' stamp: 'nice 12/28/2009 16:07'!
associate: tokens
	| result |
	result := Dictionary new.
	tokens pairsDo: [:key :value | 
					result at: key put: (value isString
						ifTrue: [value]
						ifFalse: [value collect: [:ea | self associate: ea]])].
	^ result! !

!MCMczReader methodsFor: 'private' stamp: 'nice 6/2/2013 16:20'!
contentsForMember: member
	^[(member contentStreamFromEncoding: 'utf8') text contents] on: InvalidUTF8
		do: [:exc | 
			"Case of legacy encoding, presumably it is latin-1.
			But if contents starts with a null character, it might be a case of WideString encoded in UTF-32BE"
			| str |
			str := (member contentStreamFromEncoding: 'latin1') text.
			exc return: ((str peek = Character null and: [ str size \\ 4 = 0 ])
				ifTrue: [WideString fromByteArray: str contents asByteArray]
				ifFalse: [str contents])]! !

!MCMczReader methodsFor: 'private-loading' stamp: 'nice 6/2/2013 16:08'!
extractDefinitionsFrom: member
	| reader |
	(MCSnapshotReader readerClassForFileNamed: member fileName)
		ifNotNil: [:rc |
			reader := rc on: (self contentsForMember: member) readStream.
			definitions addAll: reader definitions]
! !

!MCMczReader methodsFor: 'private-loading' stamp: 'nicolascellier 5/31/2013 23:33'!
extractDependencyFrom: zipMember
	^ MCVersionDependency
		package: (MCPackage named: (zipMember fileName copyAfterLast: $/))
		info: (self extractInfoFrom: (self parseMember: zipMember))! !

!MCMczReader methodsFor: 'private-loading' stamp: 'bf 4/18/2010 18:38'!
extractInfoFrom: dict
	^MCWorkingCopy infoFromDictionary: dict cache: self infoCache! !

!MCMczReader methodsFor: 'accessing' stamp: 'avi 1/22/2004 20:59'!
infoCache
	^ infoCache ifNil: [infoCache := Dictionary new]! !

!MCMczReader methodsFor: 'loading' stamp: 'ul 12/12/2009 14:10'!
loadDefinitions
	definitions := OrderedCollection new.
	(self zip memberNamed: 'snapshot.bin') ifNotNil:
		[:m | [^ definitions := (DataStream on: m contentStream) next definitions]
			on: Error do: [:fallThrough ]].
	"otherwise"
	(self zip membersMatching: 'snapshot/*')
		do: [:m | self extractDefinitionsFrom: m].
! !

!MCMczReader methodsFor: 'loading' stamp: 'avi 1/19/2004 16:06'!
loadDependencies
	dependencies := (self zip membersMatching: 'dependencies/*') collect: [:m | self extractDependencyFrom: m].
	dependencies := dependencies asArray.
! !

!MCMczReader methodsFor: 'loading' stamp: 'ab 8/20/2003 19:58'!
loadPackage
	| dict |
	dict := self parseMember: 'package'.
	package := MCPackage named: (dict at: #name)! !

!MCMczReader methodsFor: 'loading' stamp: 'avi 1/22/2004 21:18'!
loadVersionInfo
	info := self extractInfoFrom: (self parseMember: 'version')! !

!MCMczReader methodsFor: 'utilities' stamp: 'nice 6/2/2013 16:14'!
parseMember: memberOrName
	| member contents tokens |
	member := self zip member: memberOrName.
	contents := self contentsForMember: member.
	tokens := (self scanner scanTokens: contents) first.
	^ self associate: tokens! !

!MCMczReader methodsFor: 'constants' stamp: 'avi 1/22/2004 20:33'!
scanner
	^ MCScanner! !

!MCMczReader methodsFor: 'accessing' stamp: 'ab 8/18/2003 00:50'!
zip
	zip ifNil:
		[zip := ZipArchive new.
		zip readFrom: stream].
	^ zip! !

!MCVersionReader class methodsFor: 'testing' stamp: 'cwp 8/4/2003 00:32'!
canReadFileNamed: fileName
	^ (fileName endsWith: self extension)! !

!MCVersionReader class methodsFor: 'reading' stamp: 'ul 8/2/2011 20:03'!
file: fileName streamDo: aBlock

	^FileStream readOnlyFileNamed: fileName do: aBlock! !

!MCVersionReader class methodsFor: 'file services' stamp: 'ul 10/22/2018 22:45'!
fileReaderServicesForFile: fullName suffix: suffix
	self isAbstract ifTrue: [^ Array empty].
	^ ((suffix = self extension) or: [ suffix = '*' ])
		ifTrue: [self services]
		ifFalse: [Array empty]
		! !

!MCVersionReader class methodsFor: 'file services' stamp: 'tpr 1/7/2019 16:01'!
initialize
	"MCVersionReader initialize"
	Smalltalk 
		at: #MczInstaller
		ifPresent: [:installer | FileServices unregisterFileReader: installer].
	self concreteSubclassesDo: [:aClass | FileServices registerFileReader: aClass].

	"get rid of AnObsoleteMCMcReader and AnObsoleteMCMcvReader"
	(FileServices registeredFileReaderClasses  select: [ :ea | ea isObsolete ]) do: 
		[ :ea | FileServices unregisterFileReader: ea ]
! !

!MCVersionReader class methodsFor: 'file services' stamp: 'cmm 4/22/2011 16:54'!
loadVersionFile: fileName 
	| version |
	version := self versionFromFile: fileName.
	version workingCopy repositoryGroup addRepository: (MCDirectoryRepository path: (FileDirectory dirPathFor: fileName)).
	version load! !

!MCVersionReader class methodsFor: '*monticello-file services' stamp: 'cmm 4/22/2011 16:55'!
loadVersionStream: stream fromDirectory: directory
	| version |
	version := self versionFromStream: stream.
	directory isRemoteDirectory ifFalse: [
	version workingCopy repositoryGroup addRepository:
		(MCDirectoryRepository directory: directory). ].
	version load.
! !

!MCVersionReader class methodsFor: 'file services' stamp: 'cwp 8/1/2003 14:46'!
mergeVersionFile: fileName
	(self versionFromFile: fileName) merge! !

!MCVersionReader class methodsFor: '*monticello-file services' stamp: 'nk 2/25/2005 11:17'!
mergeVersionStream: stream
	(self versionFromStream: stream) merge! !

!MCVersionReader class methodsFor: 'reading' stamp: 'bf 3/23/2005 01:20'!
on: s fileName: f
	^ self on: s! !

!MCVersionReader class methodsFor: 'file services' stamp: 'cwp 8/1/2003 14:46'!
openVersionFile: fileName
	(self versionFromFile: fileName) open! !

!MCVersionReader class methodsFor: '*monticello-file services' stamp: 'nk 2/25/2005 11:12'!
openVersionFromStream: stream
	(self versionFromStream: stream) open! !

!MCVersionReader class methodsFor: '*monticello-file services-override-override' stamp: 'nk 2/25/2005 11:15'!
serviceLoadVersion
	^ (SimpleServiceEntry
		provider: self
		label: 'load version'
		selector: #loadVersionStream:fromDirectory:
		description: 'load a package version'
		buttonLabel: 'load')
		argumentGetter: [ :fileList | { fileList readOnlyStream . fileList directory } ]! !

!MCVersionReader class methodsFor: '*monticello-file services-override-override' stamp: 'nk 2/25/2005 11:16'!
serviceMergeVersion
	^ (SimpleServiceEntry
		provider: self
		label: 'merge version'
		selector: #mergeVersionStream:
		description: 'merge a package version into the image'
		buttonLabel: 'merge')
		argumentGetter: [ :fileList | fileList readOnlyStream ]! !

!MCVersionReader class methodsFor: '*monticello-file services-override-override' stamp: 'nk 2/25/2005 11:16'!
serviceOpenVersion
	^ (SimpleServiceEntry
		provider: self
		label: 'open version'
		selector: #openVersionFromStream:
		description: 'open a package version'
		buttonLabel: 'open')
		argumentGetter: [ :fileList | fileList readOnlyStream ]! !

!MCVersionReader class methodsFor: 'file services' stamp: 'ul 10/18/2018 01:33'!
services
	^{ 
		self serviceLoadVersion.
		self serviceMergeVersion.
		self serviceOpenVersion }! !

!MCVersionReader class methodsFor: 'file services' stamp: 'tpr 1/7/2019 16:01'!
unload
	FileServices unregisterFileReader: self ! !

!MCVersionReader class methodsFor: 'reading' stamp: 'cwp 7/31/2003 23:03'!
versionFromFile: fileName
	^ self file: fileName streamDo: [:stream | self versionFromStream: stream]! !

!MCVersionReader class methodsFor: 'reading' stamp: 'avi 1/21/2004 22:58'!
versionFromStream: aStream
	^ (self on: aStream) version! !

!MCVersionReader class methodsFor: 'reading' stamp: 'avi 1/21/2004 22:59'!
versionInfoFromStream: aStream
	^ (self on: aStream) info! !

!MCVersionReader methodsFor: 'accessing' stamp: 'avi 1/19/2004 16:52'!
basicVersion
	^ MCVersion
		package: self package
		info: self info
		snapshot: self snapshot
		dependencies: self dependencies! !

!MCVersionReader methodsFor: 'accessing' stamp: 'avi 1/21/2004 23:10'!
definitions
	definitions ifNil: [self loadDefinitions].
	^ definitions! !

!MCVersionReader methodsFor: 'accessing' stamp: 'avi 1/19/2004 14:50'!
dependencies
	dependencies ifNil: [self loadDependencies].
	^ dependencies! !

!MCVersionReader methodsFor: 'accessing' stamp: 'ab 8/20/2003 19:53'!
info
	info ifNil: [self loadVersionInfo].
	^ info! !

!MCVersionReader methodsFor: 'lifecycle' stamp: 'cwp 8/3/2003 18:48'!
initialize! !

!MCVersionReader methodsFor: 'loading' stamp: 'ab 8/20/2003 19:54'!
loadDefinitions
	self subclassResponsibility ! !

!MCVersionReader methodsFor: 'loading' stamp: 'avi 1/19/2004 14:50'!
loadDependencies
	self subclassResponsibility ! !

!MCVersionReader methodsFor: 'loading' stamp: 'ab 8/20/2003 19:54'!
loadPackage
	self subclassResponsibility ! !

!MCVersionReader methodsFor: 'loading' stamp: 'ab 8/20/2003 19:54'!
loadVersionInfo
	self subclassResponsibility! !

!MCVersionReader methodsFor: 'accessing' stamp: 'ab 8/20/2003 19:53'!
package
	package ifNil: [self loadPackage].
	^ package! !

!MCVersionReader methodsFor: 'accessing' stamp: 'ab 8/20/2003 19:54'!
snapshot
	^ MCSnapshot fromDefinitions: self definitions! !

!MCVersionReader methodsFor: 'accessing' stamp: 'avi 10/9/2003 12:38'!
version
	^ self basicVersion! !

Object subclass: #MCReorganizationPreloader
	instanceVariableNames: 'preloads previousRemovals currentRemovals'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Loading'!

!MCReorganizationPreloader commentStamp: 'bf 8/20/2013 13:51' prior: 0!
MCReorganizationPreloader ensures that definitions moved between two packages are not removed temporarily, independent of their load order.

Moves appear as deletion in one package and addition in another package. If the deletion precedes the addition, the entity will be removed from the system for a short period of time, and then recreated later. This is potentially fatal.

MCReorganizationPreloader detects these problematic definitions and combines them into MCModifications. They then get 'preloaded' (applied to the system) so when the actual packages are loaded later, they become no-ops (because the definition in the image is already in the right package).

Instance Variables
	preloads:	generated modifications
	currentRemovals:		definitions removed by the version currently being added
	previousRemovals:		definitions removed by previously added versions
!

!MCReorganizationPreloader class methodsFor: 'loading' stamp: 'bf 5/4/2013 14:48'!
preloadMovesBetween: versions
	| loader |
	loader := self new.
	versions do: [ :version |
		loader addVersion: version].
	^ loader preload! !

!MCReorganizationPreloader methodsFor: 'patch ops' stamp: 'bf 5/4/2013 15:05'!
addDefinition: new
	previousRemovals definitionLike: new
		ifPresent: [ :old | preloads add: (MCModification of: old to: new) ]
		ifAbsent: [ "ignore" ].
! !

!MCReorganizationPreloader methodsFor: 'accessing' stamp: 'bf 5/4/2013 14:58'!
addVersion: aVersion
	| patch |
	previousRemovals addAll: currentRemovals definitions.
	currentRemovals := MCDefinitionIndex new.
	patch := aVersion canOptimizeLoading
		ifTrue: [aVersion patch]
		ifFalse: [aVersion snapshot patchRelativeToBase: aVersion package snapshot].
	patch applyTo: self.


! !

!MCReorganizationPreloader methodsFor: 'initialize-release' stamp: 'bf 5/4/2013 19:45'!
initialize
	preloads := Set new.
	previousRemovals := MCDefinitionIndex new.
	currentRemovals := MCDefinitionIndex new.
! !

!MCReorganizationPreloader methodsFor: 'patch ops' stamp: 'bf 5/4/2013 14:51'!
modifyDefinition: old to: new
	"ignore"! !

!MCReorganizationPreloader methodsFor: 'loading' stamp: 'bf 5/8/2013 17:09'!
preload
	preloads ifNotEmpty: [ | loader |
		loader := MCPackageLoader new.
		(MCPatch operations: preloads) applyTo: loader.
		loader loadWithNameLike: 'mcPreload'].
! !

!MCReorganizationPreloader methodsFor: 'patch ops' stamp: 'bf 5/4/2013 15:05'!
removeDefinition: old
	currentRemovals add: old.
! !

Object subclass: #MCRepository
	instanceVariableNames: 'creationTemplate storeDiffs'
	classVariableNames: 'Settings'
	poolDictionaries: ''
	category: 'Monticello-Repositories'!

MCRepository subclass: #MCDictionaryRepository
	instanceVariableNames: 'description dict'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Repositories'!

!MCDictionaryRepository methodsFor: 'testing' stamp: 'ab 8/21/2003 12:56'!
= other
	^ self == other! !

!MCDictionaryRepository methodsFor: 'accessing' stamp: 'ab 8/20/2003 21:04'!
allVersionInfos
	^ dict values collect: [:ea | ea info]! !

!MCDictionaryRepository methodsFor: 'versions' stamp: 'ar 12/15/2010 21:59'!
allVersionNames
	"Answers all the version names in this repository"

	^ dict values collect: [:ea | ea info name]
	! !

!MCDictionaryRepository methodsFor: 'private' stamp: 'avi 8/26/2004 14:20'!
basicStoreVersion: aVersion
	dict at: aVersion info put: aVersion! !

!MCDictionaryRepository methodsFor: 'accessing' stamp: 'stephaneducasse 2/4/2006 20:47'!
closestAncestorVersionFor: anAncestry ifNone: errorBlock
	| info |
	info := anAncestry breadthFirstAncestors
			detect: [:ea | self includesVersionWithInfo: ea]
			ifNone: [^ errorBlock value].
	^ self versionWithInfo: info! !

!MCDictionaryRepository methodsFor: 'accessing' stamp: 'ab 7/26/2003 02:47'!
description

	^ description ifNil: ['cache']! !

!MCDictionaryRepository methodsFor: 'accessing' stamp: 'ab 7/26/2003 02:50'!
description: aString

	description := aString ! !

!MCDictionaryRepository methodsFor: 'accessing' stamp: 'ab 7/26/2003 02:47'!
dictionary

	^ dict! !

!MCDictionaryRepository methodsFor: 'accessing' stamp: 'ab 7/26/2003 02:47'!
dictionary: aDictionary

	dict := aDictionary! !

!MCDictionaryRepository methodsFor: 'versions' stamp: 'cmm 3/5/2011 18:32'!
includesVersionNamed: aString 
	| mcVersionName |
	mcVersionName := aString asMCVersionName.
	^ dict anySatisfy:
		[ : ea | ea info versionName = mcVersionName ]! !

!MCDictionaryRepository methodsFor: 'testing' stamp: 'ab 8/21/2003 19:49'!
includesVersionWithInfo: aVersionInfo
	^ dict includesKey: aVersionInfo! !

!MCDictionaryRepository methodsFor: 'initialize-release' stamp: 'avi 2/12/2004 19:33'!
initialize

	dict := Dictionary new.
! !

!MCDictionaryRepository methodsFor: 'user interface' stamp: 'ar 8/6/2009 18:24'!
morphicOpen: aWorkingCopy
	| names index infos |
	infos := self sortedVersionInfos.
	infos isEmpty ifTrue: [^ self inform: 'No versions'].
	names := infos collect: [:ea | ea name].
	index := UIManager default chooseFrom: names title: 'Open version:'.
	index = 0 ifFalse: [(self versionWithInfo: (infos at: index)) open]! !

!MCDictionaryRepository methodsFor: 'accessing' stamp: 'ab 8/21/2003 12:40'!
sortedVersionInfos
	| sorter |
	sorter := MCVersionSorter new.
	self allVersionInfos do: [:ea | sorter addVersionInfo: ea].
	^ sorter sortedVersionInfos
! !

!MCDictionaryRepository methodsFor: 'versions' stamp: 'ab 8/16/2003 18:22'!
versionWithInfo: aVersionInfo ifAbsent: errorBlock
	^ dict at: aVersionInfo ifAbsent: errorBlock! !

MCRepository subclass: #MCFileBasedRepository
	instanceVariableNames: 'cache allFileNamesCache allVersionNamesCache'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Repositories'!

MCFileBasedRepository subclass: #MCDirectoryRepository
	instanceVariableNames: 'directory'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Repositories'!

MCDirectoryRepository subclass: #MCCacheRepository
	instanceVariableNames: 'packageCaches seenFiles'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Repositories'!

MCCacheRepository class
	instanceVariableNames: 'default'!

MCCacheRepository class
	instanceVariableNames: 'default'!

!MCCacheRepository class methodsFor: 'accessing' stamp: 'avi 10/9/2003 12:46'!
cacheDirectory
	^ (FileDirectory default directoryNamed: 'package-cache')
		assureExistence;
		yourself! !

!MCCacheRepository class methodsFor: 'initialize-release' stamp: 'cmm 1/28/2014 10:52'!
cleanUp: aggressive
	self reset! !

!MCCacheRepository class methodsFor: 'accessing' stamp: 'cmm 8/30/2013 14:57'!
default
	(default isNil or: [ default directory exists not ]) ifTrue: [ default := self new directory: self cacheDirectory ].
	^ default! !

!MCCacheRepository class methodsFor: 'accessing' stamp: 'avi 10/9/2003 12:56'!
description
	^ nil! !

!MCCacheRepository class methodsFor: 'initialize-release' stamp: 'cmm 1/28/2014 10:52'!
reset
	default := nil! !

!MCCacheRepository methodsFor: 'private' stamp: 'bf 3/28/2011 16:07'!
basicStoreVersion: aVersion
	(aVersion isCacheable not or: [self allFileNames includes: aVersion fileName])
		ifFalse: [super basicStoreVersion: aVersion]
! !

!MCCacheRepository methodsFor: 'caching' stamp: 'avi 1/22/2004 18:13'!
cacheForPackage: aPackage
	packageCaches ifNil: [packageCaches := Dictionary new].
	^ packageCaches at: aPackage ifAbsentPut: [MCPackageCache new]! !

!MCCacheRepository methodsFor: 'accessing' stamp: 'avi 1/22/2004 18:15'!
newFileNames 
	^ self allFileNames difference: self seenFileNames! !

!MCCacheRepository methodsFor: 'private' stamp: 'pre 12/7/2018 09:12'!
rescan

	self flag: #todo. "This method seems to be unused --pre"
	self newFileNames do:
		[:ea |
		self versionReaderForFileNamed: ea do:
			[:reader |
			(self cacheForPackage: reader package)
				recordVersionInfo: reader info
				forFileNamed: ea.
			self seenFileNames add: ea]]
		displayingProgress: 'Scanning cache...'! !

!MCCacheRepository methodsFor: 'accessing' stamp: 'avi 1/22/2004 18:15'!
seenFileNames
	^ seenFiles ifNil: [seenFiles := OrderedCollection new]! !

!MCCacheRepository methodsFor: 'caching' stamp: 'eem 3/13/2017 18:50'!
versionInfoForFileNamed: aString
	^ cache at: aString ifAbsentPut: [self versionReaderForFileNamed: aString do: [:r | r info]]! !

!MCDirectoryRepository class methodsFor: 'configuring' stamp: 'cmm 4/22/2011 16:13'!
creationTemplateLocation: location 
	^ self name , ' path: {1}' format: {location printString}! !

!MCDirectoryRepository class methodsFor: 'configuring' stamp: 'ab 7/24/2003 21:20'!
description
	^ 'directory'! !

!MCDirectoryRepository class methodsFor: 'instance creation' stamp: 'cmm 4/22/2011 16:54'!
directory: aFileDirectory 
	^ self new
		 directory: aFileDirectory ;
		 yourself! !

!MCDirectoryRepository class methodsFor: 'configuring' stamp: 'cmm 4/22/2011 16:00'!
fillInTheBlankRequest
	^ 'Directory Location:'! !

!MCDirectoryRepository class methodsFor: 'configuring' stamp: 'tpr 12/23/2017 18:17'!
morphicConfigure
	^ UIManager default chooseDirectory
		 ifNotNil: [:directory | self new directory: directory]! !

!MCDirectoryRepository class methodsFor: 'instance creation' stamp: 'cmm 4/22/2011 16:58'!
path: fullyQualifiedName 
	^ self directory: (FileDirectory on: fullyQualifiedName)! !

!MCDirectoryRepository methodsFor: 'accessing' stamp: 'ul 4/26/2011 04:11'!
allFileNames

	^directory entries 
		sort: [ :a :b | a modificationTime >= b modificationTime ];
		replace: [ :ea | ea name asMCVersionName ]! !

!MCDirectoryRepository methodsFor: 'accessing' stamp: 'cmm 4/22/2011 16:59'!
creationTemplate
	^ super creationTemplate
		ifNil:
			[ self creationTemplate: (self class creationTemplateLocation: self directory pathName).
			creationTemplate ]! !

!MCDirectoryRepository methodsFor: 'user interface' stamp: 'avi 10/9/2003 12:57'!
description
	^ directory pathName! !

!MCDirectoryRepository methodsFor: 'accessing' stamp: 'ab 7/6/2003 17:49'!
directory
	^ directory! !

!MCDirectoryRepository methodsFor: 'accessing' stamp: 'ab 7/6/2003 12:56'!
directory: aDirectory
	directory := aDirectory! !

!MCDirectoryRepository methodsFor: 'comparing' stamp: 'ab 7/19/2003 21:40'!
hash
	^ directory pathName hash! !

!MCDirectoryRepository methodsFor: 'initialize-release' stamp: 'ab 8/21/2003 12:37'!
initialize
	directory := FileDirectory default! !

!MCDirectoryRepository methodsFor: 'testing' stamp: 'nk 11/2/2003 10:55'!
isValid
	^directory exists! !

!MCDirectoryRepository methodsFor: 'private' stamp: 'ab 8/21/2003 12:45'!
readStreamForFileNamed: aString do: aBlock
	| file val |
	file := FileStream readOnlyFileNamed: (directory fullNameFor: aString).
	val := aBlock value: file.
	file close.
	^ val! !

!MCDirectoryRepository methodsFor: 'private' stamp: 'avi 10/31/2003 14:35'!
writeStreamForFileNamed: aString replace: aBoolean do: aBlock
	| file sel |
	sel := aBoolean ifTrue: [#forceNewFileNamed:] ifFalse: [#newFileNamed:].
	file := FileStream perform: sel with: (directory fullNameFor: aString).
	aBlock value: file.
	file close.! !

MCDirectoryRepository subclass: #MCSubDirectoryRepository
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Repositories'!

!MCSubDirectoryRepository commentStamp: 'nk 6/11/2004 18:56' prior: 0!
A MCDirectoryRepository that looks in subdirectories too.!

!MCSubDirectoryRepository class methodsFor: 'user interface' stamp: 'nk 6/11/2004 18:48'!
description
	^ 'directory with subdirectories'! !

!MCSubDirectoryRepository methodsFor: 'enumerating' stamp: 'nk 6/11/2004 18:55'!
allDirectories
	| remaining dir dirs |
	remaining := OrderedCollection new.
	dirs := OrderedCollection new.
	remaining addLast: directory.
	[remaining isEmpty]
		whileFalse: [dir := remaining removeFirst.
			dirs add: dir.
			dir entries
				do: [:ent | ent isDirectory
						ifTrue: [remaining
								addLast: (dir directoryNamed: ent name)]]].
	^ dirs! !

!MCSubDirectoryRepository methodsFor: 'enumerating' stamp: 'ul 3/13/2017 02:59'!
allFileNames
	"sorting {entry. dirName. name}"

	| result |
	result := OrderedCollection new.
	self allDirectories
		do: [:dir | dir entries
				do: [:ent | ent isDirectory
						ifFalse: [result addLast: {ent. dir fullName. ent name}]]].
	^result
		sort: [:a :b | a first modificationTime >= b first modificationTime ];
		replace: [:ea | ea third asMCVersionName]! !

!MCSubDirectoryRepository methodsFor: 'user interface' stamp: 'ul 3/23/2011 07:21'!
description
	^ directory pathName, FileDirectory slash, '*'! !

!MCSubDirectoryRepository methodsFor: 'private' stamp: 'ul 3/13/2017 03:01'!
findFullNameForReading: aBaseName
	"Answer the latest version of aBaseName"
	| possible |
	possible := OrderedCollection new.
	self allDirectories
		do: [:dir | dir entries
				do: [:ent | ent isDirectory
						ifFalse: [
							(ent name = aBaseName) ifTrue: [ possible addLast: {ent. dir fullNameFor: ent name}]]]].
	possible isEmpty ifTrue: [ ^nil ].
	^(possible detectMin: [ :each | each first modificationTime ]) second! !

!MCSubDirectoryRepository methodsFor: 'private' stamp: 'ul 3/13/2017 03:03'!
findFullNameForWriting: aBaseName

	| possible split prefix fpattern now |
	split := directory splitNameVersionExtensionFor: aBaseName.
	fpattern := split first, '*'.
	possible := OrderedCollection new.
	now := Time totalSeconds.
	prefix := directory pathParts size.
	self allDirectories do: [:dir | | parts dirScore fileScore |
		parts := dir pathParts allButFirst: prefix.
		dirScore := (parts select: [ :part | fpattern match: part ]) size.
		fileScore := (dir entries collect: [ :ent |
			(ent isDirectory not and: [ fpattern match: ent name ])
				ifFalse: [ SmallInteger maxVal ]
				ifTrue: [ now - ent modificationTime ]]).	"minimum age"
		fileScore := fileScore isEmpty ifTrue: [ SmallInteger maxVal  ]
			ifFalse: [ fileScore min ].
		possible add: { dirScore. fileScore. dir } ].
	possible
		sort: [ :a :b |
			a first = b first
				ifTrue: [ a second = b second
						ifFalse: [ a second < b second ]
						ifTrue: [ a third fullName size < b third fullName size ]]
				ifFalse: [ a first > b first ] ].
	^(possible first third) fullNameFor: aBaseName! !

!MCSubDirectoryRepository methodsFor: 'private' stamp: 'nk 6/11/2004 20:32'!
readStreamForFileNamed: aString do: aBlock
	| file val |
	file := FileStream readOnlyFileNamed: (self findFullNameForReading: aString).
	val := aBlock value: file.
	file close.
	^ val! !

!MCSubDirectoryRepository methodsFor: 'private' stamp: 'nk 6/11/2004 20:34'!
writeStreamForFileNamed: aString replace: aBoolean do: aBlock 
	| file |
	file := aBoolean
				ifTrue: [FileStream
						forceNewFileNamed: (self findFullNameForReading: aString)]
				ifFalse: [FileStream
						newFileNamed: (self findFullNameForWriting: aString)].
	aBlock value: file.
	file close! !

!MCFileBasedRepository class methodsFor: 'release' stamp: 'ar 2/26/2010 23:03'!
cleanUp
	"Flush caches"

	self flushAllCaches.! !

!MCFileBasedRepository class methodsFor: 'release' stamp: 'avi 2/3/2005 00:43'!
flushAllCaches
	self allSubInstancesDo: [:ea | ea flushCache]! !

!MCFileBasedRepository methodsFor: 'private-files' stamp: 'ab 8/21/2003 00:35'!
allFileNames
	self subclassResponsibility! !

!MCFileBasedRepository methodsFor: 'private-files' stamp: 'cmm 3/6/2011 16:24'!
allFileNamesForVersionNamed: aMCVersionName 
	^ self
		filterFileNames: self readableFileNames
		forVersionNamed: aMCVersionName! !

!MCFileBasedRepository methodsFor: 'private-files' stamp: 'ul 5/3/2013 23:58'!
allFileNamesOrCache
	
	^allFileNamesCache ifNil: [ self allFileNames ]! !

!MCFileBasedRepository methodsFor: 'packages' stamp: 'cmm 8/8/2012 14:34'!
allPackageAndBranchNames
	^ self validNames: #packageAndBranchName! !

!MCFileBasedRepository methodsFor: 'packages' stamp: 'cmm 8/8/2012 14:34'!
allPackageNames
	^ self validNames: #packageName! !

!MCFileBasedRepository methodsFor: 'private-files' stamp: 'ul 5/3/2013 23:55'!
allVersionNames

	^allVersionNamesCache ifNil: [
		self readableFileNames collect: [ :each | each versionName ] ]! !

!MCFileBasedRepository methodsFor: 'private' stamp: 'cmm 4/24/2011 18:41'!
basicStoreVersion: aMCVersionOrConfiguration 
	self
		writeStreamForFileNamed: aMCVersionOrConfiguration fileName
		do: [ : s | aMCVersionOrConfiguration fileOutOn: s ].
	aMCVersionOrConfiguration isCacheable ifTrue:
		[ cache ifNil: [ cache := Dictionary new ].
		self resizeCache: cache.
		cache
			at: aMCVersionOrConfiguration fileName
			put: aMCVersionOrConfiguration ]! !

!MCFileBasedRepository methodsFor: 'private' stamp: 'stephaneducasse 2/4/2006 20:47'!
cache
	^ cache ifNil: [cache := Dictionary new]! !

!MCFileBasedRepository methodsFor: 'private' stamp: 'cmm 10/2/2013 22:00'!
cacheAllFileNamesDuring: aBlock 
	"Cache the result of #allFileNames and #allVersionNames during aBlock"
	allFileNamesCache ifNotNil: [ ^ aBlock value ].
	self cacheAllFilenames.
	^ aBlock ensure: [ allFileNamesCache := allVersionNamesCache := nil ]! !

!MCFileBasedRepository methodsFor: 'private' stamp: 'cmm 5/5/2014 22:15'!
cacheAllFilenames
	allFileNamesCache ifNil:
		[ allFileNamesCache := self allFileNames.
		allVersionNamesCache := self allVersionNames ]! !

!MCFileBasedRepository methodsFor: 'private' stamp: 'bf 6/9/2005 15:47'!
cachedFileNames
	^cache == nil
		ifTrue: [#()]
		ifFalse: [cache keys]! !

!MCFileBasedRepository methodsFor: 'private-files' stamp: 'avi 1/21/2004 22:57'!
canReadFileNamed: aString
	| reader |
	reader := MCVersionReader readerClassForFileNamed: aString.
	^ reader notNil! !

!MCFileBasedRepository methodsFor: 'accessing' stamp: 'avi 9/17/2005 16:40'!
closestAncestorVersionFor: anAncestry ifNone: errorBlock
	^ self cacheAllFileNamesDuring:
		[super closestAncestorVersionFor: anAncestry ifNone: errorBlock]! !

!MCFileBasedRepository methodsFor: 'private-files' stamp: 'cmm 3/6/2011 16:24'!
filterFileNames: aCollection forVersionNamed: aMCVersionName 
	^ aCollection select:
		[ : ea | aMCVersionName = ea ]! !

!MCFileBasedRepository methodsFor: 'private' stamp: 'cmm 10/2/2013 22:04'!
flushAllFilenames
	allFileNamesCache := allVersionNamesCache := nil! !

!MCFileBasedRepository methodsFor: 'private' stamp: 'cmm 10/2/2013 22:04'!
flushCache
	self flushAllFilenames.
	cache := nil! !

!MCFileBasedRepository methodsFor: 'versions' stamp: 'ul 5/3/2013 23:09'!
includesVersionNamed: aString 

	| comparable |
	comparable := ((aString endsWith: '.mcz') and: [ aString size > 4 ])
		ifTrue: [ aString allButLast: 4 ]
		ifFalse: [ aString ].
	^ self allVersionNames includes: comparable! !

!MCFileBasedRepository methodsFor: 'private-files' stamp: 'avi 1/22/2004 13:34'!
loadVersionFromFileNamed: aString
	^ self versionReaderForFileNamed: aString do: [:r | r version]! !

!MCFileBasedRepository methodsFor: 'private-files' stamp: 'avi 9/17/2005 18:37'!
loadVersionInfoFromFileNamed: aString
	^ self versionReaderForFileNamed: aString do: [:r | r info]
	! !

!MCFileBasedRepository methodsFor: 'private' stamp: 'avi 9/17/2005 16:52'!
maxCacheSize
	^ 8! !

!MCFileBasedRepository methodsFor: 'user interface' stamp: 'avi 2/28/2004 18:32'!
morphicOpen: aWorkingCopy
	(MCFileRepositoryInspector repository: self workingCopy: aWorkingCopy)
		show! !

!MCFileBasedRepository methodsFor: 'notifying' stamp: 'nice 6/11/2010 21:51'!
notifyList
	(self allFileNames includes: 'notify') ifFalse: [^ #()].
	^ self readStreamForFileNamed: 'notify' do:
		[:s |
		s upToEnd lines]! !

!MCFileBasedRepository methodsFor: 'private-files' stamp: 'ul 5/8/2011 03:26'!
readableFileNames
	"Answer an Array of MCVersionNames representing every file in this repository; first the ones from the cache (whether or not they still exist in the repository) followed by every other file in this repository that Monticello can read."
	| all cached new |
	all := self allFileNamesOrCache.	"from repository"
	cached := self cachedFileNames.	"in memory"
	"Create a sparse Set, so we can avoid #= for MCVersionNames during the difference calculation."
	new := all difference: (
		(Set new: (cached size * 8 min: all size))
			addAll: cached;
			yourself).
	^ (cached asArray, new)
		select: [:ea | self canReadFileNamed: ea]! !

!MCFileBasedRepository methodsFor: 'accessing' stamp: 'cmm 1/4/2020 15:41'!
refresh
	super refresh.
	self flushCache! !

!MCFileBasedRepository methodsFor: 'private' stamp: 'avi 9/18/2005 22:43'!
resizeCache: aDictionary
	[aDictionary size <= self maxCacheSize] whileFalse:
		[aDictionary removeKey: aDictionary keys atRandom]! !

!MCFileBasedRepository methodsFor: 'private' stamp: 'cmm 8/8/2012 14:33'!
validNames: selectorSymbol
	| answer |
	answer := Set new.
	self allFileNamesOrCache do:
		[ : each | | versionName |
		versionName := each asMCVersionName.
		versionName isValid ifTrue: [ answer add: (versionName perform: selectorSymbol) ] ].
	^ answer! !

!MCFileBasedRepository methodsFor: 'private-files' stamp: 'avi 9/17/2005 18:37'!
versionInfoFromFileNamed: aString
	self cache at: aString ifPresent: [:v | ^ v info].
	^ self loadVersionInfoFromFileNamed: aString! !

!MCFileBasedRepository methodsFor: 'versions' stamp: 'cmm 4/14/2013 21:17'!
versionNamed: aMCVersionName 
	"For FileBased repositories, aMCVersionName must have the appropriate extension!!  :-("
	| version |
	version := self cache
		at: aMCVersionName
		ifAbsent:
			[ [ self loadVersionFromFileNamed: aMCVersionName ]
				on: FileDoesNotExistException
				do: [ : err | nil ] ].
	self resizeCache: cache.
	(version notNil and: [ version isCacheable ]) ifTrue:
		[ cache
			at: aMCVersionName asMCVersionName
			put: version ].
	^ version! !

!MCFileBasedRepository methodsFor: 'versions' stamp: 'bf 5/29/2012 12:02'!
versionNamesForPackageNamed: packageName
	^ Array streamContents:
		[ : stream | | wantBranch |
		wantBranch := packageName includes: $..
		self allFileNamesOrCache do:
			[ : each | | mcVersionName branchName |
			mcVersionName := each asMCVersionName.
			branchName := wantBranch
				ifTrue: [mcVersionName packageAndBranchName]
				ifFalse: [mcVersionName packageName].
			packageName = branchName ifTrue: [ stream nextPut: mcVersionName ] ] ]! !

!MCFileBasedRepository methodsFor: 'private-files' stamp: 'ul 12/12/2009 14:10'!
versionReaderForFileNamed: aString do: aBlock
	^ self
		readStreamForFileNamed: aString
		do: [:s |
			(MCVersionReader readerClassForFileNamed: aString) ifNotNil:
				[:class | aBlock value: (class on: s fileName: aString)]]
! !

!MCFileBasedRepository methodsFor: 'versions' stamp: 'cmm 3/6/2011 16:27'!
versionWithInfo: aVersionInfo ifAbsent: errorBlock
	(self allFileNamesForVersionNamed: aVersionInfo versionName) do:
		[:fileName | | version |
		version := self versionNamed: fileName.
		version info = aVersionInfo ifTrue: [^ version]].
	^ errorBlock value! !

!MCFileBasedRepository methodsFor: 'private-files' stamp: 'avi 10/31/2003 14:32'!
writeStreamForFileNamed: aString do: aBlock
	^ self writeStreamForFileNamed: aString replace: false do: aBlock! !

MCFileBasedRepository subclass: #MCFtpRepository
	instanceVariableNames: 'host directory user password connection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Repositories'!

!MCFtpRepository class methodsFor: 'configuring' stamp: 'avi 9/16/2003 13:57'!
creationTemplate
	^
'MCFtpRepository
	host: ''modules.squeakfoundation.org''
	directory: ''mc''
	user: ''squeak''
	password: ''squeak'''
	! !

!MCFtpRepository class methodsFor: 'configuring' stamp: 'avi 9/16/2003 13:57'!
description
	^ 'FTP'! !

!MCFtpRepository class methodsFor: 'configuring' stamp: 'bkv 2/18/2004 21:02'!
fillInTheBlankRequest
	^ 'FTP Repository:'

	! !

!MCFtpRepository class methodsFor: 'instance creation' stamp: 'avi 9/16/2003 13:57'!
host: host directory: directory user: user password: password
	^ self new
		host: host;
		directory: directory;
		user: user;
		password: password! !

!MCFtpRepository class methodsFor: 'configuring' stamp: 'bkv 2/18/2004 21:02'!
morphicConfigure
	^ self fillInTheBlankConfigure! !

!MCFtpRepository class methodsFor: 'constants' stamp: 'bkv 2/18/2004 20:38'!
templateCreationSelector
	^ #host:directory:user:password: ! !

!MCFtpRepository methodsFor: 'required' stamp: 'avi 9/16/2003 14:04'!
allFileNames
	^ self clientDo:
		[:client |
		self parseDirectoryListing: client getDirectory]! !

!MCFtpRepository methodsFor: 'private' stamp: 'avi 9/16/2003 16:57'!
clientDo: aBlock
	| client |
	client := FTPClient openOnHostNamed: host.
	client loginUser: user password: password.
	directory isEmpty ifFalse: [client changeDirectoryTo: directory].
	^ [aBlock value: client] ensure: [client close]! !

!MCFtpRepository methodsFor: 'required' stamp: 'avi 9/17/2003 12:52'!
description
	^ 'ftp://', user, '@', host, '/', directory! !

!MCFtpRepository methodsFor: 'accessing' stamp: 'avi 9/16/2003 13:56'!
directory: dirPath
	directory := dirPath! !

!MCFtpRepository methodsFor: 'accessing' stamp: 'avi 9/16/2003 13:57'!
host: hostname
	host := hostname! !

!MCFtpRepository methodsFor: 'protocol handling' stamp: 'cmm 3/3/2011 14:54'!
parseDirectoryListing: aString
	| stream files line tokens |
	stream := aString readStream.
	files := OrderedCollection new.
	[stream atEnd] whileFalse:
		[line := stream nextLine.
		tokens := line findTokens: ' '.
		tokens size > 2 ifTrue: [files add: tokens last asMCVersionName]].
	^ files! !

!MCFtpRepository methodsFor: 'accessing' stamp: 'avi 9/16/2003 13:56'!
password: passwordString
	password := passwordString! !

!MCFtpRepository methodsFor: 'required' stamp: 'nice 12/27/2009 03:11'!
readStreamForFileNamed: aString do: aBlock
	
	^ self clientDo:
		[:client | | stream |
		client binary.
		stream := RWBinaryOrTextStream on: String new.
		stream nextPutAll: (client getFileNamed: aString).
		aBlock value: stream reset]! !

!MCFtpRepository methodsFor: 'accessing' stamp: 'avi 9/16/2003 13:56'!
user: userString
	user := userString! !

!MCFtpRepository methodsFor: 'required' stamp: 'avi 10/31/2003 14:35'!
writeStreamForFileNamed: aString replace: ignoreBoolean do: aBlock
	| stream |
	stream := RWBinaryOrTextStream on: String new.
	aBlock value: stream.
	self clientDo:
		[:client |
		client binary.
		client putFileStreamContents: stream reset as: aString]! !

MCFileBasedRepository subclass: #MCHttpRepository
	instanceVariableNames: 'location user password readerCache indexed webClient'
	classVariableNames: 'UseSharedWebClientInstance'
	poolDictionaries: ''
	category: 'Monticello-Repositories'!

!MCHttpRepository class methodsFor: 'class initialization' stamp: 'mt 8/9/2016 14:35'!
cleanUp: aggressive
	
	aggressive ifTrue: [ self clearCredentials ]! !

!MCHttpRepository class methodsFor: 'class initialization' stamp: 'cmm 5/18/2015 20:50'!
clearCredentials
	self allSubInstancesDo: [ : each | each clearCredentials ]! !

!MCHttpRepository class methodsFor: 'ui-support' stamp: 'cmm 1/25/2011 13:00'!
creationTemplate
	^self creationTemplateLocation: 'http://www.squeaksource.com/ProjectName'
		user: 'squeak'
		password: 'squeak'
! !

!MCHttpRepository class methodsFor: 'ui-support' stamp: 'bf 4/14/2005 15:27'!
creationTemplateLocation: location user: user password: password
	^
'MCHttpRepository
	location: {1}
	user: {2}
	password: {3}' format: {location printString. user printString. password printString}! !

!MCHttpRepository class methodsFor: 'ui-support' stamp: 'ab 7/24/2003 21:20'!
description
	^ 'HTTP'! !

!MCHttpRepository class methodsFor: 'ui-support' stamp: 'bkv 2/18/2004 21:00'!
fillInTheBlankRequest
	^ 'HTTP Repository:'
			! !

!MCHttpRepository class methodsFor: 'well-known repositories' stamp: 'mt 8/2/2016 15:20'!
inbox
	^ MCRepositoryGroup default repositories
		detect:
			[ : each | each isInbox ]
		ifNone:
			[ MCHttpRepository
				location: MCHttpRepository inboxUrlString
				user: 'squeak'
				password: 'squeak' ]! !

!MCHttpRepository class methodsFor: 'accessing' stamp: 'mt 8/2/2016 15:19'!
inboxUrlString
	^ 'http://source.squeak.org/inbox'! !

!MCHttpRepository class methodsFor: 'class initialization' stamp: 'cmm 11/27/2016 16:03'!
initialize
	self unload.
	(ServiceRegistry current serviceWithId: #browserMethodMenu) services add: self browseMethodRevisionsService.
	(ServiceRegistry current serviceWithId: #browserClassMenu) services add: self browseClassRevisionsService! !

!MCHttpRepository class methodsFor: 'ui-support' stamp: 'ab 7/24/2003 21:32'!
location: location user: user password: password
	^ self new
		location: location;
		user: user;
		password: password! !

!MCHttpRepository class methodsFor: 'ui-support' stamp: 'bkv 2/18/2004 21:01'!
morphicConfigure
	^ self fillInTheBlankConfigure! !

!MCHttpRepository class methodsFor: 'well-known repositories' stamp: 'mt 10/10/2019 09:11'!
treated
	^ MCRepositoryGroup default repositories
		detect:
			[ : each | each isTreated ]
		ifNone:
			[ MCHttpRepository
				location: MCHttpRepository treatedUrlString
				user: 'squeak'
				password: 'squeak' ]! !

!MCHttpRepository class methodsFor: 'accessing' stamp: 'mt 10/10/2019 09:10'!
treatedUrlString
	^ 'http://source.squeak.org/treated'! !

!MCHttpRepository class methodsFor: 'well-known repositories' stamp: 'cmm 10/2/2013 13:59'!
trunk
	^ MCRepositoryGroup default repositories
		detect:
			[ : each | each isTrunk ]
		ifNone:
			[ MCHttpRepository
				location: MCHttpRepository trunkUrlString
				user: 'squeak'
				password: 'squeak' ]! !

!MCHttpRepository class methodsFor: 'accessing' stamp: 'cmm 10/2/2013 13:53'!
trunkUrlString
	^ 'http://source.squeak.org/trunk'! !

!MCHttpRepository class methodsFor: 'class initialization' stamp: 'cmm 11/27/2016 16:23'!
unload
	| methodMenuServiceCategory classMenuServiceCategory |
	methodMenuServiceCategory := ServiceRegistry current serviceWithId: #browserMethodMenu.
	methodMenuServiceCategory services copy do:
		[ : each | (#(#browseMcMethodRevisions #browseMcMethodOrigin ) includes: each id) ifTrue: [ methodMenuServiceCategory services remove: each ] ].
	classMenuServiceCategory := ServiceRegistry current serviceWithId: #browserClassMenu.
	classMenuServiceCategory services copy do:
		[ : each | (#(#browseMcClassRevisions #browseMcClassOrigin ) includes: each id) ifTrue: [ classMenuServiceCategory services remove: each ] ]! !

!MCHttpRepository class methodsFor: 'preferences' stamp: 'nice 3/3/2020 20:27'!
useSharedWebClientInstance
	
	<preference: 'Use shared WebClient instance'
		category: 'Monticello'
		description: 'When true, use a shared WebClient instance to speed up downloads from MCHttpRepositories. Requires WebClient to be present.'
		type: #Boolean>
	^UseSharedWebClientInstance ifNil: [
		"There is some issue on Windows and Macos, so don't use it there by default. See http://lists.squeakfoundation.org/pipermail/squeak-dev/2019-September/thread.html#203921 for details."
		Smalltalk os platformName ~= 'Win32' and: [Smalltalk os platformName ~= 'Mac OS']]! !

!MCHttpRepository class methodsFor: 'preferences' stamp: 'ul 3/18/2018 22:33'!
useSharedWebClientInstance: aBoolean
	
	UseSharedWebClientInstance := aBoolean! !

!MCHttpRepository methodsFor: 'private-files' stamp: 'ul 3/18/2018 21:56'!
allFileNames

	| index |
	index := self displayProgress: 'Updating ', self description during: [
		self httpGet: self locationWithTrailingSlash, '?C=M;O=D' arguments: nil ].
	^index ifNotNil: [ self parseFileNamesFromStream: index ]! !

!MCHttpRepository methodsFor: 'accessing' stamp: 'bf 4/14/2005 17:49'!
asCreationTemplate
	^self class creationTemplateLocation: location user: user password: password! !

!MCHttpRepository methodsFor: 'accessing' stamp: 'mt 6/9/2020 16:24'!
clearCredentials
	user beWritableObject.
	user ifNotNil: [user atAllPut: $x].
	password beWritableObject.
	password ifNotNil: [password atAllPut: $x].
	user := password := String empty! !

!MCHttpRepository methodsFor: 'accessing' stamp: 'mt 3/17/2018 20:42'!
creationTemplate
	^ self asCreationTemplate! !

!MCHttpRepository methodsFor: 'accessing' stamp: 'mt 3/17/2018 20:41'!
creationTemplate: aString
	creationTemplate := nil.! !

!MCHttpRepository methodsFor: 'user interface' stamp: 'ab 7/24/2003 21:10'!
description
	^ location! !

!MCHttpRepository methodsFor: 'private' stamp: 'kb 5/24/2013 20:17'!
displayProgress: label during: workBlock
	| nextUpdateTime |
	nextUpdateTime := 0.
	^label displayProgressFrom: 0.0 to: 1.0 during:[:bar|
		bar value: 0.0.
		workBlock on: HTTPProgress do:[:ex|
			(ex total == nil or:[ex amount == nil]) ifFalse:[
				(nextUpdateTime < Time millisecondClockValue 
					or:[ex total = ex amount]) ifTrue:[
						bar value: ex amount asFloat / ex total asFloat.
						nextUpdateTime := Time millisecondClockValue + 100.
				].
			].
			ex resume.
		]
	].
! !

!MCHttpRepository methodsFor: 'private' stamp: 'al 12/12/2005 11:06'!
flushCache
	super flushCache.
	readerCache := nil.! !

!MCHttpRepository methodsFor: 'private' stamp: 'ct 9/17/2020 10:25'!
httpGet: url arguments: arguments

	| progress urlString client  response result |
	progress := [ :total :amount |
		HTTPProgress new 
			total: total;
			amount: amount;
			signal: 'Downloading...' translated ].
	urlString := arguments
		ifNil: [ url ]
		ifNotNil: [ 
			| queryString |
			queryString := WebUtils encodeUrlEncodedForm: arguments.
			(url includes: $?)
				ifTrue: [ url, '&', queryString ]
				ifFalse: [ url, '?', queryString ] ].
	self class useSharedWebClientInstance ifTrue: [
		"Acquire webClient by atomically storing it in the client variable and setting its value to nil."
		client := webClient.
		webClient := nil ].
	client 
		ifNil: [ client := WebClient new ]
		ifNotNil: [ 
			"Attempt to avoid an error on windows by recreating the underlying stream."
			client isConnected ifFalse: [ client close ] ].
	response := client
		username: self user;
		password: self password;
		httpGet: urlString do: [ :request |
			request
				headerAt: 'Authorization' put: 'Basic ', (self user, ':', self password) base64Encoded;
				headerAt: 'Connection' put: 'Keep-Alive';
				headerAt: 'Accept' put: '*/*' ].
	result := (response code between: 200 and: 299) 
		ifFalse: [
			response content. "Make sure content is read."
			nil ]
		ifTrue: [ (RWBinaryOrTextStream with: (response contentWithProgress: progress)) reset ].
	self class useSharedWebClientInstance
		ifTrue: [
			"Save the WebClient instance for reuse, but only if there is no client cached."
			webClient  
				ifNil: [ webClient := client ]
				ifNotNil: [ client close ] ]
		ifFalse: [ client close ].
	result ifNil: [ NetworkError signal: ('Could not access {1} (Code {2})' translated format: {location. response code}) ].
	^result! !

!MCHttpRepository methodsFor: 'private' stamp: 'cmm 9/7/2013 11:16'!
httpGet: actionString for: aMCDefinition
	^ HTTPSocket
		httpGet: self locationWithTrailingSlash
		args: 
			{ 'action'->{actionString}.
			'mc-definition'-> {self serializeForRequest: aMCDefinition}}
		user: self user
		passwd: self password! !

!MCHttpRepository methodsFor: 'testing' stamp: 'mt 8/2/2016 15:20'!
isInbox
	^ location = self class inboxUrlString! !

!MCHttpRepository methodsFor: 'private' stamp: 'cmm 6/30/2016 23:17'!
isIndexed
	^ indexed ifNil:
		[indexed := [(HTTPSocket
			httpGet: self locationWithTrailingSlash
			args: {'query' -> {'isHistorySupported'}}
			user: self user
			passwd: self password) contents = 'true']
			on: Error
			do:
				[:err | false]]! !

!MCHttpRepository methodsFor: 'testing' stamp: 'mt 10/10/2019 09:11'!
isTreated
	^ location = self class treatedUrlString! !

!MCHttpRepository methodsFor: 'testing' stamp: 'cmm 10/2/2013 13:57'!
isTrunk
	^ location = self class trunkUrlString! !

!MCHttpRepository methodsFor: 'accessing' stamp: 'ab 7/24/2003 20:41'!
location: aUrlString
	location := aUrlString! !

!MCHttpRepository methodsFor: 'accessing' stamp: 'ab 7/24/2003 22:17'!
locationWithTrailingSlash
	^ (location endsWith: '/')
		ifTrue: [location]
		ifFalse: [location, '/']! !

!MCHttpRepository methodsFor: '*monticello' stamp: 'cmm 6/30/2016 23:17'!
mcModel
	"Answer the object which can respond to #historyOf: and #originOf: or nil, if none."
	^ self isIndexed ifTrue: [self]! !

!MCHttpRepository methodsFor: 'accessing' stamp: 'cmm 7/17/2016 19:33'!
originOf: aMCDefinition 
	| reply |
	reply := self
		httpGet: 'origin'
		for: aMCDefinition.
	^ reply isString
		ifTrue:
			[ Warning signal: 'Origin request failed.  Server ''reply'' in debugger.'.
			Array empty ]
		ifFalse: [ (ReferenceStream on: reply) next ]! !

!MCHttpRepository methodsFor: 'private' stamp: 'cmm 3/3/2011 14:53'!
parseFileNamesFromStream: aStream
	| names fullName |
	names := OrderedCollection new.
	[aStream atEnd] whileFalse:
		[[aStream upTo: $<. {$a. $A. nil} includes: aStream next] whileFalse.
		aStream upTo: $".
		aStream atEnd ifFalse: [
			fullName := aStream upTo: $".
			names add: fullName unescapePercents asMCVersionName ]].
	^ names! !

!MCHttpRepository methodsFor: 'accessing' stamp: 'cmm 3/14/2014 16:16'!
password
	self userAndPasswordFromSettingsDo: [:usr :pwd | ^pwd].

	self user isEmpty ifTrue: [^password ifNil: ['']].

	password isEmptyOrNil ifTrue: [
		| answer |
		"Give the user a chance to change the login"
		answer := UIManager default request: 'User name for ', String cr, location
			initialAnswer: self user.
		answer isEmpty
			ifTrue: [^password]
			ifFalse: [self user: answer].
		
		password := UIManager default requestPassword: 'Password for "', self user, '" at ', String cr, location.
	].

	^ password! !

!MCHttpRepository methodsFor: 'accessing' stamp: 'ab 7/24/2003 20:41'!
password: passwordString
	password := passwordString! !

!MCHttpRepository methodsFor: 'private' stamp: 'tpr 1/18/2019 16:12'!
readStreamForFileNamed: aString do: aBlock

	| contents attempts|
	attempts := 0.
	self displayProgress: 'Downloading ', aString during: [
		[attempts := attempts + 1.
		contents := self httpGet: (self urlForFileNamed: aString) arguments: nil] on: NetworkError do: [:ex| 
		attempts >= 3 ifTrue:[ex pass].
		ex retry ]].
	^contents ifNotNil: [ aBlock value: contents ]! !

!MCHttpRepository methodsFor: 'accessing' stamp: 'cmm 6/30/2016 23:19'!
refresh
	super refresh.
	indexed := nil! !

!MCHttpRepository methodsFor: 'accessing' stamp: 'cmm 11/27/2016 16:12'!
revisionsOf: aMCDefinition 
	| reply |
	reply := self
		httpGet: 'history'
		for: aMCDefinition.
	^ reply isString
		ifTrue:
			[ Warning signal: 'Revisions request failed.  Server ''reply'' in debugger.'.
			Array empty ]
		ifFalse: [ (ReferenceStream on: reply) next ]! !

!MCHttpRepository methodsFor: 'private' stamp: 'cmm 9/7/2013 16:08'!
serializeForRequest: aMCDefinition 
	^ ((ReferenceStream on: (RWBinaryOrTextStream on: ByteArray new))
		 nextPut: aMCDefinition ;
		 yourself) contents asString encodeForHTTP! !

!MCHttpRepository methodsFor: 'accessing' stamp: 'ab 8/21/2003 13:08'!
urlForFileNamed: aString
	^ self locationWithTrailingSlash, aString encodeForHTTP! !

!MCHttpRepository methodsFor: 'accessing' stamp: 'bf 8/12/2009 11:21'!
user
	self userAndPasswordFromSettingsDo: [:usr :pwd | ^usr].
	"not in settings"
	^user ifNil: ['']! !

!MCHttpRepository methodsFor: 'accessing' stamp: 'ab 7/24/2003 20:41'!
user: userString
	user := userString! !

!MCHttpRepository methodsFor: 'private' stamp: 'nice 12/27/2009 03:11'!
userAndPasswordFromSettingsDo: aBlock
	"The mcSettings file in ExternalSettings preferenceDirectory should contain entries for each account:
	
		account1: *myhost.mydomain* user:password
		account2: *otherhost.mydomain/somerep* dXNlcjpwYXNzd29yZA==

	That is it must start with 'account', followed by anything to distinguish accounts, and a colon. Then comes a match expression for the repository url, and after a space the user:password string.
	
	To not have the clear text password on your disc, you can base64 encode it:
			(Base64MimeConverter mimeEncode: 'user:password' readStream) contents
	"

	
	Settings ifNotNil: [
		Settings keysAndValuesDo: [:key :value | | userAndPassword entry |
			(key asLowercase beginsWith: 'account') ifTrue: [
				entry := value findTokens: '	 '.
				(entry first match: location) ifTrue: [
					userAndPassword := entry second.
					(userAndPassword includes: $:) ifFalse: [
						userAndPassword := (Base64MimeConverter mimeDecodeToChars: userAndPassword readStream) contents].
					userAndPassword := userAndPassword findTokens: $:.
					^aBlock value: userAndPassword first 
						value: userAndPassword second 
					]
			]
		]
	].
	^nil! !

!MCHttpRepository methodsFor: 'accessing' stamp: 'stephaneducasse 2/4/2006 20:47'!
versionReaderForFileNamed: aString
	readerCache ifNil: [readerCache := Dictionary new].
	^ readerCache at: aString ifAbsent:
		[self resizeCache: readerCache.
		super versionReaderForFileNamed: aString do:
			[:r |
			r ifNotNil: [readerCache at: aString put: r]]]
	! !

!MCHttpRepository methodsFor: 'accessing' stamp: 'ul 12/12/2009 14:10'!
versionReaderForFileNamed: aString do: aBlock
	^ (self versionReaderForFileNamed: aString) ifNotNil: aBlock! !

!MCHttpRepository methodsFor: 'private' stamp: 'ar 7/21/2010 19:53'!
writeStreamForFileNamed: aString replace: ignoreBoolean do: aBlock
	| stream response statusLine code |
	stream := RWBinaryOrTextStream on: String new.
	aBlock value: stream.
	self displayProgress: 'Uploading ', aString during:[
		response := HTTPSocket
					httpPut: stream contents
					to: (self urlForFileNamed: aString)
					user: self user
					passwd: self password.
	].
	"More robust handling of HTTP responses. Instead of enumerating
	all possible return codes and http versions, do a quick parse"
	(response beginsWith: 'HTTP/') ifTrue:[
		"Looks like an HTTP header, not some error message"
		statusLine := response copyUpTo: Character cr.
		code := [(statusLine findTokens: ' ') second asInteger] on: Error do:[].
	].
	(code isInteger and:[code between: 200 and: 299]) 
		ifFalse:[self error: response].! !

MCFileBasedRepository subclass: #MCSMCacheRepository
	instanceVariableNames: 'smCache'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Repositories'!

!MCSMCacheRepository commentStamp: 'nk 1/23/2004 09:57' prior: 0!
I am a Monticello repository that reflects the caching of SqueakMap v2.

I refer write attempts to the default MCCacheRepository.!

!MCSMCacheRepository class methodsFor: 'instance creation' stamp: 'nk 1/23/2004 10:04'!
description
	^ 'SqueakMap Cache'! !

!MCSMCacheRepository class methodsFor: 'instance creation' stamp: 'nk 1/23/2004 10:05'!
morphicConfigure
	^self new! !

!MCSMCacheRepository methodsFor: 'accessing' stamp: 'nk 1/23/2004 10:03'!
allFullFileNames
	| cachedPackages |
	cachedPackages := smCache map installedPackages select: [ :ea | ea isCached ].
	^Array streamContents: [ :s |
		cachedPackages do: [ :ea | | d |
			d := ea cacheDirectory.
			(d fileNamesMatching: '*.mcz') do: [ :fn | s nextPut: (d fullNameFor: fn) ]]]! !

!MCSMCacheRepository methodsFor: 'user interface' stamp: 'nk 1/23/2004 09:40'!
description
	^ smCache directory pathName! !

!MCSMCacheRepository methodsFor: 'accessing' stamp: 'nk 1/23/2004 09:40'!
directory
	^ smCache directory! !

!MCSMCacheRepository methodsFor: 'accessing' stamp: 'nk 1/23/2004 09:40'!
directory: aDirectory
! !

!MCSMCacheRepository methodsFor: 'accessing' stamp: 'nk 1/23/2004 09:55'!
fullNameFor: aFileName
	^self allFullFileNames detect: [ :ffn | (self directory localNameFor: ffn) = aFileName ] ifNone: []! !

!MCSMCacheRepository methodsFor: 'comparing' stamp: 'nk 1/23/2004 09:55'!
hash
	^ smCache hash! !

!MCSMCacheRepository methodsFor: 'initialize-release' stamp: 'ar 1/3/2010 14:22'!
initialize
	super initialize.
	Smalltalk at: #SMSqueakMap ifPresent:[:aClass|
		smCache := aClass default cache.
	].! !

!MCSMCacheRepository methodsFor: 'testing' stamp: 'nk 1/23/2004 09:47'!
isValid
	^smCache notNil and: [ self directory exists ]! !

!MCSMCacheRepository methodsFor: 'file streaming' stamp: 'nk 1/23/2004 09:57'!
readStreamForFileNamed: aString do: aBlock
	| file fileName |
	fileName := self fullNameFor: aString.
	fileName ifNil: [
		"assume that this will come from the cache."
		^MCCacheRepository default readStreamForFileNamed: aString do: aBlock ].
	file := FileStream readOnlyFileNamed: fileName.
	^[ aBlock value: file ] ensure: [ file close ].
! !

!MCSMCacheRepository methodsFor: 'accessing' stamp: 'nk 1/23/2004 09:37'!
smCache
	^smCache! !

!MCSMCacheRepository methodsFor: 'accessing' stamp: 'nk 1/23/2004 09:45'!
smCache: aSMFileCache
	| |
	smCache := aSMFileCache.
	self directory: aSMFileCache directory.
! !

!MCSMCacheRepository methodsFor: 'file streaming' stamp: 'nk 1/23/2004 09:57'!
writeStreamForFileNamed: aString replace: aBoolean do: aBlock
	"Can't write into the SM cache, so..."
	^MCCacheRepository default writeStreamForFileNamed: aString replace: aBoolean do: aBlock! !

MCRepository subclass: #MCGOODSRepository
	instanceVariableNames: 'hostname port connection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Repositories'!

!MCGOODSRepository commentStamp: 'cmm 3/6/2011 16:56' prior: 0!
A MCGOODSRepository simply stores a Dictionary of aVersionInfo-> aMCVersion.!

!MCGOODSRepository class methodsFor: 'configuring' stamp: 'avi 2/28/2004 20:36'!
creationTemplate
	^
'MCGOODSRepository
	host: ''localhost''
	port: 6100'! !

!MCGOODSRepository class methodsFor: 'configuring' stamp: 'avi 2/28/2004 14:33'!
description
	^ 'GOODS'! !

!MCGOODSRepository class methodsFor: 'configuring' stamp: 'avi 2/28/2004 20:33'!
fillInTheBlankRequest
	^ 'GOODS Repository:'! !

!MCGOODSRepository class methodsFor: 'instance creation' stamp: 'avi 2/28/2004 20:36'!
host: hostname port: portNumber
	^ self new
		host: hostname;
		port: portNumber! !

!MCGOODSRepository class methodsFor: 'configuring' stamp: 'avi 2/28/2004 20:35'!
morphicConfigure
	^ self fillInTheBlankConfigure! !

!MCGOODSRepository methodsFor: 'packages' stamp: 'cmm 3/6/2011 16:56'!
allPackageNames
	^ self root collect:
		[ : ea | ea package name ]! !

!MCGOODSRepository methodsFor: 'private' stamp: 'avi 8/26/2004 14:20'!
basicStoreVersion: aVersion
	self root at: aVersion info put: aVersion.
	self db commit.! !

!MCGOODSRepository methodsFor: 'private' stamp: 'md 9/6/2005 18:37'!
db
	(connection isNil or: [connection isConnected not]) ifTrue: [
		connection := Smalltalk at: #KKDatabase ifPresent: [:cl | 
			cl  onHost:hostname port: port
		]
	].
	^ connection! !

!MCGOODSRepository methodsFor: 'user interface' stamp: 'avi 2/28/2004 20:35'!
description
	^ 'goods://', hostname asString, ':', port asString! !

!MCGOODSRepository methodsFor: 'accessing' stamp: 'avi 2/28/2004 20:34'!
host: aString
	hostname := aString! !

!MCGOODSRepository methodsFor: 'versions' stamp: 'cmm 3/6/2011 17:00'!
includesVersionNamed: aString 
	^ (self versionNamed: aString) notNil! !

!MCGOODSRepository methodsFor: 'accessing' stamp: 'avi 2/28/2004 20:36'!
port: aNumber
	port := aNumber! !

!MCGOODSRepository methodsFor: 'accessing' stamp: 'avi 2/28/2004 14:35'!
root
	self db root ifNil: [self db root: Dictionary new].
	^ self db root! !

!MCGOODSRepository methodsFor: 'versions' stamp: 'cmm 3/6/2011 17:00'!
versionNamed: aString 
	| versionName |
	versionName := aString asMCVersionName.
	self root keysDo:
		[ : each | each versionName = versionName ifTrue: [ ^ each ] ].
	^ nil! !

!MCGOODSRepository methodsFor: 'versions' stamp: 'cmm 3/6/2011 17:03'!
versionNamesForPackageNamed: aString 
	^ Array streamContents:
		[ : stream | self root keysDo:
			[ : each | each versionName packageName = aString ifTrue: [ stream nextPut: each ] ] ]! !

!MCGOODSRepository methodsFor: 'versions' stamp: 'avi 2/28/2004 20:21'!
versionWithInfo: aVersionInfo ifAbsent: errorBlock
	^ self root at: aVersionInfo ifAbsent: errorBlock! !

!MCRepository class methodsFor: 'configuring' stamp: 'ab 8/21/2003 00:30'!
allConcreteSubclasses
	^ self withAllSubclasses reject: [:ea | ea isAbstract]! !

!MCRepository class methodsFor: 'ui-support' stamp: 'cmm 11/27/2016 16:23'!
browseClassRevisionsService
	^ ServiceAction
		id: #browseMcClassRevisions
		text: 'browse revisions'
		button: 'mc'
		description: 'Browse revisions of this class definition from the first-listed HTTP repository of this package.'
		action:
			[ : aBrowserRequestor | aBrowserRequestor browseMcClassRevisions ]
		condition:
			[ : aBrowserRequestor | true ]! !

!MCRepository class methodsFor: 'ui-support' stamp: 'cmm 11/27/2016 17:30'!
browseMethodRevisionsService
	^ ServiceAction
		id: #browseMcMethodRevisions
		text: 'browse revisions'
		button: 'mc'
		description: 'Browse revisions of this method from the first-listed HTTP repository of this package.'
		action:
			[ : aBrowserRequestor | aBrowserRequestor browseMcMethodRevisions ]
		condition:
			[ : aBrowserRequestor | true ]! !

!MCRepository class methodsFor: 'configuring' stamp: 'bkv 2/18/2004 20:59'!
creationTemplate
	self subclassResponsibility.! !

!MCRepository class methodsFor: 'configuring' stamp: 'ab 8/21/2003 00:29'!
description
	^ nil! !

!MCRepository class methodsFor: 'external settings' stamp: 'stephaneducasse 2/4/2006 20:47'!
fetchExternalSettingsIn: aDirectory
	"Scan for settings file"
	"MCRepository fetchExternalSettingsIn: ExternalSettings preferenceDirectory"

	| stream |
	(aDirectory fileExists: self settingsFileName)
		ifFalse: [^self].
	stream := aDirectory readOnlyFileNamed: self settingsFileName.
	stream
		ifNotNil: [
			[Settings := ExternalSettings parseServerEntryArgsFrom: stream]
				ensure: [stream close]].
! !

!MCRepository class methodsFor: 'configuring' stamp: 'bkv 2/18/2004 21:05'!
fillInTheBlankConfigure
	^ self fillInTheBlankConfigure: self creationTemplate
			! !

!MCRepository class methodsFor: 'configuring' stamp: 'fbs 4/22/2013 09:31'!
fillInTheBlankConfigure: aTemplateString
	| chunk repo |
	
	aTemplateString ifNil: [ ^ false ].
	chunk := UIManager default
		multiLineRequest: self fillInTheBlankRequest
		centerAt: Sensor cursorPoint
		initialAnswer: aTemplateString
		answerHeight: 120.
			
	chunk 
		ifNotNil: [ 
			repo := self readFrom: chunk readStream.
			repo creationTemplate: chunk. 
	].

	^ repo! !

!MCRepository class methodsFor: 'configuring' stamp: 'bkv 2/18/2004 20:58'!
fillInTheBlankRequest
	self subclassResponsibility.! !

!MCRepository class methodsFor: 'well-known repositories' stamp: 'mt 8/2/2016 15:21'!
inbox
	^ MCHttpRepository inbox! !

!MCRepository class methodsFor: 'class initialization' stamp: 'bf 4/15/2005 10:19'!
initialize
	"self initialize"

	ExternalSettings registerClient: self.
! !

!MCRepository class methodsFor: 'configuring' stamp: 'ab 8/21/2003 12:59'!
isAbstract
	^ self description isNil! !

!MCRepository class methodsFor: 'instance creation' stamp: 'ar 12/16/2010 09:41'!
location: urlOrPath
	"Answer an MCRepository for the given url or path. Accepted locations are:
		- http, and ftp urls (i.e., http://source.squeak.org/trunk)
		- directory paths (i.e., C:\Squeak\packages)
		- directory matches (i.e., C:\Squeak\packages\*)
	"

	^self location: urlOrPath username: '' password: ''! !

!MCRepository class methodsFor: 'instance creation' stamp: 'cmm 8/2/2012 15:17'!
location: urlOrPath username: user password: pass
	"Answer an MCRepository for the given url or path. Accepted locations are:
		- http, and ftp urls (i.e., http://source.squeak.org/trunk)
		- directory paths (i.e., C:\Squeak\packages)
		- directory matches (i.e., C:\Squeak\packages\*)
	If provided, the supplied user name and password will be used."

	| url |
	(urlOrPath findString: '://') > 0 ifTrue:[
		url := urlOrPath asUrl.
		^ url scheme caseOf: {
			['ftp'] -> [MCFtpRepository 
							host: url authority
							 directory: url pathString allButFirst
							user: user 
							password: user].
			['http'] -> [MCHttpRepository 
							location: urlOrPath 
							user: user 
							password: pass].
		} otherwise:[self error: 'Unsupported scheme: ', url scheme].
	].

	(urlOrPath endsWith: '*') ifTrue:[
		^MCSubDirectoryRepository new 
				directory: (FileDirectory on: urlOrPath allButLast)].

	^MCDirectoryRepository
			directory: (FileDirectory on: urlOrPath)! !

!MCRepository class methodsFor: 'configuring' stamp: 'ab 7/24/2003 21:01'!
morphicConfigure
	^ self new! !

!MCRepository class methodsFor: 'well-known repositories' stamp: 'cmm 10/2/2013 14:00'!
packageCache
	^ MCCacheRepository default! !

!MCRepository class methodsFor: 'external settings' stamp: 'bf 12/17/2004 20:30'!
releaseExternalSettings
	Settings := nil.
! !

!MCRepository class methodsFor: 'external settings' stamp: 'bf 12/17/2004 20:36'!
settingsFileName
	^ 'mcSettings'! !

!MCRepository class methodsFor: 'well-known repositories' stamp: 'mt 10/10/2019 09:10'!
treated
	^ MCHttpRepository treated! !

!MCRepository class methodsFor: 'well-known repositories' stamp: 'cmm 10/2/2013 13:59'!
trunk
	^ MCHttpRepository trunk! !

!MCRepository methodsFor: 'testing' stamp: 'ab 8/21/2003 12:36'!
= other
	^ other species = self species and: [other description = self description]! !

!MCRepository methodsFor: 'packages' stamp: 'cmm 8/14/2012 16:00'!
allPackageAndBranchNames
	"Currently this function is only supported by FileBased repositories."
	^ self allPackageNames! !

!MCRepository methodsFor: 'packages' stamp: 'cmm 3/2/2011 22:32'!
allPackageNames
	self subclassResponsibility! !

!MCRepository methodsFor: 'versions' stamp: 'cmm 3/12/2011 11:41'!
allVersionsDo: aBlock
	self allPackageNames asArray sort
		do:
			[ : eachPackageName | | sortedVersions |
			sortedVersions := (self versionNamesForPackageNamed: eachPackageName) sort:
				[ : a : b | a versionNumber < b versionNumber ].
			sortedVersions
				do:
					[ : eachVersionName | (self versionNamed: eachVersionName) ifNotNil:
						[ : ver | aBlock value: ver ] ]
				displayingProgress:
					[ : eachVersionName | 'Importing ' , eachVersionName ] ]
		displayingProgress:
			[ : eachPackageName | 'Importing versions of ' , eachPackageName ]! !

!MCRepository methodsFor: 'accessing' stamp: 'avi 8/31/2004 01:08'!
alwaysStoreDiffs
	^ storeDiffs ifNil: [false]! !

!MCRepository methodsFor: 'accessing' stamp: 'bkv 2/18/2004 20:48'!
asCreationTemplate
	^ self creationTemplate! !

!MCRepository methodsFor: 'private' stamp: 'avi 8/26/2004 14:20'!
basicStoreVersion: aVersion
	self subclassResponsibility! !

!MCRepository methodsFor: 'private' stamp: 'cmm 3/6/2011 20:15'!
cacheAllFileNamesDuring: aBlock
	"FileBasedRepository's can only access all version-names, therefore this is provided to allow client-code to direct caching of getting all filenames.  Other types of repositories offer more sophisticated kinds of access, so they don't need to cache, so simply run the block."
	^ aBlock value! !

!MCRepository methodsFor: 'private' stamp: 'cmm 10/2/2013 22:02'!
cacheAllFilenames
	"No-op by default.  Subclasses override for performance."! !

!MCRepository methodsFor: 'accessing' stamp: 'ul 12/12/2009 14:10'!
closestAncestorVersionFor: anAncestry ifNone: errorBlock
	anAncestry breadthFirstAncestorsDo:
		[:ancestorInfo |
		(self versionWithInfo: ancestorInfo) ifNotNil: [:v | ^ v]].
	^ errorBlock value! !

!MCRepository methodsFor: 'versions' stamp: 'cmm 8/15/2011 17:26'!
copyAllFrom: aMCRepository 
	"Copy all MCVersions from aMCRepository to the receiver."
	self cacheAllFileNamesDuring:
		[ aMCRepository cacheAllFileNamesDuring:
			[ | pkgCount |
			pkgCount := 0.
			'Copy all package versions from ' , aMCRepository description
				displayProgressFrom: 0
				to: aMCRepository allPackageNames size
				during:
					[ : pkgBar | aMCRepository allPackageNames asArray sort do:
						[ : eachPackageName | | versions verCount |
						verCount := 0.
						versions := (aMCRepository versionNamesForPackageNamed: eachPackageName) difference: (self versionNamesForPackageNamed: eachPackageName).
						'Progress for ' , eachPackageName
							displayProgressFrom: 0
							to: versions size
							during:
								[ : verBar | versions do:
									[ : eachVersionName | (aMCRepository versionNamed: eachVersionName) ifNotNil:
										[ : ver | self storeVersion: ver ].
									verBar value: (verCount := verCount + 1) ].
								pkgBar value: (pkgCount := pkgCount + 1) ] ] ] ] ]! !

!MCRepository methodsFor: 'versions' stamp: 'cmm 8/13/2011 19:18'!
copyImageVersions
	"For each package contained in the receiver, copy the version of that package which is currently loaded in this image.  If no version of a package is loaded in the image, nothing is copied for that package."
	self cacheAllFileNamesDuring:
		[ self allPackageNames do:
			[ : eachPkgName | MCWorkingCopy allManagers
				detect: [ : each | each packageName = eachPkgName ]
				ifFound:
					[ : loaded | loaded ancestors do:
						[ : infoToCopy | (self includesVersionNamed: infoToCopy versionName) ifFalse:
							[ (MCRepositoryGroup default versionWithInfo: infoToCopy)
								ifNil: [ Warning signal: infoToCopy name , ' not found in RepositoryGroup default.' ]
								ifNotNilDo:
									[ : ver | self storeVersion: ver ] ] ] ]
				ifNone: [ "Image specifies no version to copy." ] ] ]! !

!MCRepository methodsFor: 'accessing' stamp: 'bkv 2/18/2004 20:46'!
creationTemplate
	^ creationTemplate! !

!MCRepository methodsFor: 'accessing' stamp: 'cmm 4/22/2011 15:52'!
creationTemplate: aString 
	creationTemplate := aString! !

!MCRepository methodsFor: 'user interface' stamp: 'avi 10/9/2003 12:53'!
description
	^ self class name! !

!MCRepository methodsFor: 'accessing' stamp: 'avi 8/31/2004 01:08'!
doAlwaysStoreDiffs
	storeDiffs := true! !

!MCRepository methodsFor: 'accessing' stamp: 'avi 8/31/2004 01:09'!
doNotAlwaysStoreDiffs
	storeDiffs := false! !

!MCRepository methodsFor: 'private' stamp: 'cmm 8/13/2013 21:01'!
flushCache
	"Subclasses override as desired."! !

!MCRepository methodsFor: 'testing' stamp: 'ab 8/21/2003 12:36'!
hash
	^ self description hash! !

!MCRepository methodsFor: 'versions' stamp: 'cmm 8/2/2012 23:02'!
highestNumberedVersionForPackageNamed: aString 
	^ self versionNamed: (self highestNumberedVersionNameForPackageNamed: aString)! !

!MCRepository methodsFor: 'versions' stamp: 'bf 5/31/2012 11:43'!
highestNumberedVersionNameForPackageNamed: aString 
	^ (self versionNamesForPackageNamed: aString)
		ifEmpty: [ self error: aString , ' not found in ' , self asString ]
		ifNotEmptyDo:
			[ : versionNames | versionNames detectMax:
				[ : each | each versionNumber ] ]! !

!MCRepository methodsFor: 'versions' stamp: 'cmm 8/12/2013 17:51'!
includesVersionNamed: aString
	"Subclasses may want to override for better performance."
	^ (self versionNamed: aString) notNil! !

!MCRepository methodsFor: 'testing' stamp: 'mt 8/2/2016 15:19'!
isInbox
	^ false! !

!MCRepository methodsFor: 'testing' stamp: 'mt 10/10/2019 09:11'!
isTreated
	^ false! !

!MCRepository methodsFor: 'testing' stamp: 'cmm 10/2/2013 13:51'!
isTrunk
	^ false! !

!MCRepository methodsFor: 'testing' stamp: 'nk 11/2/2003 10:55'!
isValid
	^true! !

!MCRepository methodsFor: '*monticello' stamp: 'cmm 3/15/2011 13:33'!
mcModel
	"Only Magma-based repositories employ a McModel instance."
	^ nil! !

!MCRepository methodsFor: 'versions' stamp: 'cmm 4/19/2011 21:11'!
missingAncestryDo: oneArgBlock 
	"Value oneArgBlock with each of the MCVersionInfo ancestry that is missing from this repository."
	self cacheAllFileNamesDuring:
		[ self allPackageNames do:
			[ : each | self
				missingAncestryForPackageNamed: each
				do: oneArgBlock ] ]! !

!MCRepository methodsFor: 'versions' stamp: 'cmm 4/19/2011 21:23'!
missingAncestryForPackageNamed: packageName do: oneArgBlock 
	| start |
	self cacheAllFileNamesDuring:
		[ start := self versionNamed: (self highestNumberedVersionNameForPackageNamed: packageName).
		self cacheAllFileNamesDuring:
			[ start info allAncestorsDo:
				[ : each | (self includesVersionNamed: each versionName) ifFalse: [ oneArgBlock value: each ] ] ] ]! !

!MCRepository methodsFor: 'user interface' stamp: 'lr 9/26/2003 20:03'!
morphicOpen
	self morphicOpen: nil! !

!MCRepository methodsFor: 'user interface' stamp: 'avi 2/28/2004 20:10'!
morphicOpen: aWorkingCopy
	(MCRepositoryInspector repository: self workingCopy: aWorkingCopy) show! !

!MCRepository methodsFor: 'accessing' stamp: 'cmm 4/14/2013 22:24'!
normalized
	^ (MCRepositoryGroup default repositories includes: self)
		ifTrue: [ self ]
		ifFalse: [ self copy ]! !

!MCRepository methodsFor: 'private' stamp: 'ul 10/18/2018 01:47'!
normalizedRepositories
	^{ self normalized }! !

!MCRepository methodsFor: 'notifying' stamp: 'avi 8/26/2004 14:27'!
notificationForVersion: aVersion
	^ MCVersionNotification version: aVersion repository: self! !

!MCRepository methodsFor: 'notifying' stamp: 'avi 8/26/2004 14:23'!
notifyList
	^ #()! !

!MCRepository methodsFor: 'versions' stamp: 'cmm 4/19/2011 21:42'!
obtainMissingAncestryFrom: sourceRepository 
	self cacheAllFileNamesDuring:
		[ self missingAncestryDo:
			[ : each | (sourceRepository includesVersionNamed: each versionName)
				ifTrue: [ self storeVersion: (sourceRepository versionNamed: each versionName) ]
				ifFalse: [ Notification signal: each versionName , ' not present in ' , sourceRepository asString ] ] ]! !

!MCRepository methodsFor: 'user interface' stamp: 'bf 4/14/2005 17:30'!
openAndEditTemplateCopy
	^ self class fillInTheBlankConfigure: (self asCreationTemplate ifNil: [^nil])! !

!MCRepository methodsFor: 'versions' stamp: 'bf 10/9/2014 09:06'!
possiblyNewerVersionsOfAnyOf: versionNames 
	"Answer a collection of MCVersionNames which might be newer versions of the versions identified by versionNames."
	^self cacheAllFileNamesDuring: [
		versionNames
		inject: OrderedCollection new
		into:
			[ : coll : eachVersionName | | eachPackageName |
			eachPackageName := eachVersionName packageAndBranchName.
			(self versionNamesForPackageNamed: eachPackageName) do:
				[ : eachInSelf | (eachPackageName = eachInSelf packageAndBranchName and: [
					eachInSelf versionNumber > eachVersionName versionNumber or:
					[ eachInSelf versionNumber = eachVersionName versionNumber and: [ eachInSelf author ~= eachVersionName author ] ] ]) ifTrue: [ coll add: eachInSelf ] ].
			coll ] ]! !

!MCRepository methodsFor: 'accessing' stamp: 'cmm 4/6/2019 15:41'!
prepareVersionForStorage: aVersion
	^ self alwaysStoreDiffs
		ifTrue: [aVersion asDiffAgainst:
				 (self withCache closestAncestorVersionFor: aVersion info ifNone: [^ aVersion])]
		ifFalse: [aVersion]! !

!MCRepository methodsFor: 'printing' stamp: 'mas 9/24/2003 04:21'!
printOn: aStream
	super printOn: aStream.
	aStream
		nextPut: $(;
		nextPutAll: self description;
		nextPut: $).! !

!MCRepository methodsFor: 'accessing' stamp: 'cmm 3/4/2011 14:47'!
refresh
	"Subclasses override if necessary."! !

!MCRepository methodsFor: 'accessing' stamp: 'ul 10/18/2018 01:47'!
repositories
	^{ self }! !

!MCRepository methodsFor: 'notifying' stamp: 'avi 8/26/2004 14:24'!
sendNotificationsForVersion: aVersion
	| notification notifyList |
	notifyList := self notifyList.
	notifyList isEmpty ifFalse:
		[notification := self notificationForVersion: aVersion.
		notifyList do: [:ea | notification notify: ea]]! !

!MCRepository methodsFor: 'storing' stamp: 'cmm 4/22/2011 16:34'!
storeVersion: aMCVersionOrConfiguration
	self basicStoreVersion: (self prepareVersionForStorage: aMCVersionOrConfiguration).
	self sendNotificationsForVersion: aMCVersionOrConfiguration! !

!MCRepository methodsFor: 'versions' stamp: 'cmm 3/6/2011 16:29'!
versionNamed: aMCVersionName
	"Answer the MCVersion with name, aMCVersionName, or nil if it doesn't exist in this repository."
	self subclassResponsibility! !

!MCRepository methodsFor: 'versions' stamp: 'cmm 3/3/2011 14:45'!
versionNamesForPackageNamed: aString
	"Answer a collection of MCVersionNames whose Package is named aString."
	self subclassResponsibility! !

!MCRepository methodsFor: 'versions' stamp: 'avi 10/9/2003 12:42'!
versionWithInfo: aVersionInfo
	^ self versionWithInfo: aVersionInfo ifAbsent: [nil]! !

!MCRepository methodsFor: 'versions' stamp: 'ab 8/16/2003 18:22'!
versionWithInfo: aVersionInfo ifAbsent: aBlock
	self subclassResponsibility ! !

!MCRepository methodsFor: 'user interface' stamp: 'cmm 4/25/2011 08:07'!
wasRemoved
	"Subclasses may override to, for example, disconnect a db session."! !

!MCRepository methodsFor: 'accessing' stamp: 'cmm 4/6/2019 15:40'!
withCache
	"Answer the receiver with package-cache in front of it."
	^ MCRepositoryGroup with: self! !

MCRepository subclass: #MCWriteOnlyRepository
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Repositories'!

MCWriteOnlyRepository subclass: #MCSMReleaseRepository
	instanceVariableNames: 'packageName user password'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Repositories'!

!MCSMReleaseRepository class methodsFor: 'configuring' stamp: 'avi 2/10/2004 14:15'!
creationTemplate
	^
'MCSMReleaseRepository
	package: ''mypackage''
	user: ''squeak''
	password: ''squeak'''
	! !

!MCSMReleaseRepository class methodsFor: 'configuring' stamp: 'avi 2/10/2004 13:42'!
description
	^ 'SqueakMap Release'! !

!MCSMReleaseRepository class methodsFor: 'configuring' stamp: 'bkv 2/18/2004 21:03'!
fillInTheBlankRequest
	^  'SqueakMap Release Repository:'
		! !

!MCSMReleaseRepository class methodsFor: 'configuring' stamp: 'bkv 2/18/2004 21:03'!
morphicConfigure
	^ self fillInTheBlankConfigure! !

!MCSMReleaseRepository class methodsFor: 'instance creation' stamp: 'avi 2/10/2004 14:14'!
package: packageString user: userString password: passString
	^ self basicNew initializeWithPackage: packageString user: userString password: passString! !

!MCSMReleaseRepository methodsFor: 'private' stamp: 'avi 8/26/2004 14:21'!
basicStoreVersion: aVersion
	| url |
	url := self uploadVersion: aVersion.
	self releaseVersion: aVersion url: url! !

!MCSMReleaseRepository methodsFor: 'private' stamp: 'avi 2/10/2004 14:11'!
checkResult: resultString
(#( 'HTTP/1.1 201 ' 'HTTP/1.1 200 ' 'HTTP/1.0 201 ' 'HTTP/1.0 200 ')
		anySatisfy: [:code | resultString beginsWith: code ])
			ifFalse: [self error: resultString].
! !

!MCSMReleaseRepository methodsFor: 'user interface' stamp: 'avi 2/10/2004 14:15'!
description
	^ 'sm://', packageName! !

!MCSMReleaseRepository methodsFor: 'private' stamp: 'avi 2/10/2004 14:15'!
initializeWithPackage: packageString user: userString password: passString
	packageName := packageString.
	user := userString.
	password := passString.
! !

!MCSMReleaseRepository methodsFor: 'private' stamp: 'cmm 3/5/2011 15:19'!
releaseVersion: aVersion url: urlString
	| result |
	result := HTTPSocket
		httpPost: self squeakMapUrl, '/packagebyname/', packageName, '/newrelease'
		args: {'version' -> {(aVersion info versionNumber)}.
			   'note' -> {aVersion info message}.
			   'downloadURL' -> {urlString}}
		user: user
		passwd: password.
	result contents size > 4 ifTrue: [self error: result contents]
! !

!MCSMReleaseRepository methodsFor: 'constants' stamp: 'avi 2/10/2004 14:58'!
squeakMapUrl 
	^ 'http://localhost:9070/sm'
! !

!MCSMReleaseRepository methodsFor: 'private' stamp: 'avi 2/10/2004 13:53'!
stringForVersion: aVersion
	| stream |
	stream := RWBinaryOrTextStream on: String new.
	aVersion fileOutOn: stream.
	^ stream contents! !

!MCSMReleaseRepository methodsFor: 'private' stamp: 'avi 2/12/2004 19:41'!
uploadVersion: aVersion
	| result stream |
	result := HTTPSocket
		httpPut: (self stringForVersion: aVersion)
		to: self squeakMapUrl, '/upload/', aVersion fileName
		user: user
		passwd: password.
	self checkResult: result.
	stream := result readStream.
	stream upToAll: 'http://'.
	^ 'http://', stream upToEnd! !

MCWriteOnlyRepository subclass: #MCSmtpRepository
	instanceVariableNames: 'email'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Repositories'!

!MCSmtpRepository class methodsFor: 'configuring' stamp: 'avi 10/9/2003 12:56'!
description
	^ 'SMTP'! !

!MCSmtpRepository class methodsFor: 'configuring' stamp: 'fbs 4/22/2013 09:32'!
morphicConfigure
	| address |
	address := UIManager default request: 'Email address:'.
	^ address isEmpty ifFalse: [self new emailAddress: address]! !

!MCSmtpRepository methodsFor: 'private' stamp: 'avi 8/26/2004 14:21'!
basicStoreVersion: aVersion
	MailSender sendMessage: (self messageForVersion: aVersion)! !

!MCSmtpRepository methodsFor: 'converting' stamp: 'avi 10/9/2003 13:11'!
bodyForVersion: aVersion
	^ String streamContents:
		[ :s |
		s nextPutAll: 'from version info:'; cr; cr.
		s nextPutAll:  aVersion info summary]! !

!MCSmtpRepository methodsFor: 'user interface' stamp: 'avi 10/9/2003 12:56'!
description
	^ 'mailto://', email! !

!MCSmtpRepository methodsFor: 'accessing' stamp: 'avi 10/9/2003 12:54'!
emailAddress: aString
	email := aString	! !

!MCSmtpRepository methodsFor: 'converting' stamp: 'stephaneducasse 2/4/2006 20:47'!
messageForVersion: aVersion
	| message data |
	message := MailMessage empty.
	message setField: 'from' toString: MailSender userName.
	message setField: 'to' toString: email.
	message setField: 'subject' toString: (self subjectForVersion: aVersion). 

	message body:
		(MIMEDocument
			contentType: 'text/plain'
			content: (self bodyForVersion: aVersion)).

	"Prepare the gzipped data"
	data := RWBinaryOrTextStream on: String new.
	aVersion fileOutOn: data.
	message addAttachmentFrom: data reset withName: aVersion fileName.
	^ message! !

!MCSmtpRepository methodsFor: 'converting' stamp: 'avi 10/9/2003 13:14'!
subjectForVersion: aVersion
	^ '[Package] ', aVersion info name! !

!MCWriteOnlyRepository methodsFor: 'versions' stamp: 'avi 10/9/2003 12:52'!
includesVersionNamed: aString
	^ false! !

!MCWriteOnlyRepository methodsFor: 'user interface' stamp: 'avi 10/9/2003 12:53'!
morphicOpen: aWorkingCopy
	self inform: 'This repository is write-only'! !

!MCWriteOnlyRepository methodsFor: 'versions' stamp: 'djm 9/29/2012 18:13'!
versionNamesForPackageNamed: aString
	"Answer a collection of MCVersionNames whose Package is named aString."
	^{}! !

!MCWriteOnlyRepository methodsFor: 'versions' stamp: 'avi 10/9/2003 12:52'!
versionWithInfo: aVersionInfo ifAbsent: aBlock
	^ aBlock value! !

Object subclass: #MCRepositoryGroup
	instanceVariableNames: 'repositories'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

MCRepositoryGroup class
	instanceVariableNames: 'default'!

!MCRepositoryGroup commentStamp: '<historical>' prior: 0!
A singleton class, holds the list of repositories. Can look for a requested VersionInfo among its repositories.!

MCRepositoryGroup class
	instanceVariableNames: 'default'!

!MCRepositoryGroup class methodsFor: 'accessing' stamp: 'ab 7/22/2003 00:17'!
default
	^ default ifNil: [default := self new]! !

!MCRepositoryGroup class methodsFor: 'actions' stamp: 'cmm 8/13/2013 20:50'!
flushAllCaches
	self default flushCache! !

!MCRepositoryGroup class methodsFor: 'actions' stamp: 'cmm 2/13/2014 10:49'!
reset
	default := nil! !

!MCRepositoryGroup class methodsFor: 'instance creation' stamp: 'cmm 1/29/2019 00:00'!
with: aMCRepository 
	^ self new
		 addRepository: aMCRepository ;
		 yourself! !

!MCRepositoryGroup methodsFor: 'update' stamp: 'avi 1/27/2004 00:29'!
addRepository: aRepository
	((repositories includes: aRepository) or: [aRepository == MCCacheRepository default])
		ifFalse: [repositories add: aRepository.
				self class default addRepository: aRepository].
	self changed: #repositories! !

!MCRepositoryGroup methodsFor: 'repository-api' stamp: 'cmm 4/14/2013 21:57'!
allPackageNames
	^ repositories
		inject: Set new
		into:
			[ : set : each | set
				 addAll: each allPackageNames ;
				 yourself ]! !

!MCRepositoryGroup methodsFor: 'repository-api' stamp: 'cmm 4/14/2013 21:58'!
basicStoreVersion: aVersion
	"RepositoryGroup is used for reading, not writing."
	self shouldNotImplement! !

!MCRepositoryGroup methodsFor: 'private' stamp: 'cmm 10/2/2013 22:03'!
cacheAllFilenames
	repositories do: [ : each | each cacheAllFilenames ]! !

!MCRepositoryGroup methodsFor: 'accessing' stamp: 'cmm 1/28/2019 23:58'!
closestAncestorVersionFor: anAncestry ifNone: errorBlock
	anAncestry breadthFirstAncestorsDo:
		[:ancestorInfo |
		(self versionWithInfo: ancestorInfo) ifNotNil: [:v | ^ v]].
	^ errorBlock value! !

!MCRepositoryGroup methodsFor: 'update' stamp: 'cmm 10/31/2016 15:32'!
demoteRepository: aMCRepository 
	"If aMCRepository is part of this Group, demote it to the bottom of the list so that other repositories will be checked first."
	| removed |
	removed := repositories
		remove: aMCRepository
		ifAbsent: [  ].
	removed ifNotNil:
		[ repositories addLast: aMCRepository.
		self changed: #repositories ]! !

!MCRepositoryGroup methodsFor: 'repository-api' stamp: 'cmm 4/15/2013 09:19'!
description
	^ String streamContents:
		[ : stream | stream nextPutAll: '{ '.
		repositories
			do: [ : each | stream nextPut: $';  nextPutAll: each description; nextPut: $' ]
			separatedBy: [ stream nextPutAll: '. ' ].
		stream nextPutAll: ' }' ]! !

!MCRepositoryGroup methodsFor: 'private' stamp: 'cmm 1/28/2019 23:39'!
flushAllFilenames
	self repositories do: [ : each | each flushAllFilenames ]! !

!MCRepositoryGroup methodsFor: 'repository-api' stamp: 'cmm 8/13/2013 20:50'!
flushCache
	self repositoriesDo: [ : each | each flushCache ]! !

!MCRepositoryGroup methodsFor: 'testing' stamp: 'avi 11/7/2003 00:20'!
includes: aRepository
	^ self repositories includes: aRepository! !

!MCRepositoryGroup methodsFor: 'repository-api' stamp: 'fbs 4/22/2013 09:43'!
includesVersionNamed: aString 
	^ repositories anySatisfy: [ : each | [each includesVersionNamed: aString] on: Error do: [false]]! !

!MCRepositoryGroup methodsFor: 'initialize-release' stamp: 'cmm 4/15/2013 09:09'!
initialize
	super initialize.
	repositories := OrderedCollection new! !

!MCRepositoryGroup methodsFor: 'testing' stamp: 'cmm 10/2/2013 13:51'!
isTrunk
	^ false! !

!MCRepositoryGroup methodsFor: 'ui' stamp: 'fbs 1/24/2013 13:35'!
morphicOpen: aWorkingCopy 
	^ self repositories do: [:repo | repo morphicOpen: aWorkingCopy].! !

!MCRepositoryGroup methodsFor: 'accessing' stamp: 'cmm 4/14/2013 22:24'!
normalizedRepositories
	"Find an existing instance of any active repository so that we use whatever name and password the user usually uses. If not found, answer a copy"
	^ repositories collect: [ : each | each normalized ]! !

!MCRepositoryGroup methodsFor: 'update' stamp: 'cmm 4/25/2011 08:13'!
removeRepository: aRepository
	| removed |
	removed := repositories remove: aRepository ifAbsent: [].
	removed ifNotNil: [ removed wasRemoved ].
	self changed: #repositories! !

!MCRepositoryGroup methodsFor: 'accessing' stamp: 'ul 10/18/2018 01:46'!
repositories
	^{ MCCacheRepository default }, repositories select: [ :ea | ea isValid ]! !

!MCRepositoryGroup methodsFor: 'accessing' stamp: 'avi 11/7/2003 00:51'!
repositoriesDo: aBlock
	self repositories do: [:ea | [aBlock value: ea] on: Error do: []]! !

!MCRepositoryGroup methodsFor: 'repository-api' stamp: 'cmm 4/14/2013 22:00'!
versionNamed: aMCVersionName 
	repositories do:
		[ : each | (each versionNamed: aMCVersionName) ifNotNil: [ : ver | ^ ver ] ].
	^ nil! !

!MCRepositoryGroup methodsFor: 'repository-api' stamp: 'cmm 6/27/2013 14:42'!
versionNamesForPackageNamed: aString 
	^ repositories
		inject: Set new
		into:
			[ : set : each | set
				 addAll:
					([ each versionNamesForPackageNamed: aString ]
						on: NetworkError
						do: [ : err | Array empty ]) ;
				 yourself ]! !

!MCRepositoryGroup methodsFor: 'accessing' stamp: 'klc 5/25/2008 20:20'!
versionWithInfo: aVersionInfo
	^self versionWithInfo: aVersionInfo ifNone: [ nil ]
! !

!MCRepositoryGroup methodsFor: 'repository-api' stamp: 'cmm 1/28/2019 23:31'!
versionWithInfo: aVersionInfo ifAbsent: aBlock 
	self repositories do:
		[ : each | ([each
			versionWithInfo: aVersionInfo
			ifAbsent: [ nil ]] on: NetworkError do: [ : err | nil ]) ifNotNil:
			[ : ver | ^ ver ] ].
	^ aBlock value! !

!MCRepositoryGroup methodsFor: 'accessing' stamp: 'ul 12/12/2009 14:10'!
versionWithInfo: aVersionInfo ifNone: aBlock
	self repositoriesDo: [:ea | (ea versionWithInfo: aVersionInfo) ifNotNil: [:v | ^ v]].
	^aBlock value! !

!MCRepositoryGroup methodsFor: 'accessing' stamp: 'cmm 4/6/2019 16:36'!
withCache
	^ self! !

Object subclass: #MCScanner
	instanceVariableNames: 'stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Chunk Format'!

!MCScanner class methodsFor: 'instance creation' stamp: 'avi 1/22/2004 20:14'!
scan: aStream
	^ (self new stream: aStream) next! !

!MCScanner class methodsFor: 'instance creation' stamp: 'ul 10/18/2018 01:45'!
scanTokens: aString
	"compatibility"
	^{ self scan: aString readStream }! !

!MCScanner methodsFor: 'scanning' stamp: 'ul 10/22/2018 22:57'!
next
	| c |
	stream skipSeparators.
	(c := stream peek) == $# ifTrue: [c := stream next; peek].
	c == $' ifTrue: [^ self nextString].
	c == $( ifTrue: [^ self nextArray].
	c isAlphaNumeric ifTrue: [^ self nextSymbol].
	self error: 'Unknown token type'! !

!MCScanner methodsFor: 'scanning' stamp: 'ul 10/18/2018 01:54'!
nextArray
	stream next. "("
	^ Array streamContents:
		[:s |
		[stream skipSeparators.
		(stream peek == $)) or: [stream atEnd]] whileFalse: [s nextPut: self next].
		stream next == $) ifFalse: [self error: 'Unclosed array']]! !

!MCScanner methodsFor: 'scanning' stamp: 'avi 1/22/2004 20:09'!
nextString
	^ stream nextDelimited: $'! !

!MCScanner methodsFor: 'scanning' stamp: 'avi 1/22/2004 20:16'!
nextSymbol
	^ (String streamContents:
		[:s |
		[stream peek isAlphaNumeric] whileTrue: [s nextPut: stream next]]) asSymbol
			! !

!MCScanner methodsFor: 'accessing' stamp: 'avi 1/22/2004 20:06'!
stream: aStream
	stream := aStream! !

Object subclass: #MCSnapshot
	instanceVariableNames: 'definitions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Base'!

!MCSnapshot commentStamp: 'hjh 1/22/2013 22:26' prior: 0!
A MCSnapshot is a snapshot of an MCPackage. It is a collection of MCDefinitions.

!

!MCSnapshot class methodsFor: 'as yet unclassified' stamp: 'ab 7/6/2003 23:48'!
empty
	^ self fromDefinitions: #()! !

!MCSnapshot class methodsFor: 'as yet unclassified' stamp: 'ab 7/6/2003 23:48'!
fromDefinitions: aCollection
	^ self new initializeWithDefinitions: aCollection! !

!MCSnapshot methodsFor: 'accessing' stamp: 'cmm 5/30/2012 22:29'!
= other
	self == other ifTrue: [ ^ true ].
	self species = other species ifFalse: [ ^ false ].
	^ definitions asArray = other definitions asArray! !

!MCSnapshot methodsFor: 'accessing' stamp: 'ab 12/4/2002 18:09'!
definitions
	^ definitions! !

!MCSnapshot methodsFor: 'accessing' stamp: 'ab 7/10/2003 01:05'!
hash
	^ definitions asArray hash! !

!MCSnapshot methodsFor: 'initializing' stamp: 'ab 7/6/2003 23:48'!
initializeWithDefinitions: aCollection
	definitions := aCollection.! !

!MCSnapshot methodsFor: 'loading' stamp: 'ab 7/6/2003 23:31'!
install
	MCPackageLoader installSnapshot: self! !

!MCSnapshot methodsFor: 'patching' stamp: 'ab 7/7/2003 00:37'!
patchRelativeToBase: aSnapshot
	^ MCPatch fromBase: aSnapshot target: self! !

!MCSnapshot methodsFor: 'loading' stamp: 'ab 7/7/2003 12:11'!
updatePackage: aPackage
	MCPackageLoader updatePackage: aPackage withSnapshot: self! !

Object subclass: #MCToolWindowBuilder
	instanceVariableNames: 'builder window currentFrame tool'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-UI'!

!MCToolWindowBuilder class methodsFor: 'instance creation' stamp: 'avi 2/11/2005 12:02'!
builder: aBuilder tool: aTool
	^ self basicNew initializeWithBuilder: aBuilder tool: aTool! !

!MCToolWindowBuilder methodsFor: 'building' stamp: 'avi 2/11/2005 12:03'!
build
	^ builder build: window! !

!MCToolWindowBuilder methodsFor: 'building-parts' stamp: 'avi 2/11/2005 12:04'!
buttonRow
	^ self buttonRow: tool buttonSpecs! !

!MCToolWindowBuilder methodsFor: 'building-parts' stamp: 'nice 12/27/2009 03:11'!
buttonRow: specArray
	| panel |
	panel := builder pluggablePanelSpec new.
	panel children: OrderedCollection new.
	specArray do:
		[:spec | | button |
		
		button := builder pluggableButtonSpec new.
		button model: tool.
		button label: spec first asString.
		button action: spec second.
		button help: spec third.
		button enabled: (spec at: 4 ifAbsent: [#buttonEnabled]).
		button state: (spec at: 5 ifAbsent: [#buttonSelected]).
		panel children add: button].
	panel layout: #horizontal.
	panel frame: currentFrame.
	window children add: panel! !

!MCToolWindowBuilder methodsFor: 'building-parts' stamp: 'kb 2/24/2010 19:39'!
codePane: aSymbol
	| text |
	text := builder pluggableCodePaneSpec new.
	text 
		model: tool;
		getText: aSymbol; 
		setText: (aSymbol, ':') asSymbol;
		frame: currentFrame.
	window children add: text! !

!MCToolWindowBuilder methodsFor: 'accessing' stamp: 'stephaneducasse 2/4/2006 20:47'!
frame: aLayoutFrame
	currentFrame := aLayoutFrame! !

!MCToolWindowBuilder methodsFor: 'initialize-release' stamp: 'stephaneducasse 2/4/2006 20:47'!
initializeWithBuilder: aBuilder tool: aTool
	builder := aBuilder.
	tool := aTool.
	window := builder pluggableWindowSpec new.
	window children: OrderedCollection new.
	window label: tool label asString.
	window model: tool.
	window extent: tool defaultExtent.! !

!MCToolWindowBuilder methodsFor: 'building-parts' stamp: 'nice 9/22/2010 00:42'!
innerButtonRow
	^ self buttonRow: tool innerButtonSpecs! !

!MCToolWindowBuilder methodsFor: 'building-parts' stamp: 'mt 4/7/2015 18:40'!
inputMorph: aSymbol
	| text |
	text := builder pluggableInputFieldSpec new.
	text 
		model: tool;
		getText: aSymbol; 
		setText: (aSymbol, ':') asSymbol;
		frame: currentFrame.
	window children add: text! !

!MCToolWindowBuilder methodsFor: 'building-parts' stamp: 'avi 2/11/2005 12:04'!
listMorph: listSymbol
	^ self
		listMorph: (listSymbol, 'List') asSymbol
		selection: (listSymbol, 'Selection') asSymbol
		menu: (listSymbol, 'ListMenu:') asSymbol! !

!MCToolWindowBuilder methodsFor: 'building-parts' stamp: 'avi 2/11/2005 12:04'!
listMorph: listSymbol keystroke: keystrokeSymbol
	^ (self
		listMorph: (listSymbol, 'List') asSymbol
		selection: (listSymbol, 'Selection') asSymbol
		menu: (listSymbol, 'ListMenu:') asSymbol)
		keystrokeActionSelector: keystrokeSymbol;
		yourself! !

!MCToolWindowBuilder methodsFor: 'building-parts' stamp: 'avi 2/11/2005 12:15'!
listMorph: listSymbol selection: selectionSymbol
	self listMorph: listSymbol selection: selectionSymbol menu: nil! !

!MCToolWindowBuilder methodsFor: 'building-parts' stamp: 'avi 2/11/2005 12:16'!
listMorph: listSymbol selection: selectionSymbol menu: menuSymbol
	self listMorph: listSymbol selection: selectionSymbol menu: menuSymbol keystroke: nil! !

!MCToolWindowBuilder methodsFor: 'building-parts' stamp: 'avi 2/13/2005 17:51'!
listMorph: listSymbol selection: selectionSymbol menu: menuSymbol keystroke: keystrokeSymbol
	| list |
	list := builder pluggableListSpec new.
	list 
		model: tool;
		list: listSymbol; 
		getIndex: selectionSymbol; 
		setIndex: (selectionSymbol, ':') asSymbol;
		frame: currentFrame.
	menuSymbol ifNotNil: [list menu: menuSymbol].
	keystrokeSymbol ifNotNil: [list keyPress: keystrokeSymbol].
	window children add: list
! !

!MCToolWindowBuilder methodsFor: 'building-parts' stamp: 'avi 2/13/2005 17:52'!
multiListMorph: listSymbol selection: selectionSymbol listSelection: listSelectionSymbol menu: menuSymbol
	| list |
	list := builder pluggableMultiSelectionListSpec new.
	list 
		model: tool;
		list: listSymbol; 
		getIndex: selectionSymbol; 
		setIndex: (selectionSymbol, ':') asSymbol;
		getSelectionList: listSelectionSymbol;
		setSelectionList: (listSelectionSymbol, 'put:') asSymbol;
		frame: currentFrame.
	menuSymbol ifNotNil: [list menu: menuSymbol].
	window children add: list
! !

!MCToolWindowBuilder methodsFor: 'building-parts' stamp: 'avi 2/13/2005 17:52'!
textMorph: aSymbol
	| text |
	text := builder pluggableTextSpec new.
	text 
		model: tool;
		getText: aSymbol; 
		setText: (aSymbol, ':') asSymbol;
		frame: currentFrame.
	window children add: text! !

!MCToolWindowBuilder methodsFor: 'building-parts' stamp: 'avi 2/11/2005 12:05'!
treeMorph: listSymbol
	^ self
		treeMorph: (listSymbol, 'Tree') asSymbol
		selection: (listSymbol, 'SelectionWrapper') asSymbol
		menu: (listSymbol, 'TreeMenu:') asSymbol! !

!MCToolWindowBuilder methodsFor: 'building-parts' stamp: 'avi 2/13/2005 17:52'!
treeMorph: listSymbol selection: selectionSymbol menu: menuSymbol
	self notYetImplemented! !

!MCToolWindowBuilder methodsFor: 'building-parts' stamp: 'avi 3/6/2005 22:31'!
treeOrListMorph: listSymbol
	^ self listMorph: listSymbol! !

Object subclass: #MCVariableDefinition
	instanceVariableNames: 'name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Modeling'!

MCVariableDefinition subclass: #MCClassInstanceVariableDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Modeling'!

!MCClassInstanceVariableDefinition class methodsFor: 'accessing' stamp: 'cwp 7/7/2003 22:59'!
type
	^ #classInstance! !

!MCClassInstanceVariableDefinition methodsFor: 'testing' stamp: 'cwp 7/7/2003 23:31'!
isClassInstanceVariable
	^ true! !

MCVariableDefinition subclass: #MCClassVariableDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Modeling'!

!MCClassVariableDefinition class methodsFor: 'accessing' stamp: 'cwp 7/7/2003 22:58'!
type
	^ #class! !

!MCClassVariableDefinition methodsFor: 'testing' stamp: 'cwp 7/7/2003 23:32'!
isClassVariable
	^ true! !

!MCClassVariableDefinition methodsFor: 'testing' stamp: 'bf 8/29/2006 11:41'!
isOrderDependend
	^false! !

MCVariableDefinition subclass: #MCInstanceVariableDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Modeling'!

!MCInstanceVariableDefinition class methodsFor: 'accessing' stamp: 'cwp 7/7/2003 22:59'!
type
	^ #instance! !

!MCInstanceVariableDefinition methodsFor: 'testing' stamp: 'cwp 7/7/2003 23:32'!
isInstanceVariable
	^ true! !

MCVariableDefinition subclass: #MCPoolImportDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Modeling'!

!MCPoolImportDefinition class methodsFor: 'accessing' stamp: 'cwp 7/7/2003 22:59'!
type
	^ #pool! !

!MCPoolImportDefinition methodsFor: 'testing' stamp: 'bf 8/29/2006 11:41'!
isOrderDependend
	^false! !

!MCPoolImportDefinition methodsFor: 'testing' stamp: 'cwp 7/7/2003 23:51'!
isPoolImport
	^ true! !

!MCVariableDefinition class methodsFor: 'instance creation' stamp: 'cwp 7/7/2003 23:18'!
name: aString
	^ self new name: aString
	! !

!MCVariableDefinition methodsFor: 'comparing' stamp: 'cwp 7/7/2003 23:02'!
= other
	^ (self species = other species)
		and: [self name = other name]! !

!MCVariableDefinition methodsFor: 'comparing' stamp: 'cwp 7/7/2003 23:02'!
hash
	^ name hash! !

!MCVariableDefinition methodsFor: 'testing' stamp: 'cwp 7/7/2003 23:31'!
isClassInstanceVariable
	^ false! !

!MCVariableDefinition methodsFor: 'testing' stamp: 'cwp 7/7/2003 23:08'!
isClassInstanceVariableDefinition
	^ false! !

!MCVariableDefinition methodsFor: 'testing' stamp: 'cwp 7/7/2003 23:30'!
isClassVariable
	^ false! !

!MCVariableDefinition methodsFor: 'testing' stamp: 'cwp 7/7/2003 23:31'!
isInstanceVariable
	^ false! !

!MCVariableDefinition methodsFor: 'testing' stamp: 'cwp 7/7/2003 23:10'!
isInstanceVariableDefinition
	^ false! !

!MCVariableDefinition methodsFor: 'testing' stamp: 'bf 8/29/2006 11:41'!
isOrderDependend
	^true! !

!MCVariableDefinition methodsFor: 'testing' stamp: 'cwp 7/7/2003 23:51'!
isPoolImport
	^ false! !

!MCVariableDefinition methodsFor: 'accessing' stamp: 'cwp 11/25/2002 05:57'!
name
	^ name! !

!MCVariableDefinition methodsFor: 'accessing' stamp: 'cwp 11/25/2002 06:00'!
name: aString
	name := aString! !

!MCVariableDefinition methodsFor: 'printing' stamp: 'nk 7/24/2003 14:56'!
printOn: aStream
	super printOn: aStream.
	aStream nextPut: $(; nextPutAll: self name; nextPut: $)! !

Object subclass: #MCVersion
	instanceVariableNames: 'package info snapshot dependencies'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

MCVersion subclass: #MCDiffyVersion
	instanceVariableNames: 'base patch'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

!MCDiffyVersion class methodsFor: 'name utilities' stamp: 'bf 5/30/2005 18:45'!
baseNameFrom: diffName
	| baseId verName |
	baseId := (diffName copyAfter: $() copyUpTo: $).
	baseId ifEmpty: [^baseId].
	(baseId beginsWith: '@')
		ifTrue: [^baseId copyAfter: $@].
	verName := self verNameFrom: diffName.
	^(baseId includes: $.)
		ifTrue: [(verName copyUpToLast: $-), '-', baseId]
		ifFalse: [(verName copyUpToLast: $.), '.', baseId]
! !

!MCDiffyVersion class methodsFor: 'name utilities' stamp: 'bf 4/11/2017 13:18'!
nameForVer: versionName base: baseName
	| baseId |
	baseId := versionName packageName = baseName packageName
		ifFalse: [ '@' , baseName ] 
		ifTrue: [
			versionName author = baseName author
				ifTrue: [ baseName versionNumber asString ]
				ifFalse: [ baseName author, '.', baseName versionNumber asString ] ].
	^ versionName versionName , '(' , baseId , ')'! !

!MCDiffyVersion class methodsFor: 'instance creation' stamp: 'avi 2/13/2004 23:07'!
package: aPackage info: aVersionInfo dependencies: aCollection baseInfo: baseVersionInfo patch:
aPatch
	^ self basicNew initializeWithPackage: aPackage info: aVersionInfo dependencies: aCollection baseInfo: baseVersionInfo patch:
aPatch! !

!MCDiffyVersion class methodsFor: 'instance creation' stamp: 'avi 2/13/2004 23:06'!
package: aPackage info: aVersionInfo snapshot: aSnapshot dependencies: aCollection baseVersion: aVersion
	^ self 
		package: aPackage
		info: aVersionInfo
		dependencies: aCollection
		baseInfo: aVersion info
		patch: (aSnapshot patchRelativeToBase: aVersion snapshot)! !

!MCDiffyVersion class methodsFor: 'name utilities' stamp: 'bf 5/30/2005 18:19'!
verNameFrom: diffName
	^diffName copyUpTo: $(! !

!MCDiffyVersion methodsFor: 'accessing' stamp: 'avi 2/13/2004 23:17'!
baseInfo
	^ base! !

!MCDiffyVersion methodsFor: 'accessing' stamp: 'fbs 6/9/2011 20:05'!
baseSnapshot
	| baseVersion |
	baseVersion := self workingCopy repositoryGroup versionWithInfo: base.
	baseVersion ifNil: [Error signal: 'Missing snapshot: ', self baseInfo name].
	^ baseVersion snapshot! !

!MCDiffyVersion methodsFor: 'testing' stamp: 'bf 5/23/2005 15:42'!
canOptimizeLoading
	"Answer wether I can provide a patch for the working copy without the usual diff pass"
	^ package hasWorkingCopy
		and: [package workingCopy modified not
			and: [package workingCopy ancestors includes: self baseInfo]]! !

!MCDiffyVersion methodsFor: 'accessing' stamp: 'cmm 3/3/2011 15:00'!
fileName
	^ ((self class
		nameForVer: info name
		base: base name) , '.' , self writerClass extension) asMCVersionName! !

!MCDiffyVersion methodsFor: 'actions' stamp: 'cmm 11/22/2013 15:19'!
fileOutOn: aStream
	"The whole point of diffy versions is to not store the full snapshot, so ensure we don't."
	snapshot:=nil.
	super fileOutOn: aStream! !

!MCDiffyVersion methodsFor: 'initialize-release' stamp: 'avi 2/19/2004 20:55'!
initializeWithPackage: aPackage info: aVersionInfo dependencies: aCollection baseInfo: baseVersionInfo patch: aPatch
	patch := aPatch.
	base := baseVersionInfo.
	super initializeWithPackage: aPackage info: aVersionInfo snapshot: nil dependencies: aCollection.
! !

!MCDiffyVersion methodsFor: 'testing' stamp: 'avi 2/13/2004 23:24'!
isDiffy
	^ true! !

!MCDiffyVersion methodsFor: 'accessing' stamp: 'avi 2/13/2004 23:17'!
patch
	^ patch! !

!MCDiffyVersion methodsFor: 'accessing' stamp: 'avi 2/13/2004 22:47'!
snapshot
	^ snapshot ifNil: [snapshot := MCPatcher apply: patch to: self baseSnapshot]! !

!MCDiffyVersion methodsFor: 'accessing' stamp: 'avi 2/19/2004 22:03'!
summary
	^ '(Diff against ', self baseInfo name, ')', String cr, super summary! !

!MCDiffyVersion methodsFor: 'accessing' stamp: 'avi 2/13/2004 23:17'!
writerClass
	^ MCMcdWriter ! !

!MCVersion class methodsFor: 'instance creation' stamp: 'ab 7/7/2003 16:13'!
package: aPackage
	^ self package: aPackage info: MCVersionInfo new! !

!MCVersion class methodsFor: 'instance creation' stamp: 'ab 7/7/2003 16:13'!
package: aPackage info: aVersionInfo
	^ self package: aPackage info: aVersionInfo snapshot: aPackage snapshot! !

!MCVersion class methodsFor: 'instance creation' stamp: 'cwp 11/7/2004 13:02'!
package: aPackage info: aVersionInfo snapshot: aSnapshot
	^ self package: aPackage info: aVersionInfo snapshot: aSnapshot dependencies: #()! !

!MCVersion class methodsFor: 'instance creation' stamp: 'avi 1/19/2004 13:11'!
package: aPackage info: aVersionInfo snapshot: aSnapshot dependencies: aCollection
	^ self new initializeWithPackage: aPackage info: aVersionInfo snapshot: aSnapshot dependencies: aCollection! !

!MCVersion methodsFor: 'actions' stamp: 'avi 10/9/2003 13:00'!
addToCache
	MCCacheRepository default storeVersion: self! !

!MCVersion methodsFor: 'actions' stamp: 'avi 2/12/2004 19:37'!
adopt
	self workingCopy adopt: self! !

!MCVersion methodsFor: 'enumerating' stamp: 'nice 12/27/2009 20:59'!
allAvailableDependenciesDo: aBlock
	
	self dependencies do:
		[:ea |
		[ | version |
		version := ea resolve.
		version allAvailableDependenciesDo: aBlock.
		aBlock value: version]
			on: Error do: []]! !

!MCVersion methodsFor: 'enumerating' stamp: 'cwp 11/7/2004 11:58'!
allDependenciesDo: aBlock
	self allDependenciesDo: aBlock ifUnresolved: [:ignored | true]! !

!MCVersion methodsFor: 'enumerating' stamp: 'cwp 11/7/2004 11:53'!
allDependenciesDo: aBlock ifUnresolved: failBlock
	| dict |
	dict := Dictionary new.
	self allDependenciesNotIn: dict do: aBlock ifUnresolved: failBlock! !

!MCVersion methodsFor: 'enumerating' stamp: 'nice 12/27/2009 03:11'!
allDependenciesNotIn: aDictionary do: aBlock ifUnresolved: failBlock
	
	self dependencies do: 
		[:ea | | version | 
		version := aDictionary at: ea ifAbsent: [ea resolve].
		version 
			ifNil: [failBlock value: ea]
			ifNotNil: [(aDictionary includes: version) ifFalse:
						[aDictionary at: ea put: version.
						version 
							allDependenciesNotIn: aDictionary 
							do: aBlock
							ifUnresolved: failBlock.
						aBlock value: version]]]! !

!MCVersion methodsFor: 'converting' stamp: 'avi 2/19/2004 21:00'!
asDiffAgainst: aVersion
	aVersion info = self info ifTrue: [self error: 'Cannot diff against self!!'].
	^ MCDiffyVersion
		package: self package
		info: self info
		snapshot: self snapshot
		dependencies: self dependencies
		baseVersion: aVersion! !

!MCVersion methodsFor: 'actions' stamp: 'kb 2/24/2010 19:40'!
browse
	(MCSnapshotBrowser forSnapshot: self snapshot)
		label: 'Snapshot of ', self fileName;
		show! !

!MCVersion methodsFor: 'testing' stamp: 'bf 5/23/2005 15:43'!
canOptimizeLoading
	"Answer wether I can provide a patch for the working copy without the usual diff pass"
	^false! !

!MCVersion methodsFor: 'accessing' stamp: 'avi 2/13/2004 22:42'!
changes
	^ self snapshot patchRelativeToBase: package snapshot! !

!MCVersion methodsFor: 'accessing' stamp: 'avi 1/22/2004 00:24'!
dependencies
	^ dependencies ifNil: [#()]! !

!MCVersion methodsFor: 'accessing' stamp: 'cg 3/25/2013 13:43'!
fileName
	|rawName cookedName|

	rawName := info name.
	"care for invalid filename characters"
	cookedName := rawName copy 
						replaceAll: $/ with: $_;
						replaceAll: $: with: $_..
	^ (cookedName, '.', self writerClass extension) asMCVersionName! !

!MCVersion methodsFor: 'actions' stamp: 'avi 1/22/2004 12:44'!
fileOutOn: aStream
	self writerClass fileOut: self on: aStream! !

!MCVersion methodsFor: 'accessing' stamp: 'ab 7/7/2003 14:28'!
info
	^ info! !

!MCVersion methodsFor: 'initialize-release' stamp: 'cmm 2/25/2011 16:26'!
initializeWithPackage: aPackage info: aVersionInfo snapshot: aSnapshot dependencies: aCollection 
	self
		setPackage: aPackage
		info: aVersionInfo
		snapshot: aSnapshot
		dependencies: aCollection.
	self addToCache! !

!MCVersion methodsFor: 'testing' stamp: 'bf 3/22/2005 23:00'!
isCacheable
	^true! !

!MCVersion methodsFor: 'testing' stamp: 'avi 2/13/2004 23:24'!
isDiffy
	^ false! !

!MCVersion methodsFor: 'actions' stamp: 'jr 1/14/2017 23:37'!
load
	self workingCopy withEnvironmentActiveDo: [MCVersionLoader loadVersion: self]! !

!MCVersion methodsFor: 'printing' stamp: 'bf 5/11/2012 13:45'!
logLoadingOn: aStream
	aStream
		nextPutAll: '========== ', self info name, ' =========='; cr;
		nextPutAll: self info message asString; cr;
		flush.

	package hasWorkingCopy ifFalse: [^self].

	package workingCopy ancestors do: [:each |
		(self info hasAncestor: each)
			ifTrue: [(self info allAncestorsOnPathTo: each)
				do: [:ver | aStream cr; nextPutAll: '>>> ', ver name, ' <<<'; cr;
							nextPutAll: ver message; cr; flush]]]! !

!MCVersion methodsFor: 'actions' stamp: 'jr 1/14/2017 23:38'!
merge
	self workingCopy withEnvironmentActiveDo: [MCVersionMerger mergeVersion: self]! !

!MCVersion methodsFor: 'actions' stamp: 'ab 7/12/2003 00:19'!
open
	(MCVersionInspector new version: self) show! !

!MCVersion methodsFor: 'accessing' stamp: 'ab 7/7/2003 14:19'!
package
	^ package! !

!MCVersion methodsFor: 'printing' stamp: 'nk 3/8/2004 23:54'!
printOn: aStream
	super printOn: aStream.
	aStream nextPut: $(.
	aStream nextPutAll: self info name.
	aStream nextPut: $).! !

!MCVersion methodsFor: 'actions' stamp: 'dtl 11/5/2018 21:09'!
reparent
	"Let aNode be the sole parent of this version"
	self workingCopy reparent: self! !

!MCVersion methodsFor: 'initialize-release' stamp: 'cwp 11/7/2004 13:08'!
setPackage: aPackage info: aVersionInfo snapshot: aSnapshot dependencies: aCollection
	package := aPackage.
	info := aVersionInfo.
	snapshot := aSnapshot.
	dependencies := aCollection! !

!MCVersion methodsFor: 'testing' stamp: 'cmm 8/3/2012 13:34'!
shouldMerge
	"answer true if we have to do a full merge and false if we can simply load instead"
	| pkg wc current |
	(pkg := self package) hasWorkingCopy ifFalse: [^false "no wc -> load"].
	(wc := pkg workingCopy) modified ifTrue: [^true "modified -> merge"].
	wc ancestors isEmpty ifTrue: [^true "no ancestor info -> merge"].
	current := wc ancestors first.
	(self info hasAncestor: current) ifTrue: [^false "direct descendant of wc -> load"].
	"new branch -> merge"
	^true! !

!MCVersion methodsFor: 'accessing' stamp: 'ab 7/7/2003 14:10'!
snapshot
	^ snapshot! !

!MCVersion methodsFor: 'accessing' stamp: 'avi 1/20/2004 16:07'!
summary
	^ String streamContents:
		[:s |
		s nextPutAll: info summaryHeader.
		(dependencies isNil or: [dependencies isEmpty]) ifFalse:
			[s cr; nextPutAll: 'Dependencies: '.
			dependencies
				do: [:ea | s nextPutAll: ea versionInfo name]
				separatedBy: [s nextPutAll: ', ']].
		s cr; cr; nextPutAll: info message]! !

!MCVersion methodsFor: 'enumerating' stamp: 'cwp 11/7/2004 14:24'!
withAllDependenciesDo: aBlock
	self allDependenciesDo: aBlock ifUnresolved: [:ignored].
	aBlock value: self! !

!MCVersion methodsFor: 'enumerating' stamp: 'cwp 11/7/2004 14:49'!
withAllDependenciesDo: aBlock ifUnresolved: failBlock
	| dict |
	dict := Dictionary new.
	self allDependenciesNotIn: dict do: aBlock ifUnresolved: failBlock.
	aBlock value: self! !

!MCVersion methodsFor: 'accessing' stamp: 'avi 2/12/2004 19:38'!
workingCopy
	^ package workingCopy! !

!MCVersion methodsFor: 'accessing' stamp: 'avi 1/22/2004 12:44'!
writerClass
	^ MCMczWriter ! !

Object subclass: #MCVersionDependency
	instanceVariableNames: 'package versionInfo'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

!MCVersionDependency class methodsFor: 'instance creation' stamp: 'avi 1/19/2004 13:13'!
package: aPackage info: aVersionInfo
	^ self basicNew initializeWithPackage: aPackage info: aVersionInfo! !

!MCVersionDependency methodsFor: 'comparing' stamp: 'avi 1/19/2004 16:12'!
= other
	^ other species = self species
		and: [other versionInfo = versionInfo
				and: [other package = package]]! !

!MCVersionDependency methodsFor: 'comparing' stamp: 'avi 1/19/2004 16:06'!
hash
	^ versionInfo hash! !

!MCVersionDependency methodsFor: 'initialize-release' stamp: 'avi 1/19/2004 13:12'!
initializeWithPackage: aPackage info: aVersionInfo
	package := aPackage.
	versionInfo := aVersionInfo! !

!MCVersionDependency methodsFor: 'testing' stamp: 'bf 4/19/2005 16:29'!
isCurrent
	^ package hasWorkingCopy
		and: [self isFulfilled
			and: [package workingCopy modified not]]! !

!MCVersionDependency methodsFor: 'testing' stamp: 'bf 4/19/2005 16:29'!
isFulfilled
	^package hasWorkingCopy
		and: [self isFulfilledBy: package workingCopy ancestry]! !

!MCVersionDependency methodsFor: 'testing' stamp: 'avi 3/4/2004 00:34'!
isFulfilledBy: anAncestry
	^ anAncestry ancestors includes: versionInfo! !

!MCVersionDependency methodsFor: 'testing' stamp: 'bf 4/19/2005 16:29'!
isFulfilledByAncestors
	^ package hasWorkingCopy
		and: [self isFulfilledByAncestorsOf: package workingCopy ancestry]! !

!MCVersionDependency methodsFor: 'testing' stamp: 'nk 7/13/2004 08:45'!
isFulfilledByAncestorsOf: anAncestry
	^ anAncestry hasAncestor: versionInfo! !

!MCVersionDependency methodsFor: 'testing' stamp: 'bf 4/19/2005 16:29'!
isOlder
	"Answer true if I represent an older version of a package that is loaded."
	^ package hasWorkingCopy
		and: [self isFulfilled not
			and: [ self isFulfilledByAncestors
				and: [package workingCopy modified not]]]! !

!MCVersionDependency methodsFor: 'accessing' stamp: 'avi 1/19/2004 15:40'!
package
	^ package! !

!MCVersionDependency methodsFor: 'accessing' stamp: 'bf 2/24/2012 14:40'!
packageAndBranchName
	^self versionInfo packageAndBranchName! !

!MCVersionDependency methodsFor: 'printing' stamp: 'nice 12/16/2013 22:57'!
printOn: aStream
	super printOn: aStream.
	aStream nextPut: $(.
	versionInfo printOn: aStream.
	aStream nextPut: $).! !

!MCVersionDependency methodsFor: 'accessing' stamp: 'avi 2/12/2004 19:38'!
repositoryGroup
	^ self package workingCopy repositoryGroup! !

!MCVersionDependency methodsFor: 'resolving' stamp: 'nk 6/13/2004 19:21'!
resolve
	^ self repositoryGroup
		versionWithInfo: versionInfo
		ifNone: [ MCRepositoryGroup default versionWithInfo: versionInfo ifNone: []]! !

!MCVersionDependency methodsFor: 'accessing' stamp: 'avi 1/19/2004 15:40'!
versionInfo
	^ versionInfo! !

Object subclass: #MCVersionLoader
	instanceVariableNames: 'versions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Loading'!

!MCVersionLoader class methodsFor: 'loading' stamp: 'avi 1/24/2004 20:06'!
loadVersion: aVersion
	self new
		addVersion: aVersion;
		load! !

!MCVersionLoader class methodsFor: 'instance creation' stamp: 'avi 1/24/2004 19:51'!
new
	^ self basicNew initialize! !

!MCVersionLoader methodsFor: 'loading' stamp: 'cwp 11/7/2004 17:06'!
addDependency: aDependency
	| dep |
	aDependency isCurrent ifTrue: [^ self].
	(self depAgeIsOk: aDependency) ifFalse: [^ self].
	dep := aDependency resolve.
	dep
		ifNil: [self confirmMissingDependency: aDependency]
		ifNotNil: [(versions includes: dep) ifFalse: [self addVersion: dep]]! !

!MCVersionLoader methodsFor: 'loading' stamp: 'cwp 11/7/2004 17:04'!
addVersion: aVersion
	aVersion dependencies do: [ :ea | self addDependency: ea].
	versions add: aVersion.
! !

!MCVersionLoader methodsFor: 'checking' stamp: 'avi 1/24/2004 20:15'!
checkForModifications
	| modifications |
	modifications := versions select: [:ea | ea package workingCopy modified].
	modifications isEmpty ifFalse: [self warnAboutLosingChangesTo: modifications].! !

!MCVersionLoader methodsFor: 'checking' stamp: 'cwp 11/7/2004 17:00'!
checkIfDepIsOlder: aDependency
	^ aDependency isOlder not 
		or: [self confirm: 'load older dependency ', aDependency versionInfo name , '?']! !

!MCVersionLoader methodsFor: 'checking' stamp: 'cwp 11/7/2004 17:06'!
confirmMissingDependency: aDependency
	| name |
	name := aDependency versionInfo name.
	(self confirm: 'Can''t find dependency ', name, '. ignore?')
		ifFalse: [self error: 'Can''t find dependency ', name]! !

!MCVersionLoader methodsFor: 'checking' stamp: 'cwp 11/7/2004 17:02'!
depAgeIsOk: aDependency
	^ aDependency isOlder not 
		or: [self confirm: 'load older dependency ', aDependency versionInfo name , '?']! !

!MCVersionLoader methodsFor: 'initialize-release' stamp: 'avi 1/24/2004 19:51'!
initialize
	versions := OrderedCollection new! !

!MCVersionLoader methodsFor: 'loading' stamp: 'bf 5/11/2012 13:31'!
load
	| loader |
	self checkForModifications.
	loader := MCPackageLoader new.
	versions do: [:ea |
		loader appendToPreamble: (String streamContents: [:s | ea logLoadingOn: s]).
		ea canOptimizeLoading
			ifTrue: [ea patch applyTo: loader]
			ifFalse: [loader updatePackage: ea package withSnapshot: ea snapshot]].
	loader loadWithNameLike: versions first info name.
	versions do: [:ea | ea workingCopy loaded: ea]! !

!MCVersionLoader methodsFor: 'checking' stamp: 'avi 1/24/2004 20:17'!
warnAboutLosingChangesTo: versionCollection
	self notify: (String streamContents: [:s |
		s nextPutAll: 'You are about to load new versions of the following packages that have unsaved changes in the image.  If you continue, you will lose these changes.'; cr.
		versionCollection do:
			[:ea |
			s cr; space; space; nextPutAll: ea package name]])! !

Object subclass: #MCVersionMerger
	instanceVariableNames: 'records merger'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

!MCVersionMerger class methodsFor: 'merging' stamp: 'bf 12/5/2004 12:35'!
mergeVersion: aVersion
	self new
		addVersion: aVersion;
		mergeWithNameLike: aVersion info name! !

!MCVersionMerger class methodsFor: 'instance creation' stamp: 'avi 2/13/2004 01:41'!
new
	^ self basicNew initialize! !

!MCVersionMerger methodsFor: 'accessing' stamp: 'nice 12/27/2009 03:11'!
addVersion: aVersion
	
	records add: (MCMergeRecord version: aVersion).
	aVersion dependencies do:
		[:ea | | dep |
		dep := ea resolve.
		(records anySatisfy: [:r | r version = dep]) ifFalse: [self addVersion: dep]]! !

!MCVersionMerger methodsFor: 'initialize-release' stamp: 'abc 2/13/2004 15:50'!
initialize
	records := OrderedCollection new.
	merger := MCThreeWayMerger new.! !

!MCVersionMerger methodsFor: 'merging' stamp: 'abc 2/13/2004 17:15'!
merge
	records do: [:ea | merger addBaseSnapshot: ea packageSnapshot].
	records do: [:ea | merger applyPatch: ea mergePatch].
	self resolveConflicts ifTrue:
		[merger load.
		records do: [:ea | ea updateWorkingCopy]].! !

!MCVersionMerger methodsFor: 'merging' stamp: 'bf 12/5/2004 12:32'!
mergeWithNameLike: baseName
	records do: [:ea | merger addBaseSnapshot: ea packageSnapshot].
	records do: [:ea | merger applyPatch: ea mergePatch].
	self resolveConflicts ifTrue:
		[merger loadWithNameLike: baseName.
		records do: [:ea | ea updateWorkingCopy]].! !

!MCVersionMerger methodsFor: 'testing' stamp: 'bf 4/26/2005 14:29'!
resolveConflicts
	(records allSatisfy: [:ea | ea isAncestorMerge]) ifTrue: [MCNoChangesException signal. ^ false].
	^ ((MCMergeResolutionRequest new merger: merger)
		signal: 'Merging ', records first version info name) = true! !

Object subclass: #MCVersionNotification
	instanceVariableNames: 'version ancestor repository changes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Repositories'!

!MCVersionNotification class methodsFor: 'instance creation' stamp: 'avi 8/26/2004 14:27'!
version: aVersion repository: aRepository
	^ self basicNew initializeWithVersion: aVersion repository: aRepository! !

!MCVersionNotification methodsFor: 'accessing' stamp: 'avi 8/26/2004 15:13'!
fromAddress
	^ 'monticello@beta4.com'! !

!MCVersionNotification methodsFor: 'private' stamp: 'cmm 4/6/2019 15:41'!
initializeWithVersion: aVersion repository: aRepository
	version := aVersion.
	repository := aRepository.
	ancestor := repository withCache closestAncestorVersionFor: version info ifNone: []. 
	changes := ancestor
				ifNil: [#()]
				ifNotNil: [(version snapshot patchRelativeToBase: ancestor snapshot) 							operations sorted]! !

!MCVersionNotification methodsFor: 'accessing' stamp: 'avi 8/26/2004 15:12'!
messageText
	^ String streamContents:
		[:s |
		s nextPutAll: 'Committed to repository: ', repository description; cr; cr.
		s nextPutAll: version summary.
		changes isEmpty ifFalse:
			[s cr; cr.
			s nextPutAll: '-----------------------------------------------------'; cr.
			s nextPutAll: 'Changes since ', ancestor info name, ':'; cr.
			changes do:
			[:ea |
			s cr; nextPutAll: ea summary; cr.
			s nextPutAll: ea sourceString]]]! !

!MCVersionNotification methodsFor: 'private' stamp: 'avi 8/26/2004 15:15'!
messageTo: aString
	| message |
	message := MailMessage empty.
	message setField: 'from' toString: self fromAddress.
	message setField: 'to' toString: aString.
	message setField: 'subject' toString: '[MC] ', version info name.
	message body: (MIMEDocument contentType: 'text/plain' content: self messageText).
	^ message! !

!MCVersionNotification methodsFor: 'notifying' stamp: 'ul 10/18/2018 01:44'!
notify: aString
	| message |
	message := self messageTo: aString.
	SMTPClient
		deliverMailFrom: message from
		to: { message to }
		text: message text
		usingServer: MailSender smtpServer! !

Object subclass: #MCVersionSorter
	instanceVariableNames: 'layers depthIndex depths stepparents roots'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

MCVersionSorter subclass: #MCFilteredVersionSorter
	instanceVariableNames: 'target'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

!MCFilteredVersionSorter methodsFor: 'accessing' stamp: 'bf 5/28/2005 01:14'!
addVersionInfo: aVersionInfo
	(aVersionInfo hasAncestor: target)
		ifTrue: [super addVersionInfo: aVersionInfo]
! !

!MCFilteredVersionSorter methodsFor: 'private' stamp: 'bf 12/19/2009 17:44'!
processVersionInfo: aVersionInfo
	| success |
	aVersionInfo = target ifTrue: [^ true].
	(aVersionInfo hasAncestor: target) ifFalse: [^false].
	self pushLayer.
	success := (self knownAncestorsOf: aVersionInfo) anySatisfy:
				[:ea | self processVersionInfo: ea].
	self popLayer.
	success ifTrue: [self addToCurrentLayer: aVersionInfo].
	^ success	! !

!MCFilteredVersionSorter methodsFor: 'accessing' stamp: 'avi 8/31/2003 21:27'!
target: aVersionInfo
	target := aVersionInfo! !

!MCVersionSorter class methodsFor: 'instance creation' stamp: 'avi 9/13/2004 18:03'!
new
	^ self basicNew initialize! !

!MCVersionSorter methodsFor: 'accessing' stamp: 'avi 9/11/2004 14:19'!
addAllAncestorsOf: aVersionInfo to: aSet
	(aSet includes: aVersionInfo) ifTrue: [^ self].
	aSet add: aVersionInfo.
	(self knownAncestorsOf: aVersionInfo) do:
		[:ea |
		self addAllAncestorsOf: ea to: aSet]! !

!MCVersionSorter methodsFor: 'accessing' stamp: 'avi 8/31/2003 21:30'!
addAllVersionInfos: aCollection
	aCollection do: [:ea | self addVersionInfo: ea]! !

!MCVersionSorter methodsFor: 'accessing' stamp: 'avi 3/2/2004 12:53'!
addToCurrentLayer: aVersionInfo
	| layer |
	layer := layers at: depthIndex.
	(layer includes: aVersionInfo) ifFalse:
		[depths at: aVersionInfo ifPresent:
			[:i |
			i < depthIndex
				ifTrue: [(layers at: i) remove: aVersionInfo]
				ifFalse: [^ false]].
		layer add: aVersionInfo.
		depths at: aVersionInfo put: depthIndex.
		^ true].
	^ false ! !

!MCVersionSorter methodsFor: 'accessing' stamp: 'avi 9/11/2004 10:49'!
addVersionInfo: aVersionInfo
	roots add: aVersionInfo.
	self registerStepChildrenOf: aVersionInfo seen: Set new! !

!MCVersionSorter methodsFor: 'ancestry' stamp: 'avi 9/11/2004 14:17'!
allAncestorsOf: aVersionInfo
	| all |
	all := Set new.
	self addAllAncestorsOf: aVersionInfo to: all.
	^ all! !

!MCVersionSorter methodsFor: 'initialize-release' stamp: 'avi 9/11/2004 10:37'!
initialize
	stepparents := Dictionary new.
	roots := OrderedCollection new.! !

!MCVersionSorter methodsFor: 'ancestry' stamp: 'avi 9/11/2004 14:37'!
knownAncestorsOf: aVersionInfo
	^ aVersionInfo ancestors, (self stepParentsOf: aVersionInfo) asArray! !

!MCVersionSorter methodsFor: 'accessing' stamp: 'ab 8/17/2003 15:53'!
layers
	^ layers! !

!MCVersionSorter methodsFor: 'private' stamp: 'ab 8/17/2003 15:33'!
popLayer
	depthIndex := depthIndex - 1! !

!MCVersionSorter methodsFor: 'private' stamp: 'avi 9/11/2004 10:39'!
processVersionInfo: aVersionInfo
	(self addToCurrentLayer: aVersionInfo) ifTrue:
		[self pushLayer.
		(self knownAncestorsOf: aVersionInfo) do: [:ea | self processVersionInfo: ea].
		self popLayer]
! !

!MCVersionSorter methodsFor: 'private' stamp: 'avi 8/31/2003 21:34'!
pushLayer
	depthIndex := depthIndex + 1.
	depthIndex > layers size ifTrue: [layers add: OrderedCollection new].
	! !

!MCVersionSorter methodsFor: 'private' stamp: 'avi 9/11/2004 14:34'!
registerStepChildrenOf: aVersionInfo seen: aSet
	(aSet includes: aVersionInfo) ifTrue: [^ self].
	aSet add: aVersionInfo.
	aVersionInfo stepChildren do: [:ea | (self stepParentsOf: ea) add: aVersionInfo].
	aVersionInfo ancestors do: [:ea | self registerStepChildrenOf: ea seen: aSet].! !

!MCVersionSorter methodsFor: 'versions' stamp: 'avi 9/11/2004 10:37'!
sortedVersionInfos
	layers := OrderedCollection with: OrderedCollection new.
	depthIndex := 1.
	depths := Dictionary new.
	roots do: [:ea | self processVersionInfo: ea].
	^ layers gather: [:ea | ea]! !

!MCVersionSorter methodsFor: 'ancestry' stamp: 'avi 9/11/2004 10:40'!
stepParentsOf: aVersionInfo
	^ (stepparents at: aVersionInfo ifAbsentPut: [Set new])! !

Object subclass: #MCWriter
	instanceVariableNames: 'stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Storing'!

MCWriter subclass: #MCMczWriter
	instanceVariableNames: 'zip infoWriter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Storing'!

MCMczWriter subclass: #MCMcdWriter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Storing'!

!MCMcdWriter class methodsFor: 'accessing' stamp: 'avi 2/13/2004 23:09'!
readerClass
	^ MCMcdReader! !

!MCMcdWriter methodsFor: 'visiting' stamp: 'nice 6/2/2013 16:24'!
writeBaseInfo: aVersionInfo
	| string |
	string := self serializeVersionInfo: aVersionInfo.
	self addString: string at: 'base' encodedTo: 'utf8'.
! !

!MCMcdWriter methodsFor: 'visiting' stamp: 'avi 2/17/2004 01:48'!
writeDefinitions: aVersion
	self writeBaseInfo: aVersion baseInfo.
	self writePatch: aVersion patch.! !

!MCMcdWriter methodsFor: 'visiting' stamp: 'nice 6/2/2013 16:24'!
writeNewDefinitions: aCollection
	self addString: (self serializeDefinitions: aCollection) at: 'new/source.', self snapshotWriterClass extension encodedTo: 'utf8'.! !

!MCMcdWriter methodsFor: 'visiting' stamp: 'nice 6/2/2013 16:24'!
writeOldDefinitions: aCollection
	self addString: (self serializeDefinitions: aCollection) at: 'old/source.', self snapshotWriterClass extension encodedTo: 'utf8'.! !

!MCMcdWriter methodsFor: 'visiting' stamp: 'mt 5/12/2016 11:39'!
writePatch: aPatch 
	| old new |
	old := OrderedCollection new.
	new := OrderedCollection new.
	aPatch operations do:
		[ : ea | ea isRemoval ifTrue: [ old add: ea definition ].
		ea isAddition ifTrue: [ new add: ea definition ].
		ea isModification ifTrue:
			[ old add: ea baseDefinition.
			new add: ea definition ] ].
	(old isEmpty and: [ new isEmpty ]) ifTrue: [ MCEmptyDiffyVersion signal ].
	self
		 writeOldDefinitions: old ;
		 writeNewDefinitions: new ;
		
		addString: (self serializeInBinary: aPatch)
		at: 'patch.bin'! !

!MCMczWriter class methodsFor: 'fileIn/out' stamp: 'avi 2/17/2004 02:14'!
fileOut: aVersion on: aStream
	| inst |
	inst := self on: aStream.
	inst writeVersion: aVersion.
	inst flush.
	
! !

!MCMczWriter class methodsFor: 'instance creation' stamp: 'avi 9/13/2004 18:03'!
new
	^ self basicNew initialize! !

!MCMczWriter class methodsFor: 'accessing' stamp: 'cwp 8/1/2003 12:35'!
readerClass
	^ MCMczReader! !

!MCMczWriter methodsFor: 'writing' stamp: 'cwp 8/1/2003 01:38'!
addString: string at: path
	| member |
	member := zip addString: string as: path.
	member desiredCompressionMethod: ZipArchive compressionDeflated 
	! !

!MCMczWriter methodsFor: 'writing' stamp: 'nice 6/2/2013 16:24'!
addString: string at: path encodedTo: encodingName
	| member |
	member := zip addString: (string convertToEncoding: encodingName) as: path.
	member desiredCompressionMethod: ZipArchive compressionDeflated 
	! !

!MCMczWriter methodsFor: 'writing' stamp: 'avi 2/17/2004 02:17'!
flush
	zip writeTo: stream.
	stream close! !

!MCMczWriter methodsFor: 'accessing' stamp: 'avi 2/17/2004 01:54'!
format
	^ '1'! !

!MCMczWriter methodsFor: 'initializing' stamp: 'cwp 8/1/2003 01:18'!
initialize
	zip := ZipArchive new.
! !

!MCMczWriter methodsFor: 'serializing' stamp: 'nice 6/2/2013 15:38'!
serializeDefinitions: aCollection
	^String streamContents: [:aStream |
		| writer |
		writer := self snapshotWriterClass on: aStream.
		writer writeDefinitions: aCollection]! !

!MCMczWriter methodsFor: 'serializing' stamp: 'avi 9/28/2004 14:24'!
serializeInBinary: aSnapshot
	| writer s |
	s := RWBinaryOrTextStream on: String new.
	writer := DataStream on: s.
	writer nextPut: aSnapshot.
	^ s contents! !

!MCMczWriter methodsFor: 'serializing' stamp: 'cwp 8/13/2003 01:06'!
serializePackage: aPackage
	^ '(name ''', aPackage name, ''')'! !

!MCMczWriter methodsFor: 'serializing' stamp: 'avi 2/17/2004 01:47'!
serializeVersionInfo: aVersionInfo
	infoWriter ifNil: [infoWriter := MCVersionInfoWriter new].
	^ String streamContents:
		[:s |
		infoWriter stream: s.
		infoWriter writeVersionInfo: aVersionInfo]! !

!MCMczWriter methodsFor: 'accessing' stamp: 'avi 2/17/2004 02:07'!
snapshotWriterClass
	^ MCStWriter! !

!MCMczWriter methodsFor: 'visiting' stamp: 'avi 2/17/2004 01:48'!
writeDefinitions: aVersion
	self writeSnapshot: aVersion snapshot! !

!MCMczWriter methodsFor: 'visiting' stamp: 'avi 2/17/2004 01:56'!
writeFormat
"	self addString: self format at: 'format'."! !

!MCMczWriter methodsFor: 'visiting' stamp: 'nice 6/2/2013 16:24'!
writePackage: aPackage
	self addString: (self serializePackage: aPackage) at: 'package' encodedTo: 'utf8'! !

!MCMczWriter methodsFor: 'visiting' stamp: 'mt 5/12/2016 11:40'!
writeSnapshot: aSnapshot 
	aSnapshot definitions ifEmpty: [ MCEmptyVersion signal ].
	self
		
		addString: (self serializeDefinitions: aSnapshot definitions)
		at: 'snapshot/source.' , self snapshotWriterClass extension
		encodedTo: 'utf8' ;
		
		addString: (self serializeInBinary: aSnapshot)
		at: 'snapshot.bin'! !

!MCMczWriter methodsFor: 'visiting' stamp: 'avi 9/13/2004 16:49'!
writeVersion: aVersion
	self writeFormat.
	self writePackage: aVersion package.
	self writeVersionInfo: aVersion info.
	self writeDefinitions: aVersion.
	aVersion dependencies do: [:ea | self writeVersionDependency: ea]! !

!MCMczWriter methodsFor: 'visiting' stamp: 'nice 6/2/2013 16:24'!
writeVersionDependency: aVersionDependency
	| string |
	string := (self serializeVersionInfo: aVersionDependency versionInfo).
	self addString: string at: 'dependencies/', aVersionDependency package name encodedTo: 'utf8'! !

!MCMczWriter methodsFor: 'visiting' stamp: 'nice 6/2/2013 16:24'!
writeVersionInfo: aVersionInfo
	| string |
	string := self serializeVersionInfo: aVersionInfo.
	self addString: string at: 'version' encodedTo: 'utf8'.
! !

!MCMczWriter methodsFor: 'accessing' stamp: 'cwp 8/1/2003 00:06'!
zip
	^ zip! !

MCWriter subclass: #MCStWriter
	instanceVariableNames: 'initStream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Chunk Format'!

!MCStWriter class methodsFor: 'accessing' stamp: 'avi 1/20/2004 00:16'!
readerClass
	^ MCStReader! !

!MCStWriter methodsFor: 'writing' stamp: 'cwp 8/2/2003 02:34'!
chunkContents: aBlock
	stream cr; nextChunkPut: (String streamContents: aBlock); cr! !

!MCStWriter methodsFor: 'visiting' stamp: 'cwp 8/2/2003 11:02'!
visitClassDefinition: definition
	self writeClassDefinition: definition.
	definition hasClassInstanceVariables ifTrue: [self writeMetaclassDefinition: definition].
	definition hasComment ifTrue: [self writeClassComment: definition].! !

!MCStWriter methodsFor: 'visiting' stamp: 'al 10/9/2005 19:52'!
visitClassTraitDefinition: definition
	self chunkContents: [:s | s
		nextPutAll: definition baseTrait;
		nextPutAll: ' classTrait';
		cr; tab;
		nextPutAll: 'uses: ';
		nextPutAll: (definition classTraitComposition ifNil: ['{}'])]
! !

!MCStWriter methodsFor: 'visiting' stamp: 'al 10/9/2005 19:40'!
visitMetaclassDefinition: definition
	self writeMetaclassDefinition: definition! !

!MCStWriter methodsFor: 'visiting' stamp: 'avi 2/17/2004 02:23'!
visitMethodDefinition: definition
	self writeMethodPreamble: definition.
	self writeMethodSource: definition.
	self writeMethodPostscript.
	self writeMethodInitializer: definition.! !

!MCStWriter methodsFor: 'visiting' stamp: 'cwp 8/2/2003 11:02'!
visitOrganizationDefinition: defintion
	defintion categories do: [:cat | self writeCategory: cat].
! !

!MCStWriter methodsFor: 'visiting' stamp: 'bf 8/12/2009 21:41'!
visitScriptDefinition: definition
	self writeScriptDefinition: definition
! !

!MCStWriter methodsFor: 'visiting' stamp: 'al 10/9/2005 19:40'!
visitTraitDefinition: definition
	self writeClassDefinition: definition.
	definition hasComment ifTrue: [self writeClassComment: definition].! !

!MCStWriter methodsFor: 'writing' stamp: 'cwp 8/2/2003 01:46'!
writeCategory: categoryName
	stream
		nextChunkPut: 'SystemOrganization addCategory: ', categoryName printString;
		cr! !

!MCStWriter methodsFor: 'writing' stamp: 'ab 8/17/2003 17:09'!
writeClassComment: definition
	stream
		cr;
		nextPut: $!!;
		nextPutAll: definition className;
		nextPutAll: ' commentStamp: ';
		store: definition commentStamp;
		nextPutAll: ' prior: 0!!';
		cr;
		nextChunkPut: definition comment;
		cr.! !

!MCStWriter methodsFor: 'writing' stamp: 'cwp 8/2/2003 02:16'!
writeClassDefinition: definition
	self chunkContents: [:s | definition printDefinitionOn: stream]! !

!MCStWriter methodsFor: 'writing' stamp: 'dvf 9/8/2004 10:28'!
writeDefinitions: aCollection
	"initStream is an ugly hack until we have proper init defs"
	initStream := String new writeStream.

	(MCDependencySorter sortItems: aCollection)
		do: [:ea | ea accept: self]
		displayingProgress: 'Writing definitions...'.
	
	stream nextPutAll: initStream contents.! !

!MCStWriter methodsFor: 'writing' stamp: 'al 12/2/2005 15:17'!
writeMetaclassDefinition: definition
	self chunkContents: [:str |
		str	nextPutAll: definition className;
			nextPutAll: ' class';
			cr; tab.
			definition hasClassTraitComposition ifTrue: [
				str	nextPutAll: 'uses: ';
					nextPutAll: definition classTraitCompositionString;
					cr; tab].
			str	nextPutAll: 'instanceVariableNames: ''';
				nextPutAll: definition classInstanceVariablesString;
				nextPut: $']! !

!MCStWriter methodsFor: 'writing' stamp: 'avi 2/17/2004 02:24'!
writeMethodInitializer: aMethodDefinition
	aMethodDefinition isInitializer ifTrue:
		[initStream nextChunkPut: aMethodDefinition className, ' initialize'; cr]! !

!MCStWriter methodsFor: 'writing' stamp: 'cwp 8/2/2003 12:43'!
writeMethodPostscript
	stream
		space;
		nextPut: $!!;
		cr! !

!MCStWriter methodsFor: 'writing' stamp: 'avi 9/23/2003 17:42'!
writeMethodPreamble: definition
	stream
		cr;
		nextPut: $!!;
		nextPutAll: definition fullClassName;
		nextPutAll: ' methodsFor: ';
		nextPutAll: definition category asString printString;
		nextPutAll: ' stamp: ';
		nextPutAll: definition timeStamp asString printString;
		nextPutAll: '!!';
		cr! !

!MCStWriter methodsFor: 'writing' stamp: 'cwp 8/4/2003 01:35'!
writeMethodSource: definition
	stream nextChunkPut: definition source! !

!MCStWriter methodsFor: 'writing' stamp: 'bf 8/13/2009 00:21'!
writeScriptDefinition: definition
	stream nextChunkPut: (
		'(PackageInfo named: {1}) {2}: {3}'
		format: {
			"{1}" definition packageName printString.
			"{2}" definition scriptSelector. 
			"{3}" definition script printString
		}); cr! !

!MCStWriter methodsFor: 'writing' stamp: 'avi 2/17/2004 02:25'!
writeSnapshot: aSnapshot
	self writeDefinitions: aSnapshot definitions! !

MCWriter subclass: #MCVersionInfoWriter
	instanceVariableNames: 'written'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Storing'!

!MCVersionInfoWriter methodsFor: 'testing' stamp: 'avi 1/22/2004 21:10'!
isWritten: aVersionInfo
	^ self written includes: aVersionInfo! !

!MCVersionInfoWriter methodsFor: 'writing' stamp: 'avi 9/13/2004 16:53'!
writeVersionInfo: aVersionInfo
	(self isWritten: aVersionInfo)
		ifTrue: [^ stream nextPutAll: '(id ', aVersionInfo id asString printString, ')'].
	stream nextPut: $(.
	#(name message id date time author) 
		do: [:sel | 
			stream nextPutAll: sel.
			stream nextPut: $ .
			((aVersionInfo perform: sel) ifNil: ['']) asString printOn: stream.
			stream nextPut: $ ].
	stream nextPutAll: 'ancestors ('.
	aVersionInfo ancestors do: [:ea | self writeVersionInfo: ea].
	stream nextPutAll: ') stepChildren ('.
	aVersionInfo stepChildren do: [:ea | self writeVersionInfo: ea].
	stream nextPutAll: '))'.
	self wrote: aVersionInfo! !

!MCVersionInfoWriter methodsFor: 'accessing' stamp: 'avi 1/22/2004 21:03'!
written
	^ written ifNil: [written := Set new]! !

!MCVersionInfoWriter methodsFor: 'accessing' stamp: 'avi 1/22/2004 21:10'!
wrote: aVersionInfo
	self written add: aVersionInfo! !

!MCWriter class methodsFor: 'accessing' stamp: 'cwp 8/1/2003 15:00'!
extension
	^ self readerClass extension! !

!MCWriter class methodsFor: 'writing' stamp: 'cwp 8/1/2003 01:16'!
on: aStream
	^ self new stream: aStream! !

!MCWriter class methodsFor: 'accessing' stamp: 'cwp 7/28/2003 23:46'!
readerClass
	^ self subclassResponsibility ! !

!MCWriter methodsFor: 'accessing' stamp: 'cwp 8/1/2003 01:14'!
stream
	^ stream! !

!MCWriter methodsFor: 'accessing' stamp: 'ab 7/18/2003 21:37'!
stream: aStream
	stream := aStream! !

Object subclass: #MethodAddition
	instanceVariableNames: 'text category changeStamp requestor logSource myClass methodAndNode selector compiledMethod priorMethodOrNil'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Loading'!

!MethodAddition commentStamp: 'rej 2/25/2007 19:30' prior: 0!
I represent the addition of a method to a class.  I can produce the CompiledMethod, install it, and then notify the system that the method has been added.  This allows Monticello to implement atomic addition.  A loader can compile all classes and methods first and then install all methods only after they have been all compiled, and in a way that executes little code.½!

!MethodAddition methodsFor: 'compiling' stamp: 'rej 2/26/2007 10:51'!
compile
    "This method is the how compiling a method used to work.  All these steps were done at once.
     This method should not normally be used, because the whole point of MethodAddition is to let
	you first create a compiled method and then install the method later."
	self createCompiledMethod.
	self installMethod.
	self notifyObservers.
	^selector! !

!MethodAddition methodsFor: 'initialize-release' stamp: 'rej 2/25/2007 20:36'!
compile: aString classified: aString1 withStamp: aString2 notifying: aRequestor logSource: aBoolean inClass: aClass

	text := aString.
	category := aString1.
	changeStamp := aString2.
	requestor := aRequestor.
	logSource := aBoolean.
	myClass := aClass! !

!MethodAddition methodsFor: 'compiling' stamp: 'jr 1/15/2017 01:18'!
createCompiledMethod
	| notification |
	[methodAndNode := myClass
		compile: text asString
		environment: Environment current
		notifying: requestor
		trailer: (myClass defaultMethodTrailerIfLogSource: logSource) 
		ifFail: [^nil]]
			on: SyntaxErrorNotification do: [:exc |
				notification := exc.
				exc pass].
	notification ifNotNil: [notification newSource ifNotNil: [:newSource | text := newSource]].
	selector := methodAndNode selector.
	compiledMethod := methodAndNode method.
	self writeSourceToLog.
	priorMethodOrNil := myClass compiledMethodAt: selector ifAbsent: [nil].
! !

!MethodAddition methodsFor: 'compiling' stamp: 'rej 2/25/2007 22:09'!
installMethod
	myClass addSelectorSilently: selector withMethod: compiledMethod.
! !

!MethodAddition methodsFor: 'compiling' stamp: 'bf 5/4/2013 19:38'!
notifyObservers
	SystemChangeNotifier uniqueInstance 
		doSilently: [myClass organization classify: selector under: category suppressIfDefault: false].
	priorMethodOrNil isNil
		ifTrue: [SystemChangeNotifier uniqueInstance methodAdded: compiledMethod selector: selector inProtocol: category class: myClass requestor: requestor]
		ifFalse: [SystemChangeNotifier uniqueInstance methodChangedFrom: priorMethodOrNil to: compiledMethod selector: selector inClass: myClass requestor: requestor].
	"The following code doesn't seem to do anything."
	myClass instanceSide noteCompilationOf: selector meta: myClass isClassSide.
! !

!MethodAddition methodsFor: 'compiling' stamp: 'rej 2/25/2007 20:42'!
writeSourceToLog
	logSource ifTrue: [
		myClass logMethodSource: text forMethodWithNode: methodAndNode 
			inCategory: category withStamp: changeStamp notifying: requestor.
	].
! !

!Object methodsFor: '*monticello' stamp: 'dvf 8/10/2004 23:25'!
isConflict
	^false! !

!PseudoTrait methodsFor: '*monticello' stamp: 'jr 2/11/2017 17:25'!
asMCDefinitionBy: aMCReader
	^ aMCReader traitDefinitionFrom: self! !

ListItemWrapper subclass: #MCDependentsWrapper
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-UI'!

!MCDependentsWrapper methodsFor: 'converting' stamp: 'ar 2/14/2004 02:31'!
asString
	^item description! !

!MCDependentsWrapper methodsFor: 'accessing' stamp: 'avi 9/10/2004 17:54'!
contents
	| list workingCopies |
	workingCopies := model unsortedWorkingCopies.
	list := item requiredPackages collect: 
					[:each | 
					workingCopies detect: [:wc | wc package = each] ifNone: [nil]]
				thenSelect: [:x | x notNil].
	^list collect: [:each | self class with: each model: model]! !

!MCDependentsWrapper methodsFor: 'testing' stamp: 'ar 2/14/2004 02:31'!
hasContents
	^item requiredPackages isEmpty not! !

!MCDependentsWrapper methodsFor: 'accessing' stamp: 'ar 2/14/2004 02:41'!
item
	^item! !

!Time class methodsFor: '*monticello' stamp: 'nk 11/2/2003 10:51'!
fromString: aString
	^ self readFrom: (ReadStream on: aString).
! !

!FilePackage methodsFor: '*monticello' stamp: 'jr 2/12/2017 16:22'!
classDefinition: string with: chgRec
	| tokens theClass |
		
	tokens := Scanner new scanTokens: string.

	"tokens size = 11 ifFalse:[^doIts add: chgRec]."

	theClass := self getClass: (tokens at: 3).
	theClass definition: string.
	classOrder add: theClass.! !

!FilePackage methodsFor: '*monticello' stamp: 'avi 1/19/2004 23:47'!
doIts
	^ doIts! !

!FilePackage methodsFor: '*monticello' stamp: 'jr 2/11/2017 17:12'!
traitDefinition: string with: chgRec
	| tokens theTrait |
	tokens := Scanner new scanTokens: string.
	theTrait := self getTrait: (tokens at: 3).
	theTrait definition: string.
	classOrder add: theTrait.! !

!ByteString methodsFor: '*monticello' stamp: 'cmm 3/3/2011 14:35'!
asMCVersionName 
	^ MCVersionName on: self! !

ByteString variableByteSubclass: #MCVersionName
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Modeling'!

!MCVersionName commentStamp: 'cmm 3/4/2011 13:32' prior: 0!
A MCVersionName models the Monticello file / version name, in the format [Package]-[Author].[version-number]([ancestorAuthor.ancestorVersionNumber]).[mc?].

Any file-extension, if present, is ignored.!

!MCVersionName class methodsFor: 'instance creation' stamp: 'cmm 3/3/2011 14:19'!
on: aString 
	"aString may be with or without a mc? extension."
	^ (self new: aString size)
		replaceFrom: 1
			to: aString size
			with: aString
			startingAt: 1 ;
		yourself! !

!MCVersionName methodsFor: 'comparing' stamp: 'cmm 3/12/2011 11:48'!
= aByteString 
	"Ignore any file-extension for comparison of MCVersionNames."
	| myVersionName |
	aByteString isString ifFalse: [ ^ false ].
	myVersionName := self versionName.
	^ (myVersionName
		compare: myVersionName
		with: aByteString asMCVersionName versionName
		collated: AsciiOrder) = 2! !

!MCVersionName methodsFor: 'converting' stamp: 'cmm 3/3/2011 14:33'!
asMCVersionName
	^ self! !

!MCVersionName methodsFor: 'accessing' stamp: 'ul 3/23/2011 04:55'!
author
	"The author initials embedded in the filename."
	
	^(self versionName copyAfterLast: $-) copyUpTo: $.! !

!MCVersionName methodsFor: 'accessing' stamp: 'bf 2/24/2012 15:58'!
baseVersionName
	"The version name of our ancestor, if this is a Diffy filename."

	^(MCDiffyVersion baseNameFrom: (self copyUpThrough: $))) asMCVersionName! !

!MCVersionName methodsFor: 'comparing' stamp: 'cmm 3/4/2011 10:36'!
hash
	^ self versionName hash! !

!MCVersionName methodsFor: 'testing' stamp: 'cmm 4/24/2011 20:04'!
isValid
	^ [ (self endsWith: '.mcm') or:
		[ self packageName notEmpty and:
			[ self author notEmpty and: [ self versionNumber > 0 ] ] ] ]
		on: Error
		do:
			[ : err | false ]! !

!MCVersionName methodsFor: 'as yet unclassified' stamp: 'ct 9/17/2020 10:50'!
name

	^ self! !

!MCVersionName methodsFor: 'accessing' stamp: 'bf 2/24/2012 13:20'!
packageAndBranchName
	"The MC package name, possibly with branch name, embedded into this filename."
	
	^self versionName copyUpToLast: $-! !

!MCVersionName methodsFor: 'accessing' stamp: 'bf 2/24/2012 13:22'!
packageName
	"The MC Package name embedded into this filename."
	
	^self packageAndBranchName copyUpTo: $.! !

!MCVersionName methodsFor: 'private' stamp: 'cmm 3/4/2011 10:12'!
species
	^ ByteString! !

!MCVersionName methodsFor: 'accessing' stamp: 'ul 5/4/2013 00:02'!
versionName
	"Answer my version name as a ByteString, without the file suffix or any ancestor-attributes."
	| end |
	self isEmpty ifTrue: [^ String empty]. 
	end := self indexOf: $( ifAbsent: [
		(self size > 4 
			and: [ (self at: self size - 3) == $.
			and: [ (self at: self size - 2) == $m
			and: [ (self at: self size - 1) == $c ] ] ])
				ifTrue: [self size - 3]
				ifFalse: [self size + 1]].
	^self first: end - 1! !

!MCVersionName methodsFor: 'accessing' stamp: 'bf 7/4/2012 14:55'!
versionNumber
	"The Integer version number identified by this filename."
	| v |
	v := (self versionName copyAfterLast: $-) copyAfter: $..
	^(v notEmpty and: [v first isDigit])
		ifTrue: [ v asInteger ]
	 	ifFalse: [ 0 ]! !

!ChangeRecord methodsFor: '*monticello' stamp: 'avi 9/14/2004 14:27'!
asMethodDefinition
	^ MCMethodDefinition 
		className: class
		classIsMeta: meta
		selector: self methodSelector
		category: category
		timeStamp: stamp
		source: self string! !

!PackageInfo methodsFor: '*monticello' stamp: 'cmm 3/21/2009 14:08'!
mcPackage
	^ MCPackage named: self packageName! !

!PackageInfo methodsFor: '*monticello' stamp: 'cmm 3/18/2009 20:42'!
workingCopy
	^ self mcPackage workingCopy! !

!ChangeSet methodsFor: '*Monticello-testing' stamp: 'eem 8/27/2015 16:17'!
isForPackageLoad
	| packageName |
	^(name includes: $-)
	  and: [name last isDigit
	  and: [packageName := name first: ((name indexOf: $. ifAbsent: [name size])
											min: (name lastIndexOf: $-)) - 1.
		MCWorkingCopy allManagers anySatisfy: [:wc| wc packageName = packageName]]]! !
MCRepositoryInspector initialize!
MCWorkingCopyBrowser initialize!
MCMethodDefinition initialize!
MCPackageManager initialize!
MCWorkingCopy initialize!
MCVersionReader initialize!
MCHttpRepository initialize!
MCRepository initialize!
